@@ -12,9 +12,10 @@
 # serve to show the default value.
 
 import sys
+import sympy
 
 # If your extensions are in another directory, add it here.
-sys.path.extend(['../sympy', 'ext'])
+sys.path = ['../sympy', 'ext'] + sys.path
 
 # General configuration
 # ---------------------
@@ -22,7 +23,7 @@ sys.path.extend(['../sympy', 'ext'])
 # Add any Sphinx extension module names here, as strings. They can be extensions
 # coming with Sphinx (named 'sphinx.addons.*') or your custom ones.
 extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode', 'sphinx.ext.mathjax',
-              'numpydoc', 'sympylive',]
+              'numpydoc', 'sympylive', 'sphinx.ext.graphviz', ]
 
 # Use this to use pngmath instead
 #extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode', 'sphinx.ext.pngmath', ]
@@ -41,15 +42,15 @@ master_doc = 'index'
 
 # General substitutions.
 project = 'SymPy'
-copyright = '2008, 2009, 2010, 2011, 2012 SymPy Development Team'
+copyright = '2013 SymPy Development Team'
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '0.7.2'
+version = sympy.__version__
 # The full version, including alpha/beta/rc tags.
-release = '0.7.2'
+release = version
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
@@ -57,9 +58,6 @@ release = '0.7.2'
 # Else, today_fmt is used as the format for a strftime call.
 today_fmt = '%B %d, %Y'
 
-# Translations:
-locale_dirs = ["i18n/"]
-
 # List of documents that shouldn't be included in the build.
 #unused_docs = []
 
@@ -97,7 +95,23 @@ html_last_updated_fmt = '%b %d, %Y'
 
 html_logo = '_static/sympylogo.png'
 html_favicon = '../logo/SymPy-Favicon.ico'
-html_theme_options = {'collapsiblesidebar': True}
+# See http://sphinx-doc.org/theming.html#builtin-themes.
+html_theme_options = {
+    'collapsiblesidebar': True,
+    'relbarbgcolor': '#2f441e',
+    'sidebarbgcolor': '#3b5526',
+    'sidebarbtncolor': '#4F663C',
+    'sidebarlinkcolor': '#81B953',
+    'linkcolor': '#29A329',
+    'visitedlinkcolor': '#307748',
+    'headtextcolor': '#2f441e',
+    'footerbgcolor': '#293b1b',
+    'headlinkcolor': '#AAAAAA',
+    'sidebartextcolor': '#DDDDDD',
+    'footertextcolor': '#DDDDDD',
+    'relbartextcolor': '#DDDDDD',
+    'relbarlinkcolor': '#81B953',
+}
 
 # If true, SmartyPants will be used to convert quotes and dashes to
 # typographically correct entities.
@@ -143,8 +157,9 @@ latex_documents = [('index', 'sympy-%s.tex' % release, 'SymPy Documentation',
 # Additional stuff for the LaTeX preamble.
 # Tweaked to work with XeTeX.
 latex_elements = {
-  'babel':     '',
-  'fontenc': r'''
+    'babel':     '',
+    'fontenc': r'''
+\usepackage{bm}
 \usepackage{amssymb}
 \usepackage{fontspec}
 \defaultfontfeatures{Mapping=tex-text}
@@ -152,14 +167,18 @@ latex_elements = {
 \setsansfont{DejaVu Sans}
 \setmonofont{DejaVu Sans Mono}
 ''',
-  'fontpkg':   '',
-  'inputenc':  '',
-  'utf8extra': '',
-  'preamble':  ''
+    'fontpkg':   '',
+    'inputenc':  '',
+    'utf8extra': '',
+    'preamble':  r'''
+% redefine \LaTeX to be usable in math mode
+\expandafter\def\expandafter\LaTeX\expandafter{\expandafter\text\expandafter{\LaTeX}}
+'''
 }
 
 # SymPy logo on title page
-latex_logo = '_static/sympylogo.png'
+html_logo = '_static/sympylogo.png'
+latex_logo = '_static/sympylogo_big.png'
 
 # Documents to append as an appendix to all manuals.
 #latex_appendices = []
@@ -171,18 +190,20 @@ latex_show_pagerefs = True
 latex_use_modindex = False
 
 default_role = 'math'
-pngmath_divpng_args = ['-gamma 1.5','-D 110']
+pngmath_divpng_args = ['-gamma 1.5', '-D 110']
 # Note, this is ignored by the mathjax extension
 # Any \newcommand should be defined in the file
-pngmath_latex_preamble =  '\\usepackage{amsmath}\n'+\
-              '\\usepackage{bm}\n'+\
-              '\\usepackage{amsfonts}\n'+\
-              '\\usepackage{amssymb}\n'+\
-              '\\setlength{\\parindent}{0pt}\n'
+pngmath_latex_preamble = '\\usepackage{amsmath}\n' \
+    '\\usepackage{bm}\n' \
+    '\\usepackage{amsfonts}\n' \
+    '\\usepackage{amssymb}\n' \
+    '\\setlength{\\parindent}{0pt}\n'
 
 texinfo_documents = [
-  (master_doc, 'sympy', 'SymPy Documentation',
-   'SymPy Development Team',
-   'SymPy', 'Computer algebra system (CAS) in Python', 'Programming',
-   1),
+    (master_doc, 'sympy', 'SymPy Documentation', 'SymPy Development Team',
+   'SymPy', 'Computer algebra system (CAS) in Python', 'Programming', 1),
 ]
+
+# Use svg for graphviz
+
+graphviz_output_format = 'svg'
