@@ -10,7 +10,8 @@ tensor, Laplace operator, ...
 """
 
 from sympy import var, sin, cos, pprint, Matrix, eye, trigsimp, Eq, \
-        Function, simplify, sinh, cosh, expand
+    Function, simplify, sinh, cosh, expand
+
 
 def laplace(f, g_inv, g_det, X):
     """
@@ -27,6 +28,7 @@ def laplace(f, g_inv, g_det, X):
                 f.diff(X[alpha]) / (2*g_det)
     return r
 
+
 def transform(name, X, Y, g_correct=None, recursive=False):
     """
     Transforms from cartesian coordinates X to any curvilinear coordinates Y.
@@ -44,32 +46,32 @@ def transform(name, X, Y, g_correct=None, recursive=False):
     recursive ... apply recursive trigonometric simplification (use only when
                   needed, as it is an expensive operation)
     """
-    print "_"*80
-    print "Transformation:", name
+    print("_"*80)
+    print("Transformation:", name)
     for x, y in zip(X, Y):
         pprint(Eq(y, x))
     J = X.jacobian(Y)
-    print "Jacobian:"
+    print("Jacobian:")
     pprint(J)
     g = J.T*eye(J.shape[0])*J
 
     g = g.applyfunc(expand)
     #g = g.applyfunc(trigsimp)
-    print "metric tensor g_{ij}:"
+    print("metric tensor g_{ij}:")
     pprint(g)
     if g_correct is not None:
         g = g_correct
-        print "metric tensor g_{ij} specified by hand:"
+        print("metric tensor g_{ij} specified by hand:")
         pprint(g)
-    print "inverse metric tensor g^{ij}:"
-    g_inv = g.inv("ADJ")
+    print("inverse metric tensor g^{ij}:")
+    g_inv = g.inv(method="ADJ")
     g_inv = g_inv.applyfunc(simplify)
     pprint(g_inv)
-    print "det g_{ij}:"
+    print("det g_{ij}:")
     g_det = g.det()
     pprint(g_det)
     f = Function("f")(*list(Y))
-    print "Laplace:"
+    print("Laplace:")
     pprint(laplace(f, g_inv, g_det, Y))
 
 
@@ -85,15 +87,15 @@ def main():
             Matrix([rho*sin(theta)*cos(phi), rho*sin(theta)*sin(phi),
                 rho*cos(theta)]),
             [rho, theta, phi],
-            recursive=True
-            )
+        recursive=True
+    )
 
     transform("rotating disk",
-            Matrix([t, x*cos(w*t)-y*sin(w*t), x*sin(w*t)+y*cos(w*t), z]),
+            Matrix([t, x*cos(w*t) - y*sin(w*t), x*sin(w*t) + y*cos(w*t), z]),
             [t, x, y, z])
 
     transform("parabolic",
-            Matrix([sigma*tau, (tau**2-sigma**2)/2]),
+            Matrix([sigma*tau, (tau**2 - sigma**2)/2]),
             [sigma, tau])
 
     # too complex:
@@ -106,7 +108,7 @@ def main():
     transform("elliptic",
             Matrix([a*cosh(mu)*cos(nu), a*sinh(mu)*sin(nu)]),
             [mu, nu]
-            )
+    )
 
 if __name__ == "__main__":
     main()
