@@ -1,11 +1,14 @@
+from __future__ import print_function, division
+
 from sympy.core import Basic, C, Dict, sympify
-from sympy.core.compatibility import as_int
+from sympy.core.compatibility import as_int, default_sort_key, xrange
+from sympy.functions.combinatorial.numbers import bell
 from sympy.matrices import zeros
-from sympy.utilities.misc import default_sort_key
 from sympy.utilities.iterables import has_dups, flatten, group
 
 from collections import defaultdict
 
+
 class Partition(C.FiniteSet):
     """
     This class represents an abstract partition.
@@ -53,7 +56,7 @@ class Partition(C.FiniteSet):
         if has_dups(partition):
             raise ValueError("Partition contained duplicated elements.")
 
-        obj = C.FiniteSet.__new__(cls, map(C.FiniteSet, args))
+        obj = C.FiniteSet.__new__(cls, list(map(C.FiniteSet, args)))
         obj.members = tuple(partition)
         obj.size = len(partition)
         return obj
@@ -73,7 +76,7 @@ class Partition(C.FiniteSet):
         >>> a = Partition([[1, 2]])
         >>> b = Partition([[3, 4]])
         >>> c = Partition([[1, x]])
-        >>> d = Partition([range(4)])
+        >>> d = Partition([list(range(4))])
         >>> l = [d, b, a + 1, a, c]
         >>> l.sort(key=default_sort_key); l
         [{{1, 2}}, {{1}, {2}}, {{1, x}}, {{3, 4}}, {{0, 1, 2, 3}}]
@@ -83,7 +86,7 @@ class Partition(C.FiniteSet):
         else:
             members = tuple(sorted(self.members,
                              key=lambda w: default_sort_key(w, order)))
-        return self.size, members, self.rank
+        return list(map(default_sort_key, (self.size, members, self.rank)))
 
     @property
     def partition(self):
@@ -97,7 +100,8 @@ class Partition(C.FiniteSet):
         [[1], [2, 3]]
         """
         if self._partition is None:
-            self._partition = sorted(sorted(p) for p in self.args)
+            self._partition = sorted([sorted(p, key=default_sort_key)
+                                      for p in self.args])
         return self._partition
 
     def __add__(self, other):
@@ -193,7 +197,7 @@ class Partition(C.FiniteSet):
         >>> a.rank
         13
         """
-        if self._rank != None:
+        if self._rank is not None:
             return self._rank
         self._rank = RGS_rank(self.RGS)
         return self._rank
@@ -227,7 +231,8 @@ class Partition(C.FiniteSet):
         for i, part in enumerate(partition):
             for j in part:
                 rgs[j] = i
-        return tuple([rgs[i] for i in sorted(i for p in partition for i in p)])
+        return tuple([rgs[i] for i in sorted(
+            [i for p in partition for i in p], key=default_sort_key)])
 
     @classmethod
     def from_rgs(self, rgs, elements):
@@ -264,6 +269,7 @@ class Partition(C.FiniteSet):
             raise ValueError('some blocks of the partition were empty.')
         return Partition(partition)
 
+
 class IntegerPartition(Basic):
     """
     This class represents an integer partition.
@@ -283,7 +289,7 @@ class IntegerPartition(Basic):
     sympy.utilities.iterables.partitions,
     sympy.utilities.iterables.multiset_partitions
 
-    Reference: http://en.wikipedia.org/wiki/Partition_(number_theory)
+    Reference: http://en.wikipedia.org/wiki/Partition_%28number_theory%29
     """
 
     _dict = None
@@ -305,7 +311,7 @@ class IntegerPartition(Basic):
         >>> a = IntegerPartition([5, 4, 3, 1, 1])
         >>> a
         IntegerPartition(14, (5, 4, 3, 1, 1))
-        >>> print a
+        >>> print(a)
         [5, 4, 3, 1, 1]
         >>> IntegerPartition({1:3, 2:1})
         IntegerPartition(5, (2, 1, 1, 1))
@@ -322,7 +328,7 @@ class IntegerPartition(Basic):
             integer, partition = partition, integer
         if isinstance(partition, (dict, Dict)):
             _ = []
-            for k, v in sorted(partition.items(), reverse=True):
+            for k, v in sorted(list(partition.items()), reverse=True):
                 if not v:
                     continue
                 k, v = as_int(k), as_int(v)
@@ -356,7 +362,7 @@ class IntegerPartition(Basic):
 
         >>> from sympy.combinatorics.partitions import IntegerPartition
         >>> p = IntegerPartition([4])
-        >>> print p.prev_lex()
+        >>> print(p.prev_lex())
         [3, 1]
         >>> p.partition > p.prev_lex().partition
         True
@@ -393,7 +399,7 @@ class IntegerPartition(Basic):
 
         >>> from sympy.combinatorics.partitions import IntegerPartition
         >>> p = IntegerPartition([3, 1])
-        >>> print p.next_lex()
+        >>> print(p.next_lex())
         [4]
         >>> p.partition < p.next_lex().partition
         True
@@ -517,7 +523,7 @@ class IntegerPartition(Basic):
         ========
 
         >>> from sympy.combinatorics.partitions import IntegerPartition
-        >>> print IntegerPartition([1, 1, 5]).as_ferrers()
+        >>> print(IntegerPartition([1, 1, 5]).as_ferrers())
         #####
         #
         #
@@ -527,6 +533,7 @@ class IntegerPartition(Basic):
     def __str__(self):
         return str(list(self.partition))
 
+
 def random_integer_partition(n, seed=None):
     """
     Generates a random integer partition summing to ``n`` as a list
@@ -565,6 +572,7 @@ def random_integer_partition(n, seed=None):
     partition = flatten([[k]*m for k, m in partition])
     return partition
 
+
 def RGS_generalized(m):
     """
     Computes the m + 1 generalized unrestricted growth strings
@@ -575,13 +583,14 @@ def RGS_generalized(m):
 
     >>> from sympy.combinatorics.partitions import RGS_generalized
     >>> RGS_generalized(6)
-    [  1,   1,   1,  1,  1, 1, 1]
-    [  1,   2,   3,  4,  5, 6, 0]
-    [  2,   5,  10, 17, 26, 0, 0]
-    [  5,  15,  37, 77,  0, 0, 0]
-    [ 15,  52, 151,  0,  0, 0, 0]
-    [ 52, 203,   0,  0,  0, 0, 0]
-    [203,   0,   0,  0,  0, 0, 0]
+    Matrix([
+    [  1,   1,   1,  1,  1, 1, 1],
+    [  1,   2,   3,  4,  5, 6, 0],
+    [  2,   5,  10, 17, 26, 0, 0],
+    [  5,  15,  37, 77,  0, 0, 0],
+    [ 15,  52, 151,  0,  0, 0, 0],
+    [ 52, 203,   0,  0,  0, 0, 0],
+    [203,   0,   0,  0,  0, 0, 0]])
     """
     d = zeros(m + 1)
     for i in xrange(0, m + 1):
@@ -590,12 +599,12 @@ def RGS_generalized(m):
     for i in xrange(1, m + 1):
         for j in xrange(m):
             if j <= m - i:
-                d[i, j] = j * d[i - 1, j] \
-                          + d[i - 1, j + 1]
+                d[i, j] = j * d[i - 1, j] + d[i - 1, j + 1]
             else:
                 d[i, j] = 0
     return d
 
+
 def RGS_enum(m):
     """
     RGS_enum computes the total number of restricted growth strings
@@ -616,27 +625,22 @@ def RGS_enum(m):
     We can check that the enumeration is correct by actually generating
     the partitions. Here, the 15 partitions of 4 items are generated:
 
-    >>> a = Partition([range(4)])
+    >>> a = Partition([list(range(4))])
     >>> s = set()
     >>> for i in range(20):
     ...     s.add(a)
     ...     a += 1
     ...
     >>> assert len(s) == 15
+
     """
     if (m < 1):
         return 0
     elif (m == 1):
         return 1
     else:
-        m += 1
-        b = [1] * (m)
-        for j in xrange(1, m):
-            for i in xrange(1, j):
-                b[j] += C.binomial(j - 1, i) * b[i]
+        return bell(m)
 
-        nrgf = b[m - 1]
-    return nrgf
 
 def RGS_unrank(rank, m):
     """
@@ -660,7 +664,7 @@ def RGS_unrank(rank, m):
     L = [1] * (m + 1)
     j = 1
     D = RGS_generalized(m)
-    for i in xrange(2, m+1):
+    for i in xrange(2, m + 1):
         v = D[m - i, j]
         cr = j*v
         if cr <= rank:
@@ -670,7 +674,8 @@ def RGS_unrank(rank, m):
         else:
             L[i] = int(rank / v + 1)
             rank %= v
-    return map(lambda x: x - 1, L[1:])
+    return [x - 1 for x in L[1:]]
+
 
 def RGS_rank(rgs):
     """
