@@ -1,15 +1,20 @@
+from __future__ import print_function, division
+
 import decimal
 import math
 import re as regex
 from collections import defaultdict
 
-from core import C
-from sympify import converter, sympify, _sympify, SympifyError
-from singleton import S, Singleton
-from expr import Expr, AtomicExpr
-from decorators import _sympifyit, deprecated
-from cache import cacheit, clear_cache
-from sympy.core.compatibility import as_int, HAS_GMPY, SYMPY_INTS
+from .core import C
+from .containers import Tuple
+from .sympify import converter, sympify, _sympify, SympifyError
+from .singleton import S, Singleton
+from .expr import Expr, AtomicExpr
+from .decorators import _sympifyit, deprecated
+from .cache import cacheit, clear_cache
+from sympy.core.compatibility import (
+    as_int, integer_types, long, string_types, with_metaclass, HAS_GMPY,
+    SYMPY_INTS)
 import sympy.mpmath as mpmath
 import sympy.mpmath.libmp as mlib
 from sympy.mpmath.libmp import mpf_pow, mpf_pi, mpf_e, phi_fixed
@@ -80,12 +85,7 @@ def _as_integer_ratio(p):
 
 def _decimal_to_Rational_prec(dec):
     """Convert an ordinary decimal instance to a Rational."""
-    # _is_special is needed for Python 2.5 support; is_finite for Python 3.3
-    # support
-    nonfinite = getattr(dec, '_is_special', None)
-    if nonfinite is None:
-        nonfinite = not dec.is_finite()  # Note, this is_finite is not SymPy's
-    if nonfinite:
+    if not dec.is_finite(): # NOTE: this is_finite is not SymPy's
         raise TypeError("dec must be finite, got %s." % dec)
     s, d, e = dec.as_tuple()
     prec = len(d)
@@ -218,7 +218,7 @@ class Number(AtomicExpr):
             return Rational(*obj)
         if isinstance(obj, (float, mpmath.mpf, decimal.Decimal)):
             return Float(obj)
-        if isinstance(obj, basestring):
+        if isinstance(obj, string_types):
             val = sympify(obj)
             if isinstance(val, Number):
                 return val
@@ -230,7 +230,7 @@ class Number(AtomicExpr):
         raise TypeError(msg % type(obj).__name__)
 
     def __divmod__(self, other):
-        from containers import Tuple
+        from .containers import Tuple
         from sympy.functions.elementary.complexes import sign
 
         try:
@@ -342,6 +342,8 @@ class Number(AtomicExpr):
                     return S.NegativeInfinity
                 else:
                     return S.Infinity
+        elif isinstance(other, Tuple):
+            return NotImplemented
         return AtomicExpr.__mul__(self, other)
 
     @_sympifyit('other', NotImplemented)
@@ -521,7 +523,7 @@ class Float(Number):
     ...     from sympy import Mul, Pow
     ...     s, m, e, b = f._mpf_
     ...     v = Mul(int(m), Pow(2, int(e), evaluate=False), evaluate=False)
-    ...     print '%s at prec=%s' % (v, f._prec)
+    ...     print('%s at prec=%s' % (v, f._prec))
     ...
     >>> t = Float('0.3', 3)
     >>> show(t)
@@ -566,7 +568,7 @@ class Float(Number):
     is_Float = True
 
     def __new__(cls, num, prec=15):
-        if isinstance(num, basestring):
+        if isinstance(num, string_types):
             num = num.replace(' ', '')
             if num.startswith('.') and len(num) > 1:
                 num = '0' + num
@@ -580,7 +582,7 @@ class Float(Number):
             num = num._mpf_
 
         if prec == '':
-            if not isinstance(num, basestring):
+            if not isinstance(num, string_types):
                 raise ValueError('The null string can only be used when '
                 'the number to Float is passed as a string or an integer.')
             ok = None
@@ -719,6 +721,8 @@ class Float(Number):
     def __nonzero__(self):
         return self._mpf_ != _mpf_zero
 
+    __bool__ = __nonzero__
+
     def __neg__(self):
         return Float._new(mlib.mpf_neg(self._mpf_), self._prec)
 
@@ -917,12 +921,6 @@ converter[float] = converter[decimal.Decimal] = Float
 RealNumber = Float
 
 
-@deprecated(useinstead="Float", issue=1721, deprecated_since_version="0.7.0")
-def Real(*args, **kwargs):  # pragma: no cover
-    """Deprecated alias for the Float constructor."""
-    return Float(*args, **kwargs)
-
-
 class Rational(Number):
     """Represents integers and rational numbers (p/q) of any size.
 
@@ -1001,8 +999,8 @@ class Rational(Number):
     Note that p and q return integers (not SymPy Integers) so some care
     is needed when using them in expressions:
 
-    >>> r.p//r.q
-    0
+    >>> r.p/r.q
+    0.75
 
     See Also
     ========
@@ -1022,7 +1020,7 @@ class Rational(Number):
             if isinstance(p, Rational):
                 return p
 
-            if isinstance(p, basestring):
+            if isinstance(p, string_types):
                 p = p.replace(' ', '')
                 try:
                     # we might have a Float
@@ -1395,9 +1393,9 @@ class Rational(Number):
                 args = [S.NegativeOne]
             else:
                 args = []
-            args.extend([Pow(*i, **{'evaluate':False})
+            args.extend([Pow(*i, evaluate=False)
                          for i in sorted(f.items())])
-            return Mul(*args, **{'evaluate': False})
+            return Mul(*args, evaluate=False)
 
     @_sympifyit('other', NotImplemented)
     def gcd(self, other):
@@ -1456,24 +1454,25 @@ def _intcache_printinfo():
     nmiss = _intcache_misses
 
     if nhit == 0 and nmiss == 0:
-        print
-        print 'Integer cache statistic was not collected'
+        print()
+        print('Integer cache statistic was not collected')
         return
 
     miss_ratio = float(nmiss) / (nhit + nmiss)
 
-    print
-    print 'Integer cache statistic'
-    print '-----------------------'
-    print
-    print '#items: %i' % len(ints)
-    print
-    print ' #hit   #miss               #total'
-    print
-    print '%5i   %5i (%7.5f %%)   %5i' % (
+    print()
+    print('Integer cache statistic')
+    print('-----------------------')
+    print()
+    print('#items: %i' % len(ints))
+    print()
+    print(' #hit   #miss               #total')
+    print()
+    print('%5i   %5i (%7.5f %%)   %5i' % (
         nhit, nmiss, miss_ratio*100, nhit + nmiss)
-    print
-    print ints
+    )
+    print()
+    print(ints)
 
 _intcache_hits = 0
 _intcache_misses = 0
@@ -1521,7 +1520,7 @@ class Integer(Rational):
     # TODO caching with decorator, but not to degrade performance
     @int_trace
     def __new__(cls, i):
-        if isinstance(i, basestring):
+        if isinstance(i, string_types):
             i = i.replace(' ', '')
         # whereas we cannot, in general, make a Rational from an
         # arbitrary expression, we can make an Integer unambiguously
@@ -1564,15 +1563,15 @@ class Integer(Rational):
             return Integer(-self.p)
 
     def __divmod__(self, other):
-        from containers import Tuple
+        from .containers import Tuple
         if isinstance(other, Integer):
             return Tuple(*(divmod(self.p, other.p)))
         else:
             return Number.__divmod__(self, other)
 
     def __rdivmod__(self, other):
-        from containers import Tuple
-        if isinstance(other, (int, long)):
+        from .containers import Tuple
+        if isinstance(other, integer_types):
             return Tuple(*(divmod(other, self.p)))
         else:
             try:
@@ -1586,57 +1585,57 @@ class Integer(Rational):
 
     # TODO make it decorator + bytecodehacks?
     def __add__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return Integer(self.p + other)
         elif isinstance(other, Integer):
             return Integer(self.p + other.p)
         return Rational.__add__(self, other)
 
     def __radd__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return Integer(other + self.p)
         return Rational.__add__(self, other)
 
     def __sub__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return Integer(self.p - other)
         elif isinstance(other, Integer):
             return Integer(self.p - other.p)
         return Rational.__sub__(self, other)
 
     def __rsub__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return Integer(other - self.p)
         return Rational.__rsub__(self, other)
 
     def __mul__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return Integer(self.p*other)
         elif isinstance(other, Integer):
             return Integer(self.p*other.p)
         return Rational.__mul__(self, other)
 
     def __rmul__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return Integer(other*self.p)
         return Rational.__mul__(self, other)
 
     def __mod__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return Integer(self.p % other)
         elif isinstance(other, Integer):
             return Integer(self.p % other.p)
         return Rational.__mod__(self, other)
 
     def __rmod__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return Integer(other % self.p)
         elif isinstance(other, Integer):
             return Integer(other.p % self.p)
         return Rational.__rmod__(self, other)
 
     def __eq__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return (self.p == other)
         elif isinstance(other, Integer):
             return (self.p == other.p)
@@ -1646,28 +1645,28 @@ class Integer(Rational):
         return not self.__eq__(other)
 
     def __gt__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return (self.p > other)
         elif isinstance(other, Integer):
             return (self.p > other.p)
         return Rational.__gt__(self, other)
 
     def __lt__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return (self.p < other)
         elif isinstance(other, Integer):
             return (self.p < other.p)
         return Rational.__lt__(self, other)
 
     def __ge__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return (self.p >= other)
         elif isinstance(other, Integer):
             return (self.p >= other.p)
         return Rational.__ge__(self, other)
 
     def __le__(self, other):
-        if isinstance(other, (int, long)):
+        if isinstance(other, integer_types):
             return (self.p <= other)
         elif isinstance(other, Integer):
             return (self.p <= other.p)
@@ -1775,14 +1774,14 @@ class Integer(Rational):
                 else:
                     sqr_dict[prime] = div_m
         # identify gcd of remaining powers
-        for p, ex in sqr_dict.iteritems():
+        for p, ex in sqr_dict.items():
             if sqr_gcd == 0:
                 sqr_gcd = ex
             else:
                 sqr_gcd = igcd(sqr_gcd, ex)
                 if sqr_gcd == 1:
                     break
-        for k, v in sqr_dict.iteritems():
+        for k, v in sqr_dict.items():
             sqr_int *= k**(v//sqr_gcd)
         if sqr_int == self and out_int == 1 and out_rad == 1:
             result = None
@@ -1805,7 +1804,8 @@ class Integer(Rational):
         return Integer(Integer(other).p // self.p)
 
 # Add sympify converters
-converter[int] = converter[long] = Integer
+for i_type in integer_types:
+    converter[i_type] = Integer
 
 
 class RationalConstant(Rational):
@@ -1828,9 +1828,7 @@ class IntegerConstant(Integer):
         return AtomicExpr.__new__(cls)
 
 
-class Zero(IntegerConstant):
-    __metaclass__ = Singleton
-
+class Zero(with_metaclass(Singleton, IntegerConstant)):
     p = 0
     q = 1
     is_positive = False
@@ -1873,10 +1871,10 @@ class Zero(IntegerConstant):
     def __nonzero__(self):
         return False
 
+    __bool__ = __nonzero__
 
-class One(IntegerConstant):
-    __metaclass__ = Singleton
 
+class One(with_metaclass(Singleton, IntegerConstant)):
     p = 1
     q = 1
 
@@ -1904,9 +1902,7 @@ class One(IntegerConstant):
         return {1: 1}
 
 
-class NegativeOne(IntegerConstant):
-    __metaclass__ = Singleton
-
+class NegativeOne(with_metaclass(Singleton, IntegerConstant)):
     p = -1
     q = 1
 
@@ -1943,9 +1939,7 @@ class NegativeOne(IntegerConstant):
         return
 
 
-class Half(RationalConstant):
-    __metaclass__ = Singleton
-
+class Half(with_metaclass(Singleton, RationalConstant)):
     p = 1
     q = 2
 
@@ -1956,9 +1950,8 @@ class Half(RationalConstant):
         return S.Half
 
 
-class Infinity(Number):
-    __metaclass__ = Singleton
 
+class Infinity(with_metaclass(Singleton, Number)):
     is_commutative = True
     is_positive = True
     is_bounded = False
@@ -1973,6 +1966,9 @@ class Infinity(Number):
     def __new__(cls):
         return AtomicExpr.__new__(cls)
 
+    def _latex(self, printer):
+        return r"\infty"
+
     @_sympifyit('other', NotImplemented)
     def __add__(self, other):
         if isinstance(other, Number):
@@ -2110,9 +2106,7 @@ class Infinity(Number):
 oo = S.Infinity
 
 
-class NegativeInfinity(Number):
-    __metaclass__ = Singleton
-
+class NegativeInfinity(with_metaclass(Singleton, Number)):
     is_commutative = True
     is_real = True
     is_positive = False
@@ -2127,6 +2121,9 @@ class NegativeInfinity(Number):
     def __new__(cls):
         return AtomicExpr.__new__(cls)
 
+    def _latex(self, printer):
+        return r"-\infty"
+
     @_sympifyit('other', NotImplemented)
     def __add__(self, other):
         if isinstance(other, Number):
@@ -2265,7 +2262,7 @@ class NegativeInfinity(Number):
         return other is S.NegativeInfinity
 
 
-class NaN(Number):
+class NaN(with_metaclass(Singleton, Number)):
     """
     Not a Number.
 
@@ -2299,8 +2296,6 @@ class NaN(Number):
     - http://en.wikipedia.org/wiki/NaN
 
     """
-    __metaclass__ = Singleton
-
     is_commutative = True
     is_real = None
     is_rational = None
@@ -2318,6 +2313,9 @@ class NaN(Number):
     def __new__(cls):
         return AtomicExpr.__new__(cls)
 
+    def _latex(self, printer):
+        return r"\mathrm{NaN}"
+
     @_sympifyit('other', NotImplemented)
     def __add__(self, other):
         return self
@@ -2367,9 +2365,7 @@ class NaN(Number):
 nan = S.NaN
 
 
-class ComplexInfinity(AtomicExpr):
-    __metaclass__ = Singleton
-
+class ComplexInfinity(with_metaclass(Singleton, AtomicExpr)):
     is_commutative = True
     is_bounded = False
     is_real = None
@@ -2380,6 +2376,9 @@ class ComplexInfinity(AtomicExpr):
     def __new__(cls):
         return AtomicExpr.__new__(cls)
 
+    def _latex(self, printer):
+        return r"\tilde{\infty}"
+
     @staticmethod
     def __abs__():
         return S.Infinity
@@ -2493,9 +2492,7 @@ class NumberSymbol(AtomicExpr):
         return super(NumberSymbol, self).__hash__()
 
 
-class Exp1(NumberSymbol):
-    __metaclass__ = Singleton
-
+class Exp1(with_metaclass(Singleton, NumberSymbol)):
     is_real = True
     is_positive = True
     is_negative = False  # XXX Forces is_negative/is_nonnegative
@@ -2503,6 +2500,9 @@ class Exp1(NumberSymbol):
 
     __slots__ = []
 
+    def _latex(self, printer):
+        return r"e"
+
     @staticmethod
     def __abs__():
         return S.Exp1
@@ -2536,9 +2536,7 @@ class Exp1(NumberSymbol):
 E = S.Exp1
 
 
-class Pi(NumberSymbol):
-    __metaclass__ = Singleton
-
+class Pi(with_metaclass(Singleton, NumberSymbol)):
     is_real = True
     is_positive = True
     is_negative = False
@@ -2546,6 +2544,9 @@ class Pi(NumberSymbol):
 
     __slots__ = []
 
+    def _latex(self, printer):
+        return r"\pi"
+
     @staticmethod
     def __abs__():
         return S.Pi
@@ -2568,9 +2569,7 @@ class Pi(NumberSymbol):
 pi = S.Pi
 
 
-class GoldenRatio(NumberSymbol):
-    __metaclass__ = Singleton
-
+class GoldenRatio(with_metaclass(Singleton, NumberSymbol)):
     is_real = True
     is_positive = True
     is_negative = False
@@ -2578,6 +2577,9 @@ class GoldenRatio(NumberSymbol):
 
     __slots__ = []
 
+    def _latex(self, printer):
+        return r"\phi"
+
     def __int__(self):
         return 1
 
@@ -2601,9 +2603,7 @@ class GoldenRatio(NumberSymbol):
         return sage.golden_ratio
 
 
-class EulerGamma(NumberSymbol):
-    __metaclass__ = Singleton
-
+class EulerGamma(with_metaclass(Singleton, NumberSymbol)):
     is_real = True
     is_positive = True
     is_negative = False
@@ -2611,6 +2611,9 @@ class EulerGamma(NumberSymbol):
 
     __slots__ = []
 
+    def _latex(self, printer):
+        return r"\gamma"
+
     def __int__(self):
         return 0
 
@@ -2631,9 +2634,7 @@ class EulerGamma(NumberSymbol):
         return sage.euler_gamma
 
 
-class Catalan(NumberSymbol):
-    __metaclass__ = Singleton
-
+class Catalan(with_metaclass(Singleton, NumberSymbol)):
     is_real = True
     is_positive = True
     is_negative = False
@@ -2661,9 +2662,7 @@ class Catalan(NumberSymbol):
         return sage.catalan
 
 
-class ImaginaryUnit(AtomicExpr):
-    __metaclass__ = Singleton
-
+class ImaginaryUnit(with_metaclass(Singleton, AtomicExpr)):
     is_commutative = True
     is_imaginary = True
     is_bounded = True
@@ -2672,6 +2671,9 @@ class ImaginaryUnit(AtomicExpr):
 
     __slots__ = []
 
+    def _latex(self, printer):
+        return r"i"
+
     @staticmethod
     def __abs__():
         return S.One
@@ -2754,7 +2756,7 @@ converter[mpnumeric] = sympify_mpmath
 
 
 def sympify_complex(a):
-    real, imag = map(sympify, (a.real, a.imag))
+    real, imag = list(map(sympify, (a.real, a.imag)))
     return real + S.ImaginaryUnit*imag
 
 converter[complex] = sympify_complex
@@ -2763,8 +2765,8 @@ _intcache[0] = S.Zero
 _intcache[1] = S.One
 _intcache[-1] = S.NegativeOne
 
-from power import Pow, integer_nthroot
-from mul import Mul
+from .power import Pow, integer_nthroot
+from .mul import Mul
 Mul.identity = One()
-from add import Add
+from .add import Add
 Add.identity = Zero()
