@@ -1,13 +1,13 @@
-from sympy import Symbol, Rational, exp
+from sympy import Symbol, Dummy, Rational, exp
+
 
 def test_equal():
     b = Symbol("b")
     a = Symbol("a")
-    c = Symbol("c")
-    e1 = a+b
+    e1 = a + b
     e2 = 2*a*b
     e3 = a**3*b**2
-    e4 = a*b+b*a
+    e4 = a*b + b*a
     assert not e1 == e2
     assert not e1 == e2
     assert e1 != e2
@@ -16,23 +16,24 @@ def test_equal():
     assert not e2 == e3
 
     x = Symbol("x")
-    e1 = exp(x+1/x)
+    e1 = exp(x + 1/x)
     y = Symbol("x")
-    e2 = exp(y+1/y)
+    e2 = exp(y + 1/y)
     assert e1 == e2
     assert not e1 != e2
     y = Symbol("y")
-    e2 = exp(y+1/y)
+    e2 = exp(y + 1/y)
     assert not e1 == e2
     assert e1 != e2
 
-    e5 = Rational(3)+2*x-x-x
+    e5 = Rational(3) + 2*x - x - x
     assert e5 == 3
     assert 3 == e5
     assert e5 != 4
     assert 4 != e5
-    assert e5 != 3+x
-    assert 3+x != e5
+    assert e5 != 3 + x
+    assert 3 + x != e5
+
 
 def test_expevalbug():
     x = Symbol("x")
@@ -40,6 +41,7 @@ def test_expevalbug():
     e3 = exp(x)
     assert e1 == e3
 
+
 def test_cmp_bug1():
     class T(object):
         pass
@@ -50,6 +52,7 @@ def test_cmp_bug1():
     assert not (x == t)
     assert (x != t)
 
+
 def test_cmp_bug2():
     class T(object):
         pass
@@ -58,3 +61,27 @@ def test_cmp_bug2():
 
     assert not (Symbol == t)
     assert (Symbol != t)
+
+
+def test_cmp_bug1258():
+    """ Check that Basic subclasses can be compared with sympifiable objects.
+
+    http://code.google.com/p/sympy/issues/detail?id=1258
+    """
+    assert not (Symbol == 1)
+    assert (Symbol != 1)
+    assert not (Symbol == 'x')
+    assert (Symbol != 'x')
+
+
+def test_dummy_eq():
+    x = Symbol('x')
+    y = Symbol('y')
+
+    u = Dummy('u')
+
+    assert (u**2 + 1).dummy_eq(x**2 + 1) is True
+    assert ((u**2 + 1) == (x**2 + 1)) is False
+
+    assert (u**2 + y).dummy_eq(x**2 + y, x) is True
+    assert (u**2 + y).dummy_eq(x**2 + y, y) is False
