@@ -2,7 +2,9 @@ from sympy.core import (Rational, Symbol, S, Float, Integer, Number, Pow,
 Basic, I, nan)
 from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.functions.elementary.exponential import exp
-from sympy.utilities.pytest import XFAIL
+from sympy.functions.elementary.trigonometric import sin, cos
+from sympy.series.order import O
+from sympy.utilities.pytest import XFAIL, slow
 
 
 def test_rational():
@@ -255,8 +257,17 @@ def test_issue_3109():
     assert sqrt(exp(5*I)) == -exp(5*I/2)
     assert root(exp(5*I), 3).exp == Rational(1, 3)
 
+
+def test_issue_3891():
+    x = Symbol('x')
+    a = Symbol('a')
+    b = Symbol('b')
+    assert (sqrt(a + b*x + x**2)).series(x, 0, 3).removeO() == \
+        b*x/(2*sqrt(a)) + x**2*(1/(2*sqrt(a)) - \
+        b**2/(8*a**(S(3)/2))) + sqrt(a)
+
+
 def test_issue_2969():
-    from sympy import sin, O
     x = Symbol('x')
     assert sqrt(sin(x)).series(x, 0, 7) == \
         sqrt(x) - x**(S(5)/2)/12 + x**(S(9)/2)/1440 - \
@@ -265,12 +276,39 @@ def test_issue_2969():
         sqrt(x) - x**(S(5)/2)/12 + x**(S(9)/2)/1440 - \
         x**(S(13)/2)/24192 - 67*x**(S(17)/2)/29030400 + O(x**9)
     assert sqrt(sin(x**3)).series(x, 0, 19) == \
-        sqrt(x**3) - x**6*sqrt(x**3)/12 + x**12*sqrt(x**3)/1440 + O(x**19)
+        x**(S(3)/2) - x**(S(15)/2)/12 + x**(S(27)/2)/1440 + O(x**19)
     assert sqrt(sin(x**3)).series(x, 0, 20) == \
-        sqrt(x**3) - x**6*sqrt(x**3)/12 + x**12*sqrt(x**3)/1440 - \
-        x**18*sqrt(x**3)/24192 + O(x**20)
+        x**(S(3)/2) - x**(S(15)/2)/12 + x**(S(27)/2)/1440 - \
+        x**(S(39)/2)/24192 + O(x**20)
+
 
-@XFAIL
 def test_issue_3683():
-    assert sqrt(sin(x**3)).series(x, 0, 7) == sqrt(x**3) + O(x**7)
-    assert sqrt(sin(x**4)).series(x, 0, 3) == sqrt(x**4) + O(x**4)
+    x = Symbol('x')
+    assert sqrt(sin(x**3)).series(x, 0, 7) == x**(S(3)/2) + O(x**7)
+    assert sqrt(sin(x**4)).series(x, 0, 3) == x**2 + O(x**3)
+
+
+def test_issue_3554():
+    x = Symbol('x')
+    assert (1 / sqrt(1 + cos(x) * sin(x**2))).series(x, 0, 7) == \
+        1 - x**2/2 + 5*x**4/8 - 5*x**6/8 + O(x**7)
+    assert (1 / sqrt(1 + cos(x) * sin(x**2))).series(x, 0, 8) == \
+        1 - x**2/2 + 5*x**4/8 - 5*x**6/8 + O(x**8)
+
+
+@slow
+def test_issue_3554s():
+    x = Symbol('x')
+    assert (1 / sqrt(1 + cos(x) * sin(x**2))).series(x, 0, 15) == \
+        1 - x**2/2 + 5*x**4/8 - 5*x**6/8 + 4039*x**8/5760 - 5393*x**10/6720 + \
+        13607537*x**12/14515200 - 532056047*x**14/479001600 + O(x**15)
+
+
+def test_issue_3330():
+    x = Symbol('x')
+    c = Symbol('c')
+    f = (c**2 + x)**(0.5)
+    assert f.series(x, x0=0, n=1) == (c**2)**0.5 + O(x)
+    assert f.taylor_term(0, x) == (c**2)**0.5
+    assert f.taylor_term(1, x) == 0.5*x*(c**2)**(-0.5)
+    assert f.taylor_term(2, x) == -0.125*x**2*(c**2)**(-1.5)
