@@ -1,3 +1,5 @@
+import string
+
 from sympy import (bernoulli, Symbol, symbols, Dummy, Sum, harmonic, Rational, oo,
     zoo, pi, I, bell, fibonacci, lucas, euler, catalan, binomial, gamma, sqrt,
     hyper, log, digamma, trigamma, polygamma, diff, Expr, sympify, expand_func,
@@ -184,7 +186,7 @@ def test_nC_nP_nT():
     from sympy.core.numbers import oo
     from random import choice
 
-    c = 'abcdefghijklmnopqrstuvwxyz'
+    c = string.ascii_lowercase
     for i in range(100):
         s = ''.join(choice(c) for i in range(7))
         u = len(s) == len(set(s))
@@ -198,7 +200,7 @@ def test_nC_nP_nT():
                     assert nP(len(s), i) == check
             assert nP(s) == tot
         except AssertionError:
-            print s, i, 'failed perm test'
+            print(s, i, 'failed perm test')
             raise ValueError()
 
     for i in range(100):
@@ -216,7 +218,7 @@ def test_nC_nP_nT():
             if u:
                 assert nC(len(s)) == tot
         except AssertionError:
-            print s, i, 'failed combo test'
+            print(s, i, 'failed combo test')
             raise ValueError()
 
     for i in range(1, 10):
@@ -250,7 +252,7 @@ def test_nC_nP_nT():
                 assert nT(range(len(s))) == tot
             assert nT(s) == tot
         except AssertionError:
-            print s, i, 'failed partition test'
+            print(s, i, 'failed partition test')
             raise ValueError()
 
     # tests for Stirling numbers of the first kind that are not tested in the
