@@ -1,15 +1,20 @@
+from __future__ import print_function, division
+
 from sympy.core import S, C, sympify
+from sympy.core.add import Add
 from sympy.core.basic import Basic
 from sympy.core.containers import Tuple
 from sympy.core.numbers import Rational
 from sympy.core.operations import LatticeOp, ShortCircuit
-from sympy.core.function import Application, Lambda
+from sympy.core.function import Application, Lambda, ArgumentIndexError
 from sympy.core.expr import Expr
 from sympy.core.singleton import Singleton
 from sympy.core.rules import Transform
-from sympy.core.compatibility import as_int
+from sympy.core.compatibility import as_int, with_metaclass, xrange
+from sympy.core.logic import fuzzy_and
 
-class IdentityFunction(Lambda):
+
+class IdentityFunction(with_metaclass(Singleton, Lambda)):
     """
     The identity function
 
@@ -22,9 +27,9 @@ class IdentityFunction(Lambda):
     x
 
     """
-    __metaclass__ = Singleton
     __slots__ = []
     nargs = 1
+
     def __new__(cls):
         x = C.Dummy('x')
         #construct "by hand" to avoid infinite loop
@@ -35,6 +40,7 @@ Id = S.IdentityFunction
 ############################# ROOT and SQUARE ROOT FUNCTION ###################
 ###############################################################################
 
+
 def sqrt(arg):
     """The square root function
 
@@ -60,10 +66,9 @@ def sqrt(arg):
     This is because the two are not equal to each other in general.
     For example, consider x == -1:
 
-    >>> sqrt(x**2).subs(x, -1)
-    1
-    >>> x.subs(x, -1)
-    -1
+    >>> from sympy import Eq
+    >>> Eq(sqrt(x**2), x).subs(x, -1)
+    False
 
     This is because sqrt computes the principal square root, so the square may
     put the argument in a different branch.  This identity does hold if x is
@@ -92,7 +97,7 @@ def sqrt(arg):
     See Also
     ========
 
-    sympy.polys.rootoftools.RootOf, root
+    sympy.polys.rootoftools.RootOf, root, real_root
 
     References
     ==========
@@ -105,8 +110,59 @@ def sqrt(arg):
     return C.Pow(arg, S.Half)
 
 
+
+def cbrt(arg):
+    """This function computes the principial cube root of `arg`, so
+    it's just a shortcut for `arg**Rational(1, 3)`.
+
+    Examples
+    ========
+
+    >>> from sympy import cbrt, Symbol
+    >>> x = Symbol('x')
+
+    >>> cbrt(x)
+    x**(1/3)
+
+    >>> cbrt(x)**3
+    x
+
+    Note that cbrt(x**3) does not simplify to x.
+
+    >>> cbrt(x**3)
+    (x**3)**(1/3)
+
+    This is because the two are not equal to each other in general.
+    For example, consider `x == -1`:
+
+    >>> from sympy import Eq
+    >>> Eq(cbrt(x**3), x).subs(x, -1)
+    False
+
+    This is because cbrt computes the principal cube root, this
+    identity does hold if `x` is positive:
+
+    >>> y = Symbol('y', positive=True)
+    >>> cbrt(y**3)
+    y
+
+    See Also
+    ========
+
+    sympy.polys.rootoftools.RootOf, root, real_root
+
+    References
+    ==========
+
+    * http://en.wikipedia.org/wiki/Cube_root
+    * http://en.wikipedia.org/wiki/Principal_value
+
+    """
+    return C.Pow(arg, C.Rational(1, 3))
+
+
 def root(arg, n):
-    """The n-th root function (a shortcut for arg**(1/n))
+    """The n-th root function (a shortcut for ``arg**(1/n)``)
 
     root(x, n) -> Returns the principal n-th root of x.
 
@@ -183,6 +239,7 @@ def root(arg, n):
     n = sympify(n)
     return C.Pow(arg, 1/n)
 
+
 def real_root(arg, n=None):
     """Return the real nth-root of arg if possible. If n is omitted then
     all instances of -1**(1/odd) will be changed to -1.
@@ -226,7 +283,8 @@ def real_root(arg, n=None):
 ############################# MINIMUM and MAXIMUM #############################
 ###############################################################################
 
-class MinMaxBase(LatticeOp):
+
+class MinMaxBase(Expr, LatticeOp):
     def __new__(cls, *args, **assumptions):
         if not args:
             raise ValueError("The Max/Min functions must have arguments.")
@@ -271,7 +329,7 @@ class MinMaxBase(LatticeOp):
         for arg in arg_sequence:
 
             # pre-filter, checking comparability of arguments
-            if (arg.is_real == False) or (arg is S.ComplexInfinity):
+            if (arg.is_real is False) or (arg is S.ComplexInfinity):
                 raise ValueError("The argument '%s' is not comparable." % arg)
 
             if arg == cls.zero:
@@ -342,11 +400,31 @@ class MinMaxBase(LatticeOp):
         yx = cls._rel_inversed(x, y)
         if isinstance(yx, bool):
             if yx:
-                return False # never occurs?
+                return False  # never occurs?
             return True
         return False
 
-class Max(MinMaxBase, Application, Basic):
+    def _eval_derivative(self, s):
+        # f(x).diff(s) -> x.diff(s) * f.fdiff(1)(s)
+        i = 0
+        l = []
+        for a in self.args:
+            i += 1
+            da = a.diff(s)
+            if da is S.Zero:
+                continue
+            try:
+                df = self.fdiff(i)
+            except ArgumentIndexError:
+                df = Function.fdiff(self, i)
+            l.append(df * da)
+        return Add(*l)
+
+    @property
+    def is_real(self):
+        return fuzzy_and(arg.is_real for arg in self.args)
+
+class Max(MinMaxBase, Application):
     """
     Return, if possible, the maximum value of the list.
 
@@ -427,7 +505,7 @@ class Max(MinMaxBase, Application, Basic):
     ==========
 
     .. [1] http://en.wikipedia.org/wiki/Directed_complete_partial_order
-    .. [2] http://en.wikipedia.org/wiki/Lattice_(order)
+    .. [2] http://en.wikipedia.org/wiki/Lattice_%28order%29
 
     See Also
     ========
@@ -451,8 +529,20 @@ class Max(MinMaxBase, Application, Basic):
         """
         return (x < y)
 
+    def fdiff( self, argindex ):
+        from sympy.functions.special.delta_functions import Heaviside
+        n = len(self.args)
+        if 0 < argindex and argindex <= n:
+            argindex -= 1
+            if n == 2:
+                return Heaviside( self.args[argindex] - self.args[1-argindex] )
+            newargs = tuple([self.args[i] for i in xrange(n) if i != argindex])
+            return Heaviside( self.args[argindex] - Max(*newargs) )
+        else:
+            raise ArgumentIndexError(self, argindex)
 
-class Min(MinMaxBase, Application, Basic):
+
+class Min(MinMaxBase, Application):
     """
     Return, if possible, the minimum value of the list.
 
@@ -500,3 +590,15 @@ class Min(MinMaxBase, Application, Basic):
         Check if x > y.
         """
         return (x > y)
+
+    def fdiff( self, argindex ):
+        from sympy.functions.special.delta_functions import Heaviside
+        n = len(self.args)
+        if 0 < argindex and argindex <= n:
+            argindex -= 1
+            if n == 2:
+                return Heaviside( self.args[1-argindex] - self.args[argindex] )
+            newargs = tuple([ self.args[i] for i in xrange(n) if i != argindex])
+            return Heaviside( Min(*newargs) - self.args[argindex] )
+        else:
+            raise ArgumentIndexError(self, argindex)
