@@ -1,7 +1,7 @@
 from sympy.core.symbol import Symbol
 from sympy.core.numbers import Rational
 from sympy.utilities.pytest import raises
-from sympy.functions.elementary.miscellaneous import sqrt, root, Min, Max, real_root
+from sympy.functions.elementary.miscellaneous import sqrt, cbrt, root, Min, Max, real_root
 from sympy import S, Float, I, cos, sin, oo, pi, Add
 
 
@@ -98,6 +98,10 @@ def test_Min():
     assert Min(0,-x,1-2*x).diff(x) == -Heaviside(x + Min(0, -2*x + 1)) \
         - 2*Heaviside(2*x + Min(0, -x) - 1)
 
+    a, b = Symbol('a', real=True), Symbol('b', real=True)
+    # a and b are both real, Min(a, b) should be real
+    assert Min(a, b).is_real
+
 
 def test_Max():
     from sympy.abc import x, y, z
@@ -144,6 +148,9 @@ def test_Max():
     assert Max(x**2, 1+x, 1).diff(x) == 2*x*Heaviside(x**2 - Max(1,x+1)) \
         + Heaviside(x - Max(1,x**2) + 1)
 
+    a, b = Symbol('a', real=True), Symbol('b', real=True)
+    # a and b are both real, Max(a, b) should be real
+    assert Max(a, b).is_real
 
 def test_root():
     from sympy.abc import x, y, z
@@ -152,6 +159,7 @@ def test_root():
     assert root(2, 2) == sqrt(2)
     assert root(2, 1) == 2
     assert root(2, 3) == 2**Rational(1, 3)
+    assert root(2, 3) == cbrt(2)
     assert root(2, -5) == 2**Rational(4, 5)/2
 
     assert root(-2, 1) == -2
@@ -162,6 +170,7 @@ def test_root():
     assert root(x, 2) == sqrt(x)
     assert root(x, 1) == x
     assert root(x, 3) == x**Rational(1, 3)
+    assert root(x, 3) == cbrt(x)
     assert root(x, -5) == x**Rational(-1, 5)
 
     assert root(x, n) == x**(1/n)
