@@ -26,7 +26,6 @@ def test_piecewise():
         Piecewise((x, Or(x < 1, x < 2)), (0, True))
     assert Piecewise((x, x < 1), (x, x < 2), (x, True)) == x
     assert Piecewise((x, True)) == x
-    assert Piecewise((1, Eq(x + 1/x, (x**2 + 1)/x)), (2, True)) == 1
     raises(TypeError, lambda: Piecewise(x))
     raises(TypeError, lambda: Piecewise((x, x**2)))
 
@@ -160,7 +159,9 @@ def test_piecewise_integrate():
     assert integrate(g, (x, -1, 1)) == 0
 
     g = Piecewise((1, x - y < 0), (0, True))
-    assert integrate(g, (y, -oo, oo)) == oo
+    assert integrate(g, (y, -oo, 0)) == -Min(0, x)
+    assert integrate(g, (y, 0, oo)) == oo - Max(0, x)
+    assert integrate(g, (y, -oo, oo)) == oo - x
 
     g = Piecewise((0, x < 0), (x, x <= 1), (1, True))
     assert integrate(g, (x, -5, 1)) == Rational(1, 2)
@@ -204,6 +205,21 @@ def test_piecewise_integrate():
         (y**2/2 - y + 0.5, y > 0), (-y**2/2 - y + 0.5, True))
 
 
+def test_piecewise_integrate_inequality_conditions():
+    c1, c2 = symbols("c1 c2", positive=True)
+    g = Piecewise((0, c1*x > 1), (1, c1*x > 0), (0, True))
+    assert integrate(g, (x, -oo, 0)) == 0
+    assert integrate(g, (x, -5, 0)) == 0
+    assert integrate(g, (x, 0, 5)) == Min(5, 1/c1)
+    assert integrate(g, (x, 0, oo)) == 1/c1
+
+    g = Piecewise((0, c1*x + c2*y > 1), (1, c1*x + c2*y > 0), (0, True))
+    assert integrate(g, (x, -oo, 0)).subs(y, 0) == 0
+    assert integrate(g, (x, -5, 0)).subs(y, 0) == 0
+    assert integrate(g, (x, 0, 5)).subs(y, 0) == Min(5, 1/c1)
+    assert integrate(g, (x, 0, oo)).subs(y, 0) == 1/c1
+
+
 def test_piecewise_integrate_symbolic_conditions():
     from sympy.abc import a, b, x, y
     p0 = Piecewise((0, Or(x < a, x > b)), (1, True))
@@ -247,6 +263,13 @@ def test_piecewise_integrate_independent_conditions():
         Piecewise((0, Eq(y, 0)), (4*y, True))
 
 
+def test_piecewise_simplify():
+    p = Piecewise(((x**2 + 1)/x**2, Eq(x*(1 + x) - x**2, 0)),
+                  ((-1)**x*(-1), True))
+    assert p.simplify() == \
+        Piecewise((1 + 1/x**2, Eq(x, 0)), ((-1)**(x + 1), True))
+
+
 def test_piecewise_solve():
     abs2 = Piecewise((-x, x <= 0), (x, x > 0))
     f = abs2.subs(x, x - 2)
