@@ -1,9 +1,13 @@
 """Bessel type functions"""
 
+from __future__ import print_function, division
+
 from sympy import S, pi, I
-from sympy.core.function import Function, ArgumentIndexError
-from sympy.functions.elementary.trigonometric import sin, cos
+from sympy.core.function import Function, ArgumentIndexError, expand_func
+from sympy.functions.elementary.trigonometric import sin, cos, csc, cot
 from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.complexes import re, im
+from sympy.core.compatibility import xrange
 
 # TODO
 # o Airy Ai and Bi functions
@@ -17,19 +21,20 @@ from sympy.functions.elementary.miscellaneous import sqrt
 # o More rewriting.
 # o Add solvers to ode.py (or rather add solvers for the hypergeometric equation).
 
+
 class BesselBase(Function):
     """
     Abstract base class for bessel-type functions.
 
     This class is meant to reduce code duplication.
-    All bessel type functions can 1) be differentiated, and the derivatives
+    All Bessel type functions can 1) be differentiated, and the derivatives
     expressed in terms of similar functions and 2) be rewritten in terms
     of other bessel-type functions.
 
     Here "bessel-type functions" are assumed to have one complex parameter.
 
     To use this base class, define class attributes ``_a`` and ``_b`` such that
-    ``2*F_n' = -_a*F_{n+1} b*F_{n-1}``.
+    ``2*F_n' = -_a*F_{n+1} + b*F_{n-1}``.
     """
 
     nargs = 2
@@ -48,13 +53,34 @@ class BesselBase(Function):
         if argindex != 2:
             raise ArgumentIndexError(self, argindex)
         return self._b/2 * self.__class__(self.order - 1, self.argument) \
-             - self._a/2 * self.__class__(self.order + 1, self.argument) \
+            - self._a/2 * self.__class__(self.order + 1, self.argument) \
+
+    def _eval_conjugate(self):
+        z = self.argument
+        if (z.is_real and z.is_negative) is False:
+            return self.__class__(self.order.conjugate(), z.conjugate())
+
+    def _eval_expand_func(self, **hints):
+        nu, z, f = self.order, self.argument, self.__class__
+        if nu.is_real:
+            if (nu - 1).is_positive:
+                return (-self._a*self._b*f(nu - 2, z)._eval_expand_func() +
+                        2*self._a*(nu - 1)*f(nu - 1, z)._eval_expand_func()/z)
+            elif (nu + 1).is_negative:
+                return (2*self._b*(nu + 1)*f(nu + 1, z)._eval_expand_func()/z -
+                        self._a*self._b*f(nu + 2, z)._eval_expand_func())
+        return self
+
+    def _eval_simplify(self, ratio, measure):
+        from sympy.simplify.simplify import besselsimp
+        return besselsimp(self)
+
 
 class besselj(BesselBase):
     r"""
     Bessel function of the first kind.
 
-    The Bessel J function of order :math:`\nu` is defined to be the function
+    The Bessel `J` function of order `\nu` is defined to be the function
     satisfying Bessel's differential equation
 
     .. math ::
@@ -75,7 +101,7 @@ class besselj(BesselBase):
     Examples
     ========
 
-    Create a bessel function object:
+    Create a Bessel function object:
 
     >>> from sympy import besselj, jn
     >>> from sympy.abc import z, n
@@ -86,7 +112,7 @@ class besselj(BesselBase):
     >>> b.diff(z)
     besselj(n - 1, z)/2 - besselj(n + 1, z)/2
 
-    Rewrite in terms of spherical bessel functions:
+    Rewrite in terms of spherical Bessel functions:
 
     >>> b.rewrite(jn)
     sqrt(2)*sqrt(z)*jn(n - 1/2, z)/sqrt(pi)
@@ -103,36 +129,42 @@ class besselj(BesselBase):
 
     bessely, besseli, besselk
 
-
     References
     ==========
 
-    - Abramowitz, Milton; Stegun, Irene A., eds. (1965), "Chapter 9",
-      Handbook of Mathematical Functions with Formulas, Graphs, and Mathematical
-      Tables
-    - Luke, Y. L. (1969), The Special Functions and Their Approximations,
-      Volume 1
-    - http://en.wikipedia.org/wiki/Bessel_function
+    .. [1] Abramowitz, Milton; Stegun, Irene A., eds. (1965), "Chapter 9",
+           Handbook of Mathematical Functions with Formulas, Graphs, and
+           Mathematical Tables
+    .. [2] Luke, Y. L. (1969), The Special Functions and Their
+           Approximations, Volume 1
+    .. [3] http://en.wikipedia.org/wiki/Bessel_function
+    .. [4] http://functions.wolfram.com/Bessel-TypeFunctions/BesselJ/
     """
 
     _a = S.One
     _b = S.One
 
-    def _eval_rewrite_as_jn(self, nu, z, expand=False):
-        jn_part = jn(nu - S('1/2'), self.argument)
-        if expand:
-            jn_part = jn_part._eval_expand_func()
-        return sqrt(2*z/pi) * jn_part
-
     @classmethod
     def eval(cls, nu, z):
-        if nu.is_Integer:
-            if nu < 0:
-                return S(-1)**nu*besselj(-nu, z)
-            if z.could_extract_minus_sign():
-                return S(-1)**nu*besselj(nu, -z)
+        if z.is_zero:
+            if nu.is_zero:
+                return S.One
+            elif (nu.is_integer and nu.is_zero is False) or re(nu).is_positive:
+                return S.Zero
+            elif re(nu).is_negative and not (nu.is_integer is True):
+                return S.ComplexInfinity
+            elif nu.is_imaginary:
+                return S.NaN
+        if z is S.Infinity or (z is S.NegativeInfinity):
+            return S.Zero
+
+        if z.could_extract_minus_sign():
+            return (z)**nu*(-z)**(-nu)*besselj(nu, -z)
+        if nu.is_integer:
+            if nu.could_extract_minus_sign():
+                return S(-1)**(-nu)*besselj(-nu, z)
             newz = z.extract_multiplicatively(I)
-            if newz: # NOTE we don't want to change the function if z==0
+            if newz:  # NOTE we don't want to change the function if z==0
                 return I**(nu)*besseli(nu, newz)
 
         # branch handling:
@@ -149,20 +181,28 @@ class besselj(BesselBase):
         if nu != nnu:
             return besselj(nnu, z)
 
-    def _eval_expand_func(self, **hints):
-        if self.order.is_Rational and self.order.q == 2:
-            return self._eval_rewrite_as_jn(*self.args, **{'expand':True})
-        return self
-
     def _eval_rewrite_as_besseli(self, nu, z):
         from sympy import polar_lift, exp
         return exp(I*pi*nu/2)*besseli(nu, polar_lift(-I)*z)
 
+    def _eval_rewrite_as_bessely(self, nu, z):
+        if nu.is_integer is False:
+            return csc(pi*nu)*bessely(-nu, z) - cot(pi*nu)*bessely(nu, z)
+
+    def _eval_rewrite_as_jn(self, nu, z):
+        return sqrt(2*z/pi)*jn(nu - S.Half, self.argument)
+
+    def _eval_is_real(self):
+        nu, z = self.args
+        if nu.is_integer and z.is_real:
+            return True
+
+
 class bessely(BesselBase):
     r"""
     Bessel function of the second kind.
 
-    The Bessel Y function of order :math:`\nu` is defined as
+    The Bessel `Y` function of order `\nu` is defined as
 
     .. math ::
         Y_\nu(z) = \lim_{\mu \to \nu} \frac{J_\mu(z) \cos(\pi \mu)
@@ -189,27 +229,49 @@ class bessely(BesselBase):
 
     besselj, besseli, besselk
 
+    References
+    ==========
+
+    .. [1] http://functions.wolfram.com/Bessel-TypeFunctions/BesselY/
+
     """
 
     _a = S.One
     _b = S.One
 
-    def _eval_rewrite_as_yn(self, nu, z, expand=False):
-        yn_part = yn(nu - S('1/2'), self.argument)
-        if expand:
-            yn_part = yn_part._eval_expand_func()
-        return sqrt(2*z/pi) * yn_part
-
     @classmethod
     def eval(cls, nu, z):
-        if nu.is_Integer:
-            if nu < 0:
-                return S(-1)**nu*bessely(-nu, z)
+        if z.is_zero:
+            if nu.is_zero:
+                return S.NegativeInfinity
+            elif re(nu).is_zero is False:
+                return S.ComplexInfinity
+            elif re(nu).is_zero:
+                return S.NaN
+        if z is S.Infinity or z is S.NegativeInfinity:
+            return S.Zero
+
+        if nu.is_integer:
+            if nu.could_extract_minus_sign():
+                return S(-1)**(-nu)*bessely(-nu, z)
+
+    def _eval_rewrite_as_besselj(self, nu, z):
+        if nu.is_integer is False:
+            return csc(pi*nu)*(cos(pi*nu)*besselj(nu, z) - besselj(-nu, z))
+
+    def _eval_rewrite_as_besseli(self, nu, z):
+        aj = self._eval_rewrite_as_besselj(*self.args)
+        if aj:
+            return aj.rewrite(besseli)
+
+    def _eval_rewrite_as_yn(self, nu, z):
+        return sqrt(2*z/pi) * yn(nu - S.Half, self.argument)
+
+    def _eval_is_real(self):
+        nu, z = self.args
+        if nu.is_integer and z.is_positive:
+            return True
 
-    def _eval_expand_func(self, **hints):
-        if self.order.is_Rational and self.order.q == 2:
-            return self._eval_rewrite_as_yn(*self.args, **{'expand':True})
-        return self
 
 class besseli(BesselBase):
     r"""
@@ -226,7 +288,7 @@ class besseli(BesselBase):
     .. math ::
         I_\nu(z) = i^{-\nu} J_\nu(iz),
 
-    where :math:`J_\mu(z)` is the Bessel function of the first kind.
+    where :math:`J_\nu(z)` is the Bessel function of the first kind.
 
     Examples
     ========
@@ -241,6 +303,11 @@ class besseli(BesselBase):
 
     besselj, bessely, besselk
 
+    References
+    ==========
+
+    .. [1] http://functions.wolfram.com/Bessel-TypeFunctions/BesselI/
+
     """
 
     _a = -S.One
@@ -248,9 +315,26 @@ class besseli(BesselBase):
 
     @classmethod
     def eval(cls, nu, z):
-        if nu.is_Integer:
+        if z.is_zero:
+            if nu.is_zero:
+                return S.One
+            elif (nu.is_integer and nu.is_zero is False) or re(nu).is_positive:
+                return S.Zero
+            elif re(nu).is_negative and not (nu.is_integer is True):
+                return S.ComplexInfinity
+            elif nu.is_imaginary:
+                return S.NaN
+        if z.is_imaginary:
+            if im(z) is S.Infinity or im(z) is S.NegativeInfinity:
+                return S.Zero
+
+        if z.could_extract_minus_sign():
+            return (z)**nu*(-z)**(-nu)*besseli(nu, -z)
+        if nu.is_integer:
+            if nu.could_extract_minus_sign():
+                return besseli(-nu, z)
             newz = z.extract_multiplicatively(I)
-            if newz: # NOTE we don't want to change the function if z==0
+            if newz:  # NOTE we don't want to change the function if z==0
                 return I**(-nu)*besselj(nu, -newz)
 
         # branch handling:
@@ -271,6 +355,20 @@ class besseli(BesselBase):
         from sympy import polar_lift, exp
         return exp(-I*pi*nu/2)*besselj(nu, polar_lift(I)*z)
 
+    def _eval_rewrite_as_bessely(self, nu, z):
+        aj = self._eval_rewrite_as_besselj(*self.args)
+        if aj:
+            return aj.rewrite(bessely)
+
+    def _eval_rewrite_as_jn(self, nu, z):
+        return self._eval_rewrite_as_besselj(*self.args).rewrite(jn)
+
+    def _eval_is_real(self):
+        nu, z = self.args
+        if nu.is_integer and z.is_real:
+            return True
+
+
 class besselk(BesselBase):
     r"""
     Modified Bessel function of the second kind.
@@ -299,11 +397,58 @@ class besselk(BesselBase):
 
     besselj, besseli, bessely
 
+    References
+    ==========
+
+    .. [1] http://functions.wolfram.com/Bessel-TypeFunctions/BesselK/
+
     """
 
     _a = S.One
     _b = -S.One
 
+    @classmethod
+    def eval(cls, nu, z):
+        if z.is_zero:
+            if nu.is_zero:
+                return S.Infinity
+            elif re(nu).is_zero is False:
+                return S.ComplexInfinity
+            elif re(nu).is_zero:
+                return S.NaN
+        if z.is_imaginary:
+            if im(z) is S.Infinity or im(z) is S.NegativeInfinity:
+                return S.Zero
+
+        if nu.is_integer:
+            if nu.could_extract_minus_sign():
+                return besselk(-nu, z)
+
+    def _eval_rewrite_as_besseli(self, nu, z):
+        if nu.is_integer is False:
+            return pi*csc(pi*nu)*(besseli(-nu, z) - besseli(nu, z))/2
+
+    def _eval_rewrite_as_besselj(self, nu, z):
+        ai = self._eval_rewrite_as_besseli(*self.args)
+        if ai:
+            return ai.rewrite(besselj)
+
+    def _eval_rewrite_as_bessely(self, nu, z):
+        aj = self._eval_rewrite_as_besselj(*self.args)
+        if aj:
+            return aj.rewrite(bessely)
+
+    def _eval_rewrite_as_yn(self, nu, z):
+        ay = self._eval_rewrite_as_bessely(*self.args)
+        if ay:
+            return ay.rewrite(yn)
+
+    def _eval_is_real(self):
+        nu, z = self.args
+        if nu.is_integer and z.is_positive:
+            return True
+
+
 class hankel1(BesselBase):
     r"""
     Hankel function of the first kind.
@@ -331,11 +476,22 @@ class hankel1(BesselBase):
 
     hankel2, besselj, bessely
 
+    References
+    ==========
+
+    .. [1] http://functions.wolfram.com/Bessel-TypeFunctions/HankelH1/
+
     """
 
     _a = S.One
     _b = S.One
 
+    def _eval_conjugate(self):
+        z = self.argument
+        if (z.is_real and z.is_negative) is False:
+            return hankel2(self.order.conjugate(), z.conjugate())
+
+
 class hankel2(BesselBase):
     r"""
     Hankel function of the second kind.
@@ -364,22 +520,33 @@ class hankel2(BesselBase):
 
     hankel1, besselj, bessely
 
+    References
+    ==========
+
+    .. [1] http://functions.wolfram.com/Bessel-TypeFunctions/HankelH2/
+
     """
 
     _a = S.One
     _b = S.One
 
+    def _eval_conjugate(self):
+        z = self.argument
+        if (z.is_real and z.is_negative) is False:
+            return hankel1(self.order.conjugate(), z.conjugate())
+
 from sympy.polys.orthopolys import spherical_bessel_fn as fn
 
+
 class SphericalBesselBase(BesselBase):
     """
-    Base class for spherical bessel functions.
+    Base class for spherical Bessel functions.
 
-    These are thin wrappers around ordinary bessel functions,
-    since spherical bessel functions differ from the ordinary
+    These are thin wrappers around ordinary Bessel functions,
+    since spherical Bessel functions differ from the ordinary
     ones just by a slight change in order.
 
-    To use this class, define the _rewrite and _expand methods.
+    To use this class, define the ``_rewrite`` and ``_expand`` methods.
     """
 
     def _expand(self, **hints):
@@ -387,7 +554,7 @@ class SphericalBesselBase(BesselBase):
         raise NotImplementedError('expansion')
 
     def _rewrite(self):
-        """ Rewrite self in terms of ordinary bessel functions. """
+        """ Rewrite self in terms of ordinary Bessel functions. """
         raise NotImplementedError('rewriting')
 
     def _eval_expand_func(self, **hints):
@@ -403,14 +570,14 @@ class SphericalBesselBase(BesselBase):
         if argindex != 2:
             raise ArgumentIndexError(self, argindex)
         return self.__class__(self.order - 1, self.argument) - \
-               self * (self.order + 1)/self.argument
+            self * (self.order + 1)/self.argument
 
 
 class jn(SphericalBesselBase):
     r"""
     Spherical Bessel function of the first kind.
 
-    This function is a solution to the spherical bessel equation
+    This function is a solution to the spherical Bessel equation
 
     .. math ::
         z^2 \frac{\mathrm{d}^2 w}{\mathrm{d}z^2}
@@ -428,7 +595,7 @@ class jn(SphericalBesselBase):
 
     >>> from sympy import Symbol, jn, sin, cos, expand_func
     >>> z = Symbol("z")
-    >>> print jn(0, z).expand(func=True)
+    >>> print(jn(0, z).expand(func=True))
     sin(z)/z
     >>> jn(1, z).expand(func=True) == sin(z)/z**2 - cos(z)/z
     True
@@ -459,13 +626,14 @@ class jn(SphericalBesselBase):
     def _expand(self, **hints):
         n = self.order
         z = self.argument
-        return fn(n, z) * sin(z) + (-1)**(n+1) * fn(-n-1, z) * cos(z)
+        return fn(n, z) * sin(z) + (-1)**(n + 1) * fn(-n - 1, z) * cos(z)
+
 
 class yn(SphericalBesselBase):
     r"""
     Spherical Bessel function of the second kind.
 
-    This function is another solution to the spherical bessel equation, and
+    This function is another solution to the spherical Bessel equation, and
     linearly independent from :math:`j_n`. It can be defined as
 
     .. math ::
@@ -478,7 +646,7 @@ class yn(SphericalBesselBase):
 
     >>> from sympy import Symbol, yn, sin, cos, expand_func
     >>> z = Symbol("z")
-    >>> print expand_func(yn(0, z))
+    >>> print(expand_func(yn(0, z)))
     -cos(z)/z
     >>> expand_func(yn(1, z)) == -cos(z)/z**2-sin(z)/z
     True
@@ -503,8 +671,8 @@ class yn(SphericalBesselBase):
     def _expand(self, **hints):
         n = self.order
         z = self.argument
-        return (-1)**(n+1) * \
-               (fn(-n-1, z) * sin(z) + (-1)**(-n) * fn(n, z) * cos(z))
+        return (-1)**(n + 1) * \
+               (fn(-n - 1, z) * sin(z) + (-1)**(-n) * fn(n, z) * cos(z))
 
 
 def jn_zeros(n, k, method="sympy", dps=15):
@@ -513,13 +681,17 @@ def jn_zeros(n, k, method="sympy", dps=15):
 
     This returns an array of zeros of jn up to the k-th zero.
 
-    * method = "sympy": uses mpmath besseljzero
-    * method = "scipy": uses the SciPy's sph_jn and newton to find all
+    * method = "sympy": uses :func:`mpmath.besseljzero`
+    * method = "scipy": uses the
+      `SciPy's sph_jn <http://docs.scipy.org/doc/scipy/reference/generated/scipy.special.jn.html>`_
+      and
+      `newton <http://docs.scipy.org/doc/scipy/reference/generated/scipy.optimize.newton.html>`_
+      to find all
       roots, which is faster than computing the zeros using a general
       numerical solver, but it requires SciPy and only works with low
-      precision floating point numbers.  [the function used with
+      precision floating point numbers.  [The function used with
       method="sympy" is a recent addition to mpmath, before that a general
-      solver was used]
+      solver was used.]
 
     Examples
     ========
@@ -541,14 +713,15 @@ def jn_zeros(n, k, method="sympy", dps=15):
         from sympy import Expr
         prec = dps_to_prec(dps)
         return [Expr._from_mpmath(besseljzero(S(n + 0.5)._to_mpmath(prec),
-                                              int(k)), prec) \
+                                              int(k)), prec)
                 for k in xrange(1, k + 1)]
     elif method == "scipy":
         from scipy.special import sph_jn
         from scipy.optimize import newton
-        f  = lambda x: sph_jn(n, x)[0][-1]
+        f = lambda x: sph_jn(n, x)[0][-1]
     else:
         raise NotImplementedError("Unknown method.")
+
     def solver(f, x):
         if method == "scipy":
             root = newton(f, x)
@@ -557,12 +730,12 @@ def jn_zeros(n, k, method="sympy", dps=15):
         return root
 
     # we need to approximate the position of the first root:
-    root = n+pi
+    root = n + pi
     # determine the first root exactly:
     root = solver(f, root)
     roots = [root]
-    for i in range(k-1):
+    for i in range(k - 1):
         # estimate the position of the next root using the last root + pi:
-        root = solver(f, root+pi)
+        root = solver(f, root + pi)
         roots.append(root)
     return roots
