@@ -1,25 +1,32 @@
+from __future__ import print_function, division
+
 from sympy.core import Add, S, C, sympify, oo, pi
 from sympy.core.function import Function, ArgumentIndexError
-from zeta_functions import zeta
-from error_functions import erf
-from sympy.core import Dummy,Rational
+from .zeta_functions import zeta
+from .error_functions import erf
+from sympy.core import Dummy, Rational
 from sympy.functions.elementary.exponential import log
 from sympy.functions.elementary.integers import floor
 from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.functions.combinatorial.numbers import bernoulli
 from sympy.functions.combinatorial.factorials import rf
+from sympy.functions.combinatorial.numbers import harmonic
+from sympy.core.compatibility import xrange
 
 ###############################################################################
 ############################ COMPLETE GAMMA FUNCTION ##########################
 ###############################################################################
 
+
 class gamma(Function):
     """The gamma function returns a function which passes through the integral
     values of the factorial function, i.e. though defined in the complex plane,
-    when n is an integer, `gamma(n) = (n - 1)!`
+    when n is an integer, `\Gamma(n) = (n - 1)!`
 
-    Reference:
-        http://en.wikipedia.org/wiki/Gamma_function
+    References
+    ==========
+
+    .. [1] http://en.wikipedia.org/wiki/Gamma_function
     """
 
     nargs = 1
@@ -40,7 +47,7 @@ class gamma(Function):
                 return S.Infinity
             elif arg.is_Integer:
                 if arg.is_positive:
-                    return C.factorial(arg-1)
+                    return C.factorial(arg - 1)
                 else:
                     return S.ComplexInfinity
             elif arg.is_Rational:
@@ -65,7 +72,6 @@ class gamma(Function):
                     else:
                         return 2**n*sqrt(S.Pi) / coeff
 
-
     def _eval_expand_func(self, **hints):
         arg = self.args[0]
         if arg.is_Rational:
@@ -81,7 +87,7 @@ class gamma(Function):
                 intpart = floor(coeff)
                 tail = (coeff - intpart,) + tail
                 coeff = intpart
-            tail = arg._new_rawargs(*tail, **dict(reeval=False))
+            tail = arg._new_rawargs(*tail, reeval=False)
             return gamma(tail)*C.RisingFactorial(tail, coeff)
 
         return self.func(*self.args)
@@ -99,6 +105,20 @@ class gamma(Function):
         t = self.args[0] - x0
         return (gamma(t + 1)/rf(self.args[0], -x0 + 1))._eval_nseries(x, n, logx)
 
+    def _latex(self, printer, exp=None):
+        assert len(self.args) == 1
+        aa = printer._print(self.args[0])
+        if exp:
+            return r'\Gamma^{%s}{\left(%s \right)}' % (printer._print(exp), aa)
+        else:
+            return r'\Gamma{\left(%s \right)}' % aa
+
+    @staticmethod
+    def _latex_no_arg(printer):
+        return r'\Gamma'
+
+
+
 
 ###############################################################################
 ################## LOWER and UPPER INCOMPLETE GAMMA FUNCTIONS #################
@@ -141,10 +161,10 @@ class lowergamma(Function):
     References
     ==========
 
-    - Abramowitz, Milton; Stegun, Irene A., eds. (1965), Chapter 6, Section 5,
-      Handbook of Mathematical Functions with Formulas, Graphs, and Mathematical
-      Tables
-    - http://en.wikipedia.org/wiki/Incomplete_gamma_function
+    .. [1] Abramowitz, Milton; Stegun, Irene A., eds. (1965), Chapter 6,
+           Section 5, Handbook of Mathematical Functions with Formulas, Graphs,
+           and Mathematical Tables
+    .. [2] http://en.wikipedia.org/wiki/Incomplete_gamma_function
 
     """
 
@@ -154,11 +174,11 @@ class lowergamma(Function):
         from sympy import meijerg, unpolarify
         if argindex == 2:
             a, z = self.args
-            return C.exp(-unpolarify(z))*z**(a-1)
+            return C.exp(-unpolarify(z))*z**(a - 1)
         elif argindex == 1:
             a, z = self.args
             return gamma(a)*digamma(a) - log(z)*uppergamma(a, z) \
-                   + meijerg([], [1, 1], [0, 0, a], [], z)
+                - meijerg([], [1, 1], [0, 0, a], [], z)
 
         else:
             raise ArgumentIndexError(self, argindex)
@@ -182,11 +202,11 @@ class lowergamma(Function):
         #    lowergamma(s, exp(2*I*pi*n)*x) = exp(2*pi*I*n*a)*lowergamma(a, x)
         from sympy import unpolarify, I, factorial, exp
         nx, n = x.extract_branch_factor()
-        if a.is_integer and a > 0:
+        if a.is_integer and a.is_positive:
             nx = unpolarify(x)
             if nx != x:
                 return lowergamma(a, nx)
-        elif a.is_integer and a <= 0:
+        elif a.is_integer and a.is_nonpositive:
             if n != 0:
                 return 2*pi*I*n*(-1)**(-a)/factorial(-a) + lowergamma(a, nx)
         elif n != 0:
@@ -227,6 +247,10 @@ class lowergamma(Function):
             return self
         return self.rewrite(uppergamma).rewrite(expint)
 
+    @staticmethod
+    def _latex_no_arg(printer):
+        return r'\gamma'
+
 class uppergamma(Function):
     r"""
     Upper incomplete gamma function
@@ -237,7 +261,8 @@ class uppergamma(Function):
         \Gamma(s, x) = \int_x^\infty t^{s-1} e^{-t} \mathrm{d}t
                      = \Gamma(s) - \gamma(s, x).
 
-    This can be shown to be the same as
+    where `\gamma(s, x)` is the lower incomplete gamma function,
+    :class:`lowergamma`. This can be shown to be the same as
 
     .. math ::
         \Gamma(s, x) = \Gamma(s)
@@ -246,7 +271,10 @@ class uppergamma(Function):
     where :math:`{}_1F_1` is the (confluent) hypergeometric function.
 
     The upper incomplete gamma function is also essentially equivalent to the
-    generalized exponential integral.
+    generalized exponential integral:
+
+    .. math ::
+        \operatorname{E}_{n}(x) = \int_{1}^{\infty}{\frac{e^{-xt}}{t^n} \, dt} = x^{n-1}\Gamma(1-n,x).
 
     Examples
     ========
@@ -271,11 +299,11 @@ class uppergamma(Function):
     References
     ==========
 
-    - Abramowitz, Milton; Stegun, Irene A., eds. (1965), Chapter 6, Section 5,
-      Handbook of Mathematical Functions with Formulas, Graphs, and Mathematical
-      Tables
-    - http://en.wikipedia.org/wiki/Incomplete_gamma_function
-
+    .. [1] Abramowitz, Milton; Stegun, Irene A., eds. (1965), Chapter 6,
+           Section 5, Handbook of Mathematical Functions with Formulas, Graphs,
+           and Mathematical Tables
+    .. [2] http://en.wikipedia.org/wiki/Incomplete_gamma_function
+    .. [3] http://en.wikipedia.org/wiki/Exponential_integral#Relation_with_other_functions
     """
 
     nargs = 2
@@ -284,7 +312,7 @@ class uppergamma(Function):
         from sympy import meijerg, unpolarify
         if argindex == 2:
             a, z = self.args
-            return -C.exp(-unpolarify(z))*z**(a-1)
+            return -C.exp(-unpolarify(z))*z**(a - 1)
         elif argindex == 1:
             a, z = self.args
             return uppergamma(a, z)*log(z) + meijerg([], [1, 1], [0, 0, a], [], z)
@@ -315,11 +343,11 @@ class uppergamma(Function):
 
         # We extract branching information here. C/f lowergamma.
         nx, n = z.extract_branch_factor()
-        if a.is_integer and a > 0:
+        if a.is_integer and (a > 0) is True:
             nx = unpolarify(z)
             if z != nx:
                 return uppergamma(a, nx)
-        elif a.is_integer and a <= 0:
+        elif a.is_integer and (a <= 0) is True:
             if n != 0:
                 return -2*pi*I*n*(-1)**(-a)/factorial(-a) + uppergamma(a, nx)
         elif n != 0:
@@ -331,7 +359,7 @@ class uppergamma(Function):
             if a is S.One:
                 return C.exp(-z)
             elif a is S.Half:
-                return sqrt(pi)*(1 - erf(sqrt(z))) # TODO could use erfc...
+                return sqrt(pi)*(1 - erf(sqrt(z)))  # TODO could use erfc...
             elif a.is_Integer or (2*a).is_Integer:
                 b = a - 1
                 if b.is_positive:
@@ -342,7 +370,6 @@ class uppergamma(Function):
                 if not a.is_Integer:
                     return (cls(a + 1, z) - z**a * C.exp(-z))/a
 
-
     def _eval_rewrite_as_lowergamma(self, s, x):
         return gamma(s) - lowergamma(s, x)
 
@@ -351,21 +378,44 @@ class uppergamma(Function):
         return expint(1 - s, x)*x**s
 
 
-
 ###############################################################################
 ########################### GAMMA RELATED FUNCTIONS ###########################
 ###############################################################################
-
 class polygamma(Function):
-    """The function `polygamma(n, z)` returns `log(gamma(z)).diff(n + 1)`
+    r"""The function ``polygamma(n, z)`` returns ``log(gamma(z)).diff(n + 1)``
 
-       See Also
-       ========
+    .. math ::
+        \psi^{(n)}(z) = \frac{d^n}{d x^n} \log \Gamma(z)
 
-       gamma, digamma, trigamma
+    Examples
+    ========
 
-    Reference:
-        http://en.wikipedia.org/wiki/Polygamma_function
+    We can rewrite polygamma functions in terms of harmonic numbers:
+
+    >>> from sympy import polygamma, harmonic, Symbol
+    >>> x = Symbol("x")
+
+    >>> polygamma(0, x).rewrite(harmonic)
+    harmonic(x - 1) - EulerGamma
+
+    >>> polygamma(2, x).rewrite(harmonic)
+    2*harmonic(x - 1, 3) - 2*zeta(3)
+
+    >>> ni = Symbol("n", integer=True)
+    >>> polygamma(ni, x).rewrite(harmonic)
+    (-1)**(n + 1)*(-harmonic(x - 1, n + 1) + zeta(n + 1))*factorial(n)
+
+    See Also
+    ========
+
+    gamma, digamma, trigamma
+
+    References
+    ==========
+
+    .. [1] http://en.wikipedia.org/wiki/Polygamma_function
+    .. [2] http://functions.wolfram.com/GammaBetaErf/PolyGamma/
+    .. [3] http://functions.wolfram.com/GammaBetaErf/PolyGamma2/
     """
 
     nargs = 2
@@ -373,16 +423,16 @@ class polygamma(Function):
     def fdiff(self, argindex=2):
         if argindex == 2:
             n, z = self.args[:2]
-            return polygamma(n+1, z)
+            return polygamma(n + 1, z)
         else:
             raise ArgumentIndexError(self, argindex)
 
     def _eval_is_positive(self):
-        if self.args[1].is_positive and self.args[0] > 0:
+        if self.args[1].is_positive and (self.args[0] > 0) is True:
             return self.args[0].is_odd
 
     def _eval_is_negative(self):
-        if self.args[1].is_positive and self.args[0] > 0:
+        if self.args[1].is_positive and (self.args[0] > 0) is True:
             return self.args[0].is_even
 
     def _eval_is_real(self):
@@ -390,7 +440,7 @@ class polygamma(Function):
 
     def _eval_aseries(self, n, args0, x, logx):
         if args0[1] != oo or not \
-           (self.args[0].is_Integer and self.args[0].is_nonnegative):
+                (self.args[0].is_Integer and self.args[0].is_nonnegative):
             return super(polygamma, self)._eval_aseries(n, args0, x, logx)
         z = self.args[1]
         N = self.args[0]
@@ -403,7 +453,7 @@ class polygamma(Function):
             if n < 2:
                 o = C.Order(1/z, x)
             else:
-                m = C.ceiling((n+1)//2)
+                m = C.ceiling((n + 1)//2)
                 l = [bernoulli(2*k) / (2*k*z**(2*k)) for k in range(1, m)]
                 r -= Add(*l)
                 o = C.Order(1/z**(2*m), x)
@@ -416,9 +466,9 @@ class polygamma(Function):
             #    quite a long time!
             fac = gamma(N)
             e0 = fac + N*fac/(2*z)
-            m = C.ceiling((n+1)//2)
+            m = C.ceiling((n + 1)//2)
             for k in range(1, m):
-                fac = fac*(2*k+N-1)*(2*k+N-2) / ((2*k)*(2*k-1))
+                fac = fac*(2*k + N - 1)*(2*k + N - 2) / ((2*k)*(2*k - 1))
                 e0 += bernoulli(2*k)*fac/z**(2*k)
             o = C.Order(1/z**(2*m), x)
             if n == 0:
@@ -430,7 +480,7 @@ class polygamma(Function):
 
     @classmethod
     def eval(cls, n, z):
-        n, z = map(sympify, (n, z))
+        n, z = list(map(sympify, (n, z)))
         from sympy import unpolarify
 
         if n.is_integer:
@@ -456,9 +506,9 @@ class polygamma(Function):
                             return S.ComplexInfinity
                         else:
                             if n is S.Zero:
-                                return -S.EulerGamma + C.harmonic(z-1, 1)
+                                return -S.EulerGamma + C.harmonic(z - 1, 1)
                             elif n.is_odd:
-                                return (-1)**(n+1)*C.factorial(n)*zeta(n+1, z)
+                                return (-1)**(n + 1)*C.factorial(n)*zeta(n + 1, z)
 
         if n == 0 and z.is_Rational:
             # TODO actually *any* n/m can be done, but that is messy
@@ -480,7 +530,6 @@ class polygamma(Function):
 
         # TODO n == 1 also can do some rational z
 
-
     def _eval_expand_func(self, **hints):
         n, z = self.args
 
@@ -490,51 +539,80 @@ class polygamma(Function):
                 if coeff.is_Integer:
                     e = -(n + 1)
                     if coeff > 0:
-                        tail = Add(*[C.Pow(z - i, e)  for i in xrange(1, int(coeff) + 1)])
+                        tail = Add(*[C.Pow(
+                            z - i, e) for i in xrange(1, int(coeff) + 1)])
                     else:
-                        tail = -Add(*[C.Pow(z + i, e)  for i in xrange(0, int(-coeff))])
+                        tail = -Add(*[C.Pow(
+                            z + i, e) for i in xrange(0, int(-coeff))])
                     return polygamma(n, z - coeff) + (-1)**n*C.factorial(n)*tail
 
             elif z.is_Mul:
                 coeff, z = z.as_two_terms()
                 if coeff.is_Integer and coeff.is_positive:
-                    tail = [ polygamma(n, z + C.Rational(i, coeff)) for i in xrange(0, int(coeff)) ]
+                    tail = [ polygamma(n, z + C.Rational(
+                        i, coeff)) for i in xrange(0, int(coeff)) ]
                     if n == 0:
                         return Add(*tail)/coeff + log(coeff)
                     else:
-                        return Add(*tail)/coeff**(n+1)
+                        return Add(*tail)/coeff**(n + 1)
                 z *= coeff
 
         return polygamma(n, z)
 
     def _eval_rewrite_as_zeta(self, n, z):
-        return (-1)**(n+1)*C.factorial(n)*zeta(n+1, z-1)
+        if n >= S.One:
+            return (-1)**(n + 1)*C.factorial(n)*zeta(n + 1, z)
+        else:
+            return self
+
+    def _eval_rewrite_as_harmonic(self, n, z):
+        if n.is_integer:
+            if n == S.Zero:
+                return harmonic(z - 1) - S.EulerGamma
+            else:
+                return S.NegativeOne**(n+1) * C.factorial(n) * (C.zeta(n+1) - harmonic(z-1, n+1))
+
+    def _eval_as_leading_term(self, x):
+        n, z = [a.as_leading_term(x) for a in self.args]
+        o = C.Order(z, x)
+        if n == 0 and o.contains(1/x):
+            return o.getn() * log(x)
+        else:
+            return self.func(n, z)
+
 
 class loggamma(Function):
     """
-    The loggamma function is `ln(gamma(x))`.
+    The loggamma function is `\log(\Gamma(x))`.
 
     References
     ==========
 
-    http://mathworld.wolfram.com/LogGammaFunction.html
+    .. [1] http://mathworld.wolfram.com/LogGammaFunction.html
 
     """
 
     nargs = 1
 
+    def _eval_nseries(self, x, n, logx=None):
+        x0 = self.args[0].limit(x, 0)
+        if x0 is S.Zero:
+            f = self._eval_rewrite_as_intractable(*self.args)
+            return f._eval_nseries(x, n, logx)
+        return super(loggamma, self)._eval_nseries(x, n, logx)
+
     def _eval_aseries(self, n, args0, x, logx):
         if args0[0] != oo:
             return super(loggamma, self)._eval_aseries(n, args0, x, logx)
         z = self.args[0]
-        m = min(n, C.ceiling((n+S(1))/2))
-        r = log(z)*(z-S(1)/2) - z + log(2*pi)/2
-        l = [bernoulli(2*k) / (2*k*(2*k-1)*z**(2*k-1)) for k in range(1, m)]
+        m = min(n, C.ceiling((n + S(1))/2))
+        r = log(z)*(z - S(1)/2) - z + log(2*pi)/2
+        l = [bernoulli(2*k) / (2*k*(2*k - 1)*z**(2*k - 1)) for k in range(1, m)]
         o = None
         if m == 0:
             o = C.Order(1, x)
         else:
-            o = C.Order(1/z**(2*m-1), x)
+            o = C.Order(1/z**(2*m - 1), x)
         # It is very inefficient to first add the order and then do the nseries
         return (r + Add(*l))._eval_nseries(x, n, logx) + o
 
@@ -550,12 +628,13 @@ class loggamma(Function):
         else:
             raise ArgumentIndexError(self, argindex)
 
+
 def digamma(x):
     """
     The digamma function is the logarithmic derivative of the gamma function.
 
 
-    In this case, `digamma(x) = polygamma(0, x)`.
+    In this case, ``digamma(x) = polygamma(0, x)``.
 
     See Also
     ========
@@ -565,11 +644,12 @@ def digamma(x):
     """
     return polygamma(0, x)
 
+
 def trigamma(x):
     """
     The trigamma function is the second of the polygamma functions.
 
-    In this case, `trigamma(x) = polygamma(1, x)`.
+    In this case, ``trigamma(x) = polygamma(1, x)``.
 
     See Also
     ========
@@ -579,6 +659,7 @@ def trigamma(x):
     """
     return polygamma(1, x)
 
+
 def beta(x, y):
     """
     Euler Beta function
@@ -586,4 +667,4 @@ def beta(x, y):
     ``beta(x, y) == gamma(x)*gamma(y) / gamma(x+y)``
 
     """
-    return gamma(x)*gamma(y) / gamma(x+y)
+    return gamma(x)*gamma(y) / gamma(x + y)
