@@ -6,11 +6,15 @@ combinatorial polynomials.
 
 """
 
+from __future__ import print_function, division
+
 from sympy.core.basic import C
 from sympy.core.singleton import S
 from sympy.core import Rational
 from sympy.core.function import Function, ArgumentIndexError
 from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.special.gamma_functions import gamma
+from sympy.functions.combinatorial.factorials import factorial
 
 from sympy.polys.orthopolys import (
     jacobi_poly,
@@ -42,6 +46,7 @@ class OrthogonalPolynomial(Function):
 # Jacobi polynomials
 #
 
+
 class jacobi(OrthogonalPolynomial):
     r"""
     Jacobi polynomial :math:`P_n^{\left(\alpha, \beta\right)}(x)`
@@ -70,24 +75,25 @@ class jacobi(OrthogonalPolynomial):
     jacobi(n, a, b, x)
 
     >>> jacobi(n, a, a, x)
-    RisingFactorial(a + 1, n)*gegenbauer(n, a + 1/2, x)/RisingFactorial(2*a + 1, n)
+    RisingFactorial(a + 1, n)*gegenbauer(n,
+        a + 1/2, x)/RisingFactorial(2*a + 1, n)
 
     >>> jacobi(n, 0, 0, x)
     legendre(n, x)
 
     >>> jacobi(n, S(1)/2, S(1)/2, x)
-    RisingFactorial(3/2, n)*chebyshevu(n, x)/(n + 1)!
+    RisingFactorial(3/2, n)*chebyshevu(n, x)/factorial(n + 1)
 
     >>> jacobi(n, -S(1)/2, -S(1)/2, x)
-    RisingFactorial(1/2, n)*chebyshevt(n, x)/n!
+    RisingFactorial(1/2, n)*chebyshevt(n, x)/factorial(n)
 
     >>> jacobi(n, a, b, -x)
     (-1)**n*jacobi(n, b, a, x)
 
     >>> jacobi(n, a, b, 0)
-    2**(-n)*gamma(a + n + 1)*hyper((-b - n, -n), (a + 1,), -1)/(n!*gamma(a + 1))
+    2**(-n)*gamma(a + n + 1)*hyper((-b - n, -n), (a + 1,), -1)/(factorial(n)*gamma(a + 1))
     >>> jacobi(n, a, b, 1)
-    RisingFactorial(a + 1, n)/n!
+    RisingFactorial(a + 1, n)/factorial(n)
 
     >>> conjugate(jacobi(n, a, b, x))
     jacobi(n, conjugate(a), conjugate(b), conjugate(x))
@@ -131,15 +137,15 @@ class jacobi(OrthogonalPolynomial):
             elif a == S.Zero:
                 return legendre(n, x)
             elif a == S.Half:
-                return C.RisingFactorial(3*S.Half, n) / C.factorial(n+1) * chebyshevu(n, x)
+                return C.RisingFactorial(3*S.Half, n) / C.factorial(n + 1) * chebyshevu(n, x)
             else:
-                return C.RisingFactorial(a+1, n) / C.RisingFactorial(2*a+1, n) * gegenbauer(n, a+S.Half, x)
+                return C.RisingFactorial(a + 1, n) / C.RisingFactorial(2*a + 1, n) * gegenbauer(n, a + S.Half, x)
         elif b == -a:
             # P^{a, -a}_n(x)
-            return C.gamma(n+a+1) / C.gamma(n+1) * (1+x)**(a/2) / (1-x)**(a/2) * assoc_legendre(n, -a, x)
+            return C.gamma(n + a + 1) / C.gamma(n + 1) * (1 + x)**(a/2) / (1 - x)**(a/2) * assoc_legendre(n, -a, x)
         elif a == -b:
             # P^{-b, b}_n(x)
-            return C.gamma(n-b+1) / C.gamma(n+1) * (1-x)**(b/2) / (1+x)**(b/2) * assoc_legendre(n, b, x)
+            return C.gamma(n - b + 1) / C.gamma(n + 1) * (1 - x)**(b/2) / (1 + x)**(b/2) * assoc_legendre(n, b, x)
 
         if not n.is_Number:
             # Symbolic result P^{a,b}_n(x)
@@ -148,14 +154,14 @@ class jacobi(OrthogonalPolynomial):
                 return S.NegativeOne**n * jacobi(n, b, a, -x)
             # We can evaluate for some special values of x
             if x == S.Zero:
-                return (2**(-n) * C.gamma(a+n+1) / (C.gamma(a+1) * C.factorial(n)) *
-                        C.hyper([-b-n, -n], [a+1], -1))
+                return (2**(-n) * C.gamma(a + n + 1) / (C.gamma(a + 1) * C.factorial(n)) *
+                        C.hyper([-b - n, -n], [a + 1], -1))
             if x == S.One:
-                return C.RisingFactorial(a+1, n) / C.factorial(n)
+                return C.RisingFactorial(a + 1, n) / C.factorial(n)
             elif x == S.Infinity:
                 if n.is_positive:
                     # TODO: Make sure a+b+2*n \notin Z
-                    return C.RisingFactorial(a+b+n+1, n) * S.Infinity
+                    return C.RisingFactorial(a + b + n + 1, n) * S.Infinity
         else:
             # n is a given fixed integer, evaluate into polynomial
             return jacobi_poly(n, a, b, x)
@@ -169,39 +175,99 @@ class jacobi(OrthogonalPolynomial):
             n, a, b, x = self.args
             k = C.Dummy("k")
             f1 = 1 / (a + b + n + k + 1)
-            f2 = ((a + b + 2*k + 1) * C.RisingFactorial(b + k + 1, n-k) /
-                  ((n - k) * C.RisingFactorial(a + b + k + 1, n-k)))
-            return C.Sum(f1 * (jacobi(n,a,b,x) + f2*jacobi(k,a,b,x)) , (k,0,n-1))
+            f2 = ((a + b + 2*k + 1) * C.RisingFactorial(b + k + 1, n - k) /
+                  ((n - k) * C.RisingFactorial(a + b + k + 1, n - k)))
+            return C.Sum(f1 * (jacobi(n, a, b, x) + f2*jacobi(k, a, b, x)), (k, 0, n - 1))
         elif argindex == 3:
             # Diff wrt b
             n, a, b, x = self.args
             k = C.Dummy("k")
             f1 = 1 / (a + b + n + k + 1)
-            f2 = (-1)**(n-k) * ((a + b + 2*k + 1) * C.RisingFactorial(a + k + 1, n-k) /
-                  ((n - k) * C.RisingFactorial(a + b + k + 1, n-k)))
-            return C.Sum(f1 * (jacobi(n,a,b,x) + f2*jacobi(k,a,b,x)) , (k,0,n-1))
+            f2 = (-1)**(n - k) * ((a + b + 2*k + 1) * C.RisingFactorial(a + k + 1, n - k) /
+                  ((n - k) * C.RisingFactorial(a + b + k + 1, n - k)))
+            return C.Sum(f1 * (jacobi(n, a, b, x) + f2*jacobi(k, a, b, x)), (k, 0, n - 1))
         elif argindex == 4:
             # Diff wrt x
             n, a, b, x = self.args
-            return S.Half * (a + b + n + 1) * jacobi(n-1, a+1, b+1, x)
+            return S.Half * (a + b + n + 1) * jacobi(n - 1, a + 1, b + 1, x)
         else:
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_polynomial(self, n, a, b, x):
         # TODO: Make sure n \in N
         k = C.Dummy("k")
-        kern = (C.RisingFactorial(-n, k) * C.RisingFactorial(a+b+n+1, k) * C.RisingFactorial(a+k+1, n-k) /
-                C.factorial(k) * ((1-x)/2)**k)
+        kern = (C.RisingFactorial(-n, k) * C.RisingFactorial(a + b + n + 1, k) * C.RisingFactorial(a + k + 1, n - k) /
+                C.factorial(k) * ((1 - x)/2)**k)
         return 1 / C.factorial(n) * C.Sum(kern, (k, 0, n))
 
     def _eval_conjugate(self):
         n, a, b, x = self.args
         return self.func(n, a.conjugate(), b.conjugate(), x.conjugate())
 
+
+def jacobi_normalized(n, a, b, x):
+    r"""
+    Jacobi polynomial :math:`P_n^{\left(\alpha, \beta\right)}(x)`
+
+    jacobi_normalized(n, alpha, beta, x) gives the nth Jacobi polynomial
+    in x, :math:`P_n^{\left(\alpha, \beta\right)}(x)`.
+
+    The Jacobi polynomials are orthogonal on :math:`[-1, 1]` with respect
+    to the weight :math:`\left(1-x\right)^\alpha \left(1+x\right)^\beta`.
+
+    This functions returns the polynomials normilzed:
+
+    .. math::
+
+        \int_{-1}^{1}
+          P_m^{\left(\alpha, \beta\right)}(x)
+          P_n^{\left(\alpha, \beta\right)}(x)
+          (1-x)^{\alpha} (1+x)^{\beta} \mathrm{d}x
+        = \delta_{m,n}
+
+    Examples
+    ========
+
+    >>> from sympy import jacobi_normalized
+    >>> from sympy.abc import n,a,b,x
+
+    >>> jacobi_normalized(n, a, b, x)
+    jacobi(n, a, b, x)/sqrt(2**(a + b + 1)*gamma(a + n + 1)*gamma(b + n + 1)/((a + b + 2*n + 1)*factorial(n)*gamma(a + b + n + 1)))
+
+    See Also
+    ========
+
+    gegenbauer,
+    chebyshevt_root, chebyshevu, chebyshevu_root,
+    legendre, assoc_legendre,
+    hermite,
+    laguerre, assoc_laguerre,
+    sympy.polys.orthopolys.jacobi_poly,
+    sympy.polys.orthopolys.gegenbauer_poly
+    sympy.polys.orthopolys.chebyshevt_poly
+    sympy.polys.orthopolys.chebyshevu_poly
+    sympy.polys.orthopolys.hermite_poly
+    sympy.polys.orthopolys.legendre_poly
+    sympy.polys.orthopolys.laguerre_poly
+
+    References
+    ==========
+
+    .. [1] http://en.wikipedia.org/wiki/Jacobi_polynomials
+    .. [2] http://mathworld.wolfram.com/JacobiPolynomial.html
+    .. [3] http://functions.wolfram.com/Polynomials/JacobiP/
+    """
+    nfactor = (S(2)**(a + b + 1) * (gamma(n + a + 1) * gamma(n + b + 1))
+               / (2*n + a + b + 1) / (factorial(n) * gamma(n + a + b + 1)))
+
+    return jacobi(n, a, b, x) / sqrt(nfactor)
+
+
 #----------------------------------------------------------------------------
 # Gegenbauer polynomials
 #
 
+
 class gegenbauer(OrthogonalPolynomial):
     r"""
     Gegenbauer polynomial :math:`C_n^{\left(\alpha\right)}(x)`
@@ -286,13 +352,13 @@ class gegenbauer(OrthogonalPolynomial):
         if not n.is_Number:
             # Handle this before the general sign extraction rule
             if x == S.NegativeOne:
-                if C.re(a) > S.Half:
+                if (C.re(a) > S.Half) is True:
                     return S.ComplexInfinity
                 else:
                     # No sec function available yet
                     #return (C.cos(S.Pi*(a+n)) * C.sec(S.Pi*a) * C.gamma(2*a+n) /
                     #            (C.gamma(2*a) * C.gamma(n+1)))
-                    return cls
+                    return None
 
             # Symbolic result C^a_n(x)
             # C^a_n(-x)  --->  (-1)**n * C^a_n(x)
@@ -301,12 +367,12 @@ class gegenbauer(OrthogonalPolynomial):
             # We can evaluate for some special values of x
             if x == S.Zero:
                 return (2**n * sqrt(S.Pi) * C.gamma(a + S.Half*n) /
-                        (C.gamma((1-n)/2) * C.gamma(n+1) * C.gamma(a)) )
+                        (C.gamma((1 - n)/2) * C.gamma(n + 1) * C.gamma(a)) )
             if x == S.One:
-                return C.gamma(2*a+n) / (C.gamma(2*a) * C.gamma(n+1))
+                return C.gamma(2*a + n) / (C.gamma(2*a) * C.gamma(n + 1))
             elif x == S.Infinity:
                 if n.is_positive:
-                    return C.RisingFactorial(a,n) * S.Infinity
+                    return C.RisingFactorial(a, n) * S.Infinity
         else:
             # n is a given fixed integer, evaluate into polynomial
             return gegenbauer_poly(n, a, x)
@@ -319,21 +385,23 @@ class gegenbauer(OrthogonalPolynomial):
             # Diff wrt a
             n, a, x = self.args
             k = C.Dummy("k")
-            factor1 = 2 * (1 + (-1)**(n-k)) * (k + a) / ((k + n + 2*a) * (n - k))
-            factor2 = 2*(k+1) / ((k + 2*a) * (2*k + 2*a + 1)) + 2 / (k + n + 2*a)
+            factor1 = 2 * (1 + (-1)**(n - k)) * (k + a) / ((k +
+                           n + 2*a) * (n - k))
+            factor2 = 2*(k + 1) / ((k + 2*a) * (2*k + 2*a + 1)) + \
+                2 / (k + n + 2*a)
             kern = factor1*gegenbauer(k, a, x) + factor2*gegenbauer(n, a, x)
-            return C.Sum(kern, (k, 0, n-1))
+            return C.Sum(kern, (k, 0, n - 1))
         elif argindex == 3:
             # Diff wrt x
             n, a, x = self.args
-            return 2*a*gegenbauer(n-1, a+1, x)
+            return 2*a*gegenbauer(n - 1, a + 1, x)
         else:
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_polynomial(self, n, a, x):
         k = C.Dummy("k")
-        kern = ((-1)**k * C.RisingFactorial(a, n-k) * (2*x)**(n-2*k) /
-                (C.factorial(k) * C.factorial(n-2*k)))
+        kern = ((-1)**k * C.RisingFactorial(a, n - k) * (2*x)**(n - 2*k) /
+                (C.factorial(k) * C.factorial(n - 2*k)))
         return C.Sum(kern, (k, 0, C.floor(n/2)))
 
     def _eval_conjugate(self):
@@ -344,6 +412,7 @@ class gegenbauer(OrthogonalPolynomial):
 # Chebyshev polynomials of first and second kind
 #
 
+
 class chebyshevt(OrthogonalPolynomial):
     r"""
     Chebyshev polynomial of the first kind, :math:`T_n(x)`
@@ -415,10 +484,10 @@ class chebyshevt(OrthogonalPolynomial):
             # Symbolic result T_n(x)
             # T_n(-x)  --->  (-1)**n * T_n(x)
             if x.could_extract_minus_sign():
-                return S.NegativeOne**n * chebyshevt(n,-x)
+                return S.NegativeOne**n * chebyshevt(n, -x)
             # T_{-n}(x)  --->  T_n(x)
             if n.could_extract_minus_sign():
-                return chebyshevt(-n,x)
+                return chebyshevt(-n, x)
             # We can evaluate for some special values of x
             if x == S.Zero:
                 return C.cos(S.Half * S.Pi * n)
@@ -441,15 +510,16 @@ class chebyshevt(OrthogonalPolynomial):
         elif argindex == 2:
             # Diff wrt x
             n, x = self.args
-            return n * chebyshevu(n-1, x)
+            return n * chebyshevu(n - 1, x)
         else:
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_polynomial(self, n, x):
         k = C.Dummy("k")
-        kern = C.binomial(n, 2*k) * (x**2 - 1)**k * x**(n-2*k)
+        kern = C.binomial(n, 2*k) * (x**2 - 1)**k * x**(n - 2*k)
         return C.Sum(kern, (k, 0, C.floor(n/2)))
 
+
 class chebyshevu(OrthogonalPolynomial):
     r"""
     Chebyshev polynomial of the second kind, :math:`U_n(x)`
@@ -521,13 +591,13 @@ class chebyshevu(OrthogonalPolynomial):
             # Symbolic result U_n(x)
             # U_n(-x)  --->  (-1)**n * U_n(x)
             if x.could_extract_minus_sign():
-                return S.NegativeOne**n * chebyshevu(n,-x)
+                return S.NegativeOne**n * chebyshevu(n, -x)
             # U_{-n}(x)  --->  -U_{n-2}(x)
             if n.could_extract_minus_sign():
                 if n == S.NegativeOne:
                     return S.Zero
                 else:
-                    return -chebyshevu(-n-2,x)
+                    return -chebyshevu(-n - 2, x)
             # We can evaluate for some special values of x
             if x == S.Zero:
                 return C.cos(S.Half * S.Pi * n)
@@ -542,7 +612,7 @@ class chebyshevu(OrthogonalPolynomial):
                 if n == S.NegativeOne:
                     return S.Zero
                 else:
-                    return -cls._eval_at_order(-n-2, x)
+                    return -cls._eval_at_order(-n - 2, x)
             else:
                 return cls._eval_at_order(n, x)
 
@@ -553,15 +623,17 @@ class chebyshevu(OrthogonalPolynomial):
         elif argindex == 2:
             # Diff wrt x
             n, x = self.args
-            return ((n+1) * chebyshevt(n+1, x) - x * chebyshevu(n,x)) / (x**2 - 1)
+            return ((n + 1) * chebyshevt(n + 1, x) - x * chebyshevu(n, x)) / (x**2 - 1)
         else:
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_polynomial(self, n, x):
         k = C.Dummy("k")
-        kern = S.NegativeOne**k * C.factorial(n-k) * (2*x)**(n-2*k) / (C.factorial(k) * C.factorial(n-2*k))
+        kern = S.NegativeOne**k * C.factorial(
+            n - k) * (2*x)**(n - 2*k) / (C.factorial(k) * C.factorial(n - 2*k))
         return C.Sum(kern, (k, 0, C.floor(n/2)))
 
+
 class chebyshevt_root(Function):
     r"""
     chebyshev_root(n, k) returns the kth root (indexed from zero) of
@@ -598,9 +670,11 @@ class chebyshevt_root(Function):
 
     @classmethod
     def eval(cls, n, k):
-        if not 0 <= k < n:
-            raise ValueError("must have 0 <= k < n")
-        return C.cos(S.Pi*(2*k+1)/(2*n))
+        if not ((0 <= k) is (k < n) is True):
+            raise ValueError("must have 0 <= k < n, "
+                "got k = %s and n = %s" % (k, n))
+        return C.cos(S.Pi*(2*k + 1)/(2*n))
+
 
 class chebyshevu_root(Function):
     r"""
@@ -637,14 +711,16 @@ class chebyshevu_root(Function):
 
     @classmethod
     def eval(cls, n, k):
-        if not 0 <= k < n:
-            raise ValueError("must have 0 <= k < n")
-        return C.cos(S.Pi*(k+1)/(n+1))
+        if not ((0 <= k) is (k < n) is True):
+            raise ValueError("must have 0 <= k < n, "
+                "got k = %s and n = %s" % (k, n))
+        return C.cos(S.Pi*(k + 1)/(n + 1))
 
 #----------------------------------------------------------------------------
 # Legendre polynomials and Associated Legendre polynomials
 #
 
+
 class legendre(OrthogonalPolynomial):
     r"""
     legendre(n, x) gives the nth Legendre polynomial of x, :math:`P_n(x)`
@@ -702,10 +778,10 @@ class legendre(OrthogonalPolynomial):
             # Symbolic result L_n(x)
             # L_n(-x)  --->  (-1)**n * L_n(x)
             if x.could_extract_minus_sign():
-                return S.NegativeOne**n * legendre(n,-x)
+                return S.NegativeOne**n * legendre(n, -x)
             # L_{-n}(x)  --->  L_{n-1}(x)
             if n.could_extract_minus_sign():
-                return legendre(-n - S.One,x)
+                return legendre(-n - S.One, x)
             # We can evaluate for some special values of x
             if x == S.Zero:
                 return sqrt(S.Pi)/(C.gamma(S.Half - n/2)*C.gamma(S.One + n/2))
@@ -716,7 +792,8 @@ class legendre(OrthogonalPolynomial):
         else:
             # n is a given fixed integer, evaluate into polynomial
             if n.is_negative:
-                raise ValueError("The index n must be nonnegative integer (got %r)" % n)
+                raise ValueError(
+                    "The index n must be nonnegative integer (got %r)" % n)
             else:
                 return cls._eval_at_order(n, x)
 
@@ -737,6 +814,7 @@ class legendre(OrthogonalPolynomial):
         kern = (-1)**k*C.binomial(n, k)**2*((1 + x)/2)**(n - k)*((1 - x)/2)**k
         return C.Sum(kern, (k, 0, n))
 
+
 class assoc_legendre(Function):
     r"""
     assoc_legendre(n,m, x) gives :math:`P_n^m(x)`, where n and m are
@@ -826,19 +904,21 @@ class assoc_legendre(Function):
             # Diff wrt x
             # Find better formula, this is unsuitable for x = 1
             n, m, x = self.args
-            return 1/(x**2 - 1)*(x*n*assoc_legendre(n,m,x) - (m + n)*assoc_legendre(n - 1,m,x))
+            return 1/(x**2 - 1)*(x*n*assoc_legendre(n, m, x) - (m + n)*assoc_legendre(n - 1, m, x))
         else:
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_polynomial(self, n, m, x):
         k = C.Dummy("k")
-        kern = C.factorial(2*n - 2*k)/(2**n*C.factorial(n - k)*C.factorial(k)*C.factorial(n - 2*k - m))*(-1)**k*x**(n - m - 2*k)
+        kern = C.factorial(2*n - 2*k)/(2**n*C.factorial(n - k)*C.factorial(
+            k)*C.factorial(n - 2*k - m))*(-1)**k*x**(n - m - 2*k)
         return (1 - x**2)**(m/2) * C.Sum(kern, (k, 0, C.floor((n - m)*S.Half)))
 
 #----------------------------------------------------------------------------
 # Hermite polynomials
 #
 
+
 class hermite(OrthogonalPolynomial):
     r"""
     hermite(n, x) gives the nth Hermite polynomial in x, :math:`H_n(x)`
@@ -897,7 +977,7 @@ class hermite(OrthogonalPolynomial):
             # Symbolic result H_n(x)
             # H_n(-x)  --->  (-1)**n * H_n(x)
             if x.could_extract_minus_sign():
-                return S.NegativeOne**n * hermite(n,-x)
+                return S.NegativeOne**n * hermite(n, -x)
             # We can evaluate for some special values of x
             if x == S.Zero:
                 return 2**n * sqrt(S.Pi) / C.gamma((S.One - n)/2)
@@ -906,7 +986,8 @@ class hermite(OrthogonalPolynomial):
         else:
             # n is a given fixed integer, evaluate into polynomial
             if n.is_negative:
-                raise ValueError("The index n must be nonnegative integer (got %r)" % n)
+                raise ValueError(
+                    "The index n must be nonnegative integer (got %r)" % n)
             else:
                 return cls._eval_at_order(n, x)
 
@@ -917,19 +998,20 @@ class hermite(OrthogonalPolynomial):
         elif argindex == 2:
             # Diff wrt x
             n, x = self.args
-            return 2*n*hermite(n-1, x)
+            return 2*n*hermite(n - 1, x)
         else:
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_polynomial(self, n, x):
         k = C.Dummy("k")
-        kern = (-1)**k / (C.factorial(k)*C.factorial(n-2*k)) * (2*x)**(n-2*k)
+        kern = (-1)**k / (C.factorial(k)*C.factorial(n - 2*k)) * (2*x)**(n - 2*k)
         return C.factorial(n)*C.Sum(kern, (k, 0, C.floor(n/2)))
 
 #----------------------------------------------------------------------------
 # Laguerre polynomials
 #
 
+
 class laguerre(OrthogonalPolynomial):
     r"""
     Returns the nth Laguerre polynomial in x, :math:`L_n(x)`.
@@ -992,7 +1074,7 @@ class laguerre(OrthogonalPolynomial):
             # L_{n}(-x)  --->  exp(-x) * L_{-n-1}(x)
             # L_{-n}(x)  --->  exp(x) * L_{n-1}(-x)
             if n.could_extract_minus_sign():
-                return C.exp(x) * laguerre(n-1, -x)
+                return C.exp(x) * laguerre(n - 1, -x)
             # We can evaluate for some special values of x
             if x == S.Zero:
                 return S.One
@@ -1003,7 +1085,8 @@ class laguerre(OrthogonalPolynomial):
         else:
             # n is a given fixed integer, evaluate into polynomial
             if n.is_negative:
-                raise ValueError("The index n must be nonnegative integer (got %r)" % n)
+                raise ValueError(
+                    "The index n must be nonnegative integer (got %r)" % n)
             else:
                 return laguerre_poly(n, x, 0)
 
@@ -1014,14 +1097,14 @@ class laguerre(OrthogonalPolynomial):
         elif argindex == 2:
             # Diff wrt x
             n, x = self.args
-            return -assoc_laguerre(n-1, 1, x)
+            return -assoc_laguerre(n - 1, 1, x)
         else:
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_polynomial(self, n, x):
         # TODO: Should make sure n is in N_0
         k = C.Dummy("k")
-        kern = C.RisingFactorial(-n,k) / C.factorial(k)**2 * x**k
+        kern = C.RisingFactorial(-n, k) / C.factorial(k)**2 * x**k
         return C.Sum(kern, (k, 0, n))
 
 
@@ -1051,7 +1134,8 @@ class assoc_laguerre(OrthogonalPolynomial):
     >>> assoc_laguerre(2, a, x)
     a**2/2 + 3*a/2 + x**2/2 + x*(-a - 2) + 1
     >>> assoc_laguerre(3, a, x)
-    a**3/6 + a**2 + 11*a/6 - x**3/6 + x**2*(a/2 + 3/2) + x*(-a**2/2 - 5*a/2 - 3) + 1
+    a**3/6 + a**2 + 11*a/6 - x**3/6 + x**2*(a/2 + 3/2) +
+        x*(-a**2/2 - 5*a/2 - 3) + 1
 
     >>> assoc_laguerre(n, a, 0)
     binomial(a + n, a)
@@ -1104,7 +1188,7 @@ class assoc_laguerre(OrthogonalPolynomial):
         if not n.is_Number:
             # We can evaluate for some special values of x
             if x == S.Zero:
-                return C.binomial(n+alpha, alpha)
+                return C.binomial(n + alpha, alpha)
             elif x == S.Infinity and n > S.Zero:
                 return S.NegativeOne**n * S.Infinity
             elif x == S.NegativeInfinity and n > S.Zero:
@@ -1112,7 +1196,8 @@ class assoc_laguerre(OrthogonalPolynomial):
         else:
             # n is a given fixed integer, evaluate into polynomial
             if n.is_negative:
-                raise ValueError("The index n must be nonnegative integer (got %r)" % n)
+                raise ValueError(
+                    "The index n must be nonnegative integer (got %r)" % n)
             else:
                 return laguerre_poly(n, x, alpha)
 
@@ -1124,16 +1209,17 @@ class assoc_laguerre(OrthogonalPolynomial):
             # Diff wrt alpha
             n, alpha, x = self.args
             k = C.Dummy("k")
-            return C.Sum(assoc_laguerre(k, alpha, x) / (n-alpha), (k, 0, n-1))
+            return C.Sum(assoc_laguerre(k, alpha, x) / (n - alpha), (k, 0, n - 1))
         elif argindex == 3:
             # Diff wrt x
             n, alpha, x = self.args
-            return -assoc_laguerre(n-1, alpha+1, x)
+            return -assoc_laguerre(n - 1, alpha + 1, x)
         else:
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_polynomial(self, n, x):
         # TODO: Should make sure n is in N_0
         k = C.Dummy("k")
-        kern = C.RisingFactorial(-n,k) / (C.gamma(k + alpha + 1) * C.factorial(k)) * x**k
+        kern = C.RisingFactorial(
+            -n, k) / (C.gamma(k + alpha + 1) * C.factorial(k)) * x**k
         return C.gamma(n + alpha + 1) / C.factorial(n) * C.Sum(kern, (k, 0, n))
