@@ -7,7 +7,11 @@ convex_hull
 are_similar
 
 """
+from __future__ import print_function, division
+
 from sympy import Symbol, Function, solve
+from sympy.core.compatibility import string_types
+
 
 def idiff(eq, y, x, dep=None):
     """Return dy/dx assuming that y and any other variables given in dep
@@ -34,10 +38,12 @@ def idiff(eq, y, x, dep=None):
     dep = set(dep)
     dep.add(y)
 
-    f = dict([(s, Function(s.name)(x)) for s in eq.atoms(Symbol) if s != x and s in dep])
+    f = dict([(s, Function(
+        s.name)(x)) for s in eq.atoms(Symbol) if s != x and s in dep])
     dydx = Function(y.name)(x).diff(x)
     return solve(eq.subs(f).diff(x), dydx)[0].subs(
-        [(b, a) for a, b in f.iteritems()])
+        [(b, a) for a, b in f.items()])
+
 
 def _symbol(s, matching_symbol=None):
     """Return s if s is a Symbol, else return either a new Symbol (real=True)
@@ -70,7 +76,7 @@ def _symbol(s, matching_symbol=None):
     sympy.core.symbol.Symbol
 
     """
-    if isinstance(s, basestring):
+    if isinstance(s, string_types):
         if matching_symbol and matching_symbol.name == s:
             return matching_symbol
         return Symbol(s, real=True)
@@ -79,6 +85,7 @@ def _symbol(s, matching_symbol=None):
     else:
         raise ValueError('symbol must be string for symbol name or Symbol')
 
+
 def intersection(*entities):
     """The intersection of a collection of GeometryEntity instances.
 
@@ -132,11 +139,12 @@ def intersection(*entities):
     >>> intersection(c, Point(1, 0))
     [Point(1, 0)]
     >>> intersection(c, l2)
-    [Point(-sqrt(5)/5 + 1, 2*sqrt(5)/5 + 1), Point(sqrt(5)/5 + 1, -2*sqrt(5)/5 + 1)]
+    [Point(-sqrt(5)/5 + 1, 2*sqrt(5)/5 + 1),
+     Point(sqrt(5)/5 + 1, -2*sqrt(5)/5 + 1)]
 
     """
-    from entity import GeometryEntity
-    from point import Point
+    from .entity import GeometryEntity
+    from .point import Point
 
     if len(entities) <= 1:
         return []
@@ -181,7 +189,8 @@ def convex_hull(*args):
     [1] http://en.wikipedia.org/wiki/Graham_scan
 
     [2] Andrew's Monotone Chain Algorithm
-    ( A.M. Andrew, "Another Efficient Algorithm for Convex Hulls in Two Dimensions", 1979)
+    (A.M. Andrew,
+    "Another Efficient Algorithm for Convex Hulls in Two Dimensions", 1979)
     http://softsurfer.com/Archive/algorithm_0109/algorithm_0109.htm
 
     See Also
@@ -198,10 +207,10 @@ def convex_hull(*args):
     Polygon(Point(-5, 2), Point(1, 1), Point(3, 1), Point(15, 4))
 
     """
-    from entity import GeometryEntity
-    from point import Point
-    from line import Segment
-    from polygon import Polygon
+    from .entity import GeometryEntity
+    from .point import Point
+    from .line import Segment
+    from .polygon import Polygon
 
     p = set()
     for e in args:
@@ -217,7 +226,8 @@ def convex_hull(*args):
         elif isinstance(e, Polygon):
             p.update(e.vertices)
         else:
-            raise NotImplementedError('Convex hull for %s not implemented.' % type(e))
+            raise NotImplementedError(
+                'Convex hull for %s not implemented.' % type(e))
 
     p = list(p)
     if len(p) == 1:
@@ -295,7 +305,7 @@ def are_similar(e1, e2):
     False
 
     """
-    from exceptions import GeometryError
+    from .exceptions import GeometryError
 
     if e1 == e2:
         return True
@@ -307,7 +317,9 @@ def are_similar(e1, e2):
         except AttributeError:
             n1 = e1.__class__.__name__
             n2 = e2.__class__.__name__
-            raise GeometryError("Cannot test similarity between %s and %s" % (n1, n2))
+            raise GeometryError(
+                "Cannot test similarity between %s and %s" % (n1, n2))
+
 
 def centroid(*args):
     """Find the centroid (center of mass) of the collection containing only Points,
