@@ -6,6 +6,7 @@ from sympy.abc import a, b, x, t
 
 half = S(1)/2
 
+
 def test_ratint():
     assert ratint(S(0), x) == 0
     assert ratint(S(7), x) == 7*x
@@ -14,13 +15,13 @@ def test_ratint():
     assert ratint(2*x, x) == x**2
     assert ratint(-2*x, x) == -x**2
 
-    assert ratint(8*x**7+2*x+1, x) == x**8+x**2+x
+    assert ratint(8*x**7 + 2*x + 1, x) == x**8 + x**2 + x
 
     f = S(1)
     g = x + 1
 
     assert ratint(f / g, x) == log(x + 1)
-    assert ratint((f,g), x) == log(x + 1)
+    assert ratint((f, g), x) == log(x + 1)
 
     f = x**3 - x
     g = x - 1
@@ -41,19 +42,19 @@ def test_ratint():
     assert ratint(f/g, x, real=False) == I*log(x + I)/2 - I*log(x - I)/2
 
     f = S(36)
-    g = x**5-2*x**4-2*x**3+4*x**2+x-2
+    g = x**5 - 2*x**4 - 2*x**3 + 4*x**2 + x - 2
 
     assert ratint(f/g, x) == \
         -4*log(x + 1) + 4*log(x - 2) + (12*x + 6)/(x**2 - 1)
 
-    f = x**4-3*x**2+6
-    g = x**6-5*x**4+5*x**2+4
+    f = x**4 - 3*x**2 + 6
+    g = x**6 - 5*x**4 + 5*x**2 + 4
 
     assert ratint(f/g, x) == \
         atan(x) + atan(x**3) + atan(x/2 - 3*x**S(3)/2 + S(1)/2*x**5)
 
-    f = x**7-24*x**4-4*x**2+8*x-8
-    g = x**8+6*x**6+12*x**4+8*x**2
+    f = x**7 - 24*x**4 - 4*x**2 + 8*x - 8
+    g = x**8 + 6*x**6 + 12*x**4 + 8*x**2
 
     assert ratint(f/g, x) == \
         (4 + 6*x + 8*x**2 + 3*x**3)/(4*x + 4*x**3 + x**5) + log(x)
@@ -62,65 +63,73 @@ def test_ratint():
         -(12 - 16*x + 6*x**2 - 14*x**3)/(4 + 4*x**2 + x**4) - \
         5*sqrt(2)*atan(x*sqrt(2)/2) + S(1)/2*x**2 - 3*log(2 + x**2)
 
-    f = x**5-x**4+4*x**3+x**2-x+5
-    g = x**4-2*x**3+5*x**2-4*x+4
+    f = x**5 - x**4 + 4*x**3 + x**2 - x + 5
+    g = x**4 - 2*x**3 + 5*x**2 - 4*x + 4
 
     assert ratint(f/g, x) == \
-        x + S(1)/2*x**2 + S(1)/2*log(2-x+x**2) - (4*x-9)/(14-7*x+7*x**2) + \
+        x + S(1)/2*x**2 + S(1)/2*log(2 - x + x**2) - (4*x - 9)/(14 - 7*x + 7*x**2) + \
         13*sqrt(7)*atan(-S(1)/7*sqrt(7) + 2*x*sqrt(7)/7)/49
 
-    assert ratint(1/(x**2+x+1), x) == \
+    assert ratint(1/(x**2 + x + 1), x) == \
         2*sqrt(3)*atan(sqrt(3)/3 + 2*x*sqrt(3)/3)/3
 
-    assert ratint(1/(x**3+1), x) == \
-        -log(1 - x + x**2)/6 + log(1 + x)/3 + sqrt(3)*atan(-sqrt(3)/3 + 2*x*sqrt(3)/3)/3
+    assert ratint(1/(x**3 + 1), x) == \
+        -log(1 - x + x**2)/6 + log(1 + x)/3 + sqrt(3)*atan(-sqrt(3)
+             /3 + 2*x*sqrt(3)/3)/3
 
-    assert ratint(1/(x**2+x+1), x, real=False) == \
+    assert ratint(1/(x**2 + x + 1), x, real=False) == \
         -I*3**half*log(half + x - half*I*3**half)/3 + \
         I*3**half*log(half + x + half*I*3**half)/3
 
-    assert ratint(1/(x**3+1), x, real=False) == log(1 + x)/3 + \
+    assert ratint(1/(x**3 + 1), x, real=False) == log(1 + x)/3 + \
         (-S(1)/6 + I*3**half/6)*log(-half + x + I*3**half/2) + \
         (-S(1)/6 - I*3**half/6)*log(-half + x - I*3**half/2)
 
     # Issue 1892
-    assert ratint(1/(x*(a+b*x)**3), x) == \
-        (sqrt(a**(-6))*log(x + (a - a**4*sqrt(a**(-6)))/(2*b)) +
-        (3*a + 2*b*x)/(2*a**2*b**2*x**2 + 4*b*x*a**3 + 2*a**4) -
-        sqrt(a**(-6))*log(x + (a + a**4*sqrt(a**(-6)))/(2*b)))
+    assert ratint(1/(x*(a + b*x)**3), x) == \
+        (3*a + 2*b*x)/(2*a**4 + 4*a**3*b*x + 2*a**2*b**2*x**2) + (
+            log(x) - log(a/b + x))/a**3
 
     assert ratint(x/(1 - x**2), x) == -log(x**2 - 1)/2
     assert ratint(-x/(1 - x**2), x) == log(x**2 - 1)/2
 
+    assert ratint((x/4 - 4/(1 - x)).diff(x), x) == x/4 + 4/(x - 1)
+
     ans = atan(x)
     assert ratint(1/(x**2 + 1), x, symbol=x) == ans
     assert ratint(1/(x**2 + 1), x, symbol='x') == ans
     assert ratint(1/(x**2 + 1), x, symbol=a) == ans
 
+
 def test_ratint_logpart():
-    assert ratint_logpart(x, x**2-9, x, t) == \
+    assert ratint_logpart(x, x**2 - 9, x, t) == \
         [(Poly(x**2 - 9, x), Poly(-2*t + 1, t))]
-    assert ratint_logpart(x**2, x**3-5, x, t) == \
+    assert ratint_logpart(x**2, x**3 - 5, x, t) == \
         [(Poly(x**3 - 5, x), Poly(-3*t + 1, t))]
 
+
 def test_issue_2315():
     assert ratint(1/(x**2 + 16), x) == atan(x/4)/4
 
+
 def test_issue_2150():
-    assert ratint(1/(x**2 + a**2), x) == \
-        sqrt(-1/a**2)*log(x + a**2*sqrt(-1/a**2))/2 - sqrt(-1/a**2)*log(x -
-        a**2*sqrt(-1/a**2))/2
+    assert ratint(
+        1/(x**2 + a**2), x) == (-I*log(-I*a + x)/2 + I*log(I*a + x)/2)/a
+
 
 def test_issue_2718():
     a, b, c = symbols('a,b,c', positive=True)
 
     assert simplify(ratint(a/(b*c*x**2 + a**2 + b*a), x)) == \
-        sqrt(a)*atan(sqrt(b)*sqrt(c)*x/(sqrt(a)*sqrt(a + b)))/(sqrt(b)*sqrt(c)*sqrt(a + b))
+        sqrt(a)*atan(sqrt(
+            b)*sqrt(c)*x/(sqrt(a)*sqrt(a + b)))/(sqrt(b)*sqrt(c)*sqrt(a + b))
+
 
 def test_issue_2882():
     u = symbols('u')
     assert integrate(1/(u**2 + 1)) == atan(u)
 
+
 def test_log_to_atan():
     f, g = (Poly(x + S(1)/2, x, domain='QQ'), Poly(sqrt(3)/2, x, domain='EX'))
     fg_ans = 2*atan(2*sqrt(3)*x/3 + sqrt(3)/3)
