@@ -1,18 +1,24 @@
 """ Integral Transforms """
-from sympy.integrals import integrate, Integral
+
+from __future__ import print_function, division
+
+from sympy.core import S
+from sympy.core.compatibility import reduce
+from sympy.core.function import Function
 from sympy.core.numbers import oo
 from sympy.core.symbol import Dummy
-from sympy.core.function import Function
+from sympy.integrals import integrate, Integral
+from sympy.integrals.meijerint import _dummy
 from sympy.logic.boolalg import to_cnf, conjuncts, disjuncts, Or, And
 from sympy.simplify import simplify
-from sympy.core import S
+from sympy.utilities import default_sort_key
 
-from sympy.integrals.meijerint import _dummy
 
 ##########################################################################
 # Helpers / Utilities
 ##########################################################################
 
+
 class IntegralTransformError(NotImplementedError):
     """
     Exception raised in relation to problems computing transforms.
@@ -29,6 +35,7 @@ class IntegralTransformError(NotImplementedError):
             "%s Transform could not be computed: %s." % (transform, msg))
         self.function = function
 
+
 class IntegralTransform(Function):
     """
     Base class for integral transforms.
@@ -69,7 +76,7 @@ class IntegralTransform(Function):
         is evaluated.
         """
         return self.function.free_symbols.union(set([self.transform_variable])) \
-               - set([self.function_variable])
+            - set([self.function_variable])
 
     def _compute_transform(self, f, x, s, **hints):
         raise NotImplementedError
@@ -80,7 +87,7 @@ class IntegralTransform(Function):
     def _collapse_extra(self, extra):
         from sympy import And
         cond = And(*extra)
-        if cond is False:
+        if cond == False:
             raise IntegralTransformError(self.__class__.name, None, '')
 
     def doit(self, **hints):
@@ -104,7 +111,7 @@ class IntegralTransform(Function):
         from sympy import Add, expand_mul, Mul
         from sympy.core.function import AppliedUndef
         needeval = hints.pop('needeval', False)
-        try_directly = not any(func.has(self.function_variable) \
+        try_directly = not any(func.has(self.function_variable)
                                for func in self.function.atoms(AppliedUndef))
         if try_directly:
             try:
@@ -139,7 +146,8 @@ class IntegralTransform(Function):
                 pass
 
         if needeval:
-            raise IntegralTransformError(self.__class__._name, self.function, 'needeval')
+            raise IntegralTransformError(
+                self.__class__._name, self.function, 'needeval')
 
         # TODO handle derivatives etc
 
@@ -157,12 +165,14 @@ class IntegralTransform(Function):
 
 from sympy.solvers.inequalities import _solve_inequality
 
+
 def _simplify(expr, doit):
     from sympy import powdenest, piecewise_fold
     if doit:
         return simplify(powdenest(piecewise_fold(expr), polar=True))
     return expr
 
+
 def _noconds_(default):
     """
     This is a decorator generator for dropping convergence conditions.
@@ -180,6 +190,7 @@ def _noconds_(default):
     """
     def make_wrapper(func):
         from sympy.core.decorators import wraps
+
         @wraps(func)
         def wrapper(*args, **kwargs):
             noconds = kwargs.pop('noconds', default)
@@ -199,6 +210,7 @@ _noconds = _noconds_(False)
 def _default_integrator(f, x):
     return integrate(f, (x, 0, oo))
 
+
 @_noconds
 def _mellin_transform(f, x, s_, integrator=_default_integrator, simplify=True):
     """ Backend function to compute Mellin transforms. """
@@ -206,7 +218,7 @@ def _mellin_transform(f, x, s_, integrator=_default_integrator, simplify=True):
     # We use a fresh dummy, because assumptions on s might drop conditions on
     # convergence of the integral.
     s = _dummy('s', 'mellin-transform', f)
-    F = integrator(x**(s-1) * f, x)
+    F = integrator(x**(s - 1) * f, x)
 
     if not F.has(Integral):
         return _simplify(F.subs(s, s_), simplify), (-oo, oo), True
@@ -216,7 +228,8 @@ def _mellin_transform(f, x, s_, integrator=_default_integrator, simplify=True):
 
     F, cond = F.args[0]
     if F.has(Integral):
-        raise IntegralTransformError('Mellin', f, 'integral in unexpected form')
+        raise IntegralTransformError(
+            'Mellin', f, 'integral in unexpected form')
 
     def process_conds(cond):
         """
@@ -232,15 +245,16 @@ def _mellin_transform(f, x, s_, integrator=_default_integrator, simplify=True):
             b_ = -oo
             aux_ = []
             for d in disjuncts(c):
-                d_ = d.replace(re, lambda x: x.as_real_imag()[0]).subs(re(s), t)
+                d_ = d.replace(
+                    re, lambda x: x.as_real_imag()[0]).subs(re(s), t)
                 if not d.is_Relational or \
-                   d.rel_op not in ('>', '>=', '<', '<=') \
-                   or d_.has(s) or not d_.has(t):
+                    d.rel_op not in ('>', '>=', '<', '<=') \
+                        or d_.has(s) or not d_.has(t):
                     aux_ += [d]
                     continue
                 soln = _solve_inequality(d_, t)
                 if not soln.is_Relational or \
-                   soln.rel_op not in ('>', '>=', '<', '<='):
+                        soln.rel_op not in ('>', '>=', '<', '<='):
                     aux_ += [d]
                     continue
                 if soln.lts == t:
@@ -256,8 +270,8 @@ def _mellin_transform(f, x, s_, integrator=_default_integrator, simplify=True):
         return a, b, aux
 
     conds = [process_conds(c) for c in disjuncts(cond)]
-    conds = filter(lambda x: x[2] is not False, conds)
-    conds.sort(key=lambda x: (x[0]-x[1], count_ops(x[2])))
+    conds = [x for x in conds if x[2] != False]
+    conds.sort(key=lambda x: (x[0] - x[1], count_ops(x[2])))
 
     if not conds:
         raise IntegralTransformError('Mellin', f, 'no convergence found')
@@ -265,6 +279,7 @@ def _mellin_transform(f, x, s_, integrator=_default_integrator, simplify=True):
     a, b, aux = conds[0]
     return _simplify(F.subs(s, s_), simplify), (a, b), aux
 
+
 class MellinTransform(IntegralTransform):
     """
     Class representing unevaluated Mellin transforms.
@@ -282,7 +297,7 @@ class MellinTransform(IntegralTransform):
 
     def _as_integral(self, f, x, s):
         from sympy import Integral
-        return Integral(f*x**(s-1), (x, 0, oo))
+        return Integral(f*x**(s - 1), (x, 0, oo))
 
     def _collapse_extra(self, extra):
         from sympy import And, Max, Min
@@ -294,10 +309,12 @@ class MellinTransform(IntegralTransform):
             b += [sb]
             cond += [c]
         res = (Max(*a), Min(*b)), And(*cond)
-        if (res[0][0] >= res[0][1]) is True or res[1] is False:
-            raise IntegralTransformError('Mellin', None, 'no combined convergence.')
+        if (res[0][0] >= res[0][1]) == True or res[1] == False:
+            raise IntegralTransformError(
+                'Mellin', None, 'no combined convergence.')
         return res
 
+
 def mellin_transform(f, x, s, **hints):
     r"""
     Compute the Mellin transform `F(s)` of `f(x)`,
@@ -305,17 +322,17 @@ def mellin_transform(f, x, s, **hints):
     .. math :: F(s) = \int_0^\infty x^{s-1} f(x) \mathrm{d}x.
 
     For all "sensible" functions, this converges absolutely in a strip
-      `a < Re(s) < b`.
+      `a < \operatorname{Re}(s) < b`.
 
     The Mellin transform is related via change of variables to the Fourier
     transform, and also to the (bilateral) Laplace transform.
 
-    This function returns (F, (a, b), cond)
-    where `F` is the Mellin transform of `f`, `(a, b)` is the fundamental strip
-    (as above), and cond are auxiliary convergence conditions.
+    This function returns ``(F, (a, b), cond)``
+    where ``F`` is the Mellin transform of ``f``, ``(a, b)`` is the fundamental strip
+    (as above), and ``cond`` are auxiliary convergence conditions.
 
     If the integral cannot be computed in closed form, this function returns
-    an unevaluated MellinTransform object.
+    an unevaluated :class:`MellinTransform` object.
 
     For a description of possible hints, refer to the docstring of
     :func:`sympy.integrals.transforms.IntegralTransform.doit`. If ``noconds=False``,
@@ -336,12 +353,13 @@ def mellin_transform(f, x, s, **hints):
     """
     return MellinTransform(f, x, s).doit(**hints)
 
-def _rewrite_sin((m, n), s, a, b):
+
+def _rewrite_sin(m_n, s, a, b):
     """
-    Re-write the sine function sin(m*s + n) as gamma functions, compatible
+    Re-write the sine function ``sin(m*s + n)`` as gamma functions, compatible
     with the strip (a, b).
 
-    Return (gamma1, gamma2, fac) so that f == fac/(gamma1 * gamma2).
+    Return ``(gamma1, gamma2, fac)`` so that ``f == fac/(gamma1 * gamma2)``.
 
     >>> from sympy.integrals.transforms import _rewrite_sin
     >>> from sympy import pi, S
@@ -371,17 +389,21 @@ def _rewrite_sin((m, n), s, a, b):
     # So we try to write this in such a way that the gammas are
     # eminently on the right side of the strip.
     from sympy import expand_mul, pi, ceiling, gamma, re
+    m, n = m_n
+
     m = expand_mul(m/pi)
     n = expand_mul(n/pi)
-    r = ceiling(-m*a - n.as_real_imag()[0]) # Don't use re(n), does not expand
+    r = ceiling(-m*a - n.as_real_imag()[0])  # Don't use re(n), does not expand
     return gamma(m*s + n + r), gamma(1 - n - r - m*s), (-1)**r*pi
 
+
 class MellinTransformStripError(ValueError):
     """
     Exception raised by _rewrite_gamma. Mainly for internal use.
     """
     pass
 
+
 def _rewrite_gamma(f, s, a, b):
     """
     Try to rewrite the product f(s) as a product of gamma functions,
@@ -443,6 +465,7 @@ def _rewrite_gamma(f, s, a, b):
     # 5) Combine all the exponentials.
 
     a_, b_ = S([a, b])
+
     def left(c, is_numer):
         """
         Decide whether pole at c lies to the left of the fundamental strip.
@@ -460,7 +483,7 @@ def _rewrite_gamma(f, s, a, b):
         if is_numer:
             return None
         if a_.free_symbols or b_.free_symbols or c.free_symbols:
-            return None # XXX
+            return None  # XXX
             #raise IntegralTransformError('Inverse Mellin', f,
             #                     'Could not determine position of singularity %s'
             #                     ' relative to fundamental strip' % c)
@@ -493,13 +516,14 @@ def _rewrite_gamma(f, s, a, b):
     s_multipliers = [x/common_coefficient for x in s_multipliers]
     if any(not x.is_Rational for x in s_multipliers):
         raise NotImplementedError
-    s_multiplier = common_coefficient/reduce(ilcm, [S(x.q) for x in s_multipliers], S(1))
+    s_multiplier = common_coefficient/reduce(ilcm, [S(x.q)
+                                             for x in s_multipliers], S(1))
     if s_multiplier == common_coefficient:
         if len(s_multipliers) == 0:
             s_multiplier = common_coefficient
         else:
             s_multiplier = common_coefficient \
-                           *reduce(igcd, [S(x.p) for x in s_multipliers])
+                *reduce(igcd, [S(x.p) for x in s_multipliers])
 
     exponent = S(1)
     fac = S(1)
@@ -515,7 +539,7 @@ def _rewrite_gamma(f, s, a, b):
     numer, denom = f.as_numer_denom()
     numer = Mul.make_args(numer)
     denom = Mul.make_args(denom)
-    args = zip(numer, repeat(True)) + zip(denom, repeat(False))
+    args = list(zip(numer, repeat(True))) + list(zip(denom, repeat(False)))
 
     facs = []
     dfacs = []
@@ -524,6 +548,7 @@ def _rewrite_gamma(f, s, a, b):
     denom_gammas = []
     # exponentials will contain bases for exponentials of s
     exponentials = []
+
     def exception(fact):
         return IntegralTransformError("Inverse Mellin", f, "Unrecognised form '%s'." % fact)
     while args:
@@ -551,10 +576,10 @@ def _rewrite_gamma(f, s, a, b):
         elif fact.is_Pow or isinstance(fact, exp_):
             if fact.is_Pow:
                 base = fact.base
-                exp  = fact.exp
+                exp = fact.exp
             else:
                 base = exp_polar(1)
-                exp  = fact.args[0]
+                exp = fact.args[0]
             if exp.is_Integer:
                 cond = is_numer
                 if exp < 0:
@@ -599,7 +624,8 @@ def _rewrite_gamma(f, s, a, b):
             if is_numer:
                 if (a > 0 and (left(-b/a, is_numer) is False)) or \
                    (a < 0 and (left(-b/a, is_numer) is True)):
-                    raise NotImplementedError('Gammas partially over the strip.')
+                    raise NotImplementedError(
+                        'Gammas partially over the strip.')
             ugammas += [(a, b)]
         elif isinstance(fact, sin):
             # We try to re-write all trigs as gammas. This is not in
@@ -664,13 +690,14 @@ def _rewrite_gamma(f, s, a, b):
     arg = Mul(*exponentials)
 
     # for testability, sort the arguments
-    an.sort()
-    ap.sort()
-    bm.sort()
-    bq.sort()
+    an.sort(key=default_sort_key)
+    ap.sort(key=default_sort_key)
+    bm.sort(key=default_sort_key)
+    bq.sort(key=default_sort_key)
 
     return (an, ap), (bm, bq), arg, exponent, fac
 
+
 @_noconds_(True)
 def _inverse_mellin_transform(F, s, x_, strip, as_meijerg=False):
     """ A helper for the real inverse_mellin_transform function, this one here
@@ -685,7 +712,7 @@ def _inverse_mellin_transform(F, s, x_, strip, as_meijerg=False):
         if g.is_Add:
             # do all terms separately
             ress = [_inverse_mellin_transform(G, s, x, strip, as_meijerg,
-                                              noconds=False) \
+                                              noconds=False)
                     for G in g.args]
             conds = [p[1] for p in ress]
             ress = [p[0] for p in ress]
@@ -702,11 +729,16 @@ def _inverse_mellin_transform(F, s, x_, strip, as_meijerg=False):
         if as_meijerg:
             h = G
         else:
-            h = hyperexpand(G)
+            try:
+                h = hyperexpand(G)
+            except NotImplementedError as detail:
+                raise IntegralTransformError(
+                    'Inverse Mellin', F, 'Could not calculate integral')
+
             if h.is_Piecewise and len(h.args) == 3:
                 # XXX we break modularity here!
                 h = Heaviside(x - abs(C))*h.args[0].args[0] \
-                  + Heaviside(abs(C) - x)*h.args[1].args[0]
+                    + Heaviside(abs(C) - x)*h.args[1].args[0]
         # We must ensure that the intgral along the line we want converges,
         # and return that value.
         # See [L], 5.2
@@ -716,13 +748,16 @@ def _inverse_mellin_transform(F, s, x_, strip, as_meijerg=False):
         cond += [And(Or(len(G.ap) != len(G.bq), 0 >= re(G.nu) + 1),
                      abs(arg(G.argument)) == G.delta*pi)]
         cond = Or(*cond)
-        if cond is False:
-            raise IntegralTransformError('Inverse Mellin', F, 'does not converge')
+        if cond == False:
+            raise IntegralTransformError(
+                'Inverse Mellin', F, 'does not converge')
         return (h*fac).subs(x, x_), cond
 
     raise IntegralTransformError('Inverse Mellin', F, '')
 
 _allowed = None
+
+
 class InverseMellinTransform(IntegralTransform):
     """
     Class representing unevaluated inverse Mellin transforms.
@@ -748,7 +783,7 @@ class InverseMellinTransform(IntegralTransform):
 
     @property
     def fundamental_strip(self):
-        a, b  = self.args[3], self.args[4]
+        a, b = self.args[3], self.args[4]
         if a is InverseMellinTransform._none_sentinel:
             a = None
         if b is InverseMellinTransform._none_sentinel:
@@ -759,10 +794,12 @@ class InverseMellinTransform(IntegralTransform):
         from sympy import postorder_traversal
         global _allowed
         if _allowed is None:
-            from sympy import (exp, gamma, sin, cos, tan, cot, cosh, sinh, tanh,
-                               coth, factorial, rf)
-            _allowed = set([exp, gamma, sin, cos, tan, cot, cosh, sinh, tanh, coth,
-                            factorial, rf])
+            from sympy import (
+                exp, gamma, sin, cos, tan, cot, cosh, sinh, tanh,
+                coth, factorial, rf)
+            _allowed = set(
+                [exp, gamma, sin, cos, tan, cot, cosh, sinh, tanh, coth,
+                 factorial, rf])
         for f in postorder_traversal(F):
             if f.is_Function and f.has(s) and f.func not in _allowed:
                 raise IntegralTransformError('Inverse Mellin', F,
@@ -775,6 +812,7 @@ class InverseMellinTransform(IntegralTransform):
         c = self.__class__._c
         return Integral(F*x**(-s), (s, c - I*oo, c + I*oo))
 
+
 def inverse_mellin_transform(F, s, x, strip, **hints):
     r"""
     Compute the inverse Mellin transform of `F(s)` over the fundamental
@@ -789,11 +827,11 @@ def inverse_mellin_transform(F, s, x, strip, **hints):
     this recovers `f` from its Mellin transform `F`
     (and vice versa), for positive real `x`.
 
-    One of `a` or `b` may be passed as None; a suitable `c` will be
+    One of `a` or `b` may be passed as ``None``; a suitable `c` will be
     inferred.
 
     If the integral cannot be computed in closed form, this function returns
-    an unevaluated InverseMellinTransform object.
+    an unevaluated :class:`InverseMellinTransform` object.
 
     Note that this function will assume x to be positive and real, regardless
     of the sympy assumptions!
@@ -811,11 +849,11 @@ def inverse_mellin_transform(F, s, x, strip, **hints):
 
     >>> f = 1/(s**2 - 1)
     >>> inverse_mellin_transform(f, s, x, (-oo, -1))
-    x*(1 - 1/x**2)*Heaviside(x - 1)/2
+    (x/2 - 1/(2*x))*Heaviside(x - 1)
     >>> inverse_mellin_transform(f, s, x, (-1, 1))
     -x*Heaviside(-x + 1)/2 - Heaviside(x - 1)/(2*x)
     >>> inverse_mellin_transform(f, s, x, (1, oo))
-    (-x**2/2 + 1/2)*Heaviside(-x + 1)/x
+    (-x/2 + 1/(2*x))*Heaviside(-x + 1)
 
     See Also
     ========
@@ -831,8 +869,8 @@ def inverse_mellin_transform(F, s, x, strip, **hints):
 ##########################################################################
 
 def _simplifyconds(expr, s, a):
-    """
-    Naively simplify some conditions occuring in ``expr``, given that Re(s) > a.
+    r"""
+    Naively simplify some conditions occuring in ``expr``, given that `\operatorname{Re}(s) > a`.
 
     >>> from sympy.integrals.transforms import _simplifyconds as simp
     >>> from sympy.abc import x
@@ -868,6 +906,7 @@ def _simplifyconds(expr, s, a):
         if ex.is_Pow and ex.base == s:
             return ex.exp
         return None
+
     def bigger(ex1, ex2):
         """ Return True only if |ex1| > |ex2|, False only if |ex1| < |ex2|.
             Else return None. """
@@ -886,19 +925,22 @@ def _simplifyconds(expr, s, a):
             return False
         if n < 0 and (abs(ex1) >= abs(a)**n) is True:
             return True
+
     def replie(x, y):
         """ simplify x < y """
         if not (x.is_positive or x.func is Abs) \
-           or not (y.is_positive or y.func is Abs):
+                or not (y.is_positive or y.func is Abs):
             return (x < y)
         r = bigger(x, y)
         if r is not None:
             return not r
         return (x < y)
+
     def replue(x, y):
         if bigger(x, y) in (True, False):
             return True
         return Unequality(x, y)
+
     def repl(ex, *args):
         if isinstance(ex, bool):
             return ex
@@ -908,6 +950,7 @@ def _simplifyconds(expr, s, a):
     expr = repl(expr, Unequality, replue)
     return expr
 
+
 @_noconds
 def _laplace_transform(f, t, s_, simplify=True):
     """ The backend function for Laplace transforms. """
@@ -920,11 +963,13 @@ def _laplace_transform(f, t, s_, simplify=True):
         return _simplify(F.subs(s, s_), simplify), -oo, True
 
     if not F.is_Piecewise:
-        raise IntegralTransformError('Laplace', f, 'could not compute integral')
+        raise IntegralTransformError(
+            'Laplace', f, 'could not compute integral')
 
     F, cond = F.args[0]
     if F.has(Integral):
-        raise IntegralTransformError('Laplace', f, 'integral in unexpected form')
+        raise IntegralTransformError(
+            'Laplace', f, 'integral in unexpected form')
 
     def process_conds(conds):
         """ Turn ``conds`` into a strip and auxiliary conditions. """
@@ -932,7 +977,8 @@ def _laplace_transform(f, t, s_, simplify=True):
         aux = True
         conds = conjuncts(to_cnf(conds))
         u = Dummy('u', real=True)
-        p, q, w1, w2, w3, w4, w5 = symbols('p q w1 w2 w3 w4 w5', cls=Wild, exclude=[s])
+        p, q, w1, w2, w3, w4, w5 = symbols(
+            'p q w1 w2 w3 w4 w5', cls=Wild, exclude=[s])
         for c in conds:
             a_ = oo
             aux_ = []
@@ -947,20 +993,23 @@ def _laplace_transform(f, t, s_, simplify=True):
                 if m:
                     if m[q] > 0 and m[w2]/m[p] == pi/2:
                         d = re(s + m[w3]) > 0
-                m = d.match(0 < cos(abs(arg(s**w1*w5, q))*w2)*abs(s**w3)**w4 - p)
+                m = d.match(
+                    0 < cos(abs(arg(s**w1*w5, q))*w2)*abs(s**w3)**w4 - p)
                 if not m:
-                    m = d.match(0 < cos(abs(arg(polar_lift(s)**w1*w5, q))*w2)*abs(s**w3)**w4 - p)
+                    m = d.match(0 < cos(abs(
+                        arg(polar_lift(s)**w1*w5, q))*w2)*abs(s**w3)**w4 - p)
                 if m and all(m[wild] > 0 for wild in [w1, w2, w3, w4, w5]):
                     d = re(s) > m[p]
-                d_ = d.replace(re, lambda x: x.expand().as_real_imag()[0]).subs(re(s), t)
+                d_ = d.replace(
+                    re, lambda x: x.expand().as_real_imag()[0]).subs(re(s), t)
                 if not d.is_Relational or \
-                   d.rel_op not in ('>', '>=', '<', '<=') \
-                   or d_.has(s) or not d_.has(t):
+                    d.rel_op not in ('>', '>=', '<', '<=') \
+                        or d_.has(s) or not d_.has(t):
                     aux_ += [d]
                     continue
                 soln = _solve_inequality(d_, t)
                 if not soln.is_Relational or \
-                   soln.rel_op not in ('>', '>=', '<', '<='):
+                        soln.rel_op not in ('>', '>=', '<', '<='):
                     aux_ += [d]
                     continue
                 if soln.lts == t:
@@ -975,10 +1024,11 @@ def _laplace_transform(f, t, s_, simplify=True):
         return a, aux
 
     conds = [process_conds(c) for c in disjuncts(cond)]
-    conds2 = filter(lambda x: x[1] is not False and x[0] != -oo, conds)
+    conds2 = [x for x in conds if x[1] != False and x[0] != -oo]
     if not conds2:
-        conds2 = filter(lambda x: x[1] is not False, conds)
+        conds2 = [x for x in conds if x[1] != False]
     conds = conds2
+
     def cnt(expr):
         if isinstance(expr, bool):
             return 0
@@ -998,6 +1048,7 @@ def _laplace_transform(f, t, s_, simplify=True):
         aux = _simplifyconds(aux, s, a)
     return _simplify(F.subs(s, s_), simplify), sbs(a), sbs(aux)
 
+
 class LaplaceTransform(IntegralTransform):
     """
     Class representing unevaluated Laplace transforms.
@@ -1033,10 +1084,12 @@ class LaplaceTransform(IntegralTransform):
             planes.append(plane)
         cond = And(*conds)
         plane = Max(*planes)
-        if cond is False:
-            raise IntegralTransformError('Laplace', None, 'No combined convergence.')
+        if cond == False:
+            raise IntegralTransformError(
+                'Laplace', None, 'No combined convergence.')
         return plane, cond
 
+
 def laplace_transform(f, t, s, **hints):
     r"""
     Compute the Laplace Transform `F(s)` of `f(t)`,
@@ -1044,14 +1097,14 @@ def laplace_transform(f, t, s, **hints):
     .. math :: F(s) = \int_0^\infty e^{-st} f(t) \mathrm{d}t.
 
     For all "sensible" functions, this converges absolutely in a
-    half plane  `a < Re(s)`.
+    half plane  `a < \operatorname{Re}(s)`.
 
-    This function returns (F, a, cond)
-    where `F` is the Laplace transform of `f`, `Re(s) > a` is the half-plane
-    of convergence, and cond are auxiliary convergence conditions.
+    This function returns ``(F, a, cond)``
+    where ``F`` is the Laplace transform of ``f``, `\operatorname{Re}(s) > a` is the half-plane
+    of convergence, and ``cond`` are auxiliary convergence conditions.
 
     If the integral cannot be computed in closed form, this function returns
-    an unevaluated LaplaceTransform object.
+    an unevaluated :class:`LaplaceTransform` object.
 
     For a description of possible hints, refer to the docstring of
     :func:`sympy.integrals.transforms.IntegralTransform.doit`. If ``noconds=True``,
@@ -1060,7 +1113,7 @@ def laplace_transform(f, t, s, **hints):
     >>> from sympy.integrals import laplace_transform
     >>> from sympy.abc import t, s, a
     >>> laplace_transform(t**a, t, s)
-    (s**(-a - 1)*gamma(a + 1), 0, -re(a) < 1)
+    (s**(-a)*gamma(a + 1)/s, 0, -re(a) < 1)
 
     See Also
     ========
@@ -1070,6 +1123,7 @@ def laplace_transform(f, t, s, **hints):
     """
     return LaplaceTransform(f, t, s).doit(**hints)
 
+
 @_noconds_(True)
 def _inverse_laplace_transform(F, s, t_, plane, simplify=True):
     """ The backend function for inverse Laplace transforms. """
@@ -1080,6 +1134,7 @@ def _inverse_laplace_transform(F, s, t_, plane, simplify=True):
     # 2) Use the inversion integral.
 
     t = Dummy('t', real=True)
+
     def pw_simp(*args):
         """ Simplify a piecewise expression from hyperexpand. """
         # XXX we break modularity here!
@@ -1090,7 +1145,7 @@ def _inverse_laplace_transform(F, s, t_, plane, simplify=True):
         e1 = args[0].args[0]
         e2 = args[1].args[0]
         return Heaviside(1/abs(coeff) - t**exponent)*e1 \
-             + Heaviside(t**exponent - 1/abs(coeff))*e2
+            + Heaviside(t**exponent - 1/abs(coeff))*e2
 
     try:
         f, cond = inverse_mellin_transform(F, s, exp(-t), (None, oo),
@@ -1116,6 +1171,7 @@ def _inverse_laplace_transform(F, s, t_, plane, simplify=True):
         return f.subs(t, t_), cond
 
     u = Dummy('u')
+
     def simp_heaviside(arg):
         a = arg.subs(exp(-t), u)
         if a.has(t):
@@ -1138,6 +1194,7 @@ def _inverse_laplace_transform(F, s, t_, plane, simplify=True):
 
     return _simplify(f.subs(t, t_), simplify), cond
 
+
 class InverseLaplaceTransform(IntegralTransform):
     """
     Class representing unevaluated inverse Laplace transforms.
@@ -1174,6 +1231,7 @@ class InverseLaplaceTransform(IntegralTransform):
         c = self.__class__._c
         return Integral(exp(s*t)*F, (s, c - I*oo, c + I*oo))
 
+
 def inverse_laplace_transform(F, s, t, plane=None, **hints):
     r"""
     Compute the inverse Laplace transform of `F(s)`, defined as
@@ -1181,7 +1239,7 @@ def inverse_laplace_transform(F, s, t, plane=None, **hints):
     .. math :: f(t) = \int_{c-i\infty}^{c+i\infty} e^{st} F(s) \mathrm{d}s,
 
     for `c` so large that `F(s)` has no singularites in the
-    half-plane `Re(s) > c-\epsilon`.
+    half-plane `\operatorname{Re}(s) > c-\epsilon`.
 
     The plane can be specified by
     argument ``plane``, but will be inferred if passed as None.
@@ -1191,7 +1249,7 @@ def inverse_laplace_transform(F, s, t, plane=None, **hints):
     versa.
 
     If the integral cannot be computed in closed form, this function returns
-    an unevaluated InverseLaplaceTransform object.
+    an unevaluated :class:`InverseLaplaceTransform` object.
 
     Note that this function will always assume `t` to be real,
     regardless of the sympy assumption on `t`.
@@ -1243,6 +1301,7 @@ def _fourier_transform(f, x, k, a, b, name, simplify=True):
 
     return _simplify(F, simplify), cond
 
+
 class FourierTypeTransform(IntegralTransform):
     """ Base class for Fourier transforms.
         Specify cls._a and cls._b.
@@ -1259,6 +1318,7 @@ class FourierTypeTransform(IntegralTransform):
         b = self.__class__._b
         return Integral(a*f*exp(b*I*x*k), (x, -oo, oo))
 
+
 class FourierTransform(FourierTypeTransform):
     """
     Class representing unevaluated Fourier transforms.
@@ -1273,6 +1333,7 @@ class FourierTransform(FourierTypeTransform):
     _a = 1
     _b = -2*S.Pi
 
+
 def fourier_transform(f, x, k, **hints):
     r"""
     Compute the unitary, ordinary-frequency Fourier transform of `f`, defined
@@ -1281,7 +1342,7 @@ def fourier_transform(f, x, k, **hints):
     .. math:: F(k) = \int_{-\infty}^\infty f(x) e^{-2\pi i x k} \mathrm{d} x.
 
     If the transform cannot be computed in closed form, this
-    function returns an unevaluated FourierTransform object.
+    function returns an unevaluated :class:`FourierTransform` object.
 
     For other Fourier transform conventions, see the function
     :func:`sympy.integrals.transforms._fourier_transform`.
@@ -1308,6 +1369,7 @@ def fourier_transform(f, x, k, **hints):
     """
     return FourierTransform(f, x, k).doit(**hints)
 
+
 class InverseFourierTransform(FourierTypeTransform):
     """
     Class representing unevaluated inverse Fourier transforms.
@@ -1322,6 +1384,7 @@ class InverseFourierTransform(FourierTypeTransform):
     _a = 1
     _b = 2*S.Pi
 
+
 def inverse_fourier_transform(F, k, x, **hints):
     r"""
     Compute the unitary, ordinary-frequency inverse Fourier transform of `F`,
@@ -1330,7 +1393,7 @@ def inverse_fourier_transform(F, k, x, **hints):
     .. math:: f(x) = \int_{-\infty}^\infty F(k) e^{2\pi i x k} \mathrm{d} k.
 
     If the transform cannot be computed in closed form, this
-    function returns an unevaluated InverseFourierTransform object.
+    function returns an unevaluated :class:`InverseFourierTransform` object.
 
     For other Fourier transform conventions, see the function
     :func:`sympy.integrals.transforms._fourier_transform`.
@@ -1364,6 +1427,7 @@ def inverse_fourier_transform(F, k, x, **hints):
 
 from sympy import sin, cos, sqrt, pi, I, oo
 
+
 @_noconds_(True)
 def _sine_cosine_transform(f, x, k, a, b, K, name, simplify=True):
     """
@@ -1388,6 +1452,7 @@ def _sine_cosine_transform(f, x, k, a, b, K, name, simplify=True):
 
     return _simplify(F, simplify), cond
 
+
 class SineCosineTypeTransform(IntegralTransform):
     """
     Base class for sine and cosine transforms.
@@ -1407,6 +1472,7 @@ class SineCosineTypeTransform(IntegralTransform):
         K = self.__class__._kern
         return Integral(a*f*K(b*x*k), (x, 0, oo))
 
+
 class SineTransform(SineCosineTypeTransform):
     """
     Class representing unevaluated sine transforms.
@@ -1422,6 +1488,7 @@ class SineTransform(SineCosineTypeTransform):
     _a = sqrt(2)/sqrt(pi)
     _b = 1
 
+
 def sine_transform(f, x, k, **hints):
     r"""
     Compute the unitary, ordinary-frequency sine transform of `f`, defined
@@ -1430,7 +1497,7 @@ def sine_transform(f, x, k, **hints):
     .. math:: F(k) = \sqrt{\frac{2}{\pi}} \int_{0}^\infty f(x) \sin(2\pi x k) \mathrm{d} x.
 
     If the transform cannot be computed in closed form, this
-    function returns an unevaluated SineTransform object.
+    function returns an unevaluated :class:`SineTransform` object.
 
     For a description of possible hints, refer to the docstring of
     :func:`sympy.integrals.transforms.IntegralTransform.doit`.
@@ -1454,6 +1521,7 @@ def sine_transform(f, x, k, **hints):
     """
     return SineTransform(f, x, k).doit(**hints)
 
+
 class InverseSineTransform(SineCosineTypeTransform):
     """
     Class representing unevaluated inverse sine transforms.
@@ -1469,6 +1537,7 @@ class InverseSineTransform(SineCosineTypeTransform):
     _a = sqrt(2)/sqrt(pi)
     _b = 1
 
+
 def inverse_sine_transform(F, k, x, **hints):
     r"""
     Compute the unitary, ordinary-frequency inverse sine transform of `F`,
@@ -1477,7 +1546,7 @@ def inverse_sine_transform(F, k, x, **hints):
     .. math:: f(x) = \sqrt{\frac{2}{\pi}} \int_{0}^\infty F(k) \sin(2\pi x k) \mathrm{d} k.
 
     If the transform cannot be computed in closed form, this
-    function returns an unevaluated InverseSineTransform object.
+    function returns an unevaluated :class:`InverseSineTransform` object.
 
     For a description of possible hints, refer to the docstring of
     :func:`sympy.integrals.transforms.IntegralTransform.doit`.
@@ -1485,7 +1554,8 @@ def inverse_sine_transform(F, k, x, **hints):
 
     >>> from sympy import inverse_sine_transform, exp, sqrt, gamma, pi
     >>> from sympy.abc import x, k, a
-    >>> inverse_sine_transform(2**((1-2*a)/2)*k**(a - 1)*gamma(-a/2 + 1)/gamma((a+1)/2), k, x)
+    >>> inverse_sine_transform(2**((1-2*a)/2)*k**(a - 1)*
+    ...     gamma(-a/2 + 1)/gamma((a+1)/2), k, x)
     x**(-a)
     >>> inverse_sine_transform(sqrt(2)*k*exp(-k**2/(4*a))/(4*sqrt(a)**3), k, x)
     x*exp(-a*x**2)
@@ -1501,6 +1571,7 @@ def inverse_sine_transform(F, k, x, **hints):
     """
     return InverseSineTransform(F, k, x).doit(**hints)
 
+
 class CosineTransform(SineCosineTypeTransform):
     """
     Class representing unevaluated cosine transforms.
@@ -1516,6 +1587,7 @@ class CosineTransform(SineCosineTypeTransform):
     _a = sqrt(2)/sqrt(pi)
     _b = 1
 
+
 def cosine_transform(f, x, k, **hints):
     r"""
     Compute the unitary, ordinary-frequency cosine transform of `f`, defined
@@ -1524,7 +1596,7 @@ def cosine_transform(f, x, k, **hints):
     .. math:: F(k) = \sqrt{\frac{2}{\pi}} \int_{0}^\infty f(x) \cos(2\pi x k) \mathrm{d} x.
 
     If the transform cannot be computed in closed form, this
-    function returns an unevaluated CosineTransform object.
+    function returns an unevaluated :class:`CosineTransform` object.
 
     For a description of possible hints, refer to the docstring of
     :func:`sympy.integrals.transforms.IntegralTransform.doit`.
@@ -1535,7 +1607,7 @@ def cosine_transform(f, x, k, **hints):
     >>> cosine_transform(exp(-a*x), x, k)
     sqrt(2)*a/(sqrt(pi)*(a**2 + k**2))
     >>> cosine_transform(exp(-a*sqrt(x))*cos(a*sqrt(x)), x, k)
-    a*(-sinh(a**2/(2*k)) + cosh(a**2/(2*k)))/(2*k**(3/2))
+    a*exp(-a**2/(2*k))/(2*k**(3/2))
 
     See Also
     ========
@@ -1548,6 +1620,7 @@ def cosine_transform(f, x, k, **hints):
     """
     return CosineTransform(f, x, k).doit(**hints)
 
+
 class InverseCosineTransform(SineCosineTypeTransform):
     """
     Class representing unevaluated inverse cosine transforms.
@@ -1563,6 +1636,7 @@ class InverseCosineTransform(SineCosineTypeTransform):
     _a = sqrt(2)/sqrt(pi)
     _b = 1
 
+
 def inverse_cosine_transform(F, k, x, **hints):
     r"""
     Compute the unitary, ordinary-frequency inverse cosine transform of `F`,
@@ -1571,7 +1645,7 @@ def inverse_cosine_transform(F, k, x, **hints):
     .. math:: f(x) = \sqrt{\frac{2}{\pi}} \int_{0}^\infty F(k) \cos(2\pi x k) \mathrm{d} k.
 
     If the transform cannot be computed in closed form, this
-    function returns an unevaluated InverseCosineTransform object.
+    function returns an unevaluated :class:`InverseCosineTransform` object.
 
     For a description of possible hints, refer to the docstring of
     :func:`sympy.integrals.transforms.IntegralTransform.doit`.
@@ -1580,7 +1654,7 @@ def inverse_cosine_transform(F, k, x, **hints):
     >>> from sympy import inverse_cosine_transform, exp, sqrt, pi
     >>> from sympy.abc import x, k, a
     >>> inverse_cosine_transform(sqrt(2)*a/(sqrt(pi)*(a**2 + k**2)), k, x)
-    -sinh(a*x) + cosh(a*x)
+    exp(-a*x)
     >>> inverse_cosine_transform(1/sqrt(k), k, x)
     1/sqrt(x)
 
@@ -1622,6 +1696,7 @@ def _hankel_transform(f, r, k, nu, name, simplify=True):
 
     return _simplify(F, simplify), cond
 
+
 class HankelTypeTransform(IntegralTransform):
     """
     Base class for Hankel transforms.
@@ -1650,6 +1725,7 @@ class HankelTypeTransform(IntegralTransform):
                                  self.transform_variable,
                                  self.args[3])
 
+
 class HankelTransform(HankelTypeTransform):
     """
     Class representing unevaluated Hankel transforms.
@@ -1662,6 +1738,7 @@ class HankelTransform(HankelTypeTransform):
 
     _name = 'Hankel'
 
+
 def hankel_transform(f, r, k, nu, **hints):
     r"""
     Compute the Hankel transform of `f`, defined as
@@ -1681,7 +1758,7 @@ def hankel_transform(f, r, k, nu, **hints):
 
     >>> ht = hankel_transform(1/r**m, r, k, nu)
     >>> ht
-    2**(-m + 1)*k**(m - 2)*gamma(-m/2 + nu/2 + 1)/gamma(m/2 + nu/2)
+    2*2**(-m)*k**(m - 2)*gamma(-m/2 + nu/2 + 1)/gamma(m/2 + nu/2)
 
     >>> inverse_hankel_transform(ht, k, r, nu)
     r**(-m)
@@ -1691,7 +1768,7 @@ def hankel_transform(f, r, k, nu, **hints):
     a/(k**3*(a**2/k**2 + 1)**(3/2))
 
     >>> inverse_hankel_transform(ht, k, r, 0)
-    -sinh(a*r) + cosh(a*r)
+    exp(-a*r)
 
     See Also
     ========
@@ -1704,6 +1781,7 @@ def hankel_transform(f, r, k, nu, **hints):
     """
     return HankelTransform(f, r, k, nu).doit(**hints)
 
+
 class InverseHankelTransform(HankelTypeTransform):
     """
     Class representing unevaluated inverse Hankel transforms.
@@ -1716,6 +1794,7 @@ class InverseHankelTransform(HankelTypeTransform):
 
     _name = 'Inverse Hankel'
 
+
 def inverse_hankel_transform(F, k, r, nu, **hints):
     r"""
     Compute the inverse Hankel transform of `F` defined as
@@ -1735,7 +1814,7 @@ def inverse_hankel_transform(F, k, r, nu, **hints):
 
     >>> ht = hankel_transform(1/r**m, r, k, nu)
     >>> ht
-    2**(-m + 1)*k**(m - 2)*gamma(-m/2 + nu/2 + 1)/gamma(m/2 + nu/2)
+    2*2**(-m)*k**(m - 2)*gamma(-m/2 + nu/2 + 1)/gamma(m/2 + nu/2)
 
     >>> inverse_hankel_transform(ht, k, r, nu)
     r**(-m)
@@ -1745,7 +1824,7 @@ def inverse_hankel_transform(F, k, r, nu, **hints):
     a/(k**3*(a**2/k**2 + 1)**(3/2))
 
     >>> inverse_hankel_transform(ht, k, r, 0)
-    -sinh(a*r) + cosh(a*r)
+    exp(-a*r)
 
     See Also
     ========
