@@ -1,14 +1,24 @@
-from calculus import defun
-from itertools import izip
+try:
+    from itertools import izip
+except ImportError:
+    izip = zip
+
+from ..libmp.backend import xrange
+from .calculus import defun
+
+try:
+    next = next
+except NameError:
+    next = lambda _: _.next()
 
 @defun
 def richardson(ctx, seq):
     r"""
     Given a list ``seq`` of the first `N` elements of a slowly convergent
-    infinite sequence, :func:`richardson` computes the `N`-term
+    infinite sequence, :func:`~mpmath.richardson` computes the `N`-term
     Richardson extrapolate for the limit.
 
-    :func:`richardson` returns `(v, c)` where `v` is the estimated
+    :func:`~mpmath.richardson` returns `(v, c)` where `v` is the estimated
     limit and `c` is the magnitude of the largest weight used during the
     computation. The weight provides an estimate of the precision
     lost to cancellation. Due to cancellation effects, the sequence must
@@ -18,31 +28,31 @@ def richardson(ctx, seq):
     **Applicability and issues**
 
     The `N`-step Richardson extrapolation algorithm used by
-    :func:`richardson` is described in [1].
+    :func:`~mpmath.richardson` is described in [1].
 
     Richardson extrapolation only works for a specific type of sequence,
     namely one converging like partial sums of
     `P(1)/Q(1) + P(2)/Q(2) + \ldots` where `P` and `Q` are polynomials.
     When the sequence does not convergence at such a rate
-    :func:`richardson` generally produces garbage.
+    :func:`~mpmath.richardson` generally produces garbage.
 
     Richardson extrapolation has the advantage of being fast: the `N`-term
     extrapolate requires only `O(N)` arithmetic operations, and usually
     produces an estimate that is accurate to `O(N)` digits. Contrast with
-    the Shanks transformation (see :func:`shanks`), which requires
+    the Shanks transformation (see :func:`~mpmath.shanks`), which requires
     `O(N^2)` operations.
 
-    :func:`richardson` is unable to produce an estimate for the
+    :func:`~mpmath.richardson` is unable to produce an estimate for the
     approximation error. One way to estimate the error is to perform
     two extrapolations with slightly different `N` and comparing the
     results.
 
     Richardson extrapolation does not work for oscillating sequences.
-    As a simple workaround, :func:`richardson` detects if the last
+    As a simple workaround, :func:`~mpmath.richardson` detects if the last
     three elements do not differ monotonically, and in that case
     applies extrapolation only to the even-index elements.
 
-    **Example**
+    **Examples**
 
     Applying Richardson extrapolation to the Leibniz series for `\pi`::
 
@@ -64,8 +74,7 @@ def richardson(ctx, seq):
 
     **References**
 
-    1. C. M. Bender & S. A. Orszag, "Advanced Mathematical Methods for
-       Scientists and Engineers", Springer 1999, pp. 375-376
+    1. [BenderOrszag]_ pp. 375-376
 
     """
     assert len(seq) >= 3
@@ -89,13 +98,13 @@ def richardson(ctx, seq):
 def shanks(ctx, seq, table=None, randomized=False):
     r"""
     Given a list ``seq`` of the first `N` elements of a slowly
-    convergent infinite sequence `(A_k)`, :func:`shanks` computes the iterated
+    convergent infinite sequence `(A_k)`, :func:`~mpmath.shanks` computes the iterated
     Shanks transformation `S(A), S(S(A)), \ldots, S^{N/2}(A)`. The Shanks
     transformation often provides strong convergence acceleration,
     especially if the sequence is oscillating.
 
     The iterated Shanks transformation is computed using the Wynn
-    epsilon algorithm (see [1]). :func:`shanks` returns the full
+    epsilon algorithm (see [1]). :func:`~mpmath.shanks` returns the full
     epsilon table generated by Wynn's algorithm, which can be read
     off as follows:
 
@@ -116,7 +125,7 @@ def shanks(ctx, seq, table=None, randomized=False):
     so that ``shanks(seq)[-1][-1]`` always gives an estimate of the
     limit.
 
-    Optionally, an existing table can be passed to :func:`shanks`.
+    Optionally, an existing table can be passed to :func:`~mpmath.shanks`.
     This can be used to efficiently extend a previous computation after
     new elements have been appended to the sequence. The table will
     then be updated in-place.
@@ -154,7 +163,7 @@ def shanks(ctx, seq, table=None, randomized=False):
 
     If the Shanks transformation converges to the exact limit (such
     as if the sequence is a geometric series), then a division by
-    zero occurs. By default, :func:`shanks` handles this case by
+    zero occurs. By default, :func:`~mpmath.shanks` handles this case by
     terminating the iteration and returning the table it has
     generated so far. With *randomized=True*, it will instead
     replace the zero by a pseudorandom number close to zero.
@@ -209,12 +218,9 @@ def shanks(ctx, seq, table=None, randomized=False):
 
     **References**
 
-    1. P. R. Graves-Morris, D. E. Roberts, A. Salam, "The epsilon
-       algorithm and related topics", Journal of Computational and
-       Applied Mathematics, Volume 122, Issue 1-2  (October 2000)
+    1. [GravesMorris]_
 
-    2. C. M. Bender & S. A. Orszag, "Advanced Mathematical Methods for
-       Scientists and Engineers", Springer 1999, pp. 368-375
+    2. [BenderOrszag]_ pp. 368-375
 
     """
     assert len(seq) >= 2
@@ -254,9 +260,81 @@ def shanks(ctx, seq, table=None, randomized=False):
         table.append(row)
     return table
 
+@defun
+def sumap(ctx, f, interval, integral=None, error=False):
+    r"""
+    Evaluates an infinite series of an analytic summand *f* using the
+    Abel-Plana formula
+
+    .. math ::
+
+        \sum_{k=0}^{\infty} f(k) = \int_0^{\infty} f(t) dt + \frac{1}{2} f(0) +
+            i \int_0^{\infty} \frac{f(it)-f(-it)}{e^{2\pi t}-1} dt.
+
+    Unlike the Euler-Maclaurin formula (see :func:`~mpmath.sumem`),
+    the Abel-Plana formula does not require derivatives. However,
+    it only works when `|f(it)-f(-it)|` does not
+    increase too rapidly with `t`.
+
+    **Examples**
+
+    The Abel-Plana formula is particularly useful when the summand
+    decreases like a power of `k`; for example when the sum is a pure
+    zeta function::
+
+        >>> from mpmath import *
+        >>> mp.dps = 25; mp.pretty = True
+        >>> sumap(lambda k: 1/k**2.5, [1,inf])
+        1.34148725725091717975677
+        >>> zeta(2.5)
+        1.34148725725091717975677
+        >>> sumap(lambda k: 1/(k+1j)**(2.5+2.5j), [1,inf])
+        (-3.385361068546473342286084 - 0.7432082105196321803869551j)
+        >>> zeta(2.5+2.5j, 1+1j)
+        (-3.385361068546473342286084 - 0.7432082105196321803869551j)
+
+    If the series is alternating, numerical quadrature along the real
+    line is likely to give poor results, so it is better to evaluate
+    the first term symbolically whenever possible:
+
+        >>> n=3; z=-0.75
+        >>> I = expint(n,-log(z))
+        >>> chop(sumap(lambda k: z**k / k**n, [1,inf], integral=I))
+        -0.6917036036904594510141448
+        >>> polylog(n,z)
+        -0.6917036036904594510141448
+
+    """
+    prec = ctx.prec
+    try:
+        ctx.prec += 10
+        a, b = interval
+        assert b == ctx.inf
+        g = lambda x: f(x+a)
+        if integral is None:
+            i1, err1 = ctx.quad(g, [0,ctx.inf], error=True)
+        else:
+            i1, err1 = integral, 0
+        j = ctx.j
+        p = ctx.pi * 2
+        if ctx._is_real_type(i1):
+            h = lambda t: -2 * ctx.im(g(j*t)) / ctx.expm1(p*t)
+        else:
+            h = lambda t: j*(g(j*t)-g(-j*t)) / ctx.expm1(p*t)
+        i2, err2 = ctx.quad(h, [0,ctx.inf], error=True)
+        err = err1+err2
+        v = i1+i2+0.5*g(ctx.mpf(0))
+    finally:
+        ctx.prec = prec
+    if error:
+        return +v, err
+    return +v
+
+
 @defun
 def sumem(ctx, f, interval, tol=None, reject=10, integral=None,
-    adiffs=None, bdiffs=None, verbose=False, error=False):
+    adiffs=None, bdiffs=None, verbose=False, error=False,
+    _fast_abort=False):
     r"""
     Uses the Euler-Maclaurin formula to compute an approximation accurate
     to within ``tol`` (which defaults to the present epsilon) of the sum
@@ -287,7 +365,7 @@ def sumem(ctx, f, interval, tol=None, reject=10, integral=None,
     often be "forced" if `b = \infty` by summing up to `a+N` and then
     applying the Euler-Maclaurin formula to the sum over the range
     `(a+N+1, \ldots, \infty)`. This procedure is implemented by
-    :func:`nsum`.
+    :func:`~mpmath.nsum`.
 
     By default numerical quadrature and differentiation is used.
     If the symbolic values of the integral and endpoint derivatives
@@ -307,7 +385,7 @@ def sumem(ctx, f, interval, tol=None, reject=10, integral=None,
         >>> sumem(lambda n: 1/n**2, [32, inf])
         0.03174336652030209012658168043874142714132886413417
         >>> I = mpf(1)/32
-        >>> D = adiffs=((-1)**n*fac(n+1)*32**(-2-n) for n in xrange(999))
+        >>> D = adiffs=((-1)**n*fac(n+1)*32**(-2-n) for n in range(999))
         >>> sumem(lambda n: 1/n**2, [32, inf], integral=I, adiffs=D)
         0.03174336652030209012658168043874142714132886413417
 
@@ -315,7 +393,7 @@ def sumem(ctx, f, interval, tol=None, reject=10, integral=None,
 
         >>> sumem(lambda n: n**5-12*n**2+3*n, [-100000, 200000])
         10500155000624963999742499550000.0
-        >>> print sum(n**5-12*n**2+3*n for n in xrange(-100000, 200001))
+        >>> print(sum(n**5-12*n**2+3*n for n in range(-100000, 200001)))
         10500155000624963999742499550000
 
     """
@@ -340,14 +418,16 @@ def sumem(ctx, f, interval, tol=None, reject=10, integral=None,
                 term = (db-da) * ctx.bernoulli(k+1) / ctx.factorial(k+1)
                 mag = abs(term)
                 if verbose:
-                    print "term", k, "magnitude =", ctx.nstr(mag)
+                    print("term", k, "magnitude =", ctx.nstr(mag))
                 if k > 4 and mag < tol:
                     s += term
                     break
                 elif k > 4 and abs(prev) / mag < reject:
-                    if verbose:
-                        print "Failed to converge"
                     err += mag
+                    if _fast_abort:
+                        return [s, (s, err)][error]
+                    if verbose:
+                        print("Failed to converge")
                     break
                 else:
                     s += term
@@ -357,13 +437,13 @@ def sumem(ctx, f, interval, tol=None, reject=10, integral=None,
         if b != ctx.inf: s += f(b)/2
         # Tail integral
         if verbose:
-            print "Integrating f(x) from x = %s to %s" % (ctx.nstr(a), ctx.nstr(b))
+            print("Integrating f(x) from x = %s to %s" % (ctx.nstr(a), ctx.nstr(b)))
         if integral:
             s += integral
         else:
             integral, ierr = ctx.quad(f, interval, error=True)
             if verbose:
-                print "Integration error:", ierr
+                print("Integration error:", ierr)
             s += integral
             err += ierr
     finally:
@@ -376,7 +456,10 @@ def sumem(ctx, f, interval, tol=None, reject=10, integral=None,
 @defun
 def adaptive_extrapolation(ctx, update, emfun, kwargs):
     option = kwargs.get
-    tol = option('tol', ctx.eps/2**10)
+    if ctx._fixed_precision:
+        tol = option('tol', ctx.eps*2**10)
+    else:
+        tol = option('tol', ctx.eps/2**10)
     verbose = option('verbose', False)
     maxterms = option('maxterms', ctx.dps*10)
     method = option('method', 'r+s').split('+')
@@ -400,8 +483,10 @@ def adaptive_extrapolation(ctx, update, emfun, kwargs):
     best = ctx.zero
     orig = ctx.prec
     try:
-        if TRY_RICHARDSON or TRY_SHANKS:
-            ctx.prec *= 4
+        if 'workprec' in kwargs:
+            ctx.prec = kwargs['workprec']
+        elif TRY_RICHARDSON or TRY_SHANKS:
+            ctx.prec = (ctx.prec+10) * 4
         else:
             ctx.prec += 30
         while 1:
@@ -410,12 +495,12 @@ def adaptive_extrapolation(ctx, update, emfun, kwargs):
 
             # Get new batch of terms
             try:
-                step = steps.next()
+                step = next(steps)
             except StopIteration:
                 pass
             if verbose:
-                print "-"*70
-                print "Adding terms #%i-#%i" % (index, index+step)
+                print("-"*70)
+                print("Adding terms #%i-#%i" % (index, index+step))
             update(partial, xrange(index, index+step))
             index += step
 
@@ -423,7 +508,7 @@ def adaptive_extrapolation(ctx, update, emfun, kwargs):
             best = partial[-1]
             error = abs(best - partial[-2])
             if verbose:
-                print "Direct error: %s" % ctx.nstr(error)
+                print("Direct error: %s" % ctx.nstr(error))
             if error <= tol:
                 return best
 
@@ -433,7 +518,7 @@ def adaptive_extrapolation(ctx, update, emfun, kwargs):
                 # Convergence
                 richardson_error = abs(value - last_richardson_value)
                 if verbose:
-                    print "Richardson error: %s" % ctx.nstr(richardson_error)
+                    print("Richardson error: %s" % ctx.nstr(richardson_error))
                 # Convergence
                 if richardson_error <= tol:
                     return value
@@ -441,7 +526,7 @@ def adaptive_extrapolation(ctx, update, emfun, kwargs):
                 # Unreliable due to cancellation
                 if ctx.eps*maxc > tol:
                     if verbose:
-                        print "Ran out of precision for Richardson"
+                        print("Ran out of precision for Richardson")
                     TRY_RICHARDSON = False
                 if richardson_error < error:
                     error = richardson_error
@@ -456,12 +541,12 @@ def adaptive_extrapolation(ctx, update, emfun, kwargs):
                     est1, maxc, est2 = row[-1], abs(row[-2]), row[-3]
                     shanks_error = abs(est1-est2)
                 if verbose:
-                    print "Shanks error: %s" % ctx.nstr(shanks_error)
+                    print("Shanks error: %s" % ctx.nstr(shanks_error))
                 if shanks_error <= tol:
                     return est1
                 if ctx.eps*maxc > tol:
                     if verbose:
-                        print "Ran out of precision for Shanks"
+                        print("Ran out of precision for Shanks")
                     TRY_SHANKS = False
                 if shanks_error < error:
                     error = shanks_error
@@ -477,7 +562,7 @@ def adaptive_extrapolation(ctx, update, emfun, kwargs):
                     value, em_error = emfun(index, tol)
                     value += partial[-1]
                     if verbose:
-                        print "Euler-Maclaurin error: %s" % ctx.nstr(em_error)
+                        print("Euler-Maclaurin error: %s" % ctx.nstr(em_error))
                     if em_error <= tol:
                         return value
                     if em_error < error:
@@ -487,20 +572,27 @@ def adaptive_extrapolation(ctx, update, emfun, kwargs):
     if strict:
         raise ctx.NoConvergence
     if verbose:
-        print "Warning: failed to converge to target accuracy"
+        print("Warning: failed to converge to target accuracy")
     return best
 
 @defun
-def nsum(ctx, f, interval, **kwargs):
+def nsum(ctx, f, *intervals, **options):
     r"""
     Computes the sum
 
     .. math :: S = \sum_{k=a}^b f(k)
 
     where `(a, b)` = *interval*, and where `a = -\infty` and/or
-    `b = \infty` are allowed. Two examples of
-    infinite series that can be summed by :func:`nsum`, where the
-    first converges rapidly and the second converges slowly, are::
+    `b = \infty` are allowed, or more generally
+
+    .. math :: S = \sum_{k_1=a_1}^{b_1} \cdots
+                   \sum_{k_n=a_n}^{b_n} f(k_1,\ldots,k_n)
+
+    if multiple intervals are given.
+
+    Two examples of infinite series that can be summed by :func:`~mpmath.nsum`,
+    where the first converges rapidly and the second converges slowly,
+    are::
 
         >>> from mpmath import *
         >>> mp.dps = 15; mp.pretty = True
@@ -509,11 +601,16 @@ def nsum(ctx, f, interval, **kwargs):
         >>> nsum(lambda n: 1/n**2, [1, inf])
         1.64493406684823
 
-    When appropriate, :func:`nsum` applies convergence acceleration to
-    accurately estimate the sums of slowly convergent series.
-    If the sum is finite, :func:`nsum` currently does not
-    attempt to perform any extrapolation, and simply calls
-    :func:`fsum`.
+    When appropriate, :func:`~mpmath.nsum` applies convergence acceleration to
+    accurately estimate the sums of slowly convergent series. If the series is
+    finite, :func:`~mpmath.nsum` currently does not attempt to perform any
+    extrapolation, and simply calls :func:`~mpmath.fsum`.
+
+    Multidimensional infinite series are reduced to a single-dimensional
+    series over expanding hypercubes; if both infinite and finite dimensions
+    are present, the finite ranges are moved innermost. For more advanced
+    control over the summation order, use nested calls to :func:`~mpmath.nsum`,
+    or manually rewrite the sum as a single-dimensional series.
 
     **Options**
 
@@ -540,10 +637,16 @@ def nsum(ctx, f, interval, **kwargs):
     *verbose*
         Print details about progress.
 
+    *ignore*
+        If enabled, any term that raises ``ArithmeticError``
+        or ``ValueError`` (e.g. through division by zero) is replaced
+        by a zero. This is convenient for lattice sums with
+        a singular term near the origin.
+
     **Methods**
 
     Unfortunately, an algorithm that can efficiently sum any infinite
-    series does not exist. :func:`nsum` implements several different
+    series does not exist. :func:`~mpmath.nsum` implements several different
     algorithms that each work well in different cases. The *method*
     keyword argument selects a method.
 
@@ -557,14 +660,14 @@ def nsum(ctx, f, interval, **kwargs):
     ``'richardson'`` / ``'r'``:
         Uses Richardson extrapolation. Provides useful extrapolation
         when `f(k) \sim P(k)/Q(k)` or when `f(k) \sim (-1)^k P(k)/Q(k)`
-        for polynomials `P` and `Q`. See :func:`richardson` for
+        for polynomials `P` and `Q`. See :func:`~mpmath.richardson` for
         additional information.
 
     ``'shanks'`` / ``'s'``:
         Uses Shanks transformation. Typically provides useful
         extrapolation when `f(k) \sim c^k` or when successive terms
         alternate signs. Is able to sum some divergent series.
-        See :func:`shanks` for additional information.
+        See :func:`~mpmath.shanks` for additional information.
 
     ``'euler-maclaurin'`` / ``'e'``:
         Uses the Euler-Maclaurin summation formula to approximate
@@ -572,7 +675,7 @@ def nsum(ctx, f, interval, **kwargs):
         numerical derivatives and numerical integration. The advantage
         of this algorithm is that it works regardless of the
         decay rate of `f`, as long as `f` is sufficiently smooth.
-        See :func:`sumem` for additional information.
+        See :func:`~mpmath.sumem` for additional information.
 
     ``'direct'`` / ``'d'``:
         Does not perform any extrapolation. This can be used
@@ -595,7 +698,7 @@ def nsum(ctx, f, interval, **kwargs):
         >>> nsum(lambda k: 1/(1+k**2), [-inf, inf])
         3.15334809493716
 
-    :func:`nsum` handles sums of complex numbers::
+    :func:`~mpmath.nsum` handles sums of complex numbers::
 
         >>> nsum(lambda k: (0.5+0.25j)**k, [0, inf])
         (1.6 + 0.8j)
@@ -703,7 +806,7 @@ def nsum(ctx, f, interval, **kwargs):
     series. In particular, it is often able to sum Taylor series
     beyond their radius of convergence (this is due to a relation
     between the Shanks transformation and Pade approximations;
-    see :func:`pade` for an alternative way to evaluate divergent
+    see :func:`~mpmath.pade` for an alternative way to evaluate divergent
     Taylor series).
 
     Here we apply it to `\log(1+x)` far outside the region of
@@ -725,8 +828,8 @@ def nsum(ctx, f, interval, **kwargs):
         >>> for n in range(-8, 8):
         ...     if n == 1:
         ...         continue
-        ...     print mpf(n), mpf(1)/(1-n), nsum(lambda k: n**k, [0, inf],
-        ...         method='shanks')
+        ...     print("%s %s %s" % (mpf(n), mpf(1)/(1-n),
+        ...         nsum(lambda k: n**k, [0, inf], method='shanks')))
         ...
         -8.0 0.111111111111111 0.111111111111111
         -7.0 0.125 0.125
@@ -744,16 +847,97 @@ def nsum(ctx, f, interval, **kwargs):
         6.0 -0.2 -0.2
         7.0 -0.166666666666667 -0.166666666666667
 
-    """
-    a, b = ctx._as_points(interval)
-    if a == ctx.ninf:
-        if b == ctx.inf:
-            return f(0) + ctx.nsum(lambda k: f(-k) + f(k), [1, ctx.inf], **kwargs)
-        return ctx.nsum(f, [-b, ctx.inf], **kwargs)
-    elif b != ctx.inf:
-        return ctx.fsum(f(ctx.mpf(k)) for k in xrange(int(a), int(b)+1))
+    **Multidimensional sums**
 
-    a = int(a)
+    Any combination of finite and infinite ranges is allowed for the
+    summation indices::
+
+        >>> mp.dps = 15
+        >>> nsum(lambda x,y: x+y, [2,3], [4,5])
+        28.0
+        >>> nsum(lambda x,y: x/2**y, [1,3], [1,inf])
+        6.0
+        >>> nsum(lambda x,y: y/2**x, [1,inf], [1,3])
+        6.0
+        >>> nsum(lambda x,y,z: z/(2**x*2**y), [1,inf], [1,inf], [3,4])
+        7.0
+        >>> nsum(lambda x,y,z: y/(2**x*2**z), [1,inf], [3,4], [1,inf])
+        7.0
+        >>> nsum(lambda x,y,z: x/(2**z*2**y), [3,4], [1,inf], [1,inf])
+        7.0
+
+    Some nice examples of double series with analytic solutions or
+    reductions to single-dimensional series (see [1])::
+
+        >>> nsum(lambda m, n: 1/2**(m*n), [1,inf], [1,inf])
+        1.60669515241529
+        >>> nsum(lambda n: 1/(2**n-1), [1,inf])
+        1.60669515241529
+
+        >>> nsum(lambda i,j: (-1)**(i+j)/(i**2+j**2), [1,inf], [1,inf])
+        0.278070510848213
+        >>> pi*(pi-3*ln2)/12
+        0.278070510848213
+
+        >>> nsum(lambda i,j: (-1)**(i+j)/(i+j)**2, [1,inf], [1,inf])
+        0.129319852864168
+        >>> altzeta(2) - altzeta(1)
+        0.129319852864168
+
+        >>> nsum(lambda i,j: (-1)**(i+j)/(i+j)**3, [1,inf], [1,inf])
+        0.0790756439455825
+        >>> altzeta(3) - altzeta(2)
+        0.0790756439455825
+
+        >>> nsum(lambda m,n: m**2*n/(3**m*(n*3**m+m*3**n)),
+        ...     [1,inf], [1,inf])
+        0.28125
+        >>> mpf(9)/32
+        0.28125
+
+        >>> nsum(lambda i,j: fac(i-1)*fac(j-1)/fac(i+j),
+        ...     [1,inf], [1,inf], workprec=400)
+        1.64493406684823
+        >>> zeta(2)
+        1.64493406684823
+
+    A hard example of a multidimensional sum is the Madelung constant
+    in three dimensions (see [2]). The defining sum converges very
+    slowly and only conditionally, so :func:`~mpmath.nsum` is lucky to
+    obtain an accurate value through convergence acceleration. The
+    second evaluation below uses a much more efficient, rapidly
+    convergent 2D sum::
+
+        >>> nsum(lambda x,y,z: (-1)**(x+y+z)/(x*x+y*y+z*z)**0.5,
+        ...     [-inf,inf], [-inf,inf], [-inf,inf], ignore=True)
+        -1.74756459463318
+        >>> nsum(lambda x,y: -12*pi*sech(0.5*pi * \
+        ...     sqrt((2*x+1)**2+(2*y+1)**2))**2, [0,inf], [0,inf])
+        -1.74756459463318
+
+    Another example of a lattice sum in 2D::
+
+        >>> nsum(lambda x,y: (-1)**(x+y) / (x**2+y**2), [-inf,inf],
+        ...     [-inf,inf], ignore=True)
+        -2.1775860903036
+        >>> -pi*ln2
+        -2.1775860903036
+
+    An example of an Eisenstein series::
+
+        >>> nsum(lambda m,n: (m+n*1j)**(-4), [-inf,inf], [-inf,inf],
+        ...     ignore=True)
+        (3.1512120021539 + 0.0j)
+
+    **References**
+
+    1. [Weisstein]_ http://mathworld.wolfram.com/DoubleSeries.html,
+    2. [Weisstein]_ http://mathworld.wolfram.com/MadelungConstants.html
+
+    """
+    infinite, g = standardize(ctx, f, intervals, options)
+    if not infinite:
+        return +g()
 
     def update(partial_sums, indices):
         if partial_sums:
@@ -761,7 +945,7 @@ def nsum(ctx, f, interval, **kwargs):
         else:
             psum = ctx.zero
         for k in indices:
-            psum = psum + f(a + ctx.mpf(k))
+            psum = psum + g(ctx.mpf(k))
             partial_sums.append(psum)
 
     prec = ctx.prec
@@ -769,33 +953,160 @@ def nsum(ctx, f, interval, **kwargs):
     def emfun(point, tol):
         workprec = ctx.prec
         ctx.prec = prec + 10
-        v = ctx.sumem(f, [a+point, ctx.inf], tol, error=1)
+        v = ctx.sumem(g, [point, ctx.inf], tol, error=1)
         ctx.prec = workprec
         return v
 
-    return +ctx.adaptive_extrapolation(update, emfun, kwargs)
+    return +ctx.adaptive_extrapolation(update, emfun, options)
+
+
+def wrapsafe(f):
+    def g(*args):
+        try:
+            return f(*args)
+        except (ArithmeticError, ValueError):
+            return 0
+    return g
+
+def standardize(ctx, f, intervals, options):
+    if options.get("ignore"):
+        f = wrapsafe(f)
+    finite = []
+    infinite = []
+    for k, points in enumerate(intervals):
+        a, b = ctx._as_points(points)
+        if b < a:
+            return False, (lambda: ctx.zero)
+        if a == ctx.ninf or b == ctx.inf:
+            infinite.append((k, (a,b)))
+        else:
+            finite.append((k, (int(a), int(b))))
+    if finite:
+        f = fold_finite(ctx, f, finite)
+        if not infinite:
+            return False, lambda: f(*([0]*len(intervals)))
+    if infinite:
+        f = standardize_infinite(ctx, f, infinite)
+        f = fold_infinite(ctx, f, infinite)
+        args = [0] * len(intervals)
+        d = infinite[0][0]
+        def g(k):
+            args[d] = k
+            return f(*args)
+        return True, g
+
+# backwards compatible itertools.product
+def cartesian_product(args):
+    pools = map(tuple, args)
+    result = [[]]
+    for pool in pools:
+        result = [x+[y] for x in result for y in pool]
+    for prod in result:
+        yield tuple(prod)
+
+def fold_finite(ctx, f, intervals):
+    if not intervals:
+        return f
+    indices = [v[0] for v in intervals]
+    points = [v[1] for v in intervals]
+    ranges = [xrange(a, b+1) for (a,b) in points]
+    def g(*args):
+        args = list(args)
+        s = ctx.zero
+        for xs in cartesian_product(ranges):
+            for dim, x in zip(indices, xs):
+                args[dim] = ctx.mpf(x)
+            s += f(*args)
+        return s
+    #print "Folded finite", indices
+    return g
+
+# Standardize each interval to [0,inf]
+def standardize_infinite(ctx, f, intervals):
+    if not intervals:
+        return f
+    dim, [a,b] = intervals[-1]
+    if a == ctx.ninf:
+        if b == ctx.inf:
+            def g(*args):
+                args = list(args)
+                k = args[dim]
+                if k:
+                    s = f(*args)
+                    args[dim] = -k
+                    s += f(*args)
+                    return s
+                else:
+                    return f(*args)
+        else:
+            def g(*args):
+                args = list(args)
+                args[dim] = b - args[dim]
+                return f(*args)
+    else:
+        def g(*args):
+            args = list(args)
+            args[dim] += a
+            return f(*args)
+    #print "Standardized infinity along dimension", dim, a, b
+    return standardize_infinite(ctx, g, intervals[:-1])
+
+def fold_infinite(ctx, f, intervals):
+    if len(intervals) < 2:
+        return f
+    dim1 = intervals[-2][0]
+    dim2 = intervals[-1][0]
+    # Assume intervals are [0,inf] x [0,inf] x ...
+    def g(*args):
+        args = list(args)
+        #args.insert(dim2, None)
+        n = int(args[dim1])
+        s = ctx.zero
+        #y = ctx.mpf(n)
+        args[dim2] = ctx.mpf(n) #y
+        for x in xrange(n+1):
+            args[dim1] = ctx.mpf(x)
+            s += f(*args)
+        args[dim1] = ctx.mpf(n) #ctx.mpf(n)
+        for y in xrange(n):
+            args[dim2] = ctx.mpf(y)
+            s += f(*args)
+        return s
+    #print "Folded infinite from", len(intervals), "to", (len(intervals)-1)
+    return fold_infinite(ctx, g, intervals[:-1])
 
 @defun
-def nprod(ctx, f, interval, **kwargs):
-    """
+def nprod(ctx, f, interval, nsum=False, **kwargs):
+    r"""
     Computes the product
 
-    .. math :: P = \prod_{k=a}^b f(k)
+    .. math ::
+
+        P = \prod_{k=a}^b f(k)
 
     where `(a, b)` = *interval*, and where `a = -\infty` and/or
     `b = \infty` are allowed.
 
-    This function is essentially equivalent to applying :func:`nsum`
-    to the logarithm of the product (which, of course, becomes a
-    series). All keyword arguments passed to :func:`nprod` are
-    forwarded verbatim to :func:`nsum`.
+    By default, :func:`~mpmath.nprod` uses the same extrapolation methods as
+    :func:`~mpmath.nsum`, except applied to the partial products rather than
+    partial sums, and the same keyword options as for :func:`~mpmath.nsum` are
+    supported. If ``nsum=True``, the product is instead computed via
+    :func:`~mpmath.nsum` as
+
+    .. math ::
+
+        P = \exp\left( \sum_{k=a}^b \log(f(k)) \right).
+
+    This is slower, but can sometimes yield better results. It is
+    also required (and used automatically) when Euler-Maclaurin
+    summation is requested.
 
     **Examples**
 
     A simple finite product::
 
         >>> from mpmath import *
-        >>> mp.dps = 15; mp.pretty = True
+        >>> mp.dps = 25; mp.pretty = True
         >>> nprod(lambda k: k, [1, 4])
         24.0
 
@@ -806,95 +1117,128 @@ def nprod(ctx, f, interval, **kwargs):
     A few infinite products with simple values are::
 
         >>> 2*nprod(lambda k: (4*k**2)/(4*k**2-1), [1, inf])
-        3.14159265358979
+        3.141592653589793238462643
         >>> nprod(lambda k: (1+1/k)**2/(1+2/k), [1, inf])
         2.0
         >>> nprod(lambda k: (k**3-1)/(k**3+1), [2, inf])
-        0.666666666666667
+        0.6666666666666666666666667
         >>> nprod(lambda k: (1-1/k**2), [2, inf])
         0.5
 
     Next, several more infinite products with more complicated
     values::
 
-        >>> nprod(lambda k: exp(1/k**2), [1, inf])
-        5.18066831789712
-        >>> exp(pi**2/6)
-        5.18066831789712
+        >>> nprod(lambda k: exp(1/k**2), [1, inf]); exp(pi**2/6)
+        5.180668317897115748416626
+        5.180668317897115748416626
 
-        >>> nprod(lambda k: (k**2-1)/(k**2+1), [2, inf])
-        0.272029054982133
-        >>> pi*csch(pi)
-        0.272029054982133
+        >>> nprod(lambda k: (k**2-1)/(k**2+1), [2, inf]); pi*csch(pi)
+        0.2720290549821331629502366
+        0.2720290549821331629502366
 
         >>> nprod(lambda k: (k**4-1)/(k**4+1), [2, inf])
-        0.8480540493529
+        0.8480540493529003921296502
         >>> pi*sinh(pi)/(cosh(sqrt(2)*pi)-cos(sqrt(2)*pi))
-        0.8480540493529
+        0.8480540493529003921296502
 
         >>> nprod(lambda k: (1+1/k+1/k**2)**2/(1+2/k+3/k**2), [1, inf])
-        1.84893618285824
+        1.848936182858244485224927
         >>> 3*sqrt(2)*cosh(pi*sqrt(3)/2)**2*csch(pi*sqrt(2))/pi
-        1.84893618285824
+        1.848936182858244485224927
 
-        >>> nprod(lambda k: (1-1/k**4), [2, inf])
-        0.919019477593744
-        >>> sinh(pi)/(4*pi)
-        0.919019477593744
+        >>> nprod(lambda k: (1-1/k**4), [2, inf]); sinh(pi)/(4*pi)
+        0.9190194775937444301739244
+        0.9190194775937444301739244
 
         >>> nprod(lambda k: (1-1/k**6), [2, inf])
-        0.982684277742192
+        0.9826842777421925183244759
         >>> (1+cosh(pi*sqrt(3)))/(12*pi**2)
-        0.982684277742192
+        0.9826842777421925183244759
 
-        >>> nprod(lambda k: (1+1/k**2), [2, inf])
-        1.83803895518749
-        >>> sinh(pi)/(2*pi)
-        1.83803895518749
+        >>> nprod(lambda k: (1+1/k**2), [2, inf]); sinh(pi)/(2*pi)
+        1.838038955187488860347849
+        1.838038955187488860347849
 
         >>> nprod(lambda n: (1+1/n)**n * exp(1/(2*n)-1), [1, inf])
-        1.44725592689037
+        1.447255926890365298959138
         >>> exp(1+euler/2)/sqrt(2*pi)
-        1.44725592689037
+        1.447255926890365298959138
 
     The following two products are equivalent and can be evaluated in
     terms of a Jacobi theta function. Pi can be replaced by any value
     (as long as convergence is preserved)::
 
         >>> nprod(lambda k: (1-pi**-k)/(1+pi**-k), [1, inf])
-        0.383845120748167
+        0.3838451207481672404778686
         >>> nprod(lambda k: tanh(k*log(pi)/2), [1, inf])
-        0.383845120748167
+        0.3838451207481672404778686
         >>> jtheta(4,0,1/pi)
-        0.383845120748167
+        0.3838451207481672404778686
 
     This product does not have a known closed form value::
 
         >>> nprod(lambda k: (1-1/2**k), [1, inf])
-        0.288788095086602
+        0.2887880950866024212788997
+
+    A product taken from `-\infty`::
+
+        >>> nprod(lambda k: 1-k**(-3), [-inf,-2])
+        0.8093965973662901095786805
+        >>> cosh(pi*sqrt(3)/2)/(3*pi)
+        0.8093965973662901095786805
+
+    A doubly infinite product::
+
+        >>> nprod(lambda k: exp(1/(1+k**2)), [-inf, inf])
+        23.41432688231864337420035
+        >>> exp(pi/tanh(pi))
+        23.41432688231864337420035
+
+    A product requiring the use of Euler-Maclaurin summation to compute
+    an accurate value::
+
+        >>> nprod(lambda k: (1-1/k**2.5), [2, inf], method='e')
+        0.696155111336231052898125
 
     **References**
 
-    1. E. W. Weisstein, "Infinite Product",
-       http://mathworld.wolfram.com/InfiniteProduct.html,
-       MathWorld
+    1. [Weisstein]_ http://mathworld.wolfram.com/InfiniteProduct.html
 
     """
+    if nsum or ('e' in kwargs.get('method', '')):
+        orig = ctx.prec
+        try:
+            # TODO: we are evaluating log(1+eps) -> eps, which is
+            # inaccurate. This currently works because nsum greatly
+            # increases the working precision. But we should be
+            # more intelligent and handle the precision here.
+            ctx.prec += 10
+            v = ctx.nsum(lambda n: ctx.ln(f(n)), interval, **kwargs)
+        finally:
+            ctx.prec = orig
+        return +ctx.exp(v)
+
     a, b = ctx._as_points(interval)
-    if a != ctx.ninf and b != ctx.inf:
+    if a == ctx.ninf:
+        if b == ctx.inf:
+            return f(0) * ctx.nprod(lambda k: f(-k) * f(k), [1, ctx.inf], **kwargs)
+        return ctx.nprod(f, [-b, ctx.inf], **kwargs)
+    elif b != ctx.inf:
         return ctx.fprod(f(ctx.mpf(k)) for k in xrange(int(a), int(b)+1))
 
-    orig = ctx.prec
-    try:
-        # TODO: we are evaluating log(1+eps) -> eps, which is
-        # inaccurate. This currently works because nsum greatly
-        # increases the working precision. But we should be
-        # more intelligent and handle the precision here.
-        ctx.prec += 10
-        v = ctx.nsum(lambda n: ctx.ln(f(n)), interval, **kwargs)
-    finally:
-        ctx.prec = orig
-    return +ctx.exp(v)
+    a = int(a)
+
+    def update(partial_products, indices):
+        if partial_products:
+            pprod = partial_products[-1]
+        else:
+            pprod = ctx.one
+        for k in indices:
+            pprod = pprod * f(a + ctx.mpf(k))
+            partial_products.append(pprod)
+
+    return +ctx.adaptive_extrapolation(update, None, kwargs)
+
 
 @defun
 def limit(ctx, f, x, direction=1, exp=False, **kwargs):
@@ -907,30 +1251,30 @@ def limit(ctx, f, x, direction=1, exp=False, **kwargs):
 
     where `x` may be finite or infinite.
 
-    For finite `x`, :func:`limit` evaluates `f(x + d/n)` for
+    For finite `x`, :func:`~mpmath.limit` evaluates `f(x + d/n)` for
     consecutive integer values of `n`, where the approach direction
     `d` may be specified using the *direction* keyword argument.
-    For infinite `x`, :func:`limit` evaluates values of
+    For infinite `x`, :func:`~mpmath.limit` evaluates values of
     `f(\mathrm{sign}(x) \cdot n)`.
 
     If the approach to the limit is not sufficiently fast to give
-    an accurate estimate directly, :func:`limit` attempts to find
+    an accurate estimate directly, :func:`~mpmath.limit` attempts to find
     the limit using Richardson extrapolation or the Shanks
     transformation. You can select between these methods using
-    the *method* keyword (see documentation of :func:`nsum` for
+    the *method* keyword (see documentation of :func:`~mpmath.nsum` for
     more information).
 
     **Options**
 
     The following options are available with essentially the
-    same meaning as for :func:`nsum`: *tol*, *method*, *maxterms*,
+    same meaning as for :func:`~mpmath.nsum`: *tol*, *method*, *maxterms*,
     *steps*, *verbose*.
 
     If the option *exp=True* is set, `f` will be
     sampled at exponentially spaced points `n = 2^1, 2^2, 2^3, \ldots`
     instead of the linearly spaced points `n = 1, 2, 3, \ldots`.
     This can sometimes improve the rate of convergence so that
-    :func:`limit` may return a more accurate answer (and faster).
+    :func:`~mpmath.limit` may return a more accurate answer (and faster).
     However, do note that this can only be used if `f`
     supports fast and accurate evaluation for arguments that
     are extremely close to the limit point (or if infinite,
@@ -974,7 +1318,7 @@ def limit(ctx, f, x, direction=1, exp=False, **kwargs):
 
     (which converges notoriously slowly)::
 
-        >>> f = lambda n: sum([mpf(1)/k for k in range(1,n+1)]) - log(n)
+        >>> f = lambda n: sum([mpf(1)/k for k in range(1,int(n)+1)]) - log(n)
         >>> limit(f, inf)
         0.577215664901532860606512090082
         >>> +euler
@@ -986,7 +1330,7 @@ def limit(ctx, f, x, direction=1, exp=False, **kwargs):
 
         >>> f = lambda x: sqrt(x**3+x**2)/(sqrt(x**3)+x)
         >>> limit(f, inf)
-        0.992518488562331431132360378669
+        0.992831158558330281129249686491
         >>> limit(f, inf, exp=True)
         1.0
 
