@@ -48,19 +48,13 @@ formula can not compute to the wanted precision.
 
 """
 
-# XXX: currently the loggamma function can generate inaccurate values
-# at high precision. As a temporary workaround, multiply the precision
-# in affected places by the following factor
-LOGGAMMA_BROKENNESS_FACTOR = 1.5
-
-
 import math
 
 class RSCache:
      def __init__(ctx):
         ctx._rs_cache = [0, 10, {}, {}]
 
-from functions import defun
+from .functions import defun
 
 #-------------------------------------------------------------------------------#
 #                                                                               #
@@ -451,8 +445,14 @@ def Rzeta_simul(ctx, s, der=0):
     ctx.prec = xwpd[1]+10
     xpsigma = 1-(2*xsigma)
     xd = {}
-    xd[0,0,-2]=0; xd[0,0,-1]=0; xd[0,0,0]=1; xd[0,0,1]=0
-    xd[0,-1,-2]=0; xd[0,-1,-1]=0; xd[0,-1,0]=1; xd[0,-1,1]=0
+    xd[0,0,-2]=0
+    xd[0,0,-1]=0
+    xd[0,0,0]=1
+    xd[0,0,1]=0
+    xd[0,-1,-2]=0
+    xd[0,-1,-1]=0
+    xd[0,-1,0]=1
+    xd[0,-1,1]=0
     for n in range(1,L):
         ctx.prec = xwpd[n]+10
         for k in range(0,3*n//2+1):
@@ -468,7 +468,9 @@ def Rzeta_simul(ctx, s, der=0):
                 for r in range(0,k):
                     add=xd[0,n,r]*(ctx.mpf('1.0')*ctx.fac(2*k-2*r)/ctx.fac(k-r))
                     xd[0,n,k] -= ((-1)**(k-r))*add
-        xd[0,n,-2]=0; xd[0,n,-1]=0; xd[0,n,3*n//2+1]=0
+        xd[0,n,-2]=0
+        xd[0,n,-1]=0
+        xd[0,n,3*n//2+1]=0
     for mu in range(-2,der+1):
         for n in range(-2,L):
             for k in range(-3,max(1,3*n//2+2)):
@@ -495,8 +497,14 @@ def Rzeta_simul(ctx, s, der=0):
     ctx.prec = ywpd[1]+10
     ypsigma = 1-(2*ysigma)
     yd = {}
-    yd[0,0,-2]=0; yd[0,0,-1]=0; yd[0,0,0]=1; yd[0,0,1]=0
-    yd[0,-1,-2]=0; yd[0,-1,-1]=0; yd[0,-1,0]=1; yd[0,-1,1]=0
+    yd[0,0,-2]=0
+    yd[0,0,-1]=0
+    yd[0,0,0]=1
+    yd[0,0,1]=0
+    yd[0,-1,-2]=0
+    yd[0,-1,-1]=0
+    yd[0,-1,0]=1
+    yd[0,-1,1]=0
     for n in range(1,L):
         ctx.prec = ywpd[n]+10
         for k in range(0,3*n//2+1):
@@ -512,7 +520,9 @@ def Rzeta_simul(ctx, s, der=0):
                 for r in range(0,k):
                     add=yd[0,n,r]*(ctx.mpf('1.0')*ctx.fac(2*k-2*r)/ctx.fac(k-r))
                     yd[0,n,k] -= ((-1)**(k-r))*add
-        yd[0,n,-2]=0; yd[0,n,-1]=0; yd[0,n,3*n//2+1]=0
+        yd[0,n,-2]=0
+        yd[0,n,-1]=0
+        yd[0,n,3*n//2+1]=0
 
     for mu in range(-2,der+1):
         for n in range(-2,L):
@@ -954,8 +964,14 @@ def Rzeta_set(ctx, s, derivatives=[0]):
     ctx.prec = wpd[1]+10
     psigma = 1-(2*sigma)
     d = {}
-    d[0,0,-2]=0; d[0,0,-1]=0; d[0,0,0]=1; d[0,0,1]=0
-    d[0,-1,-2]=0; d[0,-1,-1]=0; d[0,-1,0]=1; d[0,-1,1]=0
+    d[0,0,-2]=0
+    d[0,0,-1]=0
+    d[0,0,0]=1
+    d[0,0,1]=0
+    d[0,-1,-2]=0
+    d[0,-1,-1]=0
+    d[0,-1,0]=1
+    d[0,-1,1]=0
     for n in range(1,L):
         ctx.prec = wpd[n]+10
         for k in range(0,3*n//2+1):
@@ -971,7 +987,9 @@ def Rzeta_set(ctx, s, derivatives=[0]):
                 for r in range(0,k):
                     add = d[0,n,r]*(ctx.one*ctx.fac(2*k-2*r)/ctx.fac(k-r))
                     d[0,n,k] -= ((-1)**(k-r))*add
-        d[0,n,-2]=0; d[0,n,-1]=0; d[0,n,3*n//2+1]=0
+        d[0,n,-2]=0
+        d[0,n,-1]=0
+        d[0,n,3*n//2+1]=0
 
     for mu in range(-2,der+1):
         for n in range(-2,L):
@@ -1153,7 +1171,7 @@ def z_half(ctx,t,der=0):
     tt = t/(2*ctx.pi)
     wptheta = wpinitial +1 + ctx.mag(3*(tt**1.5)*ctx.ln(tt))
     wpz = wpinitial + 1 + ctx.mag(12*tt*ctx.ln(tt))
-    ctx.prec = wptheta * LOGGAMMA_BROKENNESS_FACTOR
+    ctx.prec = wptheta
     theta = ctx.siegeltheta(t)
     ctx.prec = wpz
     rz = Rzeta_set(ctx,s, range(der+1))
@@ -1211,7 +1229,7 @@ def zeta_half(ctx, s, k=0):
     wpbasic = max(wpbasic, wpbasic2)
     wptheta = max(4, 3+ctx.mag(2.7*M1*X)+wpinitial+1)
     wpR = 3+ctx.mag(1.1+2*X)+wpinitial+1
-    ctx.prec = wptheta * LOGGAMMA_BROKENNESS_FACTOR
+    ctx.prec = wptheta
     theta = ctx.siegeltheta(t-ctx.j*(sigma-ctx.mpf('0.5')))
     if k > 0: ps1 = (ctx._re(ctx.psi(0,s/2)))/2 - ctx.ln(ctx.pi)/2
     if k > 1: ps2 = -(ctx._im(ctx.psi(1,s/2)))/4
@@ -1278,7 +1296,7 @@ def zeta_offline(ctx, s, k=0):
     wpbasic = max(wpbasic, wpbasic2)
     wptheta = max(4, 3+ctx.mag(2.7*M2*X)+wpinitial+1)
     wpR = 3+ctx.mag(1.1+2*X)+wpinitial+1
-    ctx.prec = wptheta * LOGGAMMA_BROKENNESS_FACTOR
+    ctx.prec = wptheta
     theta = ctx.siegeltheta(t-ctx.j*(sigma-ctx.mpf('0.5')))
     s1 = s
     s2 = ctx.conj(1-s1)
@@ -1344,7 +1362,7 @@ def z_offline(ctx, w, k=0):
     wptheta = max(4,ctx.mag(2.04*aux2)+aux3)
     wpR = ctx.mag(4*aux1)+aux3
     # now the computations
-    ctx.prec = wptheta * LOGGAMMA_BROKENNESS_FACTOR
+    ctx.prec = wptheta
     theta = ctx.siegeltheta(w)
     ctx.prec = wpR
     xrz, yrz = Rzeta_simul(ctx,s,k)
@@ -1388,7 +1406,8 @@ def rs_zeta(ctx, s, derivative=0, **kwargs):
     if derivative > 4:
         raise NotImplementedError
     s = ctx.convert(s)
-    re = ctx._re(s); im = ctx._im(s)
+    re = ctx._re(s)
+    im = ctx._im(s)
     if im < 0:
         z = ctx.conj(ctx.rs_zeta(ctx.conj(s), derivative))
         return z
@@ -1401,7 +1420,8 @@ def rs_zeta(ctx, s, derivative=0, **kwargs):
 @defun
 def rs_z(ctx, w, derivative=0):
     w = ctx.convert(w)
-    re = ctx._re(w); im = ctx._im(w)
+    re = ctx._re(w)
+    im = ctx._im(w)
     if re < 0:
         return rs_z(ctx, -w, derivative)
     critical_line = (im == 0)
