@@ -230,6 +230,12 @@ def _gamma_complex(x):
 
 gamma = _mathfun_real(_gamma_real, _gamma_complex)
 
+def rgamma(x):
+    try:
+        return 1./gamma(x)
+    except ZeroDivisionError:
+        return x*0.0
+
 def factorial(x):
     return gamma(x+1.0)
 
@@ -267,7 +273,12 @@ def loggamma(x):
             re = z
             im = 0.0
         refloor = floor(re)
-        imsign = cmp(im, 0)
+        if im == 0.0:
+            imsign = 0
+        elif im < 0.0:
+            imsign = -1
+        else:
+            imsign = 1
         return (-pi*1j)*abs(refloor)*(1-abs(imsign)) + logpi - \
             log(sinpi(z-refloor)) - loggamma(z) + 1j*pi*refloor*imsign
     if x == 1.0 or x == 2.0:
@@ -279,13 +290,20 @@ def loggamma(x):
     s = 0.918938533204672742 + (x-0.5)*log(x) - x
     r = 1./x
     r2 = r*r
-    s += 0.083333333333333333333*r; r *= r2
-    s += -0.0027777777777777777778*r; r *= r2
-    s += 0.00079365079365079365079*r; r *= r2
-    s += -0.0005952380952380952381*r; r *= r2
-    s += 0.00084175084175084175084*r; r *= r2
-    s += -0.0019175269175269175269*r; r *= r2
-    s += 0.0064102564102564102564*r; r *= r2
+    s += 0.083333333333333333333*r
+    r *= r2
+    s += -0.0027777777777777777778*r
+    r *= r2
+    s += 0.00079365079365079365079*r
+    r *= r2
+    s += -0.0005952380952380952381*r
+    r *= r2
+    s += 0.00084175084175084175084*r
+    r *= r2
+    s += -0.0019175269175269175269*r
+    r *= r2
+    s += 0.0064102564102564102564*r
+    r *= r2
     s += -0.02955065359477124183*r
     return s + p
 
