@@ -1,3 +1,5 @@
+from ..libmp.backend import xrange
+
 # TODO: interpret list as vectors (for multiplication)
 
 rowsep = '\n'
@@ -400,41 +402,170 @@ class _matrix(object):
         s += self._toliststr(avoid_type=True) + ')'
         return s
 
-    def __getitem__(self, key):
-        if type(key) is int:
-            # only sufficent for vectors
-            if self.__rows == 1:
-                key = (0, key)
-            elif self.__cols == 1:
-                key = (key, 0)
-            else:
-                raise IndexError('insufficient indices for matrix')
+    def __get_element(self, key):
+        '''
+        Fast extraction of the i,j element from the matrix
+            This function is for private use only because is unsafe:
+                1. Does not check on the value of key it expects key to be a integer tuple (i,j)
+                2. Does not check bounds
+        '''
         if key in self.__data:
             return self.__data[key]
         else:
-            if key[0] >= self.__rows or key[1] >= self.__cols:
-                raise IndexError('matrix index out of range')
             return self.ctx.zero
 
-    def __setitem__(self, key, value):
-        if type(key) is int:
-            # only sufficent for vectors
-            if self.__rows == 1:
-                key = (0, key)
-            elif self.__cols == 1:
-                key = (key, 0)
-            else:
-                raise IndexError('insufficient indices for matrix')
-        if key[0] >= self.__rows or key[1] >= self.__cols:
-            raise IndexError('matrix index out of range')
-        value = self.ctx.convert(value)
+    def __set_element(self, key, value):
+        '''
+        Fast assignment of the i,j element in the matrix
+            This function is unsafe:
+                1. Does not check on the value of key it expects key to be a integer tuple (i,j)
+                2. Does not check bounds
+                3. Does not check the value type
+        '''
         if value: # only store non-zeros
             self.__data[key] = value
         elif key in self.__data:
             del self.__data[key]
-        # TODO: maybe do this better, if the performance impact is significant
+
+
+    def __getitem__(self, key):
+        '''
+            Getitem function for mp matrix class with slice index enabled
+            it allows the following assingments
+            scalar to a slice of the matrix
+         B = A[:,2:6]
+        '''
+        # Convert vector to matrix indexing
+        if isinstance(key, int) or isinstance(key,slice):
+            # only sufficent for vectors
+            if self.__rows == 1:
+                key = (0, key)
+            elif self.__cols == 1:
+                key = (key, 0)
+            else:
+                raise IndexError('insufficient indices for matrix')
+
+        if isinstance(key[0],slice) or isinstance(key[1],slice):
+
+            #Rows
+            if isinstance(key[0],slice):
+                #Check bounds
+                if (key[0].start is None or key[0].start >= 0) and \
+                    (key[0].stop is None or key[0].stop <= self.__rows+1):
+                    # Generate indices
+                    rows = xrange(*key[0].indices(self.__rows))
+                else:
+                    raise IndexError('Row index out of bounds')
+            else:
+                # Single row
+                rows = [key[0]]
+
+            # Columns
+            if isinstance(key[1],slice):
+                # Check bounds
+                if (key[1].start is None or key[1].start >= 0) and \
+                    (key[1].stop is None or key[1].stop <= self.__cols+1):
+                    # Generate indices
+                    columns = xrange(*key[1].indices(self.__cols))
+                else:
+                    raise IndexError('Column index out of bounds')
+
+            else:
+                # Single column
+                columns = [key[1]]
+
+            # Create matrix slice
+            m = self.ctx.matrix(len(rows),len(columns))
+
+            # Assign elements to the output matrix
+            for i,x in enumerate(rows):
+                for j,y in enumerate(columns):
+                    m.__set_element((i,j),self.__get_element((x,y)))
+
+            return m
+
+        else:
+            # single element extraction
+            if key[0] >= self.__rows or key[1] >= self.__cols:
+                raise IndexError('matrix index out of range')
+            if key in self.__data:
+                return self.__data[key]
+            else:
+                return self.ctx.zero
+
+    def __setitem__(self, key, value):
+        # setitem function for mp matrix class with slice index enabled
+        # it allows the following assingments
+        #  scalar to a slice of the matrix
+        # A[:,2:6] = 2.5
+        #  submatrix to matrix (the value matrix should be the same size as the slice size)
+        # A[3,:] = B   where A is n x m  and B is n x 1
+        # Convert vector to matrix indexing
+        if isinstance(key, int) or isinstance(key,slice):
+            # only sufficent for vectors
+            if self.__rows == 1:
+                key = (0, key)
+            elif self.__cols == 1:
+                key = (key, 0)
+            else:
+                raise IndexError('insufficient indices for matrix')
+        # Slice indexing
+        if isinstance(key[0],slice) or isinstance(key[1],slice):
+            # Rows
+            if isinstance(key[0],slice):
+                # Check bounds
+                if (key[0].start is None or key[0].start >= 0) and \
+                    (key[0].stop is None or key[0].stop <= self.__rows+1):
+                    # generate row indices
+                    rows = xrange(*key[0].indices(self.__rows))
+                else:
+                    raise IndexError('Row index out of bounds')
+            else:
+                # Single row
+                rows = [key[0]]
+            # Columns
+            if isinstance(key[1],slice):
+                # Check bounds
+                if (key[1].start is None or key[1].start >= 0) and \
+                    (key[1].stop is None or key[1].stop <= self.__cols+1):
+                    # Generate column indices
+                    columns = xrange(*key[1].indices(self.__cols))
+                else:
+                    raise IndexError('Column index out of bounds')
+            else:
+                # Single column
+                columns = [key[1]]
+            # Assign slice with a scalar
+            if isinstance(value,self.ctx.matrix):
+                # Assign elements to matrix if input and output dimensions match
+                if len(rows) == value.rows and len(columns) == value.cols:
+                    for i,x in enumerate(rows):
+                        for j,y in enumerate(columns):
+                            self.__set_element((x,y), value.__get_element((i,j)))
+                else:
+                    raise ValueError('Dimensions do not match')
+            else:
+                # Assign slice with scalars
+                value = self.ctx.convert(value)
+                for i in rows:
+                    for j in columns:
+                        self.__set_element((i,j), value)
+        else:
+            # Single element assingment
+            # Check bounds
+            if key[0] >= self.__rows or key[1] >= self.__cols:
+                raise IndexError('matrix index out of range')
+            # Convert and store value
+            value = self.ctx.convert(value)
+            if value: # only store non-zeros
+                self.__data[key] = value
+            elif key in self.__data:
+                del self.__data[key]
+
         if self._LU:
             self._LU = None
+        return
+
 
     def __iter__(self):
         for i in xrange(self.__rows):
@@ -551,7 +682,7 @@ class _matrix(object):
         return self.__rows
 
     def __setrows(self, value):
-        for key in self.__data.copy().iterkeys():
+        for key in self.__data.copy():
             if key[0] >= value:
                 del self.__data[key]
         self.__rows = value
@@ -562,7 +693,7 @@ class _matrix(object):
         return self.__cols
 
     def __setcols(self, value):
-        for key in self.__data.copy().iterkeys():
+        for key in self.__data.copy():
             if key[1] >= value:
                 del self.__data[key]
         self.__cols = value
@@ -620,7 +751,7 @@ class MatrixMethods(object):
         """
         Create square diagonal matrix using given list.
 
-        Example:
+        Examples:
         >>> from mpmath import diag, mp
         >>> mp.pretty = False
         >>> diag([1, 2, 3])
@@ -639,7 +770,7 @@ class MatrixMethods(object):
         Create matrix m x n filled with zeros.
         One given dimension will create square matrix n x n.
 
-        Example:
+        Examples:
         >>> from mpmath import zeros, mp
         >>> mp.pretty = False
         >>> zeros(2)
@@ -665,7 +796,7 @@ class MatrixMethods(object):
         Create matrix m x n filled with ones.
         One given dimension will create square matrix n x n.
 
-        Example:
+        Examples:
         >>> from mpmath import ones, mp
         >>> mp.pretty = False
         >>> ones(2)
@@ -709,7 +840,7 @@ class MatrixMethods(object):
         All values are >= min and <max.
         n defaults to m.
 
-        Example:
+        Examples:
         >>> from mpmath import randmatrix
         >>> randmatrix(2) # doctest:+SKIP
         matrix(
@@ -766,7 +897,7 @@ class MatrixMethods(object):
         ``p=inf`` gives the magnitude of the largest element.
 
         For *x* a matrix, ``p=2`` is the Frobenius norm.
-        For operator matrix norms, use :func:`mnorm` instead.
+        For operator matrix norms, use :func:`~mpmath.mnorm` instead.
 
         You can use the string 'inf' as well as float('inf') or mpf('inf')
         to specify the infinity norm.
@@ -825,7 +956,7 @@ class MatrixMethods(object):
         The Frobenius norm lacks some mathematical properties that might
         be expected of a norm.
 
-        For general elementwise `p`-norms, use :func:`norm` instead.
+        For general elementwise `p`-norms, use :func:`~mpmath.norm` instead.
 
         **Examples**
 
