@@ -35,16 +35,22 @@ def test_pretty():
     mp.pretty = True
     assert repr(mpf(2.5)) == '2.5'
     assert repr(mpc(2.5,3.5)) == '(2.5 + 3.5j)'
-    assert repr(mpi(2.5,3.5)) == '[2.5, 3.5]'
     mp.pretty = False
+    iv.pretty = True
+    assert repr(mpi(2.5,3.5)) == '[2.5, 3.5]'
+    iv.pretty = False
 
 def test_str_whitespace():
     assert mpf('1.26 ') == 1.26
 
 def test_unicode():
     mp.dps = 15
-    assert mpf(u'2.76') == 2.76
-    assert mpf(u'inf') == inf
+    try:
+        unicode = unicode
+    except NameError:
+        unicode = str
+    assert mpf(unicode('2.76')) == 2.76
+    assert mpf(unicode('inf')) == inf
 
 def test_str_format():
     assert to_str(from_float(0.1),15,strip_zeros=False) == '0.100000000000000'
@@ -73,7 +79,7 @@ def test_eval_repr_invariant():
     random.seed(123)
     for dps in [10, 15, 20, 50, 100]:
         mp.dps = dps
-        for i in xrange(1000):
+        for i in range(1000):
             a = mpf(random.random())**0.5 * 10**random.randint(-100, 100)
             assert eval(repr(a)) == a
     mp.dps = 15
@@ -138,11 +144,46 @@ def test_conversion_methods():
         assert (y+z).ae(mpc('4.3'))
         assert (z+w).ae(mpc('4.3', '1.7'))
         assert (w+z).ae(mpc('4.3', '1.7'))
-        x-y; y-x; x-w; w-x; z-y; y-z; z-w; w-z
-        x*y; y*x; x*w; w*x; z*y; y*z; z*w; w*z
-        x/y; y/x; x/w; w/x; z/y; y/z; z/w; w/z
-        x**y; y**x; x**w; w**x; z**y; y**z; z**w; w**z
-        x==y; y==x; x==w; w==x; z==y; y==z; z==w; w==z
+        x-y
+        y-x
+        x-w
+        w-x
+        z-y
+        y-z
+        z-w
+        w-z
+        x*y
+        y*x
+        x*w
+        w*x
+        z*y
+        y*z
+        z*w
+        w*z
+        x/y
+        y/x
+        x/w
+        w/x
+        z/y
+        y/z
+        z/w
+        w/z
+        x**y
+        y**x
+        x**w
+        w**x
+        z**y
+        y**z
+        z**w
+        w**z
+        x==y
+        y==x
+        x==w
+        w==x
+        z==y
+        y==z
+        z==w
+        w==z
     mp.dps = 15
     assert x.__add__(a) is NotImplemented
     assert x.__radd__(a) is NotImplemented
