@@ -1,6 +1,8 @@
 from sympy.mpmath import *
 from sympy.mpmath.libmp import round_up, from_float, mpf_zeta_int
 
+from sympy.utilities.pytest import XFAIL
+
 def test_zeta_int_bug():
     assert mpf_zeta_int(0, 10) == from_float(-0.5)
 
@@ -159,8 +161,6 @@ def test_gamma():
     assert gamma(300).ae('1.0201917073881354535e612')
     assert gamma(-0.5).ae(-3.5449077018110320546)
     assert gamma(-7.43).ae(0.00026524416464197007186)
-    #assert gamma(Rational(1,2)) == gamma(0.5)
-    #assert gamma(Rational(-7,3)).ae(gamma(mpf(-7)/3))
     assert gamma(1+1j).ae(0.49801566811835604271 - 0.15494982830181068512j)
     assert gamma(-1+0.01j).ae(-0.422733904013474115 + 99.985883082635367436j)
     assert gamma(20+30j).ae(-1453876687.5534810 + 1163777777.8031573j)
@@ -183,6 +183,18 @@ def test_gamma():
     assert factorial(3) == 6
     assert isnan(gamma(nan))
     assert gamma(1100).ae('4.8579168073569433667e2866')
+    assert rgamma(0) == 0
+    assert rgamma(-1) == 0
+    assert rgamma(2) == 1.0
+    assert rgamma(3) == 0.5
+    assert loggamma(2+8j).ae(-8.5205176753667636926 + 10.8569497125597429366j)
+    assert loggamma('1e10000').ae('2.302485092994045684017991e10004')
+    assert loggamma('1e10000j').ae(mpc('-1.570796326794896619231322e10000','2.302485092994045684017991e10004'))
+
+@XFAIL
+def test_gamma_failing():
+    assert gamma(Rational(1,2)) == gamma(0.5)
+    assert gamma(Rational(-7,3)).ae(gamma(mpf(-7)/3))
 
 def test_fac2():
     mp.dps = 15
@@ -321,8 +333,9 @@ def test_polygamma():
     psi0 = lambda z: psi(0,z)
     psi1 = lambda z: psi(1,z)
     assert psi0(3) == psi(0,3) == digamma(3)
-    #assert psi2(3) == psi(2,3) == tetragamma(3)
-    #assert psi3(3) == psi(3,3) == pentagamma(3)
+    # Not sure what this guy was doing, only place psi2, psi3, tetragamma used
+    # assert psi2(3) == psi(2,3) == tetragamma(3)
+    # assert psi3(3) == psi(3,3) == pentagamma(3)
     assert psi0(pi).ae(0.97721330794200673)
     assert psi0(-pi).ae(7.8859523853854902)
     assert psi0(-pi+1).ae(7.5676424992016996)
@@ -343,6 +356,15 @@ def test_polygamma():
     assert psi0(-100.5).ae(4.615124601338064)
     assert psi0(3+0j).ae(psi0(3))
     assert psi0(-100+3j).ae(4.6106071768714086321+3.1117510556817394626j)
+    assert isnan(psi(2,mpc(0,inf)))
+    assert isnan(psi(2,mpc(0,nan)))
+    assert isnan(psi(2,mpc(0,-inf)))
+    assert isnan(psi(2,mpc(1,inf)))
+    assert isnan(psi(2,mpc(1,nan)))
+    assert isnan(psi(2,mpc(1,-inf)))
+    assert isnan(psi(2,mpc(inf,inf)))
+    assert isnan(psi(2,mpc(nan,nan)))
+    assert isnan(psi(2,mpc(-inf,-inf)))
 
 def test_polygamma_high_prec():
     mp.dps = 100
@@ -474,8 +496,6 @@ def test_gamma_huge_5():
         "7571892628368354580620654233316841")
     mp.dps = 15
 
-"""
-XXX: fails
 def test_gamma_huge_6():
     return
     mp.dps = 500
@@ -489,7 +509,6 @@ def test_gamma_huge_6():
         "8 - 4.2996028528243336966001185406200082244961757496106e-9565705"
         "5002j)")
     mp.dps = 15
-"""
 
 def test_gamma_huge_7():
     mp.dps = 100
@@ -497,10 +516,15 @@ def test_gamma_huge_7():
     mp.dps = 15
     y = gamma(a)
     assert str(y.real) == "2.0"
-    assert str(y.imag) == "2.16735365342606e-1000"
+    assert str(y.imag) == "1.84556867019693e-1000"
     mp.dps = 50
     y = gamma(a)
     assert str(y.real) == "2.0"
+    assert str(y.imag) ==  "1.8455686701969342787869758198351951379156813281202e-1000"
+
+@XFAIL
+def test_gamma_huge_7_failing():
+    assert str(y.imag) == "2.16735365342606e-1000"
     assert str(y.imag) == "2.1673536534260596065418805612488708028522563689298e-1000"
 
 def test_stieltjes():
@@ -636,6 +660,15 @@ def test_rs_zeta():
     assert ae(fp.zeta(0.75+100000j, derivative=3), 110.61331857820103469 - 236.87847130518129926j)
     assert ae(fp.zeta(0.75+100000j, derivative=4), -1054.334275898559401 + 1769.9177890161596383j)
 
+def test_siegelz():
+    mp.dps = 15
+    assert siegelz(100000).ae(5.87959246868176504171)
+    assert siegelz(100000, derivative=2).ae(-54.1172711010126452832)
+    assert siegelz(100000, derivative=3).ae(-278.930831343966552538)
+    assert siegelz(100000+j,derivative=1).ae(678.214511857070283307-379.742160779916375413j)
+
+
+
 def test_zeta_near_1():
     # Test for a former bug in mpf_zeta and mpc_zeta
     mp.dps = 15
