@@ -30,10 +30,12 @@ def monitor(f, input='print', output='print'):
         mpf('3.1415926535897932')
         >>> len(input)  # Count number of evaluations
         9
-        >>> print input[3], output[3]
-        ((mpf('3.1415076583334066'),), {}) 8.49952562843408e-5
-        >>> print input[4], output[4]
-        ((mpf('3.1415928201669122'),), {}) -1.66577118985331e-7
+        >>> print(input[3]); print(output[3])
+        ((mpf('3.1415076583334066'),), {})
+        8.49952562843408e-5
+        >>> print(input[4]); print(output[4])
+        ((mpf('3.1415928201669122'),), {})
+        -1.66577118985331e-7
 
     """
     if not input:
@@ -42,14 +44,14 @@ def monitor(f, input='print', output='print'):
         incount = [0]
         def input(value):
             args, kwargs = value
-            print "in  %s %r %r" % (incount[0], args, kwargs)
+            print("in  %s %r %r" % (incount[0], args, kwargs))
             incount[0] += 1
     if not output:
         output = lambda v: None
     elif output == 'print':
         outcount = [0]
         def output(value):
-            print "out %s %r" % (outcount[0], value)
+            print("out %s %r" % (outcount[0], value))
             outcount[0] += 1
     def f_monitored(*args, **kwargs):
         input((args, kwargs))
@@ -78,14 +80,26 @@ def timing(f, *args, **kwargs):
     else:
         g = f
     from timeit import default_timer as clock
-    t1=clock(); v=g(); t2=clock(); t=t2-t1
+    t1=clock()
+    v=g()
+    t2=clock()
+    t=t2-t1
     if t > 0.05 or once:
         return t
     for i in range(3):
-        t1=clock();
+        t1=clock()
         # Evaluate multiple times because the timer function
         # has a significant overhead
-        g();g();g();g();g();g();g();g();g();g()
+        g()
+        g()
+        g()
+        g()
+        g()
+        g()
+        g()
+        g()
+        g()
+        g()
         t2=clock()
         t=min(t,(t2-t1)/10)
     return t
