@@ -1,22 +1,26 @@
 """
 Integer factorization
 """
+from __future__ import print_function, division
+
 import random
 import math
 
-from sympy.core import Mul
+from .primetest import isprime
+from .generate import sieve, primerange, nextprime
+from sympy.core import sympify
 from sympy.core.evalf import bitcount
-from sympy.core.numbers import igcd
+from sympy.core.numbers import igcd, oo, Rational
 from sympy.core.power import integer_nthroot, Pow
 from sympy.core.mul import Mul
-from sympy.core.compatibility import as_int
-from primetest import isprime
-from generate import sieve, primerange, nextprime
+from sympy.core.compatibility import as_int, SYMPY_INTS, xrange
 from sympy.core.singleton import S
+from sympy.core.function import Function
 
-small_trailing = [i and max(int(not i % 2**j) and j for j in range(1,8)) \
+small_trailing = [i and max(int(not i % 2**j) and j for j in range(1, 8))
     for i in range(256)]
 
+
 def smoothness(n):
     """
     Return the B-smooth and B-power smooth values of n.
@@ -39,9 +43,10 @@ def smoothness(n):
     """
 
     if n == 1:
-        return (1, 1) # not prime, but otherwise this causes headaches
+        return (1, 1)  # not prime, but otherwise this causes headaches
     facs = factorint(n)
-    return max(facs), max([m**facs[m] for m in facs])
+    return max(facs), max(m**facs[m] for m in facs)
+
 
 def smoothness_p(n, m=-1, power=0, visual=None):
     """
@@ -69,7 +74,7 @@ def smoothness_p(n, m=-1, power=0, visual=None):
 
     If visual=True then an annotated string will be returned:
 
-        >>> print smoothness_p(21477639576571, visual=1)
+        >>> print(smoothness_p(21477639576571, visual=1))
         p**i=4410317**1 has p-1 B=1787, B-pow=1787
         p**i=4869863**1 has p-1 B=2434931, B-pow=2434931
 
@@ -86,7 +91,7 @@ def smoothness_p(n, m=-1, power=0, visual=None):
     The table of the output logic is:
 
         ====== ====== ======= =======
-                       Visual
+        |              Visual
         ------ ----------------------
         Input  True   False   other
         ====== ====== ======= =======
@@ -104,14 +109,19 @@ def smoothness_p(n, m=-1, power=0, visual=None):
     """
     from sympy.utilities import flatten
 
+    # visual must be True, False or other (stored as None)
+    if visual in (1, 0):
+        visual = bool(visual)
+    elif visual not in (True, False):
+        visual = None
+
     if type(n) is str:
         if visual:
             return n
         d = {}
         for li in n.splitlines():
-            k, v = [int(i) for i in li.split('has')[0]
-                                      .split('=')[1]
-                                      .split('**')]
+            k, v = [int(i) for i in
+                    li.split('has')[0].split('=')[1].split('**')]
             d[k] = v
         if visual is not True and visual is not False:
             return d
@@ -125,13 +135,13 @@ def smoothness_p(n, m=-1, power=0, visual=None):
         k = 1
     if type(n) is not tuple:
         rv = (m, sorted([(f,
-                            tuple([M] + list(smoothness(f + m))))
-                            for f, M in [i for i in facs.items()]],
-                            key=lambda x: (x[1][k], x[0])))
+                          tuple([M] + list(smoothness(f + m))))
+                         for f, M in [i for i in facs.items()]],
+                        key=lambda x: (x[1][k], x[0])))
     else:
         rv = n
 
-    if visual is False or (visual != True) and (type(n) in [int, Mul]):
+    if visual is False or (visual is not True) and (type(n) in [int, Mul]):
         return rv
     lines = []
     for dat in rv[1]:
@@ -140,6 +150,7 @@ def smoothness_p(n, m=-1, power=0, visual=None):
         lines.append('p**i=%i**%i has p%+i B=%i, B-pow=%i' % tuple(dat))
     return '\n'.join(lines)
 
+
 def trailing(n):
     """Count the number of trailing zero digits in the binary
     representation of n, i.e. determine the largest power of 2
@@ -163,20 +174,21 @@ def trailing(n):
 
     # 2**m is quick for z up through 2**30
     z = bitcount(n) - 1
-    if type(z) is int:
+    if isinstance(z, SYMPY_INTS):
         if n == 1 << z:
             return z
 
     t = 0
     p = 8
     while not n & 1:
-        while not n & ((1<<p)-1):
+        while not n & ((1 << p) - 1):
             n >>= p
             t += p
             p *= 2
         p //= 2
     return t
 
+
 def multiplicity(p, n):
     """
     Find the greatest integer m such that p**m divides n.
@@ -185,11 +197,38 @@ def multiplicity(p, n):
     ========
 
     >>> from sympy.ntheory import multiplicity
+    >>> from sympy.core.numbers import Rational as R
     >>> [multiplicity(5, n) for n in [8, 5, 25, 125, 250]]
     [0, 1, 2, 3, 3]
+    >>> multiplicity(3, R(1, 9))
+    -2
 
     """
-    p, n = as_int(p), as_int(n)
+    try:
+        p, n = as_int(p), as_int(n)
+    except ValueError:
+        if all(isinstance(i, (SYMPY_INTS, Rational)) for i in (p, n)):
+            try:
+                p = Rational(p)
+                n = Rational(n)
+                if p.q == 1:
+                    if n.p == 1:
+                        return -multiplicity(p.p, n.q)
+                    return S.Zero
+                elif p.p == 1:
+                    return multiplicity(p.q, n.q)
+                else:
+                    like = min(
+                        multiplicity(p.p, n.p),
+                        multiplicity(p.q, n.q))
+                    cross = min(
+                        multiplicity(p.q, n.p),
+                        multiplicity(p.p, n.q))
+                    return like - cross
+            except AttributeError:
+                pass
+        raise ValueError('expecting ints or fractions, got %s and %s' % (p, n))
+
     if p == 2:
         return trailing(n)
     if p < 2:
@@ -218,6 +257,7 @@ def multiplicity(p, n):
         n, rem = divmod(n, p)
     return m
 
+
 def perfect_power(n, candidates=None, big=True, factor=True):
     """
     Return ``(b, e)`` such that ``n`` == ``b**e`` if ``n`` is a
@@ -249,8 +289,8 @@ def perfect_power(n, candidates=None, big=True, factor=True):
     if n < 3:
         return False
     logn = math.log(n, 2)
-    max_possible = int(logn) + 2 # only check values less than this
-    not_square = n % 10 in [2, 3, 7, 8] # squares cannot end in 2, 3, 7, 8
+    max_possible = int(logn) + 2  # only check values less than this
+    not_square = n % 10 in [2, 3, 7, 8]  # squares cannot end in 2, 3, 7, 8
     if not candidates:
         candidates = primerange(2 + not_square, max_possible)
 
@@ -321,6 +361,7 @@ def perfect_power(n, candidates=None, big=True, factor=True):
     else:
         return False
 
+
 def pollard_rho(n, s=2, a=1, retries=5, seed=1234, max_steps=None, F=None):
     r"""
     Use Pollard's rho method to try to extract a nontrivial factor
@@ -345,7 +386,7 @@ def pollard_rho(n, s=2, a=1, retries=5, seed=1234, max_steps=None, F=None):
     >>> n = 16843009
     >>> F = lambda x:(2048*pow(x, 2, n) + 32767) % n
     >>> for s in range(5):
-    ...     print 'loop length = %4i; leader length = %3i' % cycle_length(F, s).next()
+    ...     print('loop length = %4i; leader length = %3i' % next(cycle_length(F, s)))
     ...
     loop length = 2489; leader length =  42
     loop length =   78; leader length = 120
@@ -359,10 +400,18 @@ def pollard_rho(n, s=2, a=1, retries=5, seed=1234, max_steps=None, F=None):
     >>> x=2
     >>> for i in range(9):
     ...     x=(x**2+12)%17
-    ...     print x,
+    ...     print(x)
     ...
-    16 13 11 14 4 11 14 4 11
-    >>> cycle_length(lambda x: (x**2+12)%17, 2).next()
+    16
+    13
+    11
+    14
+    4
+    11
+    14
+    4
+    11
+    >>> next(cycle_length(lambda x: (x**2+12)%17, 2))
     (3, 2)
     >>> list(cycle_length(lambda x: (x**2+12)%17, 2, values=True))
     [16, 13, 11, 14, 4]
@@ -398,7 +447,6 @@ def pollard_rho(n, s=2, a=1, retries=5, seed=1234, max_steps=None, F=None):
 
     - Richard Crandall & Carl Pomerance (2005), "Prime Numbers:
       A Computational Perspective", Springer, 2nd edition, 229-231
-    - http://www.csh.rit.edu/~pat/math/quickies/rho/
 
     """
     n = int(n)
@@ -416,7 +464,7 @@ def pollard_rho(n, s=2, a=1, retries=5, seed=1234, max_steps=None, F=None):
                 break
             j += 1
             U = F(U)
-            V = F(F(V)) # V is 2x further along than U
+            V = F(F(V))  # V is 2x further along than U
             g = igcd(U - V, n)
             if g == 1:
                 continue
@@ -424,10 +472,11 @@ def pollard_rho(n, s=2, a=1, retries=5, seed=1234, max_steps=None, F=None):
                 break
             return int(g)
         V = prng.randint(0, n - 1)
-        a = prng.randint(1, n - 3) # for x**2 + a, a%n should not be 0 or -2
+        a = prng.randint(1, n - 3)  # for x**2 + a, a%n should not be 0 or -2
         F = None
     return None
 
+
 def pollard_pm1(n, B=10, a=2, retries=0, seed=1234):
     """
     Use Pollard's p-1 method to try to extract a nontrivial factor
@@ -519,7 +568,7 @@ def pollard_pm1(n, B=10, a=2, retries=0, seed=1234):
 
         >>> from sympy.utilities import flatten
         >>> from sympy.ntheory.factor_ import smoothness_p, factorint
-        >>> print smoothness_p(21477639576571, visual=1)
+        >>> print(smoothness_p(21477639576571, visual=1))
         p**i=4410317**1 has p-1 B=1787, B-pow=1787
         p**i=4869863**1 has p-1 B=2434931, B-pow=2434931
 
@@ -555,9 +604,7 @@ def pollard_pm1(n, B=10, a=2, retries=0, seed=1234):
 
     - Richard Crandall & Carl Pomerance (2005), "Prime Numbers:
       A Computational Perspective", Springer, 2nd edition, 236-238
-    - http://modular.math.washington.edu/edu/2007/spring/ent/ent-html/
-            node81.html
-    - http://www.math.mcgill.ca/darmon/courses/05-06/usra/charest.pdf
+    - http://modular.math.washington.edu/edu/2007/spring/ent/ent-html/node81.html
     - http://www.cs.toronto.edu/~yuvalf/Factorization.pdf
     """
 
@@ -585,6 +632,7 @@ def pollard_pm1(n, B=10, a=2, retries=0, seed=1234):
         # say 'a' should be coprime to n, but either will detect factors.
         a = prng.randint(2, n - 2)
 
+
 def _trial(factors, n, candidates, verbose=False):
     """
     Helper function for integer factorization. Trial factors ``n`
@@ -593,7 +641,7 @@ def _trial(factors, n, candidates, verbose=False):
     value of ``n`` and a flag indicating whether any factors were found.
     """
     if verbose:
-        factors0 = factors.keys()
+        factors0 = list(factors.keys())
     nfactors = len(factors)
     for d in candidates:
         if n % d == 0:
@@ -602,15 +650,12 @@ def _trial(factors, n, candidates, verbose=False):
             factors[d] = m
     if verbose:
         for k in sorted(set(factors).difference(set(factors0))):
-            print factor_msg % (k, factors[k])
+            print(factor_msg % (k, factors[k]))
     return int(n), len(factors) != nfactors
 
-def _check_termination(factors, n,
-                                   limitp1,
-                                   use_trial,
-                                   use_rho,
-                                   use_pm1,
-                                   verbose):
+
+def _check_termination(factors, n, limitp1, use_trial, use_rho, use_pm1,
+                       verbose):
     """
     Helper function for integer factorization. Checks if ``n``
     is a prime or a perfect power, and in those cases updates
@@ -618,7 +663,7 @@ def _check_termination(factors, n,
     """
 
     if verbose:
-        print 'Check for termination'
+        print('Check for termination')
 
     # since we've already been factoring there is no need to do
     # simultaneous factoring with the power check
@@ -629,15 +674,11 @@ def _check_termination(factors, n,
             limit = limitp1 - 1
         else:
             limit = limitp1
-        facs = factorint(base,
-                               limit,
-                               use_trial,
-                               use_rho,
-                               use_pm1,
-                               verbose=False)
+        facs = factorint(base, limit, use_trial, use_rho, use_pm1,
+                         verbose=False)
         for b, e in facs.items():
             if verbose:
-                print factor_msg % (b, e)
+                print(factor_msg % (b, e))
             factors[b] = exp*e
         raise StopIteration
 
@@ -656,6 +697,7 @@ factor_msg = '\t%i ** %i'
 fermat_msg = 'Close factors satisying Fermat condition found.'
 complete_msg = 'Factorization is complete.'
 
+
 def _factorint_small(factors, n, limit, fail_max):
     """
     Return the value of n and either a 0 (indicating that factorization up
@@ -756,8 +798,9 @@ def _factorint_small(factors, n, limit, fail_max):
 
     return done(n, d)
 
+
 def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
-    verbose=False, visual=None):
+              verbose=False, visual=None):
     r"""
     Given a positive integer ``n``, ``factorint(n)`` returns a dict containing
     the prime factors of ``n`` as keys and their respective multiplicities
@@ -790,9 +833,10 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
     semi-prime factor that cannot be reduced easily:
 
     >>> from sympy.ntheory import isprime
+    >>> from sympy.core.compatibility import long
     >>> a = 1407633717262338957430697921446883
     >>> f = factorint(a, limit=10000)
-    >>> f == {991: 1, 202916782076162456022877024859L: 1, 7: 1}
+    >>> f == {991: 1, long(202916782076162456022877024859): 1, 7: 1}
     True
     >>> isprime(max(f))
     False
@@ -833,7 +877,7 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
     >>> visual = factorint(1764, visual=True); pprint(visual)
      2  2  2
     2 *3 *7
-    >>> print factorint(visual)
+    >>> print(factorint(visual))
     {2: 2, 3: 2, 7: 2}
 
     If you want to send a number to be factored in a partially factored form
@@ -841,7 +885,7 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
 
     >>> factorint(factorint({4: 2, 12: 3})) # twice to toggle to dict form
     {2: 10, 3: 3}
-    >>> factorint(Mul(4, 12, **dict(evaluate=False)))
+    >>> factorint(Mul(4, 12, evaluate=False))
     {2: 4, 3: 1}
 
     The table of the output logic is:
@@ -870,7 +914,7 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
 
     >>> factors = factorint(12345678910111213141516)
     >>> for base, exp in sorted(factors.items()):
-    ...     print base, exp
+    ...     print('%s %s' % (base, exp))
     ...
     2 2
     2507191691 1
@@ -897,31 +941,22 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
     """
     factordict = {}
     if visual and not isinstance(n, Mul) and not isinstance(n, dict):
-        factordict = factorint(n,
-                     limit=limit,
-                     use_trial=use_trial,
-                     use_rho=use_rho,
-                     use_pm1=use_pm1,
-                     verbose=verbose,
-                     visual=False)
+        factordict = factorint(n, limit=limit, use_trial=use_trial,
+                               use_rho=use_rho, use_pm1=use_pm1,
+                               verbose=verbose, visual=False)
     elif isinstance(n, Mul):
         factordict = dict([(int(k), int(v)) for k, v in
-                           n.as_powers_dict().items()])
+                           list(n.as_powers_dict().items())])
     elif isinstance(n, dict):
         factordict = n
     if factordict and (isinstance(n, Mul) or isinstance(n, dict)):
         # check it
-        for k in factordict.keys():
+        for k in list(factordict.keys()):
             if isprime(k):
                 continue
             e = factordict.pop(k)
-            d = factorint(k,
-                           limit=limit,
-                           use_trial=use_trial,
-                           use_rho=use_rho,
-                           use_pm1=use_pm1,
-                           verbose=verbose,
-                           visual=False)
+            d = factorint(k, limit=limit, use_trial=use_trial, use_rho=use_rho,
+                          use_pm1=use_pm1, verbose=verbose, visual=False)
             for k, v in d.items():
                 if k in factordict:
                     factordict[k] += v*e
@@ -937,22 +972,23 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
             args = [S.NegativeOne]
         else:
             args = []
-        args.extend([Pow(*i, **{'evaluate':False})
+        args.extend([Pow(*i, evaluate=False)
                      for i in sorted(factordict.items())])
-        return Mul(*args, **{'evaluate':False})
+        return Mul(*args, evaluate=False)
     elif isinstance(n, dict) or isinstance(n, Mul):
         return factordict
 
     assert use_trial or use_rho or use_pm1
 
-    n = int(n)
+    n = as_int(n)
     if limit:
         limit = int(limit)
 
     # special cases
     if n < 0:
-        factors = factorint(-n, limit=limit, use_trial=use_trial, use_rho=use_rho,
-        use_pm1=use_pm1, verbose=verbose, visual=False)
+        factors = factorint(
+            -n, limit=limit, use_trial=use_trial, use_rho=use_rho,
+            use_pm1=use_pm1, verbose=verbose, visual=False)
         factors[-1] = 1
         return factors
 
@@ -973,10 +1009,10 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
     if verbose:
         sn = str(n)
         if len(sn) > 50:
-            print 'Factoring %s' % sn[:5] + \
-                  '..(%i other digits)..' % (len(sn) - 10) + sn[-5:]
+            print('Factoring %s' % sn[:5] + \
+                  '..(%i other digits)..' % (len(sn) - 10) + sn[-5:])
         else:
-            print 'Factoring', n
+            print('Factoring', n)
 
     if use_trial:
         # this is the preliminary factorization for small factors
@@ -984,18 +1020,18 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
         fail_max = 600
         small = min(small, limit or small)
         if verbose:
-            print trial_int_msg % (2, small, fail_max)
+            print(trial_int_msg % (2, small, fail_max))
         n, next_p = _factorint_small(factors, n, small, fail_max)
     else:
         next_p = 2
     if factors and verbose:
         for k in sorted(factors):
-            print factor_msg % (k, factors[k])
+            print(factor_msg % (k, factors[k]))
     if next_p == 0:
         if n > 1:
             factors[int(n)] = 1
         if verbose:
-            print complete_msg
+            print(complete_msg)
         return factors
 
     # continue with more advanced factorization methods
@@ -1006,14 +1042,10 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
     try:
         if limit and next_p > limit:
             if verbose:
-                print 'Exceeded limit:', limit
+                print('Exceeded limit:', limit)
 
-            _check_termination(factors, n,
-                                           limit,
-                                           use_trial,
-                                           use_rho,
-                                           use_pm1,
-                                           verbose)
+            _check_termination(factors, n, limit, use_trial, use_rho, use_pm1,
+                               verbose)
 
             if n > 1:
                 factors[int(n)] = 1
@@ -1033,34 +1065,27 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
                 b, fermat = integer_nthroot(b2, 2)
                 if fermat:
                     break
-                b2 += 2*a + 1 # equiv to (a+1)**2 - n
-                a  += 1
+                b2 += 2*a + 1  # equiv to (a+1)**2 - n
+                a += 1
             if fermat:
                 if verbose:
-                    print fermat_msg
+                    print(fermat_msg)
                 if limit:
                     limit -= 1
                 for r in [a - b, a + b]:
-                    facs = factorint(r,
-                                limit=limit,
-                                use_trial=use_trial,
-                                use_rho=use_rho,
-                                use_pm1=use_pm1,
-                                verbose=verbose)
+                    facs = factorint(r, limit=limit, use_trial=use_trial,
+                                     use_rho=use_rho, use_pm1=use_pm1,
+                                     verbose=verbose)
                     factors.update(facs)
                 raise StopIteration
 
             # ...see if factorization can be terminated
-            _check_termination(factors, n,
-                                           limit,
-                                           use_trial,
-                                           use_rho,
-                                           use_pm1,
-                                           verbose)
+            _check_termination(factors, n, limit, use_trial, use_rho, use_pm1,
+                               verbose)
 
     except StopIteration:
         if verbose:
-            print complete_msg
+            print(complete_msg)
         return factors
 
     # these are the limits for trial division which will
@@ -1081,22 +1106,18 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
             # Trial division
             if use_trial:
                 if verbose:
-                    print trial_msg % (low, high_)
+                    print(trial_msg % (low, high_))
                 ps = sieve.primerange(low, high_)
                 n, found_trial = _trial(factors, n, ps, verbose)
                 if found_trial:
-                    _check_termination(factors, n,
-                                                   limit,
-                                                   use_trial,
-                                                   use_rho,
-                                                   use_pm1,
-                                                   verbose)
+                    _check_termination(factors, n, limit, use_trial, use_rho,
+                                       use_pm1, verbose)
             else:
                 found_trial = False
 
             if high > limit:
                 if verbose:
-                    print 'Exceeded limit:', limit
+                    print('Exceeded limit:', limit)
                 if n > 1:
                     factors[int(n)] = 1
                 raise StopIteration
@@ -1109,50 +1130,40 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
                     # Pollard p-1
                     if use_pm1:
                         if verbose:
-                            print (pm1_msg % (high_root, high_))
+                            print(pm1_msg % (high_root, high_))
                         c = pollard_pm1(n, B=high_root, seed=high_)
                         if c:
                             # factor it and let _trial do the update
-                            ps = factorint(c,
-                                            limit=limit-1,
-                                            use_trial=use_trial,
-                                            use_rho=use_rho,
-                                            use_pm1=use_pm1,
-                                            verbose=verbose)
+                            ps = factorint(c, limit=limit - 1,
+                                           use_trial=use_trial,
+                                           use_rho=use_rho,
+                                           use_pm1=use_pm1,
+                                           verbose=verbose)
                             n, _ = _trial(factors, n, ps, verbose=False)
-                            _check_termination(factors, n,
-                                                           limit,
-                                                           use_trial,
-                                                           use_rho,
-                                                           use_pm1,
-                                                           verbose)
+                            _check_termination(factors, n, limit, use_trial,
+                                               use_rho, use_pm1, verbose)
 
                     # Pollard rho
                     if use_rho:
                         max_steps = high_root
                         if verbose:
-                            print (rho_msg % (1, max_steps, high_))
-                        c = pollard_rho(n, retries=1, max_steps=max_steps, \
-                                           seed=high_)
+                            print(rho_msg % (1, max_steps, high_))
+                        c = pollard_rho(n, retries=1, max_steps=max_steps,
+                                        seed=high_)
                         if c:
                             # factor it and let _trial do the update
-                            ps = factorint(c,
-                                            limit=limit-1,
-                                            use_trial=use_trial,
-                                            use_rho=use_rho,
-                                            use_pm1=use_pm1,
-                                            verbose=verbose)
+                            ps = factorint(c, limit=limit - 1,
+                                           use_trial=use_trial,
+                                           use_rho=use_rho,
+                                           use_pm1=use_pm1,
+                                           verbose=verbose)
                             n, _ = _trial(factors, n, ps, verbose=False)
-                            _check_termination(factors, n,
-                                                           limit,
-                                                           use_trial,
-                                                           use_rho,
-                                                           use_pm1,
-                                                           verbose)
+                            _check_termination(factors, n, limit, use_trial,
+                                               use_rho, use_pm1, verbose)
 
         except StopIteration:
             if verbose:
-                print complete_msg
+                print(complete_msg)
             return factors
 
         low, high = high, high*2
@@ -1191,23 +1202,23 @@ def primefactors(n, limit=None, verbose=False):
     divisors
     """
     n = int(n)
-    s = []
     factors = sorted(factorint(n, limit=limit, verbose=verbose).keys())
     s = [f for f in factors[:-1:] if f not in [-1, 0, 1]]
     if factors and isprime(factors[-1]):
         s += [factors[-1]]
     return s
 
+
 def _divisors(n):
     """Helper function for divisors which generates the divisors."""
 
     factordict = factorint(n)
     ps = sorted(factordict.keys())
 
-    def rec_gen(n = 0):
+    def rec_gen(n=0):
         if n == len(ps):
             yield 1
-        else :
+        else:
             pows = [1]
             for j in xrange(factordict[ps[n]]):
                 pows.append(pows[-1] * ps[n])
@@ -1215,9 +1226,10 @@ def _divisors(n):
                 for p in pows:
                     yield p * q
 
-    for p in rec_gen() :
+    for p in rec_gen():
         yield p
 
+
 def divisors(n, generator=False):
     r"""
     Return all divisors of n sorted from 1..n by default.
@@ -1248,18 +1260,18 @@ def divisors(n, generator=False):
     primefactors, factorint, divisor_count
     """
 
-    n = abs(n)
+    n = int(abs(n))
     if isprime(n):
         return [1, n]
-    elif n == 1:
+    if n == 1:
         return [1]
-    elif n == 0:
+    if n == 0:
         return []
-    else:
-        rv = _divisors(n)
-        if not generator:
-            return sorted(rv)
-        return rv
+    rv = _divisors(n)
+    if not generator:
+        return sorted(rv)
+    return rv
+
 
 def divisor_count(n, modulus=1):
     """
@@ -1289,9 +1301,10 @@ def divisor_count(n, modulus=1):
             return 0
     if n == 0:
         return 0
-    return Mul(*[v+1 for k, v in factorint(n).items() if k > 1])
+    return Mul(*[v + 1 for k, v in factorint(n).items() if k > 1])
 
-def totient(n):
+
+class totient(Function):
     """
     Calculate the Euler totient function phi(n)
 
@@ -1306,11 +1319,14 @@ def totient(n):
 
     divisor_count
     """
-    n = as_int(n)
-    if n < 1:
-        raise ValueError("n must be a positive integer")
-    factors = factorint(n)
-    t = 1
-    for p, k in factors.iteritems():
-        t *= (p-1) * p**(k-1)
-    return t
+    @classmethod
+    def eval(cls, n):
+        n = sympify(n)
+        if n.is_Integer:
+            if n < 1:
+                raise ValueError("n must be a positive integer")
+            factors = factorint(n)
+            t = 1
+            for p, k in factors.items():
+                t *= (p - 1) * p**(k - 1)
+            return t
