@@ -22,6 +22,8 @@ image distance
     positive for real images
 """
 
+from __future__ import print_function, division
+
 from sympy import (atan2, Expr, I, im, Matrix, oo, pi, re, sqrt, sympify,
     together)
 from sympy.utilities.misc import filldedent
@@ -30,6 +32,7 @@ from sympy.utilities.misc import filldedent
 # A, B, C, D matrices
 ###
 
+
 class RayTransferMatrix(Matrix):
     """
     Base class for a Ray Transfer Matrix.
@@ -50,12 +53,14 @@ class RayTransferMatrix(Matrix):
 
     >>> mat = RayTransferMatrix(1, 2, 3, 4)
     >>> mat
-    [1,  2]
-    [3,  4]
+    Matrix([
+    [1, 2],
+    [3, 4]])
 
     >>> RayTransferMatrix(Matrix([[1, 2], [3, 4]]))
-    [1,  2]
-    [3,  4]
+    Matrix([
+    [1, 2],
+    [3, 4]])
 
     >>> mat.A
     1
@@ -63,8 +68,9 @@ class RayTransferMatrix(Matrix):
     >>> f = Symbol('f')
     >>> lens = ThinLens(f)
     >>> lens
-    [   1, 0]
-    [-1/f, 1]
+    Matrix([
+    [   1, 0],
+    [-1/f, 1]])
 
     >>> lens.C
     -1/f
@@ -79,7 +85,7 @@ class RayTransferMatrix(Matrix):
     References
     ==========
 
-    [1] http://en.wikipedia.org/wiki/Ray_transfer_matrix_analysis
+    .. [1] http://en.wikipedia.org/wiki/Ray_transfer_matrix_analysis
     """
 
     def __new__(cls, *args):
@@ -87,8 +93,8 @@ class RayTransferMatrix(Matrix):
         if len(args) == 4:
             temp = ((args[0], args[1]), (args[2], args[3]))
         elif len(args) == 1 \
-             and isinstance(args[0], Matrix) \
-             and args[0].shape == (2, 2):
+            and isinstance(args[0], Matrix) \
+                and args[0].shape == (2, 2):
             temp = args[0]
         else:
             raise ValueError(filldedent('''
@@ -104,9 +110,9 @@ class RayTransferMatrix(Matrix):
         elif isinstance(other, BeamParameter):
             temp = self*Matrix(((other.q,), (1,)))
             q = (temp[0]/temp[1]).expand(complex=True)
-            return BeamParameter(other.wavelen, \
-                                 together(re(q)), \
-                                 z_r = together(im(q)))
+            return BeamParameter(other.wavelen,
+                                 together(re(q)),
+                                 z_r=together(im(q)))
         else:
             return Matrix.__mul__(self, other)
 
@@ -170,6 +176,7 @@ class RayTransferMatrix(Matrix):
         """
         return self[1, 1]
 
+
 class FreeSpace(RayTransferMatrix):
     """
     Ray Transfer Matrix for free space.
@@ -191,12 +198,14 @@ class FreeSpace(RayTransferMatrix):
     >>> from sympy import symbols
     >>> d = symbols('d')
     >>> FreeSpace(d)
-    [1, d]
-    [0, 1]
+    Matrix([
+    [1, d],
+    [0, 1]])
     """
     def __new__(cls, d):
         return RayTransferMatrix.__new__(cls, 1, d, 0, 1)
 
+
 class FlatRefraction(RayTransferMatrix):
     """
     Ray Transfer Matrix for refraction.
@@ -219,13 +228,15 @@ class FlatRefraction(RayTransferMatrix):
     >>> from sympy import symbols
     >>> n1, n2 = symbols('n1 n2')
     >>> FlatRefraction(n1, n2)
-    [1,     0]
-    [0, n1/n2]
+    Matrix([
+    [1,     0],
+    [0, n1/n2]])
     """
     def __new__(cls, n1, n2):
-        n1, n2 = sympify((n1, n2))
+        n1, n2 = map(sympify, (n1, n2))
         return RayTransferMatrix.__new__(cls, 1, 0, 0, n1/n2)
 
+
 class CurvedRefraction(RayTransferMatrix):
     """
     Ray Transfer Matrix for refraction on curved interface.
@@ -249,12 +260,14 @@ class CurvedRefraction(RayTransferMatrix):
     >>> from sympy import symbols
     >>> R, n1, n2 = symbols('R n1 n2')
     >>> CurvedRefraction(R, n1, n2)
-    [               1,     0]
-    [(n1 - n2)/(R*n2), n1/n2]
+    Matrix([
+    [               1,     0],
+    [(n1 - n2)/(R*n2), n1/n2]])
     """
     def __new__(cls, R, n1, n2):
-        R, n1 , n2 = sympify((R, n1, n2))
-        return RayTransferMatrix.__new__(cls, 1, 0, (n1-n2)/R/n2, n1/n2)
+        R, n1, n2 = map(sympify, (R, n1, n2))
+        return RayTransferMatrix.__new__(cls, 1, 0, (n1 - n2)/R/n2, n1/n2)
+
 
 class FlatMirror(RayTransferMatrix):
     """
@@ -270,12 +283,14 @@ class FlatMirror(RayTransferMatrix):
 
     >>> from sympy.physics.gaussopt import FlatMirror
     >>> FlatMirror()
-    [1, 0]
-    [0, 1]
+    Matrix([
+    [1, 0],
+    [0, 1]])
     """
     def __new__(cls):
         return RayTransferMatrix.__new__(cls, 1, 0, 0, 1)
 
+
 class CurvedMirror(RayTransferMatrix):
     """
     Ray Transfer Matrix for reflection from curved surface.
@@ -297,13 +312,15 @@ class CurvedMirror(RayTransferMatrix):
     >>> from sympy import symbols
     >>> R = symbols('R')
     >>> CurvedMirror(R)
-    [   1, 0]
-    [-2/R, 1]
+    Matrix([
+    [   1, 0],
+    [-2/R, 1]])
     """
     def __new__(cls, R):
         R = sympify(R)
         return RayTransferMatrix.__new__(cls, 1, 0, -2/R, 1)
 
+
 class ThinLens(RayTransferMatrix):
     """
     Ray Transfer Matrix for a thin lens.
@@ -325,8 +342,9 @@ class ThinLens(RayTransferMatrix):
     >>> from sympy import symbols
     >>> f = symbols('f')
     >>> ThinLens(f)
-    [   1, 0]
-    [-1/f, 1]
+    Matrix([
+    [   1, 0],
+    [-1/f, 1]])
     """
     def __new__(cls, f):
         f = sympify(f)
@@ -356,16 +374,19 @@ class GeometricRay(Matrix):
     >>> d, h, angle = symbols('d, h, angle')
 
     >>> GeometricRay(h, angle)
-    [    h]
-    [angle]
+    Matrix([
+    [    h],
+    [angle]])
 
     >>> FreeSpace(d)*GeometricRay(h, angle)
-    [angle*d + h]
-    [      angle]
+    Matrix([
+    [angle*d + h],
+    [      angle]])
 
     >>> GeometricRay( Matrix( ((h,), (angle,)) ) )
-    [    h]
-    [angle]
+    Matrix([
+    [    h],
+    [angle]])
 
     See Also
     ========
@@ -376,7 +397,7 @@ class GeometricRay(Matrix):
 
     def __new__(cls, *args):
         if len(args) == 1 and isinstance(args[0], Matrix) \
-                          and args[0].shape == (2, 1):
+                and args[0].shape == (2, 1):
             temp = args[0]
         elif len(args) == 2:
             temp = ((args[0],), (args[1],))
@@ -468,7 +489,7 @@ class BeamParameter(Expr):
     References
     ==========
 
-    [1] http://en.wikipedia.org/wiki/Complex_beam_parameter
+    .. [1] http://en.wikipedia.org/wiki/Complex_beam_parameter
     """
     #TODO A class Complex may be implemented. The BeamParameter may
     # subclass it. See:
@@ -477,11 +498,11 @@ class BeamParameter(Expr):
     __slots__ = ['z', 'z_r', 'wavelen']
 
     def __new__(cls, wavelen, z, **kwargs):
-        wavelen, z = sympify((wavelen, z))
+        wavelen, z = map(sympify, (wavelen, z))
         inst = Expr.__new__(cls, wavelen, z)
         inst.wavelen = wavelen
         inst.z = z
-        if len(kwargs) !=1:
+        if len(kwargs) != 1:
             raise ValueError('Constructor expects exactly one named argument.')
         elif 'z_r' in kwargs:
             inst.z_r = sympify(kwargs['z_r'])
@@ -519,7 +540,7 @@ class BeamParameter(Expr):
         >>> p.radius
         0.2809/pi**2 + 1
         """
-        return self.z*(1+(self.z/self.z_r)**2)
+        return self.z*(1 + (self.z/self.z_r)**2)
 
     @property
     def w(self):
@@ -539,7 +560,7 @@ class BeamParameter(Expr):
         >>> p.w
         0.001*sqrt(0.2809/pi**2 + 1)
         """
-        return self.w_0*sqrt(1+(self.z/self.z_r)**2)
+        return self.w_0*sqrt(1 + (self.z/self.z_r)**2)
 
     @property
     def w_0(self):
@@ -632,9 +653,10 @@ def waist2rayleigh(w, wavelen):
     >>> waist2rayleigh(w, wavelen)
     pi*w**2/wavelen
     """
-    w, wavelen = sympify((w, wavelen))
+    w, wavelen = map(sympify, (w, wavelen))
     return w**2*pi/wavelen
 
+
 def rayleigh2waist(z_r, wavelen):
     """Calculate the waist from the rayleigh range of a gaussian beam.
 
@@ -652,7 +674,7 @@ def rayleigh2waist(z_r, wavelen):
     >>> rayleigh2waist(z_r, wavelen)
     sqrt(wavelen*z_r)/sqrt(pi)
     """
-    z_r, wavelen = sympify((z_r, wavelen))
+    z_r, wavelen = map(sympify, (z_r, wavelen))
     return sqrt(z_r/pi*wavelen)
 
 
@@ -677,11 +699,12 @@ def geometric_conj_ab(a, b):
     >>> geometric_conj_ab(a, b)
     a*b/(a + b)
     """
-    a, b = sympify((a, b))
+    a, b = map(sympify, (a, b))
     if abs(a) == oo or abs(b) == oo:
         return a if abs(b) == oo else b
     else:
-        return a*b/(a+b)
+        return a*b/(a + b)
+
 
 def geometric_conj_af(a, f):
     """
@@ -707,11 +730,12 @@ def geometric_conj_af(a, f):
     >>> geometric_conj_bf(b, f)
     b*f/(b - f)
     """
-    a, f = sympify((a, f))
+    a, f = map(sympify, (a, f))
     return -geometric_conj_ab(a, -f)
 
 geometric_conj_bf = geometric_conj_af
 
+
 def gaussian_conj(s_in, z_r_in, f):
     """
     Conjugation relation for gaussian beams.
@@ -747,12 +771,13 @@ def gaussian_conj(s_in, z_r_in, f):
     >>> gaussian_conj(s_in, z_r_in, f)[2]
     1/sqrt(1 - s_in**2/f**2 + z_r_in**2/f**2)
     """
-    s_in, z_r_in, f = sympify((s_in, z_r_in, f))
-    s_out = 1 / ( -1/(s_in + z_r_in**2/(s_in-f)) + 1/f )
-    m = 1/sqrt((1-(s_in/f)**2) + (z_r_in/f)**2)
-    z_r_out = z_r_in / ((1-(s_in/f)**2) + (z_r_in/f)**2)
+    s_in, z_r_in, f = map(sympify, (s_in, z_r_in, f))
+    s_out = 1 / ( -1/(s_in + z_r_in**2/(s_in - f)) + 1/f )
+    m = 1/sqrt((1 - (s_in/f)**2) + (z_r_in/f)**2)
+    z_r_out = z_r_in / ((1 - (s_in/f)**2) + (z_r_in/f)**2)
     return (s_out, z_r_out, m)
 
+
 def conjugate_gauss_beams(wavelen, waist_in, waist_out, **kwargs):
     """
     Find the optical setup conjugating the object/image waists.
@@ -783,13 +808,14 @@ def conjugate_gauss_beams(wavelen, waist_in, waist_out, **kwargs):
     f*(-sqrt(w_i**2/w_o**2 - pi**2*w_i**4/(f**2*l**2)) + 1)
 
     >>> factor(conjugate_gauss_beams(l, w_i, w_o, f=f)[1])
-    f*w_o**2*(w_i**2/w_o**2 - sqrt(w_i**2/w_o**2 - pi**2*w_i**4/(f**2*l**2)))/w_i**2
+    f*w_o**2*(w_i**2/w_o**2 - sqrt(w_i**2/w_o**2 -
+              pi**2*w_i**4/(f**2*l**2)))/w_i**2
 
     >>> conjugate_gauss_beams(l, w_i, w_o, f=f)[2]
     f
     """
     #TODO add the other possible arguments
-    wavelen, waist_in, waist_out = sympify((wavelen, waist_in, waist_out))
+    wavelen, waist_in, waist_out = map(sympify, (wavelen, waist_in, waist_out))
     m = waist_out / waist_in
     z = waist2rayleigh(waist_in, wavelen)
     if len(kwargs) != 1:
