@@ -1,14 +1,19 @@
+from __future__ import print_function, division
+
 __all__ = ['cross',
            'dot',
            'express',
+           'time_derivative',
            'outer',
            'inertia',
            'mechanics_printing',
            'mprint',
+           'msprint',
            'mpprint',
            'mlatex',
            'kinematic_equations',
            'inertia_of_point_mass',
+           'get_motion_params',
            'partial_velocity',
            'linear_momentum',
            'angular_momentum',
@@ -17,16 +22,20 @@ __all__ = ['cross',
            'Lagrangian']
 
 from sympy.physics.mechanics.essential import (Vector, Dyadic, ReferenceFrame,
+                                               CoordinateSym,
                                                MechanicsStrPrinter,
                                                MechanicsPrettyPrinter,
                                                MechanicsLatexPrinter,
-                                               dynamicsymbols)
+                                               dynamicsymbols,
+                                               _check_frame, _check_vector)
 from sympy.physics.mechanics.particle import Particle
 from sympy.physics.mechanics.rigidbody import RigidBody
 from sympy.physics.mechanics.point import Point
-from sympy import sympify, diff, sin, cos, Matrix
+from sympy import sympify, solve, diff, sin, cos, Matrix, Symbol, integrate, \
+     trigsimp
 from sympy.core.basic import S
 
+
 def cross(vec1, vec2):
     """Cross product convenience wrapper for Vector.cross(): \n"""
     if not isinstance(vec1, (Vector, Dyadic)):
@@ -34,6 +43,7 @@ def cross(vec1, vec2):
     return vec1 ^ vec2
 cross.__doc__ += Vector.cross.__doc__
 
+
 def dot(vec1, vec2):
     """Dot product convenience wrapper for Vector.dot(): \n"""
     if not isinstance(vec1, (Vector, Dyadic)):
@@ -41,23 +51,192 @@ def dot(vec1, vec2):
     return vec1 & vec2
 dot.__doc__ += Vector.dot.__doc__
 
-def express(vec, frame, frame2=None):
-    """Express convenience wrapper for Vector.express(): \n"""
-    if not isinstance(vec, (Vector, Dyadic)):
-        raise TypeError('Can only express Vectors')
-    if isinstance(vec, Vector):
-        return vec.express(frame)
-    else:
-        return vec.express(frame, frame2)
 
-express.__doc__ += Vector.express.__doc__
+def express(expr, frame, frame2=None, variables=False):
+    """
+    Global function for 'express' functionality.
+
+    Re-expresses a Vector, scalar(sympyfiable) or Dyadic in given frame.
+
+    Refer to the local methods of Vector and Dyadic for details.
+    If 'variables' is True, then the coordinate variables (CoordinateSym
+    instances) of other frames present in the vector/scalar field or
+    dyadic expression are also substituted in terms of the base scalars of
+    this frame.
+
+    Parameters
+    ==========
+
+    expr : Vector/Dyadic/scalar(sympyfiable)
+        The expression to re-express in ReferenceFrame 'frame'
+
+    frame: ReferenceFrame
+        The reference frame to express expr in
+
+    frame2 : ReferenceFrame
+        The other frame required for re-expression(only for Dyadic expr)
+
+    variables : boolean
+        Specifies whether to substitute the coordinate variables present
+        in expr, in terms of those of frame
+
+    Examples
+    ========
+
+    >>> from sympy.physics.mechanics import ReferenceFrame, outer, dynamicsymbols
+    >>> N = ReferenceFrame('N')
+    >>> q = dynamicsymbols('q')
+    >>> B = N.orientnew('B', 'Axis', [q, N.z])
+    >>> d = outer(N.x, N.x)
+    >>> from sympy.physics.mechanics import express
+    >>> express(d, B, N)
+    cos(q)*(B.x|N.x) - sin(q)*(B.y|N.x)
+    >>> express(B.x, N)
+    cos(q)*N.x + sin(q)*N.y
+    >>> express(N[0], B, variables=True)
+    B_x*cos(q(t)) - B_y*sin(q(t))
+
+    """
+
+    _check_frame(frame)
+
+    if expr == 0:
+        return S(0)
+
+    if isinstance(expr, Vector):
+        #Given expr is a Vector
+        if variables:
+            #If variables attribute is True, substitute
+            #the coordinate variables in the Vector
+            frame_list = [x[-1] for x in expr.args]
+            subs_dict = {}
+            for f in frame_list:
+                subs_dict.update(f.variable_map(frame))
+            expr = expr.subs(subs_dict)
+        #Re-express in this frame
+        outvec = Vector([])
+        for i, v in enumerate(expr.args):
+            if v[1] != frame:
+                temp = frame.dcm(v[1]) * v[0]
+                if Vector.simp:
+                    temp = temp.applyfunc(lambda x: \
+                                          trigsimp(x, method='fu'))
+                outvec += Vector([(temp, frame)])
+            else:
+                outvec += Vector([v])
+        return outvec
+
+    if isinstance(expr, Dyadic):
+        if frame2 is None:
+            frame2 = frame
+        _check_frame(frame2)
+        ol = Dyadic(0)
+        for i, v in enumerate(expr.args):
+            ol += express(v[0], frame, variables=variables) * \
+                  (express(v[1], frame, variables=variables) | \
+                   express(v[2], frame2, variables=variables))
+        return ol
+
+    else:
+        if variables:
+            #Given expr is a scalar field
+            frame_set = set([])
+            expr = sympify(expr)
+            #Subsitute all the coordinate variables
+            for x in expr.atoms():
+                if isinstance(x, CoordinateSym)and x.frame != frame:
+                    frame_set.add(x.frame)
+            subs_dict = {}
+            for f in frame_set:
+                subs_dict.update(f.variable_map(frame))
+            return expr.subs(subs_dict)
+        return expr
+
+
+def time_derivative(expr, frame, order=1):
+    """
+    Calculate the time derivative of a vector/scalar field function
+    or dyadic expression in given frame.
+
+    References
+    ==========
+
+    http://en.wikipedia.org/wiki/Rotating_reference_frame#Time_derivatives_in_the_two_frames
+
+    Parameters
+    ==========
+
+    expr : Vector/Dyadic/sympifyable
+        The expression whose time derivative is to be calculated
+
+    frame : ReferenceFrame
+        The reference frame to calculate the time derivative in
+
+    order : integer
+        The order of the derivative to be calculated
+
+    Examples
+    ========
+
+    >>> from sympy.physics.mechanics import ReferenceFrame, Vector, dynamicsymbols
+    >>> from sympy import Symbol
+    >>> q1 = Symbol('q1')
+    >>> u1 = dynamicsymbols('u1')
+    >>> N = ReferenceFrame('N')
+    >>> A = N.orientnew('A', 'Axis', [q1, N.x])
+    >>> v = u1 * N.x
+    >>> A.set_ang_vel(N, 10*A.x)
+    >>> from sympy.physics.mechanics import time_derivative
+    >>> time_derivative(v, N)
+    u1'*N.x
+    >>> time_derivative(u1*A[0], N)
+    N_x*Derivative(u1(t), t)
+    >>> B = N.orientnew('B', 'Axis', [u1, N.z])
+    >>> from sympy.physics.mechanics import outer
+    >>> d = outer(N.x, N.x)
+    >>> time_derivative(d, B)
+    - u1'*(N.y|N.x) - u1'*(N.x|N.y)
+
+    """
+
+    t = dynamicsymbols._t
+    _check_frame(frame)
+
+    if order == 0:
+        return expr
+    if order%1 != 0 or order < 0:
+        raise ValueError("Unsupported value of order entered")
+
+    if isinstance(expr, Vector):
+        outvec = Vector(0)
+        for i, v in enumerate(expr.args):
+            if v[1] == frame:
+                outvec += Vector([(express(v[0], frame, \
+                                           variables=True).diff(t), frame)])
+            else:
+                outvec += time_derivative(Vector([v]), v[1]) + \
+                          (v[1].ang_vel_in(frame) ^ Vector([v]))
+        return time_derivative(outvec, frame, order - 1)
+
+    if isinstance(expr, Dyadic):
+        ol = Dyadic(0)
+        for i, v in enumerate(expr.args):
+            ol += (v[0].diff(t) * (v[1] | v[2]))
+            ol += (v[0] * (time_derivative(v[1], frame) | v[2]))
+            ol += (v[0] * (v[1] | time_derivative(v[2], frame)))
+        return time_derivative(ol, frame, order - 1)
+
+    else:
+        return diff(express(expr, frame, variables=True), t, order)
+
 
 def outer(vec1, vec2):
-    """Outer prodcut convenience wrapper for Vector.outer():\n"""
+    """Outer product convenience wrapper for Vector.outer():\n"""
     if not isinstance(vec1, Vector):
         raise TypeError('Outer product is between two Vectors')
     return vec1 | vec2
-outer.__doc__ += Vector.express.__doc__
+outer.__doc__ += Vector.outer.__doc__
+
 
 def inertia(frame, ixx, iyy, izz, ixy=0, iyz=0, izx=0):
     """Simple way to create inertia Dyadic object.
@@ -95,7 +274,7 @@ def inertia(frame, ixx, iyy, izz, ixy=0, iyz=0, izx=0):
 
     if not isinstance(frame, ReferenceFrame):
         raise TypeError('Need to define the inertia in a frame')
-    ol  = sympify(ixx) * (frame.x | frame.x)
+    ol = sympify(ixx) * (frame.x | frame.x)
     ol += sympify(ixy) * (frame.x | frame.y)
     ol += sympify(izx) * (frame.x | frame.z)
     ol += sympify(ixy) * (frame.y | frame.x)
@@ -106,6 +285,7 @@ def inertia(frame, ixx, iyy, izz, ixy=0, iyz=0, izx=0):
     ol += sympify(izz) * (frame.z | frame.z)
     return ol
 
+
 def inertia_of_point_mass(mass, pos_vec, frame):
     """Inertia dyadic of a point mass realtive to point O.
 
@@ -136,6 +316,7 @@ def inertia_of_point_mass(mass, pos_vec, frame):
                    (frame.z | frame.z)) * (pos_vec & pos_vec) -
                    (pos_vec | pos_vec))
 
+
 def mechanics_printing():
     """Sets up interactive printing for mechanics' derivatives.
 
@@ -171,6 +352,7 @@ def mechanics_printing():
     import sys
     sys.displayhook = mprint
 
+
 def mprint(expr, **settings):
     r"""Function for printing of expressions generated in mechanics.
 
@@ -198,14 +380,44 @@ def mprint(expr, **settings):
 
     """
 
-    pr = MechanicsStrPrinter(settings)
-    outstr = pr.doprint(expr)
+    outstr = msprint(expr, **settings)
 
-    import __builtin__
+    from sympy.core.compatibility import builtins
     if (outstr != 'None'):
-        __builtin__._ = outstr
+        builtins._ = outstr
         print(outstr)
 
+
+def msprint(expr, **settings):
+    r"""Function for displaying expressions generated in mechanics.
+
+    Returns the output of mprint() as a string.
+
+    Parameters
+    ==========
+
+    expr : valid sympy object
+        SymPy expression to print
+    settings : args
+        Same as print for SymPy
+
+    Examples
+    ========
+
+    >>> from sympy.physics.mechanics import msprint, dynamicsymbols
+    >>> u1, u2 = dynamicsymbols('u1 u2')
+    >>> u2d = dynamicsymbols('u2', level=1)
+    >>> print("%s = %s" % (u1, u2 + u2d))
+    u1(t) = u2(t) + Derivative(u2(t), t)
+    >>> print("%s = %s" % (msprint(u1), msprint(u2 + u2d)))
+    u1 = u2 + u2'
+
+    """
+
+    pr = MechanicsStrPrinter(settings)
+    return pr.doprint(expr)
+
+
 def mpprint(expr, **settings):
     r"""Function for pretty printing of expressions generated in mechanics.
 
@@ -231,6 +443,7 @@ def mpprint(expr, **settings):
     mp = MechanicsPrettyPrinter(settings)
     print(mp.doprint(expr))
 
+
 def mlatex(expr, **settings):
     r"""Function for printing latex representation of mechanics objects.
 
@@ -268,6 +481,7 @@ def mlatex(expr, **settings):
 
     return MechanicsLatexPrinter(settings).doprint(expr)
 
+
 def kinematic_equations(speeds, coords, rot_type, rot_order=''):
     """Gives equations relating the qdot's to u's for a rotation type.
 
@@ -305,18 +519,18 @@ def kinematic_equations(speeds, coords, rot_type, rot_order=''):
     # Code below is checking and sanitizing input
     approved_orders = ('123', '231', '312', '132', '213', '321', '121', '131',
                        '212', '232', '313', '323', '1', '2', '3', '')
-    rot_order = str(rot_order).upper() # Now we need to make sure XYZ = 123
-    rot_type  = rot_type.upper()
+    rot_order = str(rot_order).upper()  # Now we need to make sure XYZ = 123
+    rot_type = rot_type.upper()
     rot_order = [i.replace('X', '1') for i in rot_order]
     rot_order = [i.replace('Y', '2') for i in rot_order]
     rot_order = [i.replace('Z', '3') for i in rot_order]
     rot_order = ''.join(rot_order)
 
-    if not isinstance(speeds,(list, tuple)):
+    if not isinstance(speeds, (list, tuple)):
         raise TypeError('Need to supply speeds in a list')
     if len(speeds) != 3:
         raise TypeError('Need to supply 3 body-fixed speeds')
-    if not isinstance(coords,(list, tuple)):
+    if not isinstance(coords, (list, tuple)):
         raise TypeError('Need to supply coordinates in a list')
     if rot_type.lower() in ['body', 'space']:
         if rot_order not in approved_orders:
@@ -418,6 +632,155 @@ def kinematic_equations(speeds, coords, rot_type, rot_order=''):
     else:
         raise ValueError('Not an approved rotation type for this function')
 
+
+def get_motion_params(frame, **kwargs):
+    """
+    Returns the three motion parameters - (acceleration, velocity, and
+    position) as vectorial functions of time in the given frame.
+
+    If a higher order differential function is provided, the lower order
+    functions are used as boundary conditions. For example, given the
+    acceleration, the velocity and position parameters are taken as
+    boundary conditions.
+
+    The values of time at which the boundary conditions are specified
+    are taken from timevalue1(for position boundary condition) and
+    timevalue2(for velocity boundary condition).
+
+    If any of the boundary conditions are not provided, they are taken
+    to be zero by default (zero vectors, in case of vectorial inputs). If
+    the boundary conditions are also functions of time, they are converted
+    to constants by substituting the time values in the dynamicsymbols._t
+    time Symbol.
+
+    This function can also be used for calculating rotational motion
+    parameters. Have a look at the Parameters and Examples for more clarity.
+
+    Parameters
+    ==========
+
+    frame : ReferenceFrame
+        The frame to express the motion parameters in
+
+    acceleration : Vector
+        Acceleration of the object/frame as a function of time
+
+    velocity : Vector
+        Velocity as function of time or as boundary condition
+        of velocity at time = timevalue1
+
+    position : Vector
+        Velocity as function of time or as boundary condition
+        of velocity at time = timevalue1
+
+    timevalue1 : sympyfiable
+        Value of time for position boundary condition
+
+    timevalue2 : sympyfiable
+        Value of time for velocity boundary condition
+
+    Examples
+    ========
+
+    >>> from sympy.physics.mechanics import ReferenceFrame, get_motion_params, dynamicsymbols
+    >>> from sympy import symbols
+    >>> R = ReferenceFrame('R')
+    >>> v1, v2, v3 = dynamicsymbols('v1 v2 v3')
+    >>> v = v1*R.x + v2*R.y + v3*R.z
+    >>> get_motion_params(R, position = v)
+    (v1''*R.x + v2''*R.y + v3''*R.z, v1'*R.x + v2'*R.y + v3'*R.z, v1*R.x + v2*R.y + v3*R.z)
+    >>> a, b, c = symbols('a b c')
+    >>> v = a*R.x + b*R.y + c*R.z
+    >>> get_motion_params(R, velocity = v)
+    (0, a*R.x + b*R.y + c*R.z, a*t*R.x + b*t*R.y + c*t*R.z)
+    >>> parameters = get_motion_params(R, acceleration = v)
+    >>> parameters[1]
+    a*t*R.x + b*t*R.y + c*t*R.z
+    >>> parameters[2]
+    a*t**2/2*R.x + b*t**2/2*R.y + c*t**2/2*R.z
+
+    """
+
+    ##Helper functions
+
+    def _process_vector_differential(vectdiff, condition, \
+                                     variable, ordinate, frame):
+        """
+        Helper function for get_motion methods. Finds derivative of vectdiff wrt
+        variable, and its integral using the specified boundary condition at
+        value of variable = ordinate.
+        Returns a tuple of - (derivative, function and integral) wrt vectdiff
+
+        """
+
+        #Make sure boundary condition is independent of 'variable'
+        if condition != 0:
+            condition = express(condition, frame, variables=True)
+        #Special case of vectdiff == 0
+        if vectdiff == Vector(0):
+            return (0, 0, condition)
+        #Express vectdiff completely in condition's frame to give vectdiff1
+        vectdiff1 = express(vectdiff, frame)
+        #Find derivative of vectdiff
+        vectdiff2 = time_derivative(vectdiff, frame)
+        #Integrate and use boundary condition
+        vectdiff0 = Vector(0)
+        lims = (variable, ordinate, variable)
+        for dim in frame:
+            function1 = vectdiff1.dot(dim)
+            abscissa = dim.dot(condition).subs({variable : ordinate})
+            # Indefinite integral of 'function1' wrt 'variable', using
+            # the given initial condition (ordinate, abscissa).
+            vectdiff0 += (integrate(function1, lims) + abscissa) * dim
+        #Return tuple
+        return (vectdiff2, vectdiff, vectdiff0)
+
+    ##Function body
+
+    _check_frame(frame)
+    #Decide mode of operation based on user's input
+    if 'acceleration' in kwargs:
+        mode = 2
+    elif 'velocity' in kwargs:
+        mode = 1
+    else:
+        mode = 0
+    #All the possible parameters in kwargs
+    #Not all are required for every case
+    #If not specified, set to default values(may or may not be used in
+    #calculations)
+    conditions = ['acceleration', 'velocity', 'position',
+                  'timevalue', 'timevalue1', 'timevalue2']
+    for i, x in enumerate(conditions):
+        if x not in kwargs:
+            if i < 3:
+                kwargs[x] = Vector(0)
+            else:
+                kwargs[x] = S(0)
+        elif i < 3:
+            _check_vector(kwargs[x])
+        else:
+            kwargs[x] = sympify(kwargs[x])
+    if mode == 2:
+        vel = _process_vector_differential(kwargs['acceleration'],
+                                           kwargs['velocity'],
+                                           dynamicsymbols._t,
+                                           kwargs['timevalue2'], frame)[2]
+        pos = _process_vector_differential(vel, kwargs['position'],
+                                           dynamicsymbols._t,
+                                           kwargs['timevalue1'], frame)[2]
+        return (kwargs['acceleration'], vel, pos)
+    elif mode == 1:
+        return _process_vector_differential(kwargs['velocity'],
+                                            kwargs['position'],
+                                            dynamicsymbols._t,
+                                            kwargs['timevalue1'], frame)
+    else:
+        vel = time_derivative(kwargs['position'], frame)
+        acc = time_derivative(vel, frame)
+        return (acc, vel, kwargs['position'])
+
+
 def partial_velocity(vel_list, u_list, frame):
     """Returns a list of partial velocities.
 
@@ -469,6 +832,7 @@ def partial_velocity(vel_list, u_list, frame):
         list_of_pvlists += [pvlist]
     return list_of_pvlists
 
+
 def linear_momentum(frame, *body):
     """Linear momentum of the system.
 
@@ -510,7 +874,7 @@ def linear_momentum(frame, *body):
     if not isinstance(frame, ReferenceFrame):
         raise TypeError('Please specify a valid ReferenceFrame')
     else:
-        linear_momentum_sys = S(0)
+        linear_momentum_sys = Vector(0)
         for e in body:
             if isinstance(e, (RigidBody, Particle)):
                 linear_momentum_sys += e.linear_momentum(frame)
@@ -518,6 +882,7 @@ def linear_momentum(frame, *body):
                 raise TypeError('*body must have only Particle or RigidBody')
     return linear_momentum_sys
 
+
 def angular_momentum(point, frame, *body):
     """Angular momentum of a system
 
@@ -567,7 +932,7 @@ def angular_momentum(point, frame, *body):
     if not isinstance(point, Point):
         raise TypeError('Please specify a valid Point')
     else:
-        angular_momentum_sys = S(0)
+        angular_momentum_sys = Vector(0)
         for e in body:
             if isinstance(e, (RigidBody, Particle)):
                 angular_momentum_sys += e.angular_momentum(point, frame)
@@ -575,6 +940,7 @@ def angular_momentum(point, frame, *body):
                 raise TypeError('*body must have only Particle or RigidBody')
     return angular_momentum_sys
 
+
 def kinetic_energy(frame, *body):
     """Kinetic energy of a multibody system.
 
@@ -630,6 +996,7 @@ def kinetic_energy(frame, *body):
             raise TypeError('*body must have only Particle or RigidBody')
     return ke_sys
 
+
 def potential_energy(*body):
     """Potential energy of a multibody system.
 
@@ -681,6 +1048,7 @@ def potential_energy(*body):
             raise TypeError('*body must have only Particle or RigidBody')
     return pe_sys
 
+
 def Lagrangian(frame, *body):
     """Lagrangian of a multibody system.
 
