@@ -1,8 +1,10 @@
 """Dirac notation for states."""
 
+from __future__ import print_function, division
 
 from sympy import (cacheit, conjugate, Expr, Function, integrate, oo, sqrt,
                    Tuple)
+from sympy.core.compatibility import u
 from sympy.printing.pretty.stringpict import prettyForm, stringPict
 from sympy.physics.quantum.qexpr import QExpr, dispatch_method
 
@@ -32,10 +34,10 @@ _straight_bracket = "|"
 
 # Unicode brackets
 # MATHEMATICAL ANGLE BRACKETS
-_lbracket_ucode = u"\u27E8"
-_rbracket_ucode = u"\u27E9"
+_lbracket_ucode = u("\u27E8")
+_rbracket_ucode = u("\u27E9")
 # LIGHT VERTICAL BAR
-_straight_bracket_ucode = u"\u2758"
+_straight_bracket_ucode = u("\u2758")
 
 # Other options for unicode printing of <, > and | for Dirac notation.
 
@@ -85,12 +87,13 @@ class StateBase(QExpr):
         operator instances. See cartesian.py for examples,
         """
 
-        raise NotImplementedError("Cannot map this state to operators. Method not implemented!")
+        raise NotImplementedError(
+            "Cannot map this state to operators. Method not implemented!")
 
     @property
     def operators(self):
         """Return the operator(s) that this state is an eigenstate of"""
-        from operatorset import state_to_operators #import internally to avoid circular import errors
+        from .operatorset import state_to_operators  # import internally to avoid circular import errors
         return state_to_operators(self)
 
     def _enumerate_state(self, num_states, **options):
@@ -130,7 +133,7 @@ class StateBase(QExpr):
         # Setup for unicode vs ascii
         if use_unicode:
             lbracket, rbracket = self.lbracket_ucode, self.rbracket_ucode
-            slash, bslash, vert = u'\u2571', u'\u2572', u'\u2502'
+            slash, bslash, vert = u('\u2571'), u('\u2572'), u('\u2502')
         else:
             lbracket, rbracket = self.lbracket, self.rbracket
             slash, bslash, vert = '/', '\\', '|'
@@ -145,18 +148,22 @@ class StateBase(QExpr):
         for bracket in lbracket, rbracket:
             # Create left bracket
             if bracket in set([_lbracket, _lbracket_ucode]):
-                bracket_args = [ ' ' * (height//2-i-1) + slash for i in range(height // 2)]
-                bracket_args.extend([ ' ' * i + bslash for i in range(height // 2)])
+                bracket_args = [ ' ' * (height//2 - i - 1) +
+                                 slash for i in range(height // 2)]
+                bracket_args.extend(
+                    [ ' ' * i + bslash for i in range(height // 2)])
             # Create right bracket
             elif bracket in set([_rbracket, _rbracket_ucode]):
                 bracket_args = [ ' ' * i + bslash for i in range(height // 2)]
-                bracket_args.extend([ ' ' * (height//2-i-1) + slash for i in range(height // 2)])
+                bracket_args.extend([ ' ' * (
+                    height//2 - i - 1) + slash for i in range(height // 2)])
             # Create straight bracket
             elif bracket in set([_straight_bracket, _straight_bracket_ucode]):
                 bracket_args = [vert for i in range(height)]
             else:
                 raise ValueError(bracket)
-            brackets.append(stringPict('\n'.join(bracket_args), baseline=height//2))
+            brackets.append(
+                stringPict('\n'.join(bracket_args), baseline=height//2))
         return brackets
 
     def _sympystr(self, printer, *args):
@@ -167,7 +174,8 @@ class StateBase(QExpr):
         from sympy.printing.pretty.stringpict import prettyForm
         # Get brackets
         pform = self._print_contents_pretty(printer, *args)
-        lbracket, rbracket = self._pretty_brackets(pform.height(), printer._use_unicode)
+        lbracket, rbracket = self._pretty_brackets(
+            pform.height(), printer._use_unicode)
         # Put together state
         pform = prettyForm(*pform.left(lbracket))
         pform = prettyForm(*pform.right(rbracket))
@@ -258,6 +266,7 @@ class KetBase(StateBase):
         """
         return dispatch_method(self, '_apply_operator', op, **options)
 
+
 class BraBase(StateBase):
     """Base class for Bras.
 
@@ -283,7 +292,7 @@ class BraBase(StateBase):
 
     def _enumerate_state(self, num_states, **options):
         dual_states = self.dual._enumerate_state(num_states, **options)
-        return map(lambda x: x.dual, dual_states)
+        return [x.dual for x in dual_states]
 
     @classmethod
     def default_args(self):
@@ -320,7 +329,6 @@ class State(StateBase):
     pass
 
 
-
 class Ket(State, KetBase):
     """A general time-independent Ket in quantum mechanics.
 
@@ -385,6 +393,7 @@ class Ket(State, KetBase):
     def dual_class(self):
         return Bra
 
+
 class Bra(State, BraBase):
     """A general time-independent Bra in quantum mechanics.
 
@@ -449,6 +458,7 @@ class Bra(State, BraBase):
 # Time dependent states, bras and kets.
 #-----------------------------------------------------------------------------
 
+
 class TimeDepState(StateBase):
     """Base class for a general time-dependent quantum state.
 
@@ -518,7 +528,8 @@ class TimeDepState(StateBase):
         return printer._print_seq((label, time), delimiter=';')
 
     def _print_contents_latex(self, printer, *args):
-        label = self._print_sequence(self.label, self._label_separator, printer, *args)
+        label = self._print_sequence(
+            self.label, self._label_separator, printer, *args)
         time = self._print_time_latex(printer, *args)
         return '%s;%s' % (label, time)
 
@@ -603,6 +614,7 @@ class TimeDepBra(TimeDepState, BraBase):
     def dual_class(self):
         return TimeDepKet
 
+
 class Wavefunction(Function):
     """Class for representations in continuous bases
 
@@ -655,7 +667,7 @@ class Wavefunction(Function):
         >>> from sympy.functions import sqrt, sin
         >>> from sympy.physics.quantum.state import Wavefunction
         >>> x, L = symbols('x,L', positive=True)
-        >>> n = symbols('n', integer=True)
+        >>> n = symbols('n', integer=True, positive=True)
         >>> g = sqrt(2/L)*sin(n*pi*x/L)
         >>> f = Wavefunction(g, (x, 0, L))
         >>> f.norm
@@ -699,7 +711,7 @@ class Wavefunction(Function):
                 new_args[ct] = Tuple(*arg)
             else:
                 new_args[ct] = arg
-            ct+=1
+            ct += 1
 
         return super(Function, cls).__new__(cls, *new_args, **options)
 
@@ -707,22 +719,23 @@ class Wavefunction(Function):
         var = self.variables
 
         if len(args) != len(var):
-            raise NotImplementedError("Incorrect number of arguments to function!")
+            raise NotImplementedError(
+                "Incorrect number of arguments to function!")
 
         ct = 0
         #If the passed value is outside the specified bounds, return 0
         for v in var:
-            lower,upper = self.limits[v]
+            lower, upper = self.limits[v]
 
             #Do the comparison to limits only if the passed symbol is actually
             #a symbol present in the limits;
             #Had problems with a comparison of x > L
             if isinstance(args[ct], Expr) and \
-                   not (lower in args[ct].free_symbols \
-                        or upper in args[ct].free_symbols):
+                not (lower in args[ct].free_symbols
+                     or upper in args[ct].free_symbols):
                 continue
 
-            if args[ct] < lower or args[ct] > upper:
+            if (args[ct] < lower) is True or (args[ct] > upper) is True:
                 return 0
 
             ct += 1
@@ -810,7 +823,7 @@ class Wavefunction(Function):
             {x: (-oo, oo), y: (-1, 2)}
 
         """
-        limits = [(g[1], g[2]) if isinstance(g, Tuple) else (-oo, oo) \
+        limits = [(g[1], g[2]) if isinstance(g, Tuple) else (-oo, oo)
                   for g in self._args[1:]]
         return dict(zip(self.variables, tuple(limits)))
 
@@ -844,7 +857,7 @@ class Wavefunction(Function):
             >>> from sympy.functions import sqrt, sin
             >>> from sympy.physics.quantum.state import Wavefunction
             >>> x, L = symbols('x,L', positive=True)
-            >>> n = symbols('n', integer=True)
+            >>> n = symbols('n', integer=True, positive=True)
             >>> g = sqrt(2/L)*sin(n*pi*x/L)
             >>> f = Wavefunction(g, (x, 0, L))
             >>> f.is_normalized
@@ -870,7 +883,7 @@ class Wavefunction(Function):
             >>> from sympy.functions import sqrt, sin
             >>> from sympy.physics.quantum.state import Wavefunction
             >>> x, L = symbols('x,L', positive=True)
-            >>> n = symbols('n', integer=True)
+            >>> n = symbols('n', integer=True, positive=True)
             >>> g = sqrt(2/L)*sin(n*pi*x/L)
             >>> f = Wavefunction(g, (x, 0, L))
             >>> f.norm
@@ -902,8 +915,9 @@ class Wavefunction(Function):
             >>> from sympy import symbols, pi
             >>> from sympy.functions import sqrt, sin
             >>> from sympy.physics.quantum.state import Wavefunction
-            >>> x, L = symbols('x,L', real=True)
-            >>> n = symbols('n', integer=True)
+            >>> x = symbols('x', real=True)
+            >>> L = symbols('L', positive=True)
+            >>> n = symbols('n', integer=True, positive=True)
             >>> g = sin(n*pi*x/L)
             >>> f = Wavefunction(g, (x, 0, L))
             >>> f.normalize()
