@@ -22,48 +22,39 @@ if you care at all about performance. A new backend instance is initialized
 every time you call ``show()`` and the old one is left to the garbage collector.
 """
 
+from __future__ import print_function, division
+
 from inspect import getargspec
 from itertools import chain
+from collections import Callable
+import warnings
+
 from sympy import sympify, Expr, Tuple, Dummy
 from sympy.external import import_module
-from sympy.core.compatibility import set_union
-import warnings
-from experimental_lambdify import (vectorized_lambdify, lambdify)
+from sympy.utilities.decorator import doctest_depends_on
+from .experimental_lambdify import (vectorized_lambdify, lambdify)
 
-#TODO probably all of the imports after this line can be put inside function to
-# speed up the `from sympy import *` command.
-np = import_module('numpy')
-
-# Backend specific imports - matplotlib
-matplotlib = import_module('matplotlib',
-    __import__kwargs={'fromlist':['pyplot', 'cm', 'collections']},
-    min_module_version='1.0.0', catch=(RuntimeError,))
-if matplotlib:
-    plt = matplotlib.pyplot
-    cm = matplotlib.cm
-    LineCollection = matplotlib.collections.LineCollection
-    mpl_toolkits = import_module('mpl_toolkits',
-            __import__kwargs={'fromlist':['mplot3d']})
-    Axes3D = mpl_toolkits.mplot3d.Axes3D
-    art3d = mpl_toolkits.mplot3d.art3d
-    ListedColormap = matplotlib.colors.ListedColormap
+# N.B.
+# When changing the minimum module version for matplotlib, please change
+# the same in the `SymPyDocTestFinder`` in `sympy/utilities/runtests.py`
 
 # Backend specific imports - textplot
 from sympy.plotting.textplot import textplot
 
 # Global variable
-# Set to False when running tests / doctests so that the plots don't
-# show.
-_show=True
+# Set to False when running tests / doctests so that the plots don't show.
+_show = True
+
 
 def unset_show():
     global _show
-    _show=False
+    _show = False
 
 ##############################################################################
 # The public interface
 ##############################################################################
 
+
 class Plot(object):
     """The central class of the plotting module.
 
@@ -181,7 +172,7 @@ class Plot(object):
         self.backend = DefaultBackend
 
         # The keyword arguments should only contain options for the plot.
-        for key, val in kwargs.iteritems():
+        for key, val in kwargs.items():
             if hasattr(self, key):
                 setattr(self, key, val)
 
@@ -200,14 +191,14 @@ class Plot(object):
 
     def __str__(self):
         series_strs = [('[%d]: ' % i) + str(s)
-                          for i, s in enumerate(self._series)]
+                       for i, s in enumerate(self._series)]
         return 'Plot object containing:\n' + '\n'.join(series_strs)
 
     def __getitem__(self, index):
         return self._series[index]
 
     def __setitem__(self, index, *args):
-        if len(args)==1 and isinstance(args[0], BaseSeries):
+        if len(args) == 1 and isinstance(args[0], BaseSeries):
             self._series[index] = args
 
     def __delitem__(self, index):
@@ -331,8 +322,9 @@ class Line2DBaseSeries(BaseSeries):
         self.line_color = None
 
     def get_segments(self):
+        np = import_module('numpy')
         points = self.get_points()
-        if self.steps == True:
+        if self.steps is True:
             x = np.array((points[0], points[0])).T.flatten()[1:]
             y = np.array((points[1], points[1])).T.flatten()[:-1]
             points = (x, y)
@@ -340,6 +332,7 @@ class Line2DBaseSeries(BaseSeries):
         return np.ma.concatenate([points[:-1], points[1:]], axis=1)
 
     def get_color_array(self):
+        np = import_module('numpy')
         c = self.line_color
         if hasattr(c, '__call__'):
             f = np.vectorize(c)
@@ -348,12 +341,12 @@ class Line2DBaseSeries(BaseSeries):
                 x = self.get_parameter_points()
                 return f(centers_of_segments(x))
             else:
-                variables = map(centers_of_segments, self.get_points())
+                variables = list(map(centers_of_segments, self.get_points()))
                 if arity == 1:
                     return f(variables[0])
                 elif arity == 2:
                     return f(*variables[:2])
-                else: # only if the line is 3D (otherwise raises an error)
+                else:  # only if the line is 3D (otherwise raises an error)
                     return f(*variables)
         else:
             return c*np.ones(self.nb_of_points)
@@ -363,6 +356,7 @@ class List2DSeries(Line2DBaseSeries):
     """Representation for a line consisting of list of points."""
 
     def __init__(self, list_x, list_y):
+        np = import_module('numpy')
         super(List2DSeries, self).__init__()
         self.list_x = np.array(list_x)
         self.list_y = np.array(list_y)
@@ -390,12 +384,9 @@ class LineOver1DRangeSeries(Line2DBaseSeries):
         self.depth = kwargs.get('depth', 12)
         self.line_color = kwargs.get('line_color', None)
 
-
     def __str__(self):
         return 'cartesian line: %s for %s over %s' % (
-                str(self.expr),
-                str(self.var),
-                str((self.start, self.end)))
+            str(self.expr), str(self.var), str((self.start, self.end)))
 
     def get_segments(self):
         """
@@ -416,12 +407,14 @@ class LineOver1DRangeSeries(Line2DBaseSeries):
         else:
             f = lambdify([self.var], self.expr)
             list_segments = []
+
             def sample(p, q, depth):
                 """ Samples recursively if three points are almost collinear.
                 For depth < 6, points are added irrespective of whether they
                 satisfy the collinearity condition or not. The maximum depth
                 allowed is 12.
                 """
+                np = import_module('numpy')
                 #Randomly sample to avoid aliasing.
                 random = 0.45 + np.random.rand() * 0.1
                 xnew = p[0] + random * (q[0] - p[0])
@@ -443,16 +436,17 @@ class LineOver1DRangeSeries(Line2DBaseSeries):
                 #sample those points further.
                 elif p[1] is None and q[1] is None:
                     xarray = np.linspace(p[0], q[0], 10)
-                    yarray = map(f, xarray)
+                    yarray = list(map(f, xarray))
                     if any(y is not None for y in yarray):
-                        for i in len(yarray) - 1:
-                            if yarray[i] is None or yarray[i + 1] is None:
+                        for i in range(len(yarray) - 1):
+                            if yarray[i] is not None or yarray[i + 1] is not None:
                                 sample([xarray[i], yarray[i]],
                                     [xarray[i + 1], yarray[i + 1]], depth + 1)
 
                 #Sample further if one of the end points in None( i.e. a complex
                 #value) or the three points are not almost collinear.
-                elif p[1] is None or q[1] is None or not flat(p, new_point, q):
+                elif (p[1] is None or q[1] is None or new_point[1] is None
+                        or not flat(p, new_point, q)):
                     sample(p, new_point, depth + 1)
                     sample(new_point, q, depth + 1)
                 else:
@@ -464,9 +458,10 @@ class LineOver1DRangeSeries(Line2DBaseSeries):
             return list_segments
 
     def get_points(self):
-        if self.only_integers == True:
+        np = import_module('numpy')
+        if self.only_integers is True:
             list_x = np.linspace(int(self.start), int(self.end),
-                    num=int(self.end)-int(self.start)+1)
+                    num=int(self.end) - int(self.start) + 1)
         else:
             list_x = np.linspace(self.start, self.end, num=self.nb_of_points)
         f = vectorized_lambdify([self.var], self.expr)
@@ -495,12 +490,11 @@ class Parametric2DLineSeries(Line2DBaseSeries):
 
     def __str__(self):
         return 'parametric cartesian line: (%s, %s) for %s over %s' % (
-                str(self.expr_x),
-                str(self.expr_y),
-                str(self.var),
-                str((self.start, self.end)))
+            str(self.expr_x), str(self.expr_y), str(self.var),
+            str((self.start, self.end)))
 
     def get_parameter_points(self):
+        np = import_module('numpy')
         return np.linspace(self.start, self.end, num=self.nb_of_points)
 
     def get_points(self):
@@ -531,6 +525,7 @@ class Parametric2DLineSeries(Line2DBaseSeries):
         f_x = lambdify([self.var], self.expr_x)
         f_y = lambdify([self.var], self.expr_y)
         list_segments = []
+
         def sample(param_p, param_q, p, q, depth):
             """ Samples recursively if three points are almost collinear.
             For depth < 6, points are added irrespective of whether they
@@ -538,6 +533,7 @@ class Parametric2DLineSeries(Line2DBaseSeries):
             allowed is 12.
             """
             #Randomly sample to avoid aliasing.
+            np = import_module('numpy')
             random = 0.45 + np.random.rand() * 0.1
             param_new = param_p + random * (param_q - param_p)
             xnew = f_x(param_new)
@@ -560,17 +556,17 @@ class Parametric2DLineSeries(Line2DBaseSeries):
             elif ((p[0] is None and q[1] is None) or
                     (p[1] is None and q[1] is None)):
                 param_array = np.linspace(param_p, param_q, 10)
-                x_array = map(f_x, param_array)
-                y_array = map(f_y, param_array)
+                x_array = list(map(f_x, param_array))
+                y_array = list(map(f_y, param_array))
                 if any(x is not None and y is not None
                         for x, y in zip(x_array, y_array)):
-                    for i in len(y_array) - 1:
+                    for i in range(len(y_array) - 1):
                         if ((x_array[i] is not None and y_array[i] is not None) or
-                                (x_array[i+1] is not None and y_array[i] is not None)):
+                                (x_array[i + 1] is not None and y_array[i + 1] is not None)):
                             point_a = [x_array[i], y_array[i]]
                             point_b = [x_array[i + 1], y_array[i + 1]]
                             sample(param_array[i], param_array[i], point_a,
-                                    point_b, depth + 1)
+                                   point_b, depth + 1)
 
             #Sample further if one of the end points in None( ie a complex
             #value) or the three points are not almost collinear.
@@ -624,13 +620,11 @@ class Parametric3DLineSeries(Line3DBaseSeries):
 
     def __str__(self):
         return '3D parametric cartesian line: (%s, %s, %s) for %s over %s' % (
-                str(self.expr_x),
-                str(self.expr_y),
-                str(self.expr_z),
-                str(self.var),
-                str((self.start, self.end)))
+            str(self.expr_x), str(self.expr_y), str(self.expr_z),
+            str(self.var), str((self.start, self.end)))
 
     def get_parameter_points(self):
+        np = import_module('numpy')
         return np.linspace(self.start, self.end, num=self.nb_of_points)
 
     def get_points(self):
@@ -655,17 +649,18 @@ class SurfaceBaseSeries(BaseSeries):
         self.surface_color = None
 
     def get_color_array(self):
+        np = import_module('numpy')
         c = self.surface_color
-        if callable(c):
+        if isinstance(c, Callable):
             f = np.vectorize(c)
             arity = len(getargspec(c)[0])
             if self.is_parametric:
-                variables = map(centers_of_faces, self.get_parameter_meshes())
+                variables = list(map(centers_of_faces, self.get_parameter_meshes()))
                 if arity == 1:
                     return f(variables[0])
                 elif arity == 2:
                     return f(*variables)
-            variables = map(centers_of_faces, self.get_meshes())
+            variables = list(map(centers_of_faces, self.get_meshes()))
             if arity == 1:
                 return f(variables[0])
             elif arity == 2:
@@ -695,13 +690,14 @@ class SurfaceOver2DRangeSeries(SurfaceBaseSeries):
     def __str__(self):
         return ('cartesian surface: %s for'
                 ' %s over %s and %s over %s') % (
-                str(self.expr),
-                str(self.var_x),
-                str((self.start_x, self.end_x)),
-                str(self.var_y),
-                str((self.start_y, self.end_y)))
+                    str(self.expr),
+                    str(self.var_x),
+                    str((self.start_x, self.end_x)),
+                    str(self.var_y),
+                    str((self.start_y, self.end_y)))
 
     def get_meshes(self):
+        np = import_module('numpy')
         mesh_x, mesh_y = np.meshgrid(np.linspace(self.start_x, self.end_x,
                                                  num=self.nb_of_points_x),
                                      np.linspace(self.start_y, self.end_y,
@@ -716,8 +712,9 @@ class ParametricSurfaceSeries(SurfaceBaseSeries):
 
     is_parametric = True
 
-    def __init__(self, expr_x, expr_y, expr_z, var_start_end_u, var_start_end_v,
-                    **kwargs):
+    def __init__(
+        self, expr_x, expr_y, expr_z, var_start_end_u, var_start_end_v,
+            **kwargs):
         super(ParametricSurfaceSeries, self).__init__()
         self.expr_x = sympify(expr_x)
         self.expr_y = sympify(expr_y)
@@ -735,15 +732,16 @@ class ParametricSurfaceSeries(SurfaceBaseSeries):
     def __str__(self):
         return ('parametric cartesian surface: (%s, %s, %s) for'
                 ' %s over %s and %s over %s') % (
-                str(self.expr_x),
-                str(self.expr_y),
-                str(self.expr_z),
-                str(self.var_u),
-                str((self.start_u, self.end_u)),
-                str(self.var_v),
-                str((self.start_v, self.end_v)))
+                    str(self.expr_x),
+                    str(self.expr_y),
+                    str(self.expr_z),
+                    str(self.var_u),
+                    str((self.start_u, self.end_u)),
+                    str(self.var_v),
+                    str((self.start_v, self.end_v)))
 
     def get_parameter_meshes(self):
+        np = import_module('numpy')
         return np.meshgrid(np.linspace(self.start_u, self.end_u,
                                        num=self.nb_of_points_u),
                            np.linspace(self.start_v, self.end_v,
@@ -783,13 +781,14 @@ class ContourSeries(BaseSeries):
     def __str__(self):
         return ('contour: %s for '
                 '%s over %s and %s over %s') % (
-                str(self.expr),
-                str(self.var_x),
-                str((self.start_x, self.end_x)),
-                str(self.var_y),
-                str((self.start_y, self.end_y)))
+                    str(self.expr),
+                    str(self.var_x),
+                    str((self.start_x, self.end_x)),
+                    str(self.var_y),
+                    str((self.start_y, self.end_y)))
 
     def get_meshes(self):
+        np = import_module('numpy')
         mesh_x, mesh_y = np.meshgrid(np.linspace(self.start_x, self.end_x,
                                                  num=self.nb_of_points_x),
                                      np.linspace(self.start_y, self.end_y,
@@ -808,14 +807,22 @@ class BaseBackend(object):
         self.parent = parent
 
 
+## don't have to check for the success of importing matplotlib in each case;
+## we will only be using this backend if we can successfully import matploblib
 class MatplotlibBackend(BaseBackend):
     def __init__(self, parent):
         super(MatplotlibBackend, self).__init__(parent)
         are_3D = [s.is_3D for s in self.parent._series]
+        self.matplotlib = import_module('matplotlib',
+            __import__kwargs={'fromlist': ['pyplot', 'cm', 'collections']},
+            min_module_version='1.1.0', catch=(RuntimeError,))
+        self.plt = self.matplotlib.pyplot
+        self.cm = self.matplotlib.cm
+        self.LineCollection = self.matplotlib.collections.LineCollection
         if any(are_3D) and not all(are_3D):
             raise ValueError('The matplotlib backend can not mix 2D and 3D.')
         elif not any(are_3D):
-            self.fig = plt.figure()
+            self.fig = self.plt.figure()
             self.ax = self.fig.add_subplot(111)
             self.ax.spines['left'].set_position('zero')
             self.ax.spines['right'].set_color('none')
@@ -826,7 +833,11 @@ class MatplotlibBackend(BaseBackend):
             self.ax.xaxis.set_ticks_position('bottom')
             self.ax.yaxis.set_ticks_position('left')
         elif all(are_3D):
-            self.fig = plt.figure()
+            ## mpl_toolkits.mplot3d is necessary for
+            ##      projection='3d'
+            mpl_toolkits = import_module('mpl_toolkits',
+                                     __import__kwargs={'fromlist': ['mplot3d']})
+            self.fig = self.plt.figure()
             self.ax = self.fig.add_subplot(111, projection='3d')
 
     def process_series(self):
@@ -835,12 +846,15 @@ class MatplotlibBackend(BaseBackend):
         for s in self.parent._series:
             # Create the collections
             if s.is_2Dline:
-                collection = LineCollection(s.get_segments())
+                collection = self.LineCollection(s.get_segments())
                 self.ax.add_collection(collection)
             elif s.is_contour:
                 self.ax.contour(*s.get_meshes())
             elif s.is_3Dline:
                 # TODO too complicated, I blame matplotlib
+                mpl_toolkits = import_module('mpl_toolkits',
+                    __import__kwargs={'fromlist': ['mplot3d']})
+                art3d = mpl_toolkits.mplot3d.art3d
                 collection = art3d.Line3DCollection(s.get_segments())
                 self.ax.add_collection(collection)
                 x, y, z = s.get_points()
@@ -849,7 +863,7 @@ class MatplotlibBackend(BaseBackend):
                 self.ax.set_zlim((min(z), max(z)))
             elif s.is_3Dsurface:
                 x, y, z = s.get_meshes()
-                collection = self.ax.plot_surface(x, y, z, cmap=cm.jet,
+                collection = self.ax.plot_surface(x, y, z, cmap=self.cm.jet,
                                                   rstride=1, cstride=1,
                                                   linewidth=0.1)
             elif s.is_implicit:
@@ -865,6 +879,7 @@ class MatplotlibBackend(BaseBackend):
                     # use contourf or contour depending on whether it is
                     # an inequality or equality.
                     #XXX: ``contour`` plots multiple lines. Should be fixed.
+                    ListedColormap = self.matplotlib.colors.ListedColormap
                     colormap = ListedColormap(["white", "blue"])
                     xarray, yarray, zarray, plot_type = points
                     if plot_type == 'contour':
@@ -882,15 +897,15 @@ class MatplotlibBackend(BaseBackend):
             if hasattr(s, 'label'):
                 collection.set_label(s.label)
             if s.is_line and s.line_color:
-                if isinstance(s.line_color, (float, int)) or callable(s.line_color):
+                if isinstance(s.line_color, (float, int)) or isinstance(s.line_color, Callable):
                     color_array = s.get_color_array()
                     collection.set_array(color_array)
                 else:
                     collection.set_color(s.line_color)
             if s.is_3Dsurface and s.surface_color:
-                if matplotlib.__version__ < "1.2.0": #TODO in the distant future remove this check
+                if self.matplotlib.__version__ < "1.2.0":  # TODO in the distant future remove this check
                     warnings.warn('The version of matplotlib is too old to use surface coloring.')
-                elif isinstance(s.surface_color, (float, int)) or callable(s.surface_color):
+                elif isinstance(s.surface_color, (float, int)) or isinstance(s.surface_color, Callable):
                     color_array = s.get_color_array()
                     color_array = color_array.reshape(color_array.size)
                     collection.set_array(color_array)
@@ -900,15 +915,19 @@ class MatplotlibBackend(BaseBackend):
         # Set global options.
         # TODO The 3D stuff
         # XXX The order of those is important.
+
+        mpl_toolkits = import_module('mpl_toolkits',
+            __import__kwargs={'fromlist': ['mplot3d']})
+        Axes3D = mpl_toolkits.mplot3d.Axes3D
         if parent.xscale and not isinstance(self.ax, Axes3D):
             self.ax.set_xscale(parent.xscale)
-        if parent.yscale and  not isinstance(self.ax, Axes3D):
+        if parent.yscale and not isinstance(self.ax, Axes3D):
             self.ax.set_yscale(parent.yscale)
         if parent.xlim:
             self.ax.set_xlim(parent.xlim)
         if parent.ylim:
             self.ax.set_ylim(parent.ylim)
-        if not isinstance(self.ax, Axes3D) or matplotlib.__version__ >= '1.2.0': #XXX in the distant future remove this check
+        if not isinstance(self.ax, Axes3D) or self.matplotlib.__version__ >= '1.2.0':  # XXX in the distant future remove this check
             self.ax.set_autoscale_on(parent.autoscale)
         if parent.axis_center:
             val = parent.axis_center
@@ -948,14 +967,14 @@ class MatplotlibBackend(BaseBackend):
         # you can uncomment the next line and remove the pyplot.show() call
         #self.fig.show()
         if _show:
-            plt.show()
+            self.plt.show()
 
     def save(self, path):
         self.process_series()
         self.fig.savefig(path)
 
     def close(self):
-        plt.close(self.fig)
+        self.plt.close(self.fig)
 
 
 class TextBackend(BaseBackend):
@@ -964,9 +983,11 @@ class TextBackend(BaseBackend):
 
     def show(self):
         if len(self.parent._series) != 1:
-            raise ValueError('The TextBackend supports only one graph per Plot.')
+            raise ValueError(
+                'The TextBackend supports only one graph per Plot.')
         elif not isinstance(self.parent._series[0], LineOver1DRangeSeries):
-            raise ValueError('The TextBackend supports only expressions over a 1D range')
+            raise ValueError(
+                'The TextBackend supports only expressions over a 1D range')
         else:
             ser = self.parent._series[0]
             textplot(ser.expr, ser.start, ser.end)
@@ -977,6 +998,7 @@ class TextBackend(BaseBackend):
 
 class DefaultBackend(BaseBackend):
     def __new__(cls, parent):
+        matplotlib = import_module('matplotlib', min_module_version='1.1.0', catch=(RuntimeError,))
         if matplotlib:
             return MatplotlibBackend(parent)
         else:
@@ -984,10 +1006,10 @@ class DefaultBackend(BaseBackend):
 
 
 plot_backends = {
-        'matplotlib' : MatplotlibBackend,
-        'text' : TextBackend,
-        'default': DefaultBackend
-        }
+    'matplotlib': MatplotlibBackend,
+    'text': TextBackend,
+    'default': DefaultBackend
+}
 
 
 ##############################################################################
@@ -995,17 +1017,22 @@ plot_backends = {
 ##############################################################################
 
 def centers_of_segments(array):
+    np = import_module('numpy')
     return np.average(np.vstack((array[:-1], array[1:])), 0)
 
+
 def centers_of_faces(array):
+    np = import_module('numpy')
     return np.average(np.dstack((array[:-1, :-1],
-                                 array[1: , :-1],
+                                 array[1:, :-1],
                                  array[:-1, 1: ],
                                  array[:-1, :-1],
                                  )), 2)
 
+
 def flat(x, y, z, eps=1e-3):
     """Checks whether three points are almost collinear"""
+    np = import_module('numpy')
     vector_a = x - y
     vector_b = z - y
     dot_product = np.dot(vector_a, vector_b)
@@ -1014,6 +1041,7 @@ def flat(x, y, z, eps=1e-3):
     cos_theta = dot_product / (vector_a_norm * vector_b_norm)
     return abs(cos_theta + 1) < eps
 
+
 def _matplotlib_list(interval_list):
     """
     Returns lists for matplotlib ``fill`` command from a list of bounding
@@ -1026,9 +1054,9 @@ def _matplotlib_list(interval_list):
             intervalx = intervals[0]
             intervaly = intervals[1]
             xlist.extend([intervalx.start, intervalx.start,
-                            intervalx.end, intervalx.end, None])
+                          intervalx.end, intervalx.end, None])
             ylist.extend([intervaly.start, intervaly.end,
-                            intervaly.end, intervaly.start, None])
+                          intervaly.end, intervaly.start, None])
     else:
         #XXX Ugly hack. Matplotlib does not accept empty lists for ``fill``
         xlist.extend([None, None, None, None])
@@ -1042,9 +1070,12 @@ def _matplotlib_list(interval_list):
 # TODO: Add more plotting options for 3d plots.
 # TODO: Adaptive sampling for 3D plots.
 
+@doctest_depends_on(modules=('numpy', 'matplotlib',))
 def plot(*args, **kwargs):
     """
-    Plots a function of a single variable.
+    Plots a function of a single variable and returns an instance of
+    the ``Plot`` class (also, see the description of the
+    ``show`` keyword argument below).
 
     The plotting uses an adaptive algorithm which samples recursively to
     accurately plot the plot. The adaptive algorithm uses a random point near
@@ -1085,6 +1116,14 @@ def plot(*args, **kwargs):
     Keyword Arguments
     =================
 
+    Arguments for ``plot`` function:
+
+    ``show``: Boolean. The default value is set to ``True``. Set show to
+    ``False`` and the function will not display the plot. The returned
+    instance of the ``Plot`` class can then be used to save or display
+    the plot by calling the ``save()`` and ``show()`` methods
+    respectively.
+
     Arguments for ``LineOver1DRangeSeries`` class:
 
     ``adaptive``: Boolean. The default value is set to True. Set adaptive to False and
@@ -1134,19 +1173,31 @@ def plot(*args, **kwargs):
 
     Single Plot
 
-    >>> plot(x**2, (x, -5, 5))# doctest: +SKIP
+    >>> plot(x**2, (x, -5, 5))
+    Plot object containing:
+    [0]: cartesian line: x**2 for x over (-5.0, 5.0)
 
     Multiple plots with single range.
 
-    >>> plot(x, x**2, x**3, (x, -5, 5))# doctest: +SKIP
+    >>> plot(x, x**2, x**3, (x, -5, 5))
+    Plot object containing:
+    [0]: cartesian line: x for x over (-5.0, 5.0)
+    [1]: cartesian line: x**2 for x over (-5.0, 5.0)
+    [2]: cartesian line: x**3 for x over (-5.0, 5.0)
+
 
     Multiple plots with different ranges.
 
-    >>> plot((x**2, (x, -6, 6)), (x, (x, -5, 5)))# doctest: +SKIP
+    >>> plot((x**2, (x, -6, 6)), (x, (x, -5, 5)))
+    Plot object containing:
+    [0]: cartesian line: x**2 for x over (-6.0, 6.0)
+    [1]: cartesian line: x for x over (-5.0, 5.0)
 
     No adaptive sampling.
 
-    >>> plot(x**2, adaptive=False, nb_of_points=400)# doctest: +SKIP
+    >>> plot(x**2, adaptive=False, nb_of_points=400)
+    Plot object containing:
+    [0]: cartesian line: x**2 for x over (-10.0, 10.0)
 
     See Also
     ========
@@ -1154,7 +1205,7 @@ def plot(*args, **kwargs):
     Plot, LineOver1DRangeSeries.
 
     """
-    args = sympify([arg for arg in args])
+    args = list(map(sympify, args))
     show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 1, 1)
@@ -1166,6 +1217,7 @@ def plot(*args, **kwargs):
     return plots
 
 
+@doctest_depends_on(modules=('numpy', 'matplotlib',))
 def plot_parametric(*args, **kwargs):
     """
     Plots a 2D parametric plot.
@@ -1243,8 +1295,8 @@ def plot_parametric(*args, **kwargs):
 
     ``yscale``: {'linear', 'log'} Sets the scaling if the y - axis.
 
-    ``axis_center``: tuple of two floats denoting the coordinates of the center or
-    {'center', 'auto'}
+    ``axis_center``: tuple of two floats denoting the coordinates of the center
+    or {'center', 'auto'}
 
     ``xlim`` : tuple of two floats, denoting the x - axis limits.
 
@@ -1252,28 +1304,40 @@ def plot_parametric(*args, **kwargs):
 
     Examples
     ========
+
     >>> from sympy import symbols, cos, sin
     >>> from sympy.plotting import plot_parametric
     >>> u = symbols('u')
 
     Single Parametric plot
 
-    >>> plot_parametric(cos(u), sin(u), (u, -5, 5))# doctest: +SKIP
+    >>> plot_parametric(cos(u), sin(u), (u, -5, 5))
+    Plot object containing:
+    [0]: parametric cartesian line: (cos(u), sin(u)) for u over (-5.0, 5.0)
+
 
     Multiple parametric plot with single range.
 
-    >>> plot_parametric((cos(u), sin(u)), (u, cos(u)))# doctest: +SKIP
+    >>> plot_parametric((cos(u), sin(u)), (u, cos(u)))
+    Plot object containing:
+    [0]: parametric cartesian line: (cos(u), sin(u)) for u over (-10.0, 10.0)
+    [1]: parametric cartesian line: (u, cos(u)) for u over (-10.0, 10.0)
 
     Multiple parametric plots.
 
-    >>> plot_parametric((cos(u), sin(u), (u, -5, 5)), (cos(u), u, (u, -5, 5)))# doctest: +SKIP
+    >>> plot_parametric((cos(u), sin(u), (u, -5, 5)),
+    ...     (cos(u), u, (u, -5, 5)))
+    Plot object containing:
+    [0]: parametric cartesian line: (cos(u), sin(u)) for u over (-5.0, 5.0)
+    [1]: parametric cartesian line: (cos(u), u) for u over (-5.0, 5.0)
+
 
     See Also
     ========
     Plot, Parametric2DLineSeries
 
     """
-    args = sympify([arg for arg in args])
+    args = list(map(sympify, args))
     show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 2, 1)
@@ -1283,6 +1347,8 @@ def plot_parametric(*args, **kwargs):
         plots.show()
     return plots
 
+
+@doctest_depends_on(modules=('numpy', 'matplotlib',))
 def plot3d_parametric_line(*args, **kwargs):
     """
     Plots a 3D parametric line plot.
@@ -1347,11 +1413,19 @@ def plot3d_parametric_line(*args, **kwargs):
 
     Single plot.
 
-    >>> plot3d_parametric_line(cos(u), sin(u), u, (u, -5, 5))# doctest: +SKIP
+    >>> plot3d_parametric_line(cos(u), sin(u), u, (u, -5, 5))
+    Plot object containing:
+    [0]: 3D parametric cartesian line: (cos(u), sin(u), u) for u over (-5.0, 5.0)
+
 
     Multiple plots.
 
-    >>> plot3d_parametric_line((cos(u), sin(u), u, (u, -5, 5)), (sin(u), u**2, u, (u, -5, 5)))# doctest: +SKIP
+    >>> plot3d_parametric_line((cos(u), sin(u), u, (u, -5, 5)),
+    ...     (sin(u), u**2, u, (u, -5, 5)))
+    Plot object containing:
+    [0]: 3D parametric cartesian line: (cos(u), sin(u), u) for u over (-5.0, 5.0)
+    [1]: 3D parametric cartesian line: (sin(u), u**2, u) for u over (-5.0, 5.0)
+
 
     See Also
     ========
@@ -1359,7 +1433,7 @@ def plot3d_parametric_line(*args, **kwargs):
     Plot, Parametric3DLineSeries
 
     """
-    args = sympify([arg for arg in args])
+    args = list(map(sympify, args))
     show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 3, 1)
@@ -1369,6 +1443,8 @@ def plot3d_parametric_line(*args, **kwargs):
         plots.show()
     return plots
 
+
+@doctest_depends_on(modules=('numpy', 'matplotlib',))
 def plot3d(*args, **kwargs):
     """
     Plots a 3D surface plot.
@@ -1441,15 +1517,27 @@ def plot3d(*args, **kwargs):
 
     Single plot
 
-    >>> plot3d(x*y, (x, -5, 5), (y, -5, 5)) # doctest: +SKIP
+    >>> plot3d(x*y, (x, -5, 5), (y, -5, 5))
+    Plot object containing:
+    [0]: cartesian surface: x*y for x over (-5.0, 5.0) and y over (-5.0, 5.0)
+
 
     Multiple plots with same range
 
-    >>> plot3d(x*y, -x*y, (x, -5, 5), (y, -5, 5))# doctest: +SKIP
+    >>> plot3d(x*y, -x*y, (x, -5, 5), (y, -5, 5))
+    Plot object containing:
+    [0]: cartesian surface: x*y for x over (-5.0, 5.0) and y over (-5.0, 5.0)
+    [1]: cartesian surface: -x*y for x over (-5.0, 5.0) and y over (-5.0, 5.0)
+
 
     Multiple plots with different ranges.
 
-    >>> plot3d((x**2 + y**2, (x, -5, 5), (y, -5, 5)), (x*y, (x, -3, 3), (y, -3, 3)))# doctest: +SKIP
+    >>> plot3d((x**2 + y**2, (x, -5, 5), (y, -5, 5)),
+    ...     (x*y, (x, -3, 3), (y, -3, 3)))
+    Plot object containing:
+    [0]: cartesian surface: x**2 + y**2 for x over (-5.0, 5.0) and y over (-5.0, 5.0)
+    [1]: cartesian surface: x*y for x over (-3.0, 3.0) and y over (-3.0, 3.0)
+
 
     See Also
     ========
@@ -1457,7 +1545,7 @@ def plot3d(*args, **kwargs):
 
     """
 
-    args = sympify([arg for arg in args])
+    args = list(map(sympify, args))
     show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 1, 2)
@@ -1468,6 +1556,7 @@ def plot3d(*args, **kwargs):
     return plots
 
 
+@doctest_depends_on(modules=('numpy', 'matplotlib',))
 def plot3d_parametric_surface(*args, **kwargs):
     """
     Plots a 3D parametric surface plot.
@@ -1539,7 +1628,11 @@ def plot3d_parametric_surface(*args, **kwargs):
 
     Single plot.
 
-    >>> plot3d_parametric_surface(cos(u + v), sin(u - v), u - v, (u, -5, 5), (v, -5, 5)) # doctest: +SKIP
+    >>> plot3d_parametric_surface(cos(u + v), sin(u - v), u - v,
+    ...     (u, -5, 5), (v, -5, 5))
+    Plot object containing:
+    [0]: parametric cartesian surface: (cos(u + v), sin(u - v), u - v) for u over (-5.0, 5.0) and v over (-5.0, 5.0)
+
 
     See Also
     ========
@@ -1547,7 +1640,7 @@ def plot3d_parametric_surface(*args, **kwargs):
 
     """
 
-    args = sympify([arg for arg in args])
+    args = list(map(sympify, args))
     show = kwargs.pop('show', True)
     series = []
     plot_expr = check_arguments(args, 3, 2)
@@ -1557,6 +1650,7 @@ def plot3d_parametric_surface(*args, **kwargs):
         plots.show()
     return plots
 
+
 def check_arguments(args, expr_len, nb_of_free_symbols):
     """
     Checks the arguments and converts into tuples of the
@@ -1583,7 +1677,7 @@ def check_arguments(args, expr_len, nb_of_free_symbols):
             i = len(args) + 1
 
         exprs = Tuple(*args[:i])
-        free_symbols = list(set_union(*[e.free_symbols for e in exprs]))
+        free_symbols = list(set.union(*[e.free_symbols for e in exprs]))
         if len(args) == expr_len + nb_of_free_symbols:
             #Ranges given
             plots = [exprs + Tuple(*args[expr_len:])]
@@ -1599,8 +1693,8 @@ def check_arguments(args, expr_len, nb_of_free_symbols):
         return plots
 
     if isinstance(args[0], Expr) or (isinstance(args[0], Tuple) and
-                                        len(args[0]) == expr_len and
-                                        expr_len != 3):
+                                     len(args[0]) == expr_len and
+                                     expr_len != 3):
         # Cannot handle expressions with number of expression = 3. It is
         # not possible to differentiate between expressions and ranges.
         #Series of plots with same range
@@ -1614,14 +1708,15 @@ def check_arguments(args, expr_len, nb_of_free_symbols):
 
         exprs = args[:i]
         assert all(isinstance(e, Expr) for expr in exprs for e in expr)
-        free_symbols = list(set_union(*[e.free_symbols for expr in exprs
+        free_symbols = list(set.union(*[e.free_symbols for expr in exprs
                                         for e in expr]))
 
         if len(free_symbols) > nb_of_free_symbols:
-            raise ValueError("The number of free_symbols in the expression"
-                                "is greater than %d" % nb_of_free_symbols)
+            raise ValueError("The number of free_symbols in the expression "
+                             "is greater than %d" % nb_of_free_symbols)
         if len(args) == i + nb_of_free_symbols and isinstance(args[i], Tuple):
-            ranges = Tuple(*[range_expr for range_expr in args[i:i + nb_of_free_symbols]])
+            ranges = Tuple(*[range_expr for range_expr in args[
+                           i:i + nb_of_free_symbols]])
             plots = [expr + ranges for expr in exprs]
             return plots
         else:
@@ -1643,9 +1738,9 @@ def check_arguments(args, expr_len, nb_of_free_symbols):
             for i in range(expr_len):
                 if not isinstance(arg[i], Expr):
                     raise ValueError("Expected an expression, given %s" %
-                                        str(arg[i]))
+                                     str(arg[i]))
             for i in range(nb_of_free_symbols):
                 if not len(arg[i + expr_len]) == 3:
-                    raise ValueError("The ranges should be a tuple of"
+                    raise ValueError("The ranges should be a tuple of "
                                      "length 3, got %s" % str(arg[i + expr_len]))
         return args
