@@ -1,7 +1,8 @@
+from __future__ import print_function, division
+
 from sympy import Basic, Symbol, symbols, lambdify
 from util import interpolate, rinterpolate, create_bounds, update_bounds
-
-from sympy.core.compatibility import callable
+from sympy.core.compatibility import xrange
 
 
 class ColorGradient(object):
@@ -131,7 +132,7 @@ class ColorScheme(object):
         if gargs:
             try:
                 gradient = ColorGradient(*gargs)
-            except Exception, ex:
+            except Exception as ex:
                 raise ValueError(("Could not initialize a gradient "
                                   "with arguments %s. Inner "
                                   "exception: %s") % (gargs, str(ex)))
@@ -171,7 +172,7 @@ class ColorScheme(object):
         # when vars are given explicitly, any vars
         # not given are marked 'unbound' as to not
         # be accidentally used in an expression
-        vars = [Symbol('unbound%i' % (i)) for i in xrange(1, 6)]
+        vars = [Symbol('unbound%i' % (i)) for i in range(1, 6)]
         # interpret as t
         if len(args) == 1:
             vars[3] = args[0]
@@ -214,18 +215,18 @@ class ColorScheme(object):
         try:
             result = self.f(0, 0, 0, 0, 0)
             assert len(result) == 3
-        except TypeError, te:
+        except TypeError as te:
             raise ValueError("Color function needs to accept x,y,z,u,v, "
                              "as arguments even if it doesn't use all of them.")
-        except AssertionError, ae:
+        except AssertionError as ae:
             raise ValueError("Color function needs to return 3-tuple r,g,b.")
-        except Exception, ie:
+        except Exception as ie:
             pass  # color function probably not valid at 0,0,0,0,0
 
     def __call__(self, x, y, z, u, v):
         try:
             return self.f(x, y, z, u, v)
-        except Exception, e:
+        except Exception as e:
             #print e
             return None
 
@@ -257,7 +258,7 @@ class ColorScheme(object):
         # scale and apply gradient
         for _u in xrange(len(u_set)):
             if cverts[_u] is not None:
-                for _c in xrange(3):
+                for _c in range(3):
                     # scale from [f_min, f_max] to [0,1]
                     cverts[_u][_c] = rinterpolate(bounds[_c][0], bounds[_c][1],
                                                   cverts[_u][_c])
@@ -300,7 +301,7 @@ class ColorScheme(object):
             for _v in xrange(len(v_set)):
                 if cverts[_u][_v] is not None:
                     # scale from [f_min, f_max] to [0,1]
-                    for _c in xrange(3):
+                    for _c in range(3):
                         cverts[_u][_v][_c] = rinterpolate(bounds[_c][0],
                                              bounds[_c][1], cverts[_u][_v][_c])
                     # apply gradient
