@@ -17,14 +17,16 @@ non-implemented methods. They may also supply new implementations of the
 convenience methods, for example if there are faster algorithms available.
 """
 
+from __future__ import print_function, division
+
 from copy import copy
 
 from sympy.polys.polyerrors import CoercionFailed
-from sympy.polys.monomialtools import ProductOrder, monomial_key
-from sympy.polys.domains import Field
+from sympy.polys.orderings import ProductOrder, monomial_key
+from sympy.polys.domains.field import Field
 from sympy.polys.agca.ideals import Ideal
 
-from sympy.core.compatibility import iterable
+from sympy.core.compatibility import iterable, reduce
 
 # TODO
 # - module saturation
@@ -46,7 +48,7 @@ class Module(object):
 
     >>> from sympy import QQ
     >>> from sympy.abc import x
-    >>> QQ[x].free_module(2)
+    >>> QQ.old_poly_ring(x).free_module(2)
     QQ[x]**2
 
     Attributes:
@@ -110,7 +112,7 @@ class Module(object):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> F.subset([(1, x), (x, 2)])
         True
         >>> F.subset([(1/x, x), (x, 2)])
@@ -306,7 +308,7 @@ class FreeModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> M = F.submodule([2, x])
         >>> F.is_submodule(F)
         True
@@ -330,7 +332,7 @@ class FreeModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> F.convert([1, 0])
         [1, 0]
         """
@@ -360,9 +362,9 @@ class FreeModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> QQ[x].free_module(0).is_zero()
+        >>> QQ.old_poly_ring(x).free_module(0).is_zero()
         True
-        >>> QQ[x].free_module(1).is_zero()
+        >>> QQ.old_poly_ring(x).free_module(1).is_zero()
         False
         """
         return self.rank == 0
@@ -373,7 +375,7 @@ class FreeModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> QQ[x].free_module(3).basis()
+        >>> QQ.old_poly_ring(x).free_module(3).basis()
         ([1, 0, 0], [0, 1, 0], [0, 0, 1])
         """
         from sympy.matrices import eye
@@ -386,13 +388,13 @@ class FreeModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> M = QQ[x].free_module(2)
+        >>> M = QQ.old_poly_ring(x).free_module(2)
         >>> M.quotient_module(M.submodule([1, x], [x, 2]))
         QQ[x]**2/<[1, x], [x, 2]>
 
         Or more conicisely, using the overloaded division operator:
 
-        >>> QQ[x].free_module(2) / [[1, x], [x, 2]]
+        >>> QQ.old_poly_ring(x).free_module(2) / [[1, x], [x, 2]]
         QQ[x]**2/<[1, x], [x, 2]>
         """
         return QuotientModule(self.ring, self, submodule)
@@ -403,8 +405,8 @@ class FreeModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> I = QQ[x].ideal(x)
-        >>> F = QQ[x].free_module(2)
+        >>> I = QQ.old_poly_ring(x).ideal(x)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> F.multiply_ideal(I)
         <[x, 0], [0, x]>
         """
@@ -416,7 +418,7 @@ class FreeModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> QQ[x].free_module(2).identity_hom()
+        >>> QQ.old_poly_ring(x).free_module(2).identity_hom()
         Matrix([
         [1, 0], : QQ[x]**2 -> QQ[x]**2
         [0, 1]])
@@ -433,7 +435,7 @@ class FreeModulePolyRing(FreeModule):
 
     >>> from sympy.abc import x
     >>> from sympy import QQ
-    >>> F = QQ[x].free_module(3)
+    >>> F = QQ.old_poly_ring(x).free_module(3)
     >>> F
     QQ[x]**3
     >>> F.contains([x, 1, 0])
@@ -458,7 +460,7 @@ class FreeModulePolyRing(FreeModule):
 
         >>> from sympy.abc import x, y
         >>> from sympy import QQ
-        >>> M = QQ[x, y].free_module(2).submodule([x, x + y])
+        >>> M = QQ.old_poly_ring(x, y).free_module(2).submodule([x, x + y])
         >>> M
         <[x, x + y]>
         >>> M.contains([2*x, 2*x + 2*y])
@@ -477,7 +479,7 @@ class FreeModuleQuotientRing(FreeModule):
 
     >>> from sympy.abc import x
     >>> from sympy import QQ
-    >>> F = (QQ[x]/[x**2 + 1]).free_module(3)
+    >>> F = (QQ.old_poly_ring(x)/[x**2 + 1]).free_module(3)
     >>> F
     (QQ[x]/<x**2 + 1>)**3
 
@@ -487,7 +489,7 @@ class FreeModuleQuotientRing(FreeModule):
     """
 
     def __init__(self, ring, rank):
-        from sympy.polys.domains import QuotientRing
+        from sympy.polys.domains.quotientring import QuotientRing
         FreeModule.__init__(self, ring, rank)
         if not isinstance(ring, QuotientRing):
             raise NotImplementedError('This implementation only works over '
@@ -504,7 +506,7 @@ class FreeModuleQuotientRing(FreeModule):
 
         >>> from sympy.abc import x, y
         >>> from sympy import QQ
-        >>> M = (QQ[x, y]/[x**2 - y**2]).free_module(2).submodule([x, x + y])
+        >>> M = (QQ.old_poly_ring(x, y)/[x**2 - y**2]).free_module(2).submodule([x, x + y])
         >>> M
         <[x + <x**2 - y**2>, x + y + <x**2 - y**2>]>
         >>> M.contains([y**2, x**2 + x*y])
@@ -523,7 +525,7 @@ class FreeModuleQuotientRing(FreeModule):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = (QQ[x]/[x**2 + 1]).free_module(2)
+        >>> F = (QQ.old_poly_ring(x)/[x**2 + 1]).free_module(2)
         >>> e = F.convert([1, 0])
         >>> e
         [1 + <x**2 + 1>, 0 + <x**2 + 1>]
@@ -544,7 +546,7 @@ class FreeModuleQuotientRing(FreeModule):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = (QQ[x]/[x**2 + 1]).free_module(2)
+        >>> F = (QQ.old_poly_ring(x)/[x**2 + 1]).free_module(2)
         >>> e = F.convert([1, 0])
         >>> l = F.lift(e)
         >>> e == l
@@ -614,7 +616,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> M = QQ[x].free_module(2).submodule([1, x])
+        >>> M = QQ.old_poly_ring(x).free_module(2).submodule([1, x])
         >>> M.convert([2, 2*x])
         [2, 2*x]
         """
@@ -642,7 +644,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x, y
         >>> from sympy import QQ
-        >>> F = QQ[x, y].free_module(2)
+        >>> F = QQ.old_poly_ring(x, y).free_module(2)
         >>> F.submodule([x, x]).intersect(F.submodule([y, y]))
         <[x*y, x*y]>
 
@@ -677,7 +679,7 @@ class SubModule(Module):
 
         >>> from sympy import QQ
         >>> from sympy.abc import x, y
-        >>> F = QQ[x, y].free_module(2)
+        >>> F = QQ.old_poly_ring(x, y).free_module(2)
         >>> S = F.submodule([x*y, x*y])
         >>> T = F.submodule([x, x])
         >>> S.module_quotient(T)
@@ -711,7 +713,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(1)
+        >>> F = QQ.old_poly_ring(x).free_module(1)
         >>> M = F.submodule([x**2 + x]) # <x(x+1)>
         >>> N = F.submodule([x**2 - 1]) # <(x-1)(x+1)>
         >>> M.union(N) == F.submodule([x+1])
@@ -730,7 +732,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> F.submodule([x, 1]).is_zero()
         False
         >>> F.submodule([0, 0]).is_zero()
@@ -744,7 +746,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> M = QQ[x].free_module(2).submodule([x, 1])
+        >>> M = QQ.old_poly_ring(x).free_module(2).submodule([x, 1])
         >>> M.submodule([x**2, x])
         <[x**2, x]>
         """
@@ -758,7 +760,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> F.submodule([x, 1]).is_full_module()
         False
         >>> F.submodule([1, 1], [1, 2]).is_full_module()
@@ -772,7 +774,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> M = F.submodule([2, x])
         >>> N = M.submodule([2*x, x**2])
         >>> M.is_submodule(M)
@@ -803,13 +805,13 @@ class SubModule(Module):
 
         >>> from sympy.abc import x, y
         >>> from sympy import QQ
-        >>> QQ[x].free_module(2).submodule([1, 0], [1, 1]).syzygy_module().is_zero()
+        >>> QQ.old_poly_ring(x).free_module(2).submodule([1, 0], [1, 1]).syzygy_module().is_zero()
         True
 
         A slightly more interesting example:
 
-        >>> M = QQ[x, y].free_module(2).submodule([x, 2*x], [y, 2*y])
-        >>> S = QQ[x, y].free_module(2).submodule([y, -x])
+        >>> M = QQ.old_poly_ring(x, y).free_module(2).submodule([x, 2*x], [y, 2*y])
+        >>> S = QQ.old_poly_ring(x, y).free_module(2).submodule([y, -x])
         >>> M.syzygy_module() == S
         True
         """
@@ -826,7 +828,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> M = F.submodule([1, 0], [1, 1])
         >>> M.in_terms_of_generators([x, x**2])
         [-x**2 + x, x**2]
@@ -855,7 +857,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> S1 = F.submodule([x, 1])
         >>> S2 = F.submodule([x**2, x])
         >>> S1.quotient_module(S2)
@@ -882,8 +884,8 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> I = QQ[x].ideal(x**2)
-        >>> M = QQ[x].free_module(2).submodule([1, 1])
+        >>> I = QQ.old_poly_ring(x).ideal(x**2)
+        >>> M = QQ.old_poly_ring(x).free_module(2).submodule([1, 1])
         >>> I*M
         <[x**2, x**2]>
         """
@@ -897,7 +899,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> QQ[x].free_module(2).submodule([x, x]).inclusion_hom()
+        >>> QQ.old_poly_ring(x).free_module(2).submodule([x, x]).inclusion_hom()
         Matrix([
         [1, 0], : <[x, x]> -> QQ[x]**2
         [0, 1]])
@@ -910,7 +912,7 @@ class SubModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> QQ[x].free_module(2).submodule([x, x]).identity_hom()
+        >>> QQ.old_poly_ring(x).free_module(2).submodule([x, x]).identity_hom()
         Matrix([
         [1, 0], : <[x, x]> -> <[x, x]>
         [0, 1]])
@@ -930,7 +932,7 @@ class SubQuotientModule(SubModule):
 
     >>> from sympy.abc import x
     >>> from sympy import QQ
-    >>> F = QQ[x].free_module(2)
+    >>> F = QQ.old_poly_ring(x).free_module(2)
     >>> S = F.submodule([1, 0], [1, x])
     >>> Q = F/[(1, 0)]
     >>> S/[(1, 0)] == Q.submodule([5, x])
@@ -975,7 +977,7 @@ class SubQuotientModule(SubModule):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> F.submodule([x, 1]).is_full_module()
         False
         >>> F.submodule([1, 1], [1, 2]).is_full_module()
@@ -991,7 +993,7 @@ class SubQuotientModule(SubModule):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> M = (QQ[x].free_module(2) / [(1, x)]).submodule([1, 0])
+        >>> M = (QQ.old_poly_ring(x).free_module(2) / [(1, x)]).submodule([1, 0])
         >>> M.quotient_hom()
         Matrix([
         [1, 0], : <[1, 0], [1, x]> -> <[1, 0] + <[1, x]>, [1, x] + <[1, x]>>
@@ -1022,7 +1024,7 @@ class SubModulePolyRing(SubModule):
 
     >>> from sympy.abc import x, y
     >>> from sympy import QQ
-    >>> F = QQ[x, y].free_module(2)
+    >>> F = QQ.old_poly_ring(x, y).free_module(2)
     >>> F.submodule([x, y], [1, 0])
     <[x, y], [1, 0]>
 
@@ -1102,7 +1104,7 @@ class SubModulePolyRing(SubModule):
             newgens.append(Rkr.convert(m))
         # Note: we need *descending* order on module index, and TOP=False to
         #       get an eliminetaion order
-        F = Rkr.submodule(*newgens, **{'order': 'ilex', 'TOP': False})
+        F = Rkr.submodule(*newgens, order='ilex', TOP=False)
 
         # Second bullet point: standard basis of F
         G = F._groebner_vec()
@@ -1122,7 +1124,7 @@ class SubModulePolyRing(SubModule):
             order="ilex", TOP=False)  # We want decreasing order!
         G = S._groebner_vec()
         # This list cannot not be empty since e is an element
-        e = list(filter(lambda x: self.ring.is_unit(x[0]), G))[0]
+        e = [x for x in G if self.ring.is_unit(x[0])][0]
         return [-x/e[0] for x in e[1:]]
 
     def reduce_element(self, x, NF=None):
@@ -1176,7 +1178,7 @@ class SubModulePolyRing(SubModule):
             gi = [list(x) + [0] for x in self.gens]
             # NOTE: We *need* to use an elimination order
             M = self.ring.free_module(self.rank + 1).submodule(*([g1] + gi),
-                                            **{'order': 'ilex', 'TOP': False})
+                                            order='ilex', TOP=False)
             if not relations:
                 return self.ring.ideal(*[x[-1] for x in M._groebner_vec() if
                                          all(y == self.ring.zero for y in x[:-1])])
@@ -1201,7 +1203,7 @@ class SubModuleQuotientRing(SubModule):
 
     >>> from sympy.abc import x, y
     >>> from sympy import QQ
-    >>> M = (QQ[x, y]/[x**2 - y**2]).free_module(2).submodule([x, x + y])
+    >>> M = (QQ.old_poly_ring(x, y)/[x**2 - y**2]).free_module(2).submodule([x, x + y])
     >>> M
     <[x + <x**2 - y**2>, x + y + <x**2 - y**2>]>
     >>> M.contains([y**2, x**2 + x*y])
@@ -1283,7 +1285,7 @@ class QuotientModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2)
+        >>> F = QQ.old_poly_ring(x).free_module(2)
         >>> (F/[(1, 0)]).is_zero()
         False
         >>> (F/[(1, 0), (0, 1)]).is_zero()
@@ -1297,7 +1299,7 @@ class QuotientModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> Q = QQ[x].free_module(2) / [(x, x)]
+        >>> Q = QQ.old_poly_ring(x).free_module(2) / [(x, x)]
         >>> S = Q.submodule([1, 0])
         >>> Q.is_submodule(S)
         True
@@ -1320,7 +1322,7 @@ class QuotientModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> Q = QQ[x].free_module(2) / [(x, x)]
+        >>> Q = QQ.old_poly_ring(x).free_module(2) / [(x, x)]
         >>> Q.submodule([x, 0])
         <[x, 0] + <[x, x]>>
         """
@@ -1335,7 +1337,7 @@ class QuotientModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> F = QQ[x].free_module(2) / [(1, 2), (1, x)]
+        >>> F = QQ.old_poly_ring(x).free_module(2) / [(1, 2), (1, x)]
         >>> F.convert([1, 0])
         [1, 0] + <[1, 2], [1, x]>
         """
@@ -1353,7 +1355,7 @@ class QuotientModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> M = QQ[x].free_module(2) / [(1, 2), (1, x)]
+        >>> M = QQ.old_poly_ring(x).free_module(2) / [(1, 2), (1, x)]
         >>> M.identity_hom()
         Matrix([
         [1, 0], : QQ[x]**2/<[1, 2], [1, x]> -> QQ[x]**2/<[1, 2], [1, x]>
@@ -1371,7 +1373,7 @@ class QuotientModule(Module):
 
         >>> from sympy.abc import x
         >>> from sympy import QQ
-        >>> M = QQ[x].free_module(2) / [(1, 2), (1, x)]
+        >>> M = QQ.old_poly_ring(x).free_module(2) / [(1, 2), (1, x)]
         >>> M.quotient_hom()
         Matrix([
         [1, 0], : QQ[x]**2 -> QQ[x]**2/<[1, 2], [1, x]>
