@@ -1,5 +1,7 @@
 """Euclidean algorithms, GCDs, LCMs and polynomial remainder sequences. """
 
+from __future__ import print_function, division
+
 from sympy.polys.densebasic import (
     dup_strip, dmp_raise,
     dmp_zero, dmp_one, dmp_ground,
@@ -48,10 +50,10 @@ from sympy.polys.polyerrors import (
 
 from sympy.polys.polyconfig import query
 
-from sympy.utilities import cythonized
-
 from sympy.ntheory import nextprime
 
+from sympy.core.compatibility import xrange
+
 
 def dup_half_gcdex(f, g, K):
     """
@@ -72,7 +74,7 @@ def dup_half_gcdex(f, g, K):
     (-1/5*x + 3/5, x + 1)
 
     """
-    if not (K.has_Field or not K.is_Exact):
+    if not K.has_Field:
         raise DomainError("can't compute half extended GCD over %s" % K)
 
     a, b = [K.one], []
@@ -311,7 +313,6 @@ def dmp_primitive_prs(f, g, u, K):
         raise MultivariatePolynomialError(f, g)
 
 
-@cythonized("n,m,d,k")
 def dup_inner_subresultants(f, g, K):
     """
     Subresultant PRS algorithm in `K[x]`.
@@ -395,7 +396,6 @@ def dup_subresultants(f, g, K):
     return dup_inner_subresultants(f, g, K)[0]
 
 
-@cythonized("s,i,du,dv,dw")
 def dup_prs_resultant(f, g, K):
     """
     Resultant algorithm in `K[x]` using subresultant PRS.
@@ -442,7 +442,6 @@ def dup_prs_resultant(f, g, K):
     i = dup_degree(R[-2])
 
     res = dup_LC(R[-1], K)**i
-
     res = K.quo(res*p, q)
 
     return res, R
@@ -467,7 +466,6 @@ def dup_resultant(f, g, K, includePRS=False):
     return dup_prs_resultant(f, g, K)[0]
 
 
-@cythonized("u,v,n,m,d,k")
 def dmp_inner_subresultants(f, g, u, K):
     """
     Subresultant PRS algorithm in `K[X]`.
@@ -546,7 +544,6 @@ def dmp_inner_subresultants(f, g, u, K):
     return R, B, D
 
 
-@cythonized("u")
 def dmp_subresultants(f, g, u, K):
     """
     Computes subresultant PRS of two polynomials in `K[X]`.
@@ -570,7 +567,6 @@ def dmp_subresultants(f, g, u, K):
     return dmp_inner_subresultants(f, g, u, K)[0]
 
 
-@cythonized("u,v,s,i,d,du,dv,dw")
 def dmp_prs_resultant(f, g, u, K):
     """
     Resultant algorithm in `K[X]` using subresultant PRS.
@@ -642,7 +638,6 @@ def dmp_prs_resultant(f, g, u, K):
     return res, R
 
 
-@cythonized("u,v,n,m,N,M,B")
 def dmp_zz_modular_resultant(f, g, p, u, K):
     """
     Compute resultant of `f` and `g` modulo a prime `p`.
@@ -721,7 +716,6 @@ def _collins_crt(r, R, P, p, K):
     return gf_int(gf_crt([r, R], [P, p], K), P*p)
 
 
-@cythonized("u,v,n,m")
 def dmp_zz_collins_resultant(f, g, u, K):
     """
     Collins's modular resultant algorithm in `Z[X]`.
@@ -781,7 +775,6 @@ def dmp_zz_collins_resultant(f, g, u, K):
     return r
 
 
-@cythonized("u,n,m")
 def dmp_qq_collins_resultant(f, g, u, K0):
     """
     Collins's modular resultant algorithm in `Q[X]`.
@@ -821,7 +814,6 @@ def dmp_qq_collins_resultant(f, g, u, K0):
     return dmp_quo_ground(r, c, u - 1, K0)
 
 
-@cythonized("u")
 def dmp_resultant(f, g, u, K, includePRS=False):
     """
     Computes resultant of two polynomials in `K[X]`.
@@ -855,7 +847,6 @@ def dmp_resultant(f, g, u, K, includePRS=False):
     return dmp_prs_resultant(f, g, u, K)[0]
 
 
-@cythonized("d,s")
 def dup_discriminant(f, K):
     """
     Computes discriminant of a polynomial in `K[x]`.
@@ -883,7 +874,6 @@ def dup_discriminant(f, K):
         return K.quo(r, c*K(s))
 
 
-@cythonized("u,v,d,s")
 def dmp_discriminant(f, u, K):
     """
     Computes discriminant of a polynomial in `K[X]`.
@@ -945,7 +935,6 @@ def _dup_ff_trivial_gcd(f, g, K):
         return None
 
 
-@cythonized("u")
 def _dmp_rr_trivial_gcd(f, g, u, K):
     """Handle trivial cases in GCD algorithm over a ring. """
     zero_f = dmp_zero_p(f, u)
@@ -969,7 +958,6 @@ def _dmp_rr_trivial_gcd(f, g, u, K):
         return None
 
 
-@cythonized("u")
 def _dmp_ff_trivial_gcd(f, g, u, K):
     """Handle trivial cases in GCD algorithm over a field. """
     zero_f = dmp_zero_p(f, u)
@@ -991,7 +979,6 @@ def _dmp_ff_trivial_gcd(f, g, u, K):
         return None
 
 
-@cythonized("u,v,df,dg")
 def _dmp_simplify_gcd(f, g, u, K):
     """Try to eliminate `x_0` from GCD computation in `K[X]`. """
     df = dmp_degree(f, u)
@@ -1092,7 +1079,6 @@ def dup_ff_prs_gcd(f, g, K):
     return h, cff, cfg
 
 
-@cythonized("u")
 def dmp_rr_prs_gcd(f, g, u, K):
     """
     Computes polynomial GCD using subresultants over a ring.
@@ -1139,7 +1125,6 @@ def dmp_rr_prs_gcd(f, g, u, K):
     return h, cff, cfg
 
 
-@cythonized("u")
 def dmp_ff_prs_gcd(f, g, u, K):
     """
     Computes polynomial GCD using subresultants over a field.
@@ -1202,7 +1187,6 @@ def _dup_zz_gcd_interpolate(h, x, K):
     return f
 
 
-@cythonized("i,df,dg")
 def dup_zz_heu_gcd(f, g, K):
     """
     Heuristic polynomial GCD in `Z[x]`.
@@ -1309,7 +1293,6 @@ def dup_zz_heu_gcd(f, g, K):
     raise HeuristicGCDFailed('no luck')
 
 
-@cythonized("v")
 def _dmp_zz_gcd_interpolate(h, x, v, K):
     """Interpolate polynomial GCD from integer GCD. """
     f = []
@@ -1327,7 +1310,6 @@ def _dmp_zz_gcd_interpolate(h, x, v, K):
         return f
 
 
-@cythonized("u,v,i,dg,df")
 def dmp_zz_heu_gcd(f, g, u, K):
     """
     Heuristic polynomial GCD in `Z[X]`.
@@ -1484,7 +1466,6 @@ def dup_qq_heu_gcd(f, g, K0):
     return h, cff, cfg
 
 
-@cythonized("u")
 def dmp_qq_heu_gcd(f, g, u, K0):
     """
     Heuristic polynomial GCD in `Q[X]`.
@@ -1551,7 +1532,23 @@ def dup_inner_gcd(f, g, K):
     (x - 1, x + 1, x - 2)
 
     """
-    if K.has_Field or not K.is_Exact:
+    if not K.is_Exact:
+        try:
+            exact = K.get_exact()
+        except DomainError:
+            return [K.one], f, g
+
+        f = dup_convert(f, K, exact)
+        g = dup_convert(g, K, exact)
+
+        h, cff, cfg = dup_inner_gcd(f, g, exact)
+
+        h = dup_convert(h, exact, K)
+        cff = dup_convert(cff, exact, K)
+        cfg = dup_convert(cfg, exact, K)
+
+        return h, cff, cfg
+    elif K.has_Field:
         if K.is_QQ and query('USE_HEU_GCD'):
             try:
                 return dup_qq_heu_gcd(f, g, K)
@@ -1569,10 +1566,25 @@ def dup_inner_gcd(f, g, K):
         return dup_rr_prs_gcd(f, g, K)
 
 
-@cythonized("u")
 def _dmp_inner_gcd(f, g, u, K):
     """Helper function for `dmp_inner_gcd()`. """
-    if K.has_Field or not K.is_Exact:
+    if not K.is_Exact:
+        try:
+            exact = K.get_exact()
+        except DomainError:
+            return dmp_one(u, K), f, g
+
+        f = dmp_convert(f, u, K, exact)
+        g = dmp_convert(g, u, K, exact)
+
+        h, cff, cfg = _dmp_inner_gcd(f, g, u, exact)
+
+        h = dmp_convert(h, u, exact, K)
+        cff = dmp_convert(cff, u, exact, K)
+        cfg = dmp_convert(cfg, u, exact, K)
+
+        return h, cff, cfg
+    elif K.has_Field:
         if K.is_QQ and query('USE_HEU_GCD'):
             try:
                 return dmp_qq_heu_gcd(f, g, u, K)
@@ -1590,7 +1602,6 @@ def _dmp_inner_gcd(f, g, u, K):
         return dmp_rr_prs_gcd(f, g, u, K)
 
 
-@cythonized("u")
 def dmp_inner_gcd(f, g, u, K):
     """
     Computes polynomial GCD and cofactors of `f` and `g` in `K[X]`.
@@ -1639,7 +1650,6 @@ def dup_gcd(f, g, K):
     return dup_inner_gcd(f, g, K)[0]
 
 
-@cythonized("u")
 def dmp_gcd(f, g, u, K):
     """
     Computes polynomial GCD of `f` and `g` in `K[X]`.
@@ -1722,13 +1732,12 @@ def dup_lcm(f, g, K):
     x**3 - 2*x**2 - x + 2
 
     """
-    if K.has_Field or not K.is_Exact:
+    if K.has_Field:
         return dup_ff_lcm(f, g, K)
     else:
         return dup_rr_lcm(f, g, K)
 
 
-@cythonized("u")
 def dmp_rr_lcm(f, g, u, K):
     """
     Computes polynomial LCM over a ring in `K[X]`.
@@ -1757,7 +1766,6 @@ def dmp_rr_lcm(f, g, u, K):
     return dmp_mul_ground(h, c, u, K)
 
 
-@cythonized("u")
 def dmp_ff_lcm(f, g, u, K):
     """
     Computes polynomial LCM over a field in `K[X]`.
@@ -1781,7 +1789,6 @@ def dmp_ff_lcm(f, g, u, K):
     return dmp_ground_monic(h, u, K)
 
 
-@cythonized("u")
 def dmp_lcm(f, g, u, K):
     """
     Computes polynomial LCM of `f` and `g` in `K[X]`.
@@ -1802,13 +1809,12 @@ def dmp_lcm(f, g, u, K):
     if not u:
         return dup_lcm(f, g, K)
 
-    if K.has_Field or not K.is_Exact:
+    if K.has_Field:
         return dmp_ff_lcm(f, g, u, K)
     else:
         return dmp_rr_lcm(f, g, u, K)
 
 
-@cythonized("u,v")
 def dmp_content(f, u, K):
     """
     Returns GCD of multivariate coefficients.
@@ -1840,7 +1846,6 @@ def dmp_content(f, u, K):
         return cont
 
 
-@cythonized("u,v")
 def dmp_primitive(f, u, K):
     """
     Returns multivariate content and a primitive polynomial.
@@ -1907,6 +1912,8 @@ def dmp_cancel(f, g, u, K, include=True):
     _, p, q = dmp_inner_gcd(f, g, u, K)
 
     if K0 is not None:
+        _, cp, cq = K.cofactors(cp, cq)
+
         p = dmp_convert(p, u, K, K0)
         q = dmp_convert(q, u, K, K0)
 
