@@ -1,8 +1,14 @@
 """OO layer for several polynomial representations. """
 
+from __future__ import print_function, division
+
+from sympy.core.sympify import CantSympify
+
 from sympy.polys.polyutils import PicklableWithSlots
 from sympy.polys.polyerrors import CoercionFailed, NotReversible
 
+from sympy import oo
+
 class GenericPoly(PicklableWithSlots):
     """Base class for low-level polynomial representations. """
 
@@ -130,10 +136,12 @@ from sympy.polys.polyerrors import (
     UnificationFailed,
     PolynomialError)
 
+
 def init_normal_DMP(rep, lev, dom):
     return DMP(dmp_normal(rep, lev, dom), dom, lev)
 
-class DMP(PicklableWithSlots):
+
+class DMP(PicklableWithSlots, CantSympify):
     """Dense Multivariate Polynomials over `K`. """
 
     __slots__ = ['rep', 'lev', 'dom', 'ring']
@@ -232,7 +240,7 @@ class DMP(PicklableWithSlots):
         """Convert ``f`` to a dict representation with SymPy coefficients. """
         rep = dmp_to_dict(f.rep, f.lev, f.dom, zero=zero)
 
-        for k, v in rep.iteritems():
+        for k, v in rep.items():
             rep[k] = f.dom.to_sympy(v)
 
         return rep
@@ -252,10 +260,10 @@ class DMP(PicklableWithSlots):
 
     @classmethod
     def from_monoms_coeffs(cls, monoms, coeffs, lev, dom, ring=None):
-        return DMP(dict(zip(monoms, coeffs)), dom, lev, ring)
+        return DMP(dict(list(zip(monoms, coeffs))), dom, lev, ring)
 
     def to_ring(f):
-        """Make the ground domain a field. """
+        """Make the ground domain a ring. """
         return f.convert(f.dom.get_ring())
 
     def to_field(f):
@@ -307,7 +315,7 @@ class DMP(PicklableWithSlots):
             if n < 0:
                 return [(0,)]
             else:
-                return [ (n-i,) for i, c in enumerate(f.rep) ]
+                return [ (n - i,) for i, c in enumerate(f.rep) ]
         else:
             raise PolynomialError('multivariate polynomials not supported')
 
@@ -319,7 +327,7 @@ class DMP(PicklableWithSlots):
             if n < 0:
                 return [((0,), f.dom.zero)]
             else:
-                return [ ((n-i,), c) for i, c in enumerate(f.rep) ]
+                return [ ((n - i,), c) for i, c in enumerate(f.rep) ]
         else:
             raise PolynomialError('multivariate polynomials not supported')
 
@@ -340,7 +348,7 @@ class DMP(PicklableWithSlots):
     def eject(f, dom, front=False):
         """Eject selected generators into the ground domain. """
         F = dmp_eject(f.rep, f.lev, dom, front=front)
-        return f.__class__(F, dom, f.lev - len(dom.gens))
+        return f.__class__(F, dom, f.lev - len(dom.symbols))
 
     def exclude(f):
         r"""
@@ -410,7 +418,7 @@ class DMP(PicklableWithSlots):
         return f.per(dmp_abs(f.rep, f.lev, f.dom))
 
     def neg(f):
-        """Negate all cefficients in ``f``. """
+        """Negate all coefficients in ``f``. """
         return f.per(dmp_neg(f.rep, f.lev, f.dom))
 
     def add(f, g):
@@ -437,7 +445,7 @@ class DMP(PicklableWithSlots):
         if isinstance(n, int):
             return f.per(dmp_pow(f.rep, n, f.lev, f.dom))
         else:
-            raise TypeError("`int` expected, got %s" % type(n))
+            raise TypeError("``int`` expected, got %s" % type(n))
 
     def pdiv(f, g):
         """Polynomial pseudo-division of ``f`` and ``g``. """
@@ -486,11 +494,11 @@ class DMP(PicklableWithSlots):
         return res
 
     def degree(f, j=0):
-        """Returns the leading degree of `f` in `x_j`. """
+        """Returns the leading degree of ``f`` in ``x_j``. """
         if isinstance(j, int):
             return dmp_degree_in(f.rep, j, f.lev)
         else:
-            raise TypeError("`int` expected, got %s" % type(j))
+            raise TypeError("``int`` expected, got %s" % type(j))
 
     def degree_list(f):
         """Returns a list of degrees of ``f``. """
@@ -498,12 +506,31 @@ class DMP(PicklableWithSlots):
 
     def total_degree(f):
         """Returns the total degree of ``f``. """
-        return max([sum(m) for m in f.monoms()])
+        return max(sum(m) for m in f.monoms())
+
+    def homogenize(f, s):
+        """Return homogeneous polynomial of ``f``"""
+        td = f.total_degree()
+        result = {}
+        new_symbol = (s == len(f.terms()[0][0]))
+        for term in f.terms():
+            d = sum(term[0])
+            if d < td:
+                i = td - d
+            else:
+                i = 0
+            if new_symbol:
+                result[term[0] + (i,)] = term[1]
+            else:
+                l = list(term[0])
+                l[s] += i
+                result[tuple(l)] = term[1]
+        return DMP(result, f.dom, f.lev + int(new_symbol), f.ring)
 
     def homogeneous_order(f):
         """Returns the homogeneous order of ``f``. """
         if f.is_zero:
-            return -1
+            return -oo
 
         monoms = f.monoms()
         tdeg = sum(monoms[0])
@@ -545,29 +572,29 @@ class DMP(PicklableWithSlots):
         return coeff, f.per(F)
 
     def integrate(f, m=1, j=0):
-        """Computes indefinite integral of ``f``. """
+        """Computes the ``m``-th order indefinite integral of ``f`` in ``x_j``. """
         if not isinstance(m, int):
-            raise TypeError("`int` expected, got %s" % type(m))
+            raise TypeError("``int`` expected, got %s" % type(m))
 
         if not isinstance(j, int):
-            raise TypeError("`int` expected, got %s" % type(j))
+            raise TypeError("``int`` expected, got %s" % type(j))
 
         return f.per(dmp_integrate_in(f.rep, m, j, f.lev, f.dom))
 
     def diff(f, m=1, j=0):
-        """Computes `m`-th order derivative of `f` in `x_j`. """
+        """Computes the ``m``-th order derivative of ``f`` in ``x_j``. """
         if not isinstance(m, int):
-            raise TypeError("`int` expected, got %s" % type(m))
+            raise TypeError("``int`` expected, got %s" % type(m))
 
         if not isinstance(j, int):
-            raise TypeError("`int` expected, got %s" % type(j))
+            raise TypeError("``int`` expected, got %s" % type(j))
 
         return f.per(dmp_diff_in(f.rep, m, j, f.lev, f.dom))
 
     def eval(f, a, j=0):
-        """Evaluates `f` at the given point `a` in `x_j`. """
+        """Evaluates ``f`` at the given point ``a`` in ``x_j``. """
         if not isinstance(j, int):
-            raise TypeError("`int` expected, got %s" % type(j))
+            raise TypeError("``int`` expected, got %s" % type(j))
 
         return f.per(dmp_eval_in(f.rep,
             f.dom.convert(a), j, f.lev, f.dom), kill=True)
@@ -612,11 +639,14 @@ class DMP(PicklableWithSlots):
         """Computes subresultant PRS sequence of ``f`` and ``g``. """
         lev, dom, per, F, G = f.unify(g)
         R = dmp_subresultants(F, G, lev, dom)
-        return map(per, R)
+        return list(map(per, R))
 
-    def resultant(f, g):
+    def resultant(f, g, includePRS=False):
         """Computes resultant of ``f`` and ``g`` via PRS. """
         lev, dom, per, F, G = f.unify(g)
+        if includePRS:
+            res, R = dmp_resultant(F, G, lev, dom, includePRS=includePRS)
+            return per(res, kill=True), list(map(per, R))
         return per(dmp_resultant(F, G, lev, dom), kill=True)
 
     def discriminant(f):
@@ -660,7 +690,7 @@ class DMP(PicklableWithSlots):
         return f.per(dmp_ground_trunc(f.rep, f.dom.convert(p), f.lev, f.dom))
 
     def monic(f):
-        """Divides all coefficients by `LC(f)`. """
+        """Divides all coefficients by ``LC(f)``. """
         return f.per(dmp_ground_monic(f.rep, f.lev, f.dom))
 
     def content(f):
@@ -680,7 +710,7 @@ class DMP(PicklableWithSlots):
     def decompose(f):
         """Computes functional decomposition of ``f``. """
         if not f.lev:
-            return map(f.per, dup_decompose(f.rep, f.dom))
+            return list(map(f.per, dup_decompose(f.rep, f.dom)))
         else:
             raise ValueError('univariate polynomial expected')
 
@@ -694,7 +724,7 @@ class DMP(PicklableWithSlots):
     def sturm(f):
         """Computes the Sturm sequence of ``f``. """
         if not f.lev:
-            return map(f.per, dup_sturm(f.rep, f.dom))
+            return list(map(f.per, dup_sturm(f.rep, f.dom)))
         else:
             raise ValueError('univariate polynomial expected')
 
@@ -748,14 +778,21 @@ class DMP(PicklableWithSlots):
                 else:
                     return dup_isolate_all_roots_sqf(f.rep, f.dom, eps=eps, inf=inf, sup=sup, fast=fast)
         else:
-            raise PolynomialError("can't isolate roots of a multivariate polynomial")
+            raise PolynomialError(
+                "can't isolate roots of a multivariate polynomial")
 
     def refine_root(f, s, t, eps=None, steps=None, fast=False):
-        """Refine an isolating interval to the given precision. """
+        """
+        Refine an isolating interval to the given precision.
+
+        ``eps`` should be a rational number.
+
+        """
         if not f.lev:
             return dup_refine_real_root(f.rep, s, t, f.dom, eps=eps, steps=steps, fast=fast)
         else:
-            raise PolynomialError("can't refine a root of a multivariate polynomial")
+            raise PolynomialError(
+                "can't refine a root of a multivariate polynomial")
 
     def count_real_roots(f, inf=None, sup=None):
         """Return the number of real roots of ``f`` in ``[inf, sup]``. """
@@ -792,7 +829,7 @@ class DMP(PicklableWithSlots):
 
     @property
     def is_primitive(f):
-        """Returns ``True`` if GCD of coefficients of ``f`` is one. """
+        """Returns ``True`` if the GCD of the coefficients of ``f`` is one. """
         return f.dom.is_one(dmp_ground_content(f.rep, f.lev, f.dom))
 
     @property
@@ -961,8 +998,8 @@ class DMP(PicklableWithSlots):
 
     def _strict_eq(f, g):
         return isinstance(g, f.__class__) and f.lev == g.lev \
-                                          and f.dom == g.dom \
-                                          and f.rep == g.rep
+            and f.dom == g.dom \
+            and f.rep == g.rep
 
     def __lt__(f, g):
         _, _, _, F, G = f.unify(g)
@@ -983,11 +1020,15 @@ class DMP(PicklableWithSlots):
     def __nonzero__(f):
         return not dmp_zero_p(f.rep, f.lev)
 
+    __bool__ = __nonzero__
+
+
 def init_normal_DMF(num, den, lev, dom):
     return DMF(dmp_normal(num, lev, dom),
                dmp_normal(den, lev, dom), dom, lev)
 
-class DMF(PicklableWithSlots):
+
+class DMF(PicklableWithSlots, CantSympify):
     """Dense Multivariate Fractions over `K`. """
 
     __slots__ = ['num', 'den', 'lev', 'dom', 'ring']
@@ -1179,11 +1220,11 @@ class DMF(PicklableWithSlots):
         return cls.new(1, dom, lev, ring=ring)
 
     def numer(f):
-        """Returns numerator of ``f``. """
+        """Returns the numerator of ``f``. """
         return f.half_per(f.num)
 
     def denom(f):
-        """Returns denominator of ``f``. """
+        """Returns the denominator of ``f``. """
         return f.half_per(f.den)
 
     def cancel(f):
@@ -1191,7 +1232,7 @@ class DMF(PicklableWithSlots):
         return f.per(f.num, f.den)
 
     def neg(f):
-        """Negate all cefficients in ``f``. """
+        """Negate all coefficients in ``f``. """
         return f.per(dmp_neg(f.num, f.lev, f.dom), f.den, cancel=False)
 
     def add(f, g):
@@ -1244,7 +1285,7 @@ class DMF(PicklableWithSlots):
             return f.per(dmp_pow(f.num, n, f.lev, f.dom),
                          dmp_pow(f.den, n, f.lev, f.dom), cancel=False)
         else:
-            raise TypeError("`int` expected, got %s" % type(n))
+            raise TypeError("``int`` expected, got %s" % type(n))
 
     def quo(f, g):
         """Computes quotient of fractions ``f`` and ``g``. """
@@ -1282,7 +1323,7 @@ class DMF(PicklableWithSlots):
     def is_one(f):
         """Returns ``True`` if ``f`` is a unit fraction. """
         return dmp_one_p(f.num, f.lev, f.dom) and \
-               dmp_one_p(f.den, f.lev, f.dom)
+            dmp_one_p(f.den, f.lev, f.dom)
 
     def __neg__(f):
         return f.neg()
@@ -1426,11 +1467,15 @@ class DMF(PicklableWithSlots):
     def __nonzero__(f):
         return not dmp_zero_p(f.num, f.lev)
 
+    __bool__ = __nonzero__
+
+
 def init_normal_ANP(rep, mod, dom):
     return ANP(dup_normal(rep, dom),
                dup_normal(mod, dom), dom)
 
-class ANP(PicklableWithSlots):
+
+class ANP(PicklableWithSlots, CantSympify):
     """Dense Algebraic Number Polynomials over a field. """
 
     __slots__ = ['rep', 'mod', 'dom']
@@ -1504,7 +1549,7 @@ class ANP(PicklableWithSlots):
         """Convert ``f`` to a dict representation with SymPy coefficients. """
         rep = dmp_to_dict(f.rep, 0, f.dom)
 
-        for k, v in rep.iteritems():
+        for k, v in rep.items():
             rep[k] = f.dom.to_sympy(v)
 
         return rep
@@ -1527,7 +1572,7 @@ class ANP(PicklableWithSlots):
 
     @classmethod
     def from_list(cls, rep, mod, dom):
-        return ANP(dup_strip(map(dom.convert, rep)), mod, dom)
+        return ANP(dup_strip(list(map(dom.convert, rep))), mod, dom)
 
     def neg(f):
         return f.per(dup_neg(f.rep, f.dom))
@@ -1554,7 +1599,7 @@ class ANP(PicklableWithSlots):
 
             return f.per(dup_rem(dup_pow(F, n, f.dom), f.mod, f.dom))
         else:
-            raise TypeError("`int` expected, got %s" % type(n))
+            raise TypeError("``int`` expected, got %s" % type(n))
 
     def div(f, g):
         dom, per, F, G, mod = f.unify(g)
@@ -1602,7 +1647,7 @@ class ANP(PicklableWithSlots):
         else:
             try:
                 return f.add(f.per(g))
-            except TypeError:
+            except (CoercionFailed, TypeError):
                 return NotImplemented
 
     def __radd__(f, g):
@@ -1614,7 +1659,7 @@ class ANP(PicklableWithSlots):
         else:
             try:
                 return f.sub(f.per(g))
-            except TypeError:
+            except (CoercionFailed, TypeError):
                 return NotImplemented
 
     def __rsub__(f, g):
@@ -1626,7 +1671,7 @@ class ANP(PicklableWithSlots):
         else:
             try:
                 return f.mul(f.per(g))
-            except TypeError:
+            except (CoercionFailed, TypeError):
                 return NotImplemented
 
     def __rmul__(f, g):
@@ -1647,7 +1692,7 @@ class ANP(PicklableWithSlots):
         else:
             try:
                 return f.quo(f.per(g))
-            except TypeError:
+            except (CoercionFailed, TypeError):
                 return NotImplemented
 
     __truediv__ = __div__
@@ -1686,3 +1731,5 @@ class ANP(PicklableWithSlots):
 
     def __nonzero__(f):
         return bool(f.rep)
+
+    __bool__ = __nonzero__
