@@ -1,8 +1,8 @@
 from sympy import diff, Integral, Limit, sin, Symbol, Integer, Rational, cos, \
     tan, asin, acos, atan, sinh, cosh, tanh, asinh, acosh, atanh, E, I, oo, \
     pi, GoldenRatio, EulerGamma, Sum, Eq, Ne, Ge, Lt, Float
+from sympy.core.compatibility import u
 from sympy.printing.mathml import mathml, MathMLPrinter
-from xml.dom.minidom import parseString
 
 from sympy.utilities.pytest import raises
 
@@ -10,16 +10,18 @@ x = Symbol('x')
 y = Symbol('y')
 mp = MathMLPrinter()
 
+
 def test_printmethod():
-    assert mp.doprint(1+x) == '<apply><plus/><ci>x</ci><cn>1</cn></apply>'
+    assert mp.doprint(1 + x) == '<apply><plus/><ci>x</ci><cn>1</cn></apply>'
+
 
 def test_mathml_core():
-    mml_1 = mp._print(1+x)
+    mml_1 = mp._print(1 + x)
     assert mml_1.nodeName == 'apply'
     nodes = mml_1.childNodes
     assert len(nodes) == 3
     assert nodes[0].nodeName == 'plus'
-    assert nodes[0].hasChildNodes() == False
+    assert nodes[0].hasChildNodes() is False
     assert nodes[0].nodeValue is None
     assert nodes[1].nodeName in ['cn', 'ci']
     if nodes[1].nodeName == 'cn':
@@ -42,13 +44,14 @@ def test_mathml_core():
     assert nodes[1].childNodes[0].nodeValue == '2'
     assert nodes[2].childNodes[0].nodeValue == 'x'
 
-    mml = mp._print(Float(1.0,2)*x)
+    mml = mp._print(Float(1.0, 2)*x)
     assert mml.nodeName == 'apply'
     nodes = mml.childNodes
     assert nodes[0].nodeName == 'times'
     assert nodes[1].childNodes[0].nodeValue == '1.0'
     assert nodes[2].childNodes[0].nodeValue == 'x'
 
+
 def test_mathml_functions():
     mml_1 = mp._print(sin(x))
     assert mml_1.nodeName == 'apply'
@@ -59,7 +62,16 @@ def test_mathml_functions():
     assert mml_2.nodeName == 'apply'
     assert mml_2.childNodes[0].nodeName == 'diff'
     assert mml_2.childNodes[1].nodeName == 'bvar'
-    assert mml_2.childNodes[1].childNodes[0].nodeName == 'ci'  # below bvar there's <ci>x/ci>
+    assert mml_2.childNodes[1].childNodes[
+        0].nodeName == 'ci'  # below bvar there's <ci>x/ci>
+
+    mml_3 = mp._print(diff(cos(x*y), x, evaluate=False))
+    assert mml_3.nodeName == 'apply'
+    assert mml_3.childNodes[0].nodeName == 'partialdiff'
+    assert mml_3.childNodes[1].nodeName == 'bvar'
+    assert mml_3.childNodes[1].childNodes[
+        0].nodeName == 'ci'  # below bvar there's <ci>x/ci>
+
 
 def test_mathml_limits():
     # XXX No unevaluated limits
@@ -70,6 +82,7 @@ def test_mathml_limits():
     assert mml_1.childNodes[2].nodeName == 'lowlimit'
     assert mml_1.childNodes[3].toxml() == mp._print(lim_fun).toxml()
 
+
 def test_mathml_integrals():
     integrand = x
     mml_1 = mp._print(Integral(integrand, (x, 0, 1)))
@@ -79,6 +92,7 @@ def test_mathml_integrals():
     assert mml_1.childNodes[3].nodeName == 'uplimit'
     assert mml_1.childNodes[4].toxml() == mp._print(integrand).toxml()
 
+
 def test_mathml_sums():
     summand = x
     mml_1 = mp._print(Sum(summand, (x, 1, 10)))
@@ -88,6 +102,7 @@ def test_mathml_sums():
     assert mml_1.childNodes[3].nodeName == 'uplimit'
     assert mml_1.childNodes[4].toxml() == mp._print(summand).toxml()
 
+
 def test_mathml_tuples():
     mml_1 = mp._print([2])
     assert mml_1.nodeName == 'list'
@@ -100,8 +115,6 @@ def test_mathml_tuples():
     assert mml_2.childNodes[1].nodeName == 'cn'
     assert len(mml_2.childNodes) == 2
 
-def test_mathml_matrices():
-    pass #TODO
 
 def test_mathml_add():
     mml = mp._print(x**5 - x**4 + x)
@@ -109,14 +122,16 @@ def test_mathml_add():
     assert mml.childNodes[1].childNodes[0].nodeName == 'minus'
     assert mml.childNodes[1].childNodes[1].nodeName == 'apply'
 
+
 def test_mathml_Rational():
-    mml_1 = mp._print(Rational(1,1))
+    mml_1 = mp._print(Rational(1, 1))
     """should just return a number"""
     assert mml_1.nodeName == 'cn'
 
-    mml_2 = mp._print(Rational(2,5))
+    mml_2 = mp._print(Rational(2, 5))
     assert mml_2.childNodes[0].nodeName == 'divide'
 
+
 def test_mathml_constants():
     mml = mp._print(I)
     assert mml.nodeName == 'imaginaryi'
@@ -135,6 +150,7 @@ def test_mathml_constants():
     mml = mathml(EulerGamma)
     assert mml == '<eulergamma/>'
 
+
 def test_mathml_trig():
     mml = mp._print(sin(x))
     assert mml.childNodes[0].nodeName == 'sin'
@@ -172,8 +188,9 @@ def test_mathml_trig():
     mml = mp._print(acosh(x))
     assert mml.childNodes[0].nodeName == 'arccosh'
 
+
 def test_mathml_relational():
-    mml_1 = mp._print(Eq(x,1))
+    mml_1 = mp._print(Eq(x, 1))
     assert mml_1.nodeName == 'apply'
     assert mml_1.childNodes[0].nodeName == 'eq'
     assert mml_1.childNodes[1].nodeName == 'ci'
@@ -181,7 +198,7 @@ def test_mathml_relational():
     assert mml_1.childNodes[2].nodeName == 'cn'
     assert mml_1.childNodes[2].childNodes[0].nodeValue == '1'
 
-    mml_2 = mp._print(Ne(1,x))
+    mml_2 = mp._print(Ne(1, x))
     assert mml_2.nodeName == 'apply'
     assert mml_2.childNodes[0].nodeName == 'neq'
     assert mml_2.childNodes[1].nodeName == 'cn'
@@ -189,7 +206,7 @@ def test_mathml_relational():
     assert mml_2.childNodes[2].nodeName == 'ci'
     assert mml_2.childNodes[2].childNodes[0].nodeValue == 'x'
 
-    mml_3 = mp._print(Ge(1,x))
+    mml_3 = mp._print(Ge(1, x))
     assert mml_3.nodeName == 'apply'
     assert mml_3.childNodes[0].nodeName == 'geq'
     assert mml_3.childNodes[1].nodeName == 'cn'
@@ -197,7 +214,7 @@ def test_mathml_relational():
     assert mml_3.childNodes[2].nodeName == 'ci'
     assert mml_3.childNodes[2].childNodes[0].nodeValue == 'x'
 
-    mml_4 = mp._print(Lt(1,x))
+    mml_4 = mp._print(Lt(1, x))
     assert mml_4.nodeName == 'apply'
     assert mml_4.childNodes[0].nodeName == 'lt'
     assert mml_4.childNodes[1].nodeName == 'cn'
@@ -206,14 +223,6 @@ def test_mathml_relational():
     assert mml_4.childNodes[2].childNodes[0].nodeValue == 'x'
 
 
-def test_c2p():
-    """This tests some optional routines that depend on libxslt1 (which is optional)"""
-    try:
-        from sympy.modules.mathml import c2p
-        assert c2p(f.mathml) == result
-    except ImportError:
-        pass
-
 def test_symbol():
     mml = mp._print(Symbol("x"))
     assert mml.nodeName == 'ci'
@@ -276,11 +285,14 @@ def test_symbol():
     assert mml.childNodes[0].childNodes[0].childNodes[0].nodeValue == 'x'
     assert mml.childNodes[0].childNodes[1].nodeName == 'mml:mrow'
     assert mml.childNodes[0].childNodes[1].childNodes[0].nodeName == 'mml:mi'
-    assert mml.childNodes[0].childNodes[1].childNodes[0].childNodes[0].nodeValue == '2'
+    assert mml.childNodes[0].childNodes[1].childNodes[0].childNodes[
+        0].nodeValue == '2'
     assert mml.childNodes[0].childNodes[1].childNodes[1].nodeName == 'mml:mo'
-    assert mml.childNodes[0].childNodes[1].childNodes[1].childNodes[0].nodeValue == ' '
+    assert mml.childNodes[0].childNodes[1].childNodes[1].childNodes[
+        0].nodeValue == ' '
     assert mml.childNodes[0].childNodes[1].childNodes[2].nodeName == 'mml:mi'
-    assert mml.childNodes[0].childNodes[1].childNodes[2].childNodes[0].nodeValue == 'a'
+    assert mml.childNodes[0].childNodes[1].childNodes[2].childNodes[
+        0].nodeValue == 'a'
     del mml
 
     mml = mp._print(Symbol("x^2^a"))
@@ -290,11 +302,14 @@ def test_symbol():
     assert mml.childNodes[0].childNodes[0].childNodes[0].nodeValue == 'x'
     assert mml.childNodes[0].childNodes[1].nodeName == 'mml:mrow'
     assert mml.childNodes[0].childNodes[1].childNodes[0].nodeName == 'mml:mi'
-    assert mml.childNodes[0].childNodes[1].childNodes[0].childNodes[0].nodeValue == '2'
+    assert mml.childNodes[0].childNodes[1].childNodes[0].childNodes[
+        0].nodeValue == '2'
     assert mml.childNodes[0].childNodes[1].childNodes[1].nodeName == 'mml:mo'
-    assert mml.childNodes[0].childNodes[1].childNodes[1].childNodes[0].nodeValue == ' '
+    assert mml.childNodes[0].childNodes[1].childNodes[1].childNodes[
+        0].nodeValue == ' '
     assert mml.childNodes[0].childNodes[1].childNodes[2].nodeName == 'mml:mi'
-    assert mml.childNodes[0].childNodes[1].childNodes[2].childNodes[0].nodeValue == 'a'
+    assert mml.childNodes[0].childNodes[1].childNodes[2].childNodes[
+        0].nodeValue == 'a'
     del mml
 
     mml = mp._print(Symbol("x__2__a"))
@@ -304,17 +319,21 @@ def test_symbol():
     assert mml.childNodes[0].childNodes[0].childNodes[0].nodeValue == 'x'
     assert mml.childNodes[0].childNodes[1].nodeName == 'mml:mrow'
     assert mml.childNodes[0].childNodes[1].childNodes[0].nodeName == 'mml:mi'
-    assert mml.childNodes[0].childNodes[1].childNodes[0].childNodes[0].nodeValue == '2'
+    assert mml.childNodes[0].childNodes[1].childNodes[0].childNodes[
+        0].nodeValue == '2'
     assert mml.childNodes[0].childNodes[1].childNodes[1].nodeName == 'mml:mo'
-    assert mml.childNodes[0].childNodes[1].childNodes[1].childNodes[0].nodeValue == ' '
+    assert mml.childNodes[0].childNodes[1].childNodes[1].childNodes[
+        0].nodeValue == ' '
     assert mml.childNodes[0].childNodes[1].childNodes[2].nodeName == 'mml:mi'
-    assert mml.childNodes[0].childNodes[1].childNodes[2].childNodes[0].nodeValue == 'a'
+    assert mml.childNodes[0].childNodes[1].childNodes[2].childNodes[
+        0].nodeValue == 'a'
     del mml
 
+
 def test_mathml_greek():
     mml = mp._print(Symbol('alpha'))
     assert mml.nodeName == 'ci'
-    assert mml.childNodes[0].nodeValue == u'\u03b1'
+    assert mml.childNodes[0].nodeValue == u('\u03b1')
 
     assert mp.doprint(Symbol('alpha')) == '<ci>&#945;</ci>'
     assert mp.doprint(Symbol('beta')) == '<ci>&#946;</ci>'
@@ -333,7 +352,7 @@ def test_mathml_greek():
     assert mp.doprint(Symbol('omicron')) == '<ci>&#959;</ci>'
     assert mp.doprint(Symbol('pi')) == '<ci>&#960;</ci>'
     assert mp.doprint(Symbol('rho')) == '<ci>&#961;</ci>'
-    assert mp.doprint(Symbol('varsigma')) == '<ci>&#962;</ci>'
+    assert mp.doprint(Symbol('varsigma')) == '<ci>&#962;</ci>', mp.doprint(Symbol('varsigma'))
     assert mp.doprint(Symbol('sigma')) == '<ci>&#963;</ci>'
     assert mp.doprint(Symbol('tau')) == '<ci>&#964;</ci>'
     assert mp.doprint(Symbol('upsilon')) == '<ci>&#965;</ci>'
@@ -367,6 +386,7 @@ def test_mathml_greek():
     assert mp.doprint(Symbol('Psi')) == '<ci>&#936;</ci>'
     assert mp.doprint(Symbol('Omega')) == '<ci>&#937;</ci>'
 
+
 def test_mathml_order():
     expr = x**3 + x**2*y + 3*x*y**3 + y**4
 
@@ -392,13 +412,16 @@ def test_mathml_order():
     assert mml.childNodes[4].childNodes[1].childNodes[0].data == 'x'
     assert mml.childNodes[4].childNodes[2].childNodes[0].data == '3'
 
+
 def test_settings():
     raises(TypeError, lambda: mathml(Symbol("x"), method="garbage"))
 
+
 def test_toprettyxml_hooking():
     # test that the patch doesn't influence the behavior of the standard library
     import xml.dom.minidom
-    doc = xml.dom.minidom.parseString("<apply><plus/><ci>x</ci><cn>1</cn></apply>")
+    doc = xml.dom.minidom.parseString(
+        "<apply><plus/><ci>x</ci><cn>1</cn></apply>")
     prettyxml_old = doc.toprettyxml()
 
     mp.apply_patch()
