@@ -1,36 +1,41 @@
-from sympy.utilities.iterables import preorder_traversal
+from __future__ import print_function, division
+
 
 def pprint_nodes(subtrees):
     """
     Prettyprints systems of nodes.
 
-    Example:
+    Examples
+    ========
 
-    >> print pprint_nodes(["a", "b1\nb2", "c"])
+    >>> from sympy.printing.tree import pprint_nodes
+    >>> print(pprint_nodes(["a", "b1\\nb2", "c"]))
     +-a
     +-b1
     | b2
     +-c
-    >>
 
     """
-    def indent(s,type=1):
+    def indent(s, type=1):
         x = s.split("\n")
-        r = "+-%s\n"%x[0]
+        r = "+-%s\n" % x[0]
         for a in x[1:]:
-            if a=="": continue
-            if type==1:
-                r += "| %s\n"%a
+            if a == "":
+                continue
+            if type == 1:
+                r += "| %s\n" % a
             else:
-                r += "  %s\n"%a
+                r += "  %s\n" % a
         return r
-    if len(subtrees)==0: return ""
-    f="";
+    if len(subtrees) == 0:
+        return ""
+    f = ""
     for a in subtrees[:-1]:
         f += indent(a)
-    f += indent(subtrees[-1],2)
+    f += indent(subtrees[-1], 2)
     return f
 
+
 def print_node(node):
     """
     Returns an information about the "node".
@@ -43,6 +48,7 @@ def print_node(node):
             s += "%s: %s\n" % (a, node._assumptions[a])
     return s
 
+
 def tree(node):
     """
     Returns a tree representation of "node" as a string.
@@ -54,27 +60,30 @@ def tree(node):
     subtrees = []
     for arg in node.args:
         subtrees.append(tree(arg))
-    s = print_node(node)+pprint_nodes(subtrees)
+    s = print_node(node) + pprint_nodes(subtrees)
     return s
 
+
 def print_tree(node):
     """
     Prints a tree representation of "node".
 
-    In [1]: print_tree(x**2)
+    Examples
+    ========
+
+    >>> from sympy.printing import print_tree
+    >>> from sympy.abc import x
+    >>> print_tree(x**2) # doctest: +SKIP
     Pow: x**2
     +-Symbol: x
     | comparable: False
-    | noncommutative: False
-    | commutative: True
     +-Integer: 2
       real: True
+      nonzero: True
       comparable: True
       commutative: True
       infinitesimal: False
-      nonzero: True
       unbounded: False
-      noncommutative: False
       noninteger: False
       zero: False
       complex: True
@@ -84,8 +93,8 @@ def print_tree(node):
       imaginary: False
       finite: True
       irrational: False
-
+    <BLANKLINE>
 
     See also: tree()
     """
-    print tree(node)
+    print(tree(node))
