@@ -116,7 +116,8 @@ And check manually which line is wrong. Then go to the source code and
 debug this function to figure out the exact problem.
 
 """
-from sympy import SYMPY_DEBUG
+from __future__ import print_function, division
+
 from sympy.core import Basic, S, oo, Symbol, I, Dummy, Wild
 from sympy.functions import log, exp
 from sympy.series.order import Order
@@ -125,78 +126,11 @@ from sympy import cacheit
 
 from sympy.core.compatibility import reduce
 
-O = Order
-
-def debug(func):
-    """Only for debugging purposes: prints a tree
-
-    It will print a nice execution tree with arguments and results
-    of all decorated functions.
-    """
-    if not SYMPY_DEBUG:
-        #normal mode - do nothing
-        return func
-
-    #debug mode
-    def decorated(*args, **kwargs):
-        #r = func(*args, **kwargs)
-        r = maketree(func, *args, **kwargs)
-        #print "%s = %s(%s, %s)" % (r, func.__name__, args, kwargs)
-        return r
-
-    return decorated
-
 from sympy.utilities.timeutils import timethis
 timeit = timethis('gruntz')
 
-def tree(subtrees):
-    """Only debugging purposes: prints a tree"""
-    def indent(s, type=1):
-        x = s.split("\n")
-        r = "+-%s\n"%x[0]
-        for a in x[1:]:
-            if a == "":
-                continue
-            if type == 1:
-                r += "| %s\n"%a
-            else:
-                r += "  %s\n"%a
-        return r
-    if len(subtrees) == 0:
-        return ""
-    f = []
-    for a in subtrees[:-1]:
-        f.append(indent(a))
-    f.append(indent(subtrees[-1], 2))
-    return ''.join(f)
+from sympy.utilities.misc import debug_decorator as debug
 
-tmp = []
-iter = 0
-def maketree(f, *args, **kw):
-    """Only debugging purposes: prints a tree"""
-    global tmp
-    global iter
-    oldtmp = tmp
-    tmp = []
-    iter += 1
-
-    # If there is a bug and the algorithm enters an infinite loop, enable the
-    # following line. It will print the names and parameters of all major functions
-    # that are called, *before* they are called
-    #print "%s%s %s%s" % (iter, reduce(lambda x, y: x + y,map(lambda x: '-',range(1,2+iter))), f.func_name, args)
-
-    r = f(*args, **kw)
-
-    iter -= 1
-    s = "%s%s = %s\n" % (f.func_name, args, r)
-    if tmp != []:
-        s += tree(tmp)
-    tmp = oldtmp
-    tmp.append(s)
-    if iter == 0:
-        print tmp[0]
-        tmp = []
-    return r
 
 def compare(a, b, x):
     """Returns "<" if a<b, "=" for a == b, ">" for a>b"""
@@ -215,6 +149,7 @@ def compare(a, b, x):
     else:
         return "="
 
+
 class SubsSet(dict):
     """
     Stores (expr, dummy) pairs, and how to rewrite expr-s.
@@ -271,35 +206,37 @@ class SubsSet(dict):
         return dict.__getitem__(self, key)
 
     def do_subs(self, e):
-        for expr, var in self.iteritems():
+        for expr, var in self.items():
             e = e.subs(var, expr)
         return e
 
     def meets(self, s2):
         """Tell whether or not self and s2 have non-empty intersection"""
-        return set(self.keys()).intersection(s2.keys()) != set()
+        return set(self.keys()).intersection(list(s2.keys())) != set()
 
     def union(self, s2, exps=None):
         """Compute the union of self and s2, adjusting exps"""
         res = self.copy()
         tr = {}
-        for expr, var in s2.iteritems():
+        for expr, var in s2.items():
             if expr in self:
-                if exps: exps = exps.subs(var, res[expr])
+                if exps:
+                    exps = exps.subs(var, res[expr])
                 tr[var] = res[expr]
             else:
                 res[expr] = var
-        for var, rewr in s2.rewrites.iteritems():
+        for var, rewr in s2.rewrites.items():
             res.rewrites[var] = rewr.subs(tr)
         return res, exps
 
     def copy(self):
         r = SubsSet()
         r.rewrites = self.rewrites.copy()
-        for expr, var in self.iteritems():
+        for expr, var in self.items():
             r[expr] = var
         return r
 
+
 @debug
 def mrv(e, x):
     """Returns a SubsSet of most rapidly varying (mrv) subexpressions of 'e',
@@ -312,7 +249,7 @@ def mrv(e, x):
         s = SubsSet()
         return s, s[x]
     elif e.is_Mul or e.is_Add:
-        i, d = e.as_independent(x) # throw away x-independent terms
+        i, d = e.as_independent(x)  # throw away x-independent terms
         if d.func != e.func:
             s, expr = mrv(d, x)
             return s, e.func(i, expr)
@@ -353,13 +290,15 @@ def mrv(e, x):
             raise NotImplementedError("MRV set computation for functions in"
                                       " several variables not implemented.")
         s, ss = l2[0], SubsSet()
-        args = map(lambda x: ss.do_subs(x[1]), l)
+        args = [ss.do_subs(x[1]) for x in l]
         return s, e.func(*args)
     elif e.is_Derivative:
         raise NotImplementedError("MRV set computation for derviatives"
                                   " not implemented yet.")
         return mrv(e.args[0], x)
-    raise NotImplementedError("Don't know how to calculate the mrv of '%s'" % e)
+    raise NotImplementedError(
+        "Don't know how to calculate the mrv of '%s'" % e)
+
 
 def mrv_max3(f, expsf, g, expsg, union, expsboth, x):
     """Computes the maximum of two sets of expressions f and g, which
@@ -377,7 +316,7 @@ def mrv_max3(f, expsf, g, expsg, union, expsboth, x):
     elif f.meets(g):
         return union, expsboth
 
-    c = compare(f.keys()[0], g.keys()[0], x)
+    c = compare(list(f.keys())[0], list(g.keys())[0], x)
     if c == ">":
         return f, expsf
     elif c == "<":
@@ -386,6 +325,7 @@ def mrv_max3(f, expsf, g, expsg, union, expsboth, x):
         assert c == "="
         return union, expsboth
 
+
 def mrv_max1(f, g, exps, x):
     """Computes the maximum of two sets of expressions f and g, which
     are in the same comparability class, i.e. mrv_max1() compares (two elements of)
@@ -397,6 +337,7 @@ def mrv_max1(f, g, exps, x):
     return mrv_max3(f, g.do_subs(exps), g, f.do_subs(exps),
                     u, b, x)
 
+
 @debug
 @cacheit
 @timeit
@@ -446,12 +387,13 @@ def sign(e, x):
         if e.exp.is_Integer:
             return s**e.exp
     elif e.func is log:
-        return sign(e.args[0] -1, x)
+        return sign(e.args[0] - 1, x)
 
     # if all else fails, do it the hard way
     c0, e0 = mrv_leadterm(e, x)
     return sign(c0, x)
 
+
 @debug
 @timeit
 @cacheit
@@ -461,7 +403,9 @@ def limitinf(e, x):
     e = e.rewrite('tractable', deep=True)
 
     if not e.has(x):
-        return e #e is a constant
+        return e  # e is a constant
+    if e.has(Order):
+        e = e.expand().removeO()
     if not x.is_positive:
         # We make sure that x.is_positive is True so we
         # get all the correct mathematical bechavior from the expression.
@@ -472,8 +416,8 @@ def limitinf(e, x):
     c0, e0 = mrv_leadterm(e, x)
     sig = sign(e0, x)
     if sig == 1:
-        return S.Zero # e0>0: lim f = 0
-    elif sig == -1: #e0<0: lim f = +-oo (the sign depends on the sign of c0)
+        return S.Zero  # e0>0: lim f = 0
+    elif sig == -1:  # e0<0: lim f = +-oo (the sign depends on the sign of c0)
         if c0.match(I*Wild("a", exclude=[I])):
             return c0*oo
         s = sign(c0, x)
@@ -481,38 +425,39 @@ def limitinf(e, x):
         assert s != 0
         return s*oo
     elif sig == 0:
-        return limitinf(c0, x) #e0=0: lim f = lim c0
+        return limitinf(c0, x)  # e0=0: lim f = lim c0
+
 
 def moveup2(s, x):
     r = SubsSet()
-    for expr, var in s.iteritems():
+    for expr, var in s.items():
         r[expr.subs(x, exp(x))] = var
-    for var, expr in s.rewrites.iteritems():
+    for var, expr in s.rewrites.items():
         r.rewrites[var] = s.rewrites[var].subs(x, exp(x))
     return r
 
+
 def moveup(l, x):
     return [e.subs(x, exp(x)) for e in l]
 
 
 @debug
 @timeit
-def calculate_series(e, x, skip_abs=False, logx=None):
+def calculate_series(e, x, logx=None):
     """ Calculates at least one term of the series of "e" in "x".
 
     This is a place that fails most often, so it is in its own function.
     """
-    n = 1
-    while 1:
-        series = e.nseries(x, n=n, logx=logx)
-        if not series.has(O):
-            # The series expansion is locally exact.
-            return series
+    from sympy.polys import cancel
+
+    for t in e.lseries(x, logx=logx):
+        t = cancel(t)
+
+        if t:
+            break
+
+    return t
 
-        series = series.removeO()
-        if series and ((not skip_abs) or series.has(x)):
-            return series
-        n *= 2
 
 @debug
 @timeit
@@ -549,9 +494,9 @@ def mrv_leadterm(e, x):
     w = Dummy("w", real=True, positive=True, bounded=True)
     f, logw = rewrite(exps, Omega, x, w)
     series = calculate_series(f, w, logx=logw)
-    series = series.subs(log(w), logw) # this should not be necessary
     return series.leadterm(w)
 
+
 def build_expression_tree(Omega, rewrites):
     r""" Helper function for rewrite.
 
@@ -570,7 +515,7 @@ def build_expression_tree(Omega, rewrites):
     class Node:
         def ht(self):
             return reduce(lambda x, y: x + y,
-                          map(lambda x: x.ht(), self.before), 1)
+                          [x.ht() for x in self.before], 1)
     nodes = {}
     for expr, v in Omega:
         n = Node()
@@ -588,6 +533,7 @@ def build_expression_tree(Omega, rewrites):
 
     return nodes
 
+
 @debug
 @timeit
 def rewrite(e, Omega, x, wsym):
@@ -605,17 +551,19 @@ def rewrite(e, Omega, x, wsym):
     for t in Omega.keys():
         assert t.func is exp
     rewrites = Omega.rewrites
-    Omega = Omega.items()
+    Omega = list(Omega.items())
 
     nodes = build_expression_tree(Omega, rewrites)
     Omega.sort(key=lambda x: nodes[x[1]].ht(), reverse=True)
 
-    g, _ = Omega[-1] #g is going to be the "w" - the simplest one in the mrv set
-    sig = sign(g.args[0], x)
+    # make sure we know the sign of each exp() term; after the loop,
+    # g is going to be the "w" - the simplest one in the mrv set
+    for g, _ in Omega:
+        sig = sign(g.args[0], x)
+        if sig != 1 and sig != -1:
+            raise NotImplementedError('Result depends on the sign of %s' % sig)
     if sig == 1:
-        wsym = 1/wsym #if g goes to oo, substitute 1/w
-    elif sig != -1:
-        raise NotImplementedError('Result depends on the sign of %s' % sig)
+        wsym = 1/wsym  # if g goes to oo, substitute 1/w
     #O2 is a list, which results by rewriting each item in Omega using "w"
     O2 = []
     denominators = []
@@ -645,7 +593,7 @@ def rewrite(e, Omega, x, wsym):
     #finally compute the logarithm of w (logw).
     logw = g.args[0]
     if sig == 1:
-        logw = -logw     #log(w)->log(1/w)=-log(w)
+        logw = -logw  # log(w)->log(1/w)=-log(w)
 
     # Some parts of sympy have difficulty computing series expansions with
     # non-integral exponents. The following heuristic improves the situation:
@@ -655,6 +603,7 @@ def rewrite(e, Omega, x, wsym):
 
     return f, logw
 
+
 def gruntz(e, z, z0, dir="+"):
     """
     Compute the limit of e(z) at the point z0 using the Gruntz algorithm.
