@@ -3,9 +3,10 @@ This module provides convenient functions to transform sympy expressions to
 lambda functions which can be used to calculate numerical values very fast.
 """
 
-from __future__ import division
+from __future__ import print_function, division
+
 from sympy.external import import_module
-from sympy.core.compatibility import is_sequence
+from sympy.core.compatibility import exec_, is_sequence, iterable, string_types
 
 import inspect
 
@@ -26,64 +27,74 @@ SYMPY_DEFAULT = {}
 
 # Mappings between sympy and other modules function names.
 MATH_TRANSLATIONS = {
-    "Abs":"fabs",
-    "ceiling":"ceil",
-    "E":"e",
-    "ln":"log",
+    "Abs": "fabs",
+    "ceiling": "ceil",
+    "E": "e",
+    "ln": "log",
 }
 
 MPMATH_TRANSLATIONS = {
-    "ceiling":"ceil",
-    "chebyshevt":"chebyt",
-    "chebyshevu":"chebyu",
-    "E":"e",
-    "I":"j",
-    "ln":"log",
+    "elliptic_k": "ellipk",
+    "elliptic_f": "ellipf",
+    "elliptic_e": "ellipe",
+    "elliptic_pi": "ellippi",
+    "ceiling": "ceil",
+    "chebyshevt": "chebyt",
+    "chebyshevu": "chebyu",
+    "E": "e",
+    "I": "j",
+    "ln": "log",
     #"lowergamma":"lower_gamma",
-    "oo":"inf",
+    "oo": "inf",
     #"uppergamma":"upper_gamma",
-    "LambertW":"lambertw",
-    "Matrix":"matrix",
-    "conjugate":"conj",
-    "dirichlet_eta":"altzeta",
-    "Ei":"ei",
-    "Shi":"shi",
-    "Chi":"chi",
-    "Si":"si",
-    "Ci":"ci"
+    "LambertW": "lambertw",
+    "Matrix": "matrix",
+    "MutableDenseMatrix": "matrix",
+    "ImmutableMatrix": "matrix",
+    "conjugate": "conj",
+    "dirichlet_eta": "altzeta",
+    "Ei": "ei",
+    "Shi": "shi",
+    "Chi": "chi",
+    "Si": "si",
+    "Ci": "ci"
 }
 
 NUMPY_TRANSLATIONS = {
-    "Abs":"abs",
-    "acos":"arccos",
-    "acosh":"arccosh",
-    "arg":"angle",
-    "asin":"arcsin",
-    "asinh":"arcsinh",
-    "atan":"arctan",
-    "atan2":"arctan2",
-    "atanh":"arctanh",
-    "ceiling":"ceil",
-    "E":"e",
-    "im":"imag",
-    "ln":"log",
-    "Matrix":"matrix",
-    "Max":"amax",
-    "Min":"amin",
-    "oo":"inf",
-    "re":"real",
+    "Abs": "abs",
+    "acos": "arccos",
+    "acosh": "arccosh",
+    "arg": "angle",
+    "asin": "arcsin",
+    "asinh": "arcsinh",
+    "atan": "arctan",
+    "atan2": "arctan2",
+    "atanh": "arctanh",
+    "ceiling": "ceil",
+    "E": "e",
+    "im": "imag",
+    "ln": "log",
+    "Matrix": "matrix",
+    "MutableDenseMatrix": "matrix",
+    "ImmutableMatrix": "matrix",
+    "Max": "amax",
+    "Min": "amin",
+    "oo": "inf",
+    "re": "real",
 }
 
 # Available modules:
 MODULES = {
-    "math"   : (MATH,   MATH_DEFAULT,   MATH_TRANSLATIONS,   ("from math import *",)),
-    "mpmath" : (MPMATH, MPMATH_DEFAULT, MPMATH_TRANSLATIONS, ("from sympy.mpmath import *",)),
-    "numpy"  : (NUMPY,  NUMPY_DEFAULT,  NUMPY_TRANSLATIONS,  ("import_module('numpy')",)),
-    "sympy"  : (SYMPY,  SYMPY_DEFAULT,  {},                  ("from sympy.functions import *",
-                                                              "from sympy.matrices import Matrix",
-                                                              "from sympy import Integral, pi, oo, nan, zoo, E, I",)),
+    "math": (MATH, MATH_DEFAULT, MATH_TRANSLATIONS, ("from math import *",)),
+    "mpmath": (MPMATH, MPMATH_DEFAULT, MPMATH_TRANSLATIONS, ("from sympy.mpmath import *",)),
+    "numpy": (NUMPY, NUMPY_DEFAULT, NUMPY_TRANSLATIONS, ("import_module('numpy')",)),
+    "sympy": (SYMPY, SYMPY_DEFAULT, {}, (
+        "from sympy.functions import *",
+        "from sympy.matrices import *",
+        "from sympy import Integral, pi, oo, nan, zoo, E, I",)),
 }
 
+
 def _import(module, reload="False"):
     """
     Creates a global translation dictionary for module.
@@ -94,9 +105,11 @@ def _import(module, reload="False"):
     other modules.
     """
     try:
-        namespace, namespace_default, translations, import_commands = MODULES[module]
+        namespace, namespace_default, translations, import_commands = MODULES[
+            module]
     except KeyError:
-        raise NameError("'%s' module can't be used for lambdification" % module)
+        raise NameError(
+            "'%s' module can't be used for lambdification" % module)
 
     # Clear namespace or exit
     if namespace != namespace_default:
@@ -116,53 +129,43 @@ def _import(module, reload="False"):
                 continue
         else:
             try:
-                exec import_command in {}, namespace
+                exec_(import_command, {}, namespace)
                 continue
             except ImportError:
                 pass
 
-        raise ImportError("can't import '%s' with '%s' command" % (module, import_command))
+        raise ImportError(
+            "can't import '%s' with '%s' command" % (module, import_command))
 
     # Add translated names to namespace
-    for sympyname, translation in translations.iteritems():
+    for sympyname, translation in translations.items():
         namespace[sympyname] = namespace[translation]
 
+
 def lambdify(args, expr, modules=None, printer=None, use_imps=True):
     """
     Returns a lambda function for fast calculation of numerical values.
 
-    Usage:
-
-    >>> from sympy import sqrt, sin
-    >>> from sympy.utilities.lambdify import lambdify
-    >>> from sympy.abc import x, y, z
-    >>> f = lambdify(x, x**2)
-    >>> f(2)
-    4
-    >>> f = lambdify((x,y,z), [z,y,x])
-    >>> f(1,2,3)
-    [3, 2, 1]
-    >>> f = lambdify(x, sqrt(x))
-    >>> f(4)
-    2.0
-    >>> f = lambdify((x,y), sin(x*y)**2)
-    >>> f(0, 5)
-    0.0
-
     If not specified differently by the user, SymPy functions are replaced as
     far as possible by either python-math, numpy (if available) or mpmath
-    functions - exactly in this order.
-    To change this behavior, the "modules" argument can be used.
-    It accepts:
+    functions - exactly in this order. To change this behavior, the "modules"
+    argument can be used. It accepts:
 
      - the strings "math", "mpmath", "numpy", "sympy"
      - any modules (e.g. math)
      - dictionaries that map names of sympy functions to arbitrary functions
-     - lists that contain a mix of the arguments above. (Entries that are first
-        in the list have higher priority)
+     - lists that contain a mix of the arguments above, with higher priority
+       given to entries appearing first.
 
-    Examples
-    ========
+    The default behavior is to substitute all arguments in the provided
+    expression with dummy symbols. This allows for applied functions (e.g.
+    f(t)) to be supplied as arguments. Call the function with dummify=False if
+    dummy substitution is unwanted.
+    If you want to view the lambdified function or provide "sympy" as the
+    module, you should probably set dummify=False.
+
+    Usage
+    =====
 
     (1) Use one of the provided modules:
 
@@ -201,27 +204,49 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True):
 
         >> lambda x: my_cool_function(x)
 
+    Examples
+    ========
+
+    >>> from sympy.utilities.lambdify import implemented_function, lambdify
+    >>> from sympy import sqrt, sin, Matrix
+    >>> from sympy import Function
+    >>> from sympy.abc import x, y, z
+
+    >>> f = lambdify(x, x**2)
+    >>> f(2)
+    4
+    >>> f = lambdify((x, y, z), [z, y, x])
+    >>> f(1,2,3)
+    [3, 2, 1]
+    >>> f = lambdify(x, sqrt(x))
+    >>> f(4)
+    2.0
+    >>> f = lambdify((x, y), sin(x*y)**2)
+    >>> f(0, 5)
+    0.0
+    >>> f = lambdify((x, y), Matrix((x, x + y)).T, modules='sympy')
+    >>> f(1, 2)
+    Matrix([[1, 3]])
+
     Functions present in `expr` can also carry their own numerical
     implementations, in a callable attached to the ``_imp_``
     attribute.  Usually you attach this using the
     ``implemented_function`` factory:
 
-    >>> from sympy.abc import x, y, z
-    >>> from sympy.utilities.lambdify import lambdify, implemented_function
-    >>> from sympy import Function
     >>> f = implemented_function(Function('f'), lambda x: x+1)
     >>> func = lambdify(x, f(x))
     >>> func(4)
     5
 
-    ``lambdify`` always prefers ``_imp_`` implementations to
-    implementations in other namespaces, unless the ``use_imps`` input
-    parameter is False.
+    ``lambdify`` always prefers ``_imp_`` implementations to implementations
+    in other namespaces, unless the ``use_imps`` input parameter is False.
     """
     from sympy.core.symbol import Symbol
 
     # If the user hasn't specified any modules, use what is available.
+    module_provided = True
     if modules is None:
+        module_provided = False
         # Use either numpy (if available) or python.math where possible.
         # XXX: This leads to different behaviour on different systems and
         #      might be the reason for irreproducible errors.
@@ -234,6 +259,7 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True):
         else:
             modules.insert(1, "numpy")
 
+
     # Get the needed namespaces.
     namespaces = []
     # First find any function implementations
@@ -258,10 +284,10 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True):
             namespace.update({str(term): term})
 
     # Create lambda function.
-    lstr = lambdastr(args, expr, printer=printer)
-
+    lstr = lambdastr(args, expr, printer=printer, dummify=True)
     return eval(lstr, namespace)
 
+
 def _get_namespace(m):
     """
     This is used by _lambdify to parse its arguments.
@@ -276,7 +302,8 @@ def _get_namespace(m):
     else:
         raise TypeError("Argument must be either a string, dict or module but it is: %s" % m)
 
-def lambdastr(args, expr, printer=None):
+
+def lambdastr(args, expr, printer=None, dummify=False):
     """
     Returns a string that can be evaluated to a lambda function.
 
@@ -288,6 +315,10 @@ def lambdastr(args, expr, printer=None):
     'lambda x,y,z: ([z, y, x])'
 
     """
+    # Transforming everything to strings.
+    from sympy.matrices import DeferredVector
+    from sympy import Dummy, sympify, Symbol, Function
+
     if printer is not None:
         if inspect.isfunction(printer):
             lambdarepr = printer
@@ -300,17 +331,61 @@ def lambdastr(args, expr, printer=None):
         #XXX: This has to be done here because of circular imports
         from sympy.printing.lambdarepr import lambdarepr
 
-    # Transform everything to strings.
-    expr = lambdarepr(expr)
-    if isinstance(args, str):
-        pass
-    elif hasattr(args, "__iter__"):
-        args = ",".join(str(a) for a in args)
+    def sub_args(args, dummies_dict):
+        if isinstance(args, str):
+            return args
+        elif isinstance(args, DeferredVector):
+            return str(args)
+        elif iterable(args):
+            flatten = lambda *n: (e for a in n for e in
+                                  (flatten(*a) if iterable(a) else (a,)))
+            dummies = flatten([sub_args(a, dummies_dict) for a in args])
+            return ",".join(str(a) for a in dummies)
+        else:
+            if isinstance(args, Function):
+                dummies = Dummy()
+                dummies_dict.update({args : dummies})
+                return str(dummies)
+            else:
+                return str(args)
+
+    def sub_expr(expr, dummies_dict):
+        try:
+            expr = sympify(expr).xreplace(dummies_dict)
+        except:
+            if isinstance(expr, DeferredVector):
+                pass
+            elif isinstance(expr, dict):
+                k = [sub_expr(sympify(a), dummies_dict) for a in expr.keys()]
+                v = [sub_expr(sympify(a), dummies_dict) for a in expr.values()]
+                expr = dict(zip(k, v))
+            elif isinstance(expr, tuple):
+                expr = tuple(sub_expr(sympify(a), dummies_dict) for a in expr)
+            elif isinstance(expr, list):
+                expr = [sub_expr(sympify(a), dummies_dict) for a in expr]
+        return expr
+
+    # Transform args
+    dummies_dict = {}
+    if dummify:
+        args = sub_args(args, dummies_dict)
     else:
-        args = str(args)
+        if isinstance(args, str):
+            pass
+        elif iterable(args, exclude=DeferredVector):
+            args = ",".join(str(a) for a in args)
+
+    # Transform expr
+    if dummify:
+        if isinstance(expr, str):
+            pass
+        else:
+            expr = sub_expr(expr, dummies_dict)
+    expr = lambdarepr(expr)
 
     return "lambda %s: (%s)" % (args, expr)
 
+
 def _imp_namespace(expr, namespace=None):
     """ Return namespace dict with function implementations
 
@@ -336,7 +411,7 @@ def _imp_namespace(expr, namespace=None):
 
     Examples
     --------
-    >>> from sympy.abc import x, y, z
+    >>> from sympy.abc import x
     >>> from sympy.utilities.lambdify import implemented_function, _imp_namespace
     >>> from sympy import Function
     >>> f = implemented_function(Function('f'), lambda x: x+1)
@@ -377,6 +452,7 @@ def _imp_namespace(expr, namespace=None):
             _imp_namespace(arg, namespace)
     return namespace
 
+
 def implemented_function(symfunc, implementation):
     """ Add numerical ``implementation`` to function ``symfunc``.
 
@@ -404,7 +480,7 @@ def implemented_function(symfunc, implementation):
 
     Examples
     --------
-    >>> from sympy.abc import x, y, z
+    >>> from sympy.abc import x
     >>> from sympy.utilities.lambdify import lambdify, implemented_function
     >>> from sympy import Function
     >>> f = implemented_function(Function('f'), lambda x: x+1)
@@ -415,7 +491,7 @@ def implemented_function(symfunc, implementation):
     # Delayed import to avoid circular imports
     from sympy.core.function import UndefinedFunction
     # if name, create function to hold implementation
-    if isinstance(symfunc, basestring):
+    if isinstance(symfunc, string_types):
         symfunc = UndefinedFunction(symfunc)
     elif not isinstance(symfunc, UndefinedFunction):
         raise ValueError('symfunc should be either a string or'
