@@ -1,56 +1,69 @@
 from sympy.utilities.pytest import XFAIL, raises
-from sympy import (symbols, lambdify, sqrt, sin, cos, pi, atan, Rational, Float,
-        Matrix, Lambda, exp, Integral, oo, I, Abs)
+from sympy import (
+    symbols, lambdify, sqrt, sin, cos, pi, atan, Rational, Float,
+    Matrix, Lambda, exp, Integral, oo, I, Abs, Function, true, false)
 from sympy.printing.lambdarepr import LambdaPrinter
 from sympy import mpmath
 from sympy.utilities.lambdify import implemented_function
 from sympy.utilities.pytest import skip
 from sympy.utilities.decorator import conserve_mpmath_dps
 from sympy.external import import_module
-import math, sympy
+import math
+import sympy
 
-numpy = import_module('numpy', min_python_version=(2, 6))
+MutableDenseMatrix = Matrix
+
+numpy = import_module('numpy')
+
+x, y, z = symbols('x,y,z')
+
+#================== Test different arguments =======================
 
-x,y,z = symbols('x,y,z')
 
-#================== Test different arguments ==============
 def test_no_args():
     f = lambdify([], 1)
     raises(TypeError, lambda: f(-1))
     assert f() == 1
 
+
 def test_single_arg():
     f = lambdify(x, 2*x)
     assert f(1) == 2
 
+
 def test_list_args():
-    f = lambdify([x,y], x+y)
-    assert f(1,2) == 3
+    f = lambdify([x, y], x + y)
+    assert f(1, 2) == 3
+
 
 def test_str_args():
     f = lambdify('x,y,z', 'z,y,x')
-    assert f(3,2,1) == (1,2,3)
-    assert f(1.0,2.0,3.0) == (3.0,2.0,1.0)
+    assert f(3, 2, 1) == (1, 2, 3)
+    assert f(1.0, 2.0, 3.0) == (3.0, 2.0, 1.0)
     # make sure correct number of args required
     raises(TypeError, lambda: f(0))
 
+
 def test_own_namespace():
-    myfunc = lambda x:1
-    f = lambdify(x, sin(x), {"sin":myfunc})
+    myfunc = lambda x: 1
+    f = lambdify(x, sin(x), {"sin": myfunc})
     assert f(0.1) == 1
     assert f(100) == 1
 
+
 def test_own_module():
     f = lambdify(x, sin(x), math)
-    assert f(0)==0.0
+    assert f(0) == 0.0
     f = lambdify(x, sympy.ceiling(x), math)
     raises(NameError, lambda: f(4.5))
 
+
 def test_bad_args():
     # no vargs given
     raises(TypeError, lambda: lambdify(1))
     # same with vector exprs
-    raises(TypeError, lambda: lambdify([1,2]))
+    raises(TypeError, lambda: lambdify([1, 2]))
+
 
 def test_atoms():
     # Non-Symbol atoms should not be pulled out from the expression namespace
@@ -59,9 +72,11 @@ def test_atoms():
     f = lambdify(x, I + x, {"I": 1j})
     assert f(1) == 1 + 1j
 
-#================== Test different modules ================
+#================== Test different modules =========================
 
 # high precision output of sin(0.2*pi) is used to detect if precision is lost unwanted
+
+
 @conserve_mpmath_dps
 def test_sympy_lambda():
     mpmath.mp.dps = 50
@@ -69,10 +84,11 @@ def test_sympy_lambda():
     f = lambdify(x, sin(x), "sympy")
     assert f(x) == sin(x)
     prec = 1e-15
-    assert -prec < f(Rational(1,5)).evalf() - Float(str(sin02)) < prec
+    assert -prec < f(Rational(1, 5)).evalf() - Float(str(sin02)) < prec
     # arctan is in numpy module and should not be available
     raises(NameError, lambda: lambdify(x, arctan(x), "sympy"))
 
+
 @conserve_mpmath_dps
 def test_math_lambda():
     mpmath.mp.dps = 50
@@ -80,16 +96,20 @@ def test_math_lambda():
     f = lambdify(x, sin(x), "math")
     prec = 1e-15
     assert -prec < f(0.2) - sin02 < prec
-    raises(ValueError, lambda: f(x)) # if this succeeds, it can't be a python math function
+    raises(ValueError, lambda: f(x))
+           # if this succeeds, it can't be a python math function
+
 
 @conserve_mpmath_dps
 def test_mpmath_lambda():
     mpmath.mp.dps = 50
     sin02 = mpmath.mpf("0.19866933079506121545941262711838975037020672954020")
     f = lambdify(x, sin(x), "mpmath")
-    prec = 1e-49 # mpmath precision is around 50 decimal places
+    prec = 1e-49  # mpmath precision is around 50 decimal places
     assert -prec < f(mpmath.mpf("0.2")) - sin02 < prec
-    raises(TypeError, lambda: f(x)) # if this succeeds, it can't be a mpmath function
+    raises(TypeError, lambda: f(x))
+           # if this succeeds, it can't be a mpmath function
+
 
 @conserve_mpmath_dps
 @XFAIL
@@ -97,43 +117,49 @@ def test_number_precision():
     mpmath.mp.dps = 50
     sin02 = mpmath.mpf("0.19866933079506121545941262711838975037020672954020")
     f = lambdify(x, sin02, "mpmath")
-    prec = 1e-49 # mpmath precision is around 50 decimal places
+    prec = 1e-49  # mpmath precision is around 50 decimal places
     assert -prec < f(0) - sin02 < prec
 
-#================== Test Translations =====================
+#================== Test Translations ==============================
 # We can only check if all translated functions are valid. It has to be checked
 # by hand if they are complete.
 
+
 def test_math_transl():
     from sympy.utilities.lambdify import MATH_TRANSLATIONS
-    for sym, mat in MATH_TRANSLATIONS.iteritems():
+    for sym, mat in MATH_TRANSLATIONS.items():
         assert sym in sympy.__dict__
         assert mat in math.__dict__
 
+
 def test_mpmath_transl():
     from sympy.utilities.lambdify import MPMATH_TRANSLATIONS
-    for sym, mat in MPMATH_TRANSLATIONS.iteritems():
+    for sym, mat in MPMATH_TRANSLATIONS.items():
         assert sym in sympy.__dict__ or sym == 'Matrix'
         assert mat in mpmath.__dict__
 
+
 def test_numpy_transl():
     if not numpy:
-        skip("numpy not installed or Python too old.")
+        skip("numpy not installed.")
 
     from sympy.utilities.lambdify import NUMPY_TRANSLATIONS
-    for sym, nump in NUMPY_TRANSLATIONS.iteritems():
+    for sym, nump in NUMPY_TRANSLATIONS.items():
         assert sym in sympy.__dict__
         assert nump in numpy.__dict__
 
+
 def test_numpy_translation_abs():
     if not numpy:
-        skip("numpy not installed or Python too old.")
+        skip("numpy not installed.")
 
     f = lambdify(x, Abs(x), "numpy")
     assert f(-1) == 1
     assert f(1) == 1
 
-#================== Test some functions ===================
+#================== Test some functions ============================
+
+
 def test_exponentiation():
     f = lambdify(x, x**2)
     assert f(-1) == 1
@@ -143,6 +169,7 @@ def test_exponentiation():
     assert f(2) == 4
     assert f(2.5) == 6.25
 
+
 def test_sqrt():
     f = lambdify(x, sqrt(x))
     assert f(0) == 0.0
@@ -151,25 +178,29 @@ def test_sqrt():
     assert abs(f(2) - 1.414) < 0.001
     assert f(6.25) == 2.5
 
+
 def test_trig():
-    f = lambdify([x], [cos(x),sin(x)])
+    f = lambdify([x], [cos(x), sin(x)])
     d = f(pi)
     prec = 1e-11
-    assert -prec < d[0]+1 < prec
+    assert -prec < d[0] + 1 < prec
     assert -prec < d[1] < prec
     d = f(3.14159)
     prec = 1e-5
-    assert -prec < d[0]+1 < prec
+    assert -prec < d[0] + 1 < prec
     assert -prec < d[1] < prec
 
-#================== Test vectors ==========================
+#================== Test vectors ===================================
+
+
 def test_vector_simple():
-    f = lambdify((x,y,z), (z,y,x))
-    assert f(3,2,1) == (1,2,3)
-    assert f(1.0,2.0,3.0) == (3.0,2.0,1.0)
+    f = lambdify((x, y, z), (z, y, x))
+    assert f(3, 2, 1) == (1, 2, 3)
+    assert f(1.0, 2.0, 3.0) == (3.0, 2.0, 1.0)
     # make sure correct number of args required
     raises(TypeError, lambda: f(0))
 
+
 def test_vector_discontinuous():
     f = lambdify(x, (-1/x, 1/x))
     raises(ZeroDivisionError, lambda: f(0))
@@ -177,65 +208,82 @@ def test_vector_discontinuous():
     assert f(2) == (-0.5, 0.5)
     assert f(-2) == (0.5, -0.5)
 
+
 def test_trig_symbolic():
-    f = lambdify([x], [cos(x),sin(x)])
+    f = lambdify([x], [cos(x), sin(x)])
     d = f(pi)
-    assert abs(d[0]+1) < 0.0001
-    assert abs(d[1]-0) < 0.0001
+    assert abs(d[0] + 1) < 0.0001
+    assert abs(d[1] - 0) < 0.0001
+
 
 def test_trig_float():
-    f = lambdify([x], [cos(x),sin(x)])
+    f = lambdify([x], [cos(x), sin(x)])
     d = f(3.14159)
-    assert abs(d[0]+1) < 0.0001
-    assert abs(d[1]-0) < 0.0001
+    assert abs(d[0] + 1) < 0.0001
+    assert abs(d[1] - 0) < 0.0001
+
 
 def test_docs():
     f = lambdify(x, x**2)
     assert f(2) == 4
-    f = lambdify([x,y,z], [z,y,x])
+    f = lambdify([x, y, z], [z, y, x])
     assert f(1, 2, 3) == [3, 2, 1]
     f = lambdify(x, sqrt(x))
     assert f(4) == 2.0
-    f = lambdify((x,y), sin(x*y)**2)
+    f = lambdify((x, y), sin(x*y)**2)
     assert f(0, 5) == 0
 
+
 def test_math():
     f = lambdify((x, y), sin(x), modules="math")
     assert f(0, 5) == 0
 
+
 def test_sin():
     f = lambdify(x, sin(x)**2)
     assert isinstance(f(2), float)
     f = lambdify(x, sin(x)**2, modules="math")
     assert isinstance(f(2), float)
 
+
 def test_matrix():
-    A = Matrix([[x, x*y], [sin(z)+4, x**z]])
-    sol = Matrix([[1, 2], [sin(3)+4, 1]])
-    f = lambdify((x,y,z), A, modules="sympy")
-    assert f(1,2,3) == sol
-    f = lambdify((x,y,z), (A, [A]), modules="sympy")
-    assert f(1,2,3) == (sol,[sol])
+    A = Matrix([[x, x*y], [sin(z) + 4, x**z]])
+    sol = Matrix([[1, 2], [sin(3) + 4, 1]])
+    f = lambdify((x, y, z), A, modules="sympy")
+    assert f(1, 2, 3) == sol
+    f = lambdify((x, y, z), (A, [A]), modules="sympy")
+    assert f(1, 2, 3) == (sol, [sol])
+    J = Matrix((x, x + y)).jacobian((x, y))
+    v = Matrix((x, y))
+    sol = Matrix([[1, 0], [1, 1]])
+    assert lambdify(v, J, modules='sympy')(1, 2) == sol
+    assert lambdify(v.T, J, modules='sympy')(1, 2) == sol
+
 
 def test_integral():
     f = Lambda(x, exp(-x**2))
     l = lambdify(x, Integral(f(x), (x, -oo, oo)), modules="sympy")
     assert l(x) == Integral(exp(-x**2), (x, -oo, oo))
 
-#########Test Symbolic###########
+#================== Test symbolic ==================================
+
+
 def test_sym_single_arg():
     f = lambdify(x, x * y)
     assert f(z) == z * y
 
+
 def test_sym_list_args():
-    f = lambdify([x,y], x + y + z)
-    assert f(1,2) == 3 + z
+    f = lambdify([x, y], x + y + z)
+    assert f(1, 2) == 3 + z
+
 
 def test_sym_integral():
     f = Lambda(x, exp(-x**2))
     l = lambdify(x, Integral(f(x), (x, -oo, oo)), modules="sympy")
     assert l(y).doit() == sqrt(pi)
 
+
 def test_namespace_order():
     # lambdify had a bug, such that module dictionaries or cached module
     # dictionaries would pull earlier namespaces into themselves.
@@ -243,9 +291,9 @@ def test_namespace_order():
     # generated lambda, this meant that the behavior of a previously
     # generated lambda function could change as a result of later calls
     # to lambdify.
-    n1 = {'f': lambda x:'first f'}
-    n2 = {'f': lambda x:'second f',
-          'g': lambda x:'function g'}
+    n1 = {'f': lambda x: 'first f'}
+    n2 = {'f': lambda x: 'second f',
+          'g': lambda x: 'function g'}
     f = sympy.Function('f')
     g = sympy.Function('g')
     if1 = lambdify(x, f(x), modules=(n1, "sympy"))
@@ -254,6 +302,7 @@ def test_namespace_order():
     # previously gave 'second f'
     assert if1(1) == 'first f'
 
+
 def test_imps():
     # Here we check if the default returned functions are anonymous - in
     # the sense that we can have more than one function with the same name
@@ -273,8 +322,10 @@ def test_imps():
     f2 = implemented_function("f", lambda x: x + 101)
     raises(ValueError, lambda: lambdify(x, f(f2(x))))
 
+
 def test_imps_wrong_args():
-    raises(ValueError, lambda: implemented_function(sin, lambda x:x))
+    raises(ValueError, lambda: implemented_function(sin, lambda x: x))
+
 
 def test_lambdify_imps():
     # Test lambdify with implemented functions
@@ -309,7 +360,23 @@ def test_lambdify_imps():
     lam = lambdify(x, f(x), d, use_imps=False)
     assert lam(3) == 102
 
+def test_dummification():
+    t = symbols('t')
+    F = Function('F')
+    G = Function('G')
+    some_expr = 2 * F(t)**2 / G(t)
+    lam = lambdify((F(t), G(t)), some_expr)
+    assert lam(3, 9) == 2
+    lam = lambdify(sin(t), 2 * sin(t)**2)
+    assert lam(F(t)) == 2 * F(t)**2
+    raises(SyntaxError, lambda: lambdify(F(t) * G(t), F(t) * G(t) + 5))
+    raises(SyntaxError, lambda: lambdify(2 * F(t), 2 * F(t) + 5))
+    raises(SyntaxError, lambda: lambdify(2 * F(t), 4 * F(t) + 5))
+
+
 #================== Test special printers ==========================
+
+
 def test_special_printers():
     class IntervalPrinter(LambdaPrinter):
         """Use ``lambda`` printer but print numbers as ``mpi`` intervals. """
@@ -334,3 +401,8 @@ def test_special_printers():
     assert isinstance(func0(), mpi)
     assert isinstance(func1(), mpi)
     assert isinstance(func2(), mpi)
+
+def test_true_false():
+    # We want exact is comparison here, not just ==
+    assert lambdify([], true)() is True
+    assert lambdify([], false)() is False
